@@ -1,3 +1,4 @@
+
 /* Dia -- an diagram creation/manipulation program
  * Copyright (C) 1998 Alexander Larsson
  *
@@ -30,10 +31,6 @@
 #include <unistd.h>
 #endif
 
-#ifdef GNOME
-#undef GTK_DISABLE_DEPRECATED /* gtk_signal_connect[_object] */
-#endif
-
 #include <gtk/gtk.h>
 #include <gmodule.h>
 
@@ -43,10 +40,18 @@
 
 #ifdef GNOME
 #include <gnome.h>
-#else
-#ifdef HAVE_POPT_H
-#include <popt.h>
+#else 
+#  ifdef HAVE_POPT_H
+#    include <popt.h>
+#  else
+/* sorry about the mess, but one should not use conditional defined types in 
+ * unconditional function call in the first place ... --hb */
+typedef void* poptContext;
+#  endif
 #endif
+
+#ifdef HAVE_FREETYPE
+#include <pango/pangoft2.h>
 #endif
 
 #include <libxml/parser.h>
@@ -81,6 +86,7 @@
 #include "dia_image.h"
 #include "persistence.h"
 #include "sheets.h"
+#include "utils.h"
 
 #if defined(HAVE_LIBPNG) && defined(HAVE_LIBART)
 extern DiaExportFilter png_export_filter;
@@ -89,12 +95,51 @@ extern DiaExportFilter png_export_filter;
 static gboolean
 handle_initial_diagram(const char *input_file_name, 
 		       const char *export_file_name,
-		       const char *export_file_format);
+		       const char *export_file_format,
+		       const char *size);
 
 static void create_user_dirs(void);
 static PluginInitResult internal_plugin_init(PluginInfo *info);
+static void process_opts(int argc, char **argv,
+#ifdef HAVE_POPT
+			 poptContext poptCtx, struct poptOption options[],
+#endif
+			 GSList **files, char **export_file_name,
+			 char **export_file_format, char **size);
+static gboolean handle_all_diagrams(GSList *files, char *export_file_name,
+				    char *export_file_format, char *size);
+static void print_credits(gboolean credits);
 
 static gboolean dia_is_interactive = TRUE;
+static void
+stderr_message_internal(char *title, const char *fmt,
+                        va_list *args,  va_list *args2);
+
+static void
+stderr_message_internal(char *title, const char *fmt,
+                        va_list *args,  va_list *args2)
+{
+  static gchar *buf = NULL;
+  static gint   alloc = 0;
+  gint len;
+
+  len = format_string_length_upper_bound (fmt, args);
+
+  if (len >= alloc) {
+    if (buf)
+      g_free (buf);
+    
+    alloc = nearest_pow (MAX(len + 1, 1024));
+    
+    buf = g_new (char, alloc);
+  }
+  
+  vsprintf (buf, fmt, *args2);
+  
+  fprintf(stderr,
+          "%s: %s\n", 
+          title,buf);
+}
 
 #ifdef GNOME
 
@@ -154,10 +199,12 @@ const char *argv0 = NULL;
 
 /** Convert infname to outfname, using input filter inf and export filter
  * ef.  If either is null, try to guess them.
+ * size might be NULL.
  */
 gboolean
 do_convert(const char *infname, 
-	   const char *outfname, DiaExportFilter  *ef)
+	   const char *outfname, DiaExportFilter *ef,
+	   const char *size)
 {
   DiaImportFilter *inf;
   DiagramData *diagdata = NULL;
@@ -169,8 +216,7 @@ do_convert(const char *infname,
   if (ef == NULL) {
     ef = filter_guess_export_filter(outfname);
     if (!ef) {
-      fprintf(stderr,
-	      _("%s error: don't know how to export into %s\n"),
+      g_error(_("%s error: don't know how to export into %s\n"),
 	      argv0,outfname);
       exit(1);
     }
@@ -179,20 +225,32 @@ do_convert(const char *infname,
   dia_is_interactive = FALSE;
 
   if (0==strcmp(infname,outfname)) {
-    fprintf(stderr,
-            _("%s error: input and output file name is identical: %s"),
+    g_error(_("%s error: input and output file name is identical: %s"),
             argv0, infname);
     exit(1);
   }
   
   diagdata = new_diagram_data(&prefs.new_diagram);
-  if (!inf->import(infname,diagdata,inf->user_data)) {
-    fprintf(stderr,
-            _("%s error: need valid input file %s\n"),
-            argv0,infname);
-            exit(1);
+  if (!inf->import_func(infname,diagdata,inf->user_data)) {
+    g_error(_("%s error: need valid input file %s\n"),
+            argv0, infname);
+    exit(1);
   }
-  ef->export(diagdata, outfname, infname, ef->user_data);
+  /* Do our best in providing the size to the filter, but don't abuse user_data 
+   * too much for it. It _must not_ be changed after initialization and there 
+   * are quite some filter selecting their output format by it. --hb
+   */
+  if (size) {
+    if (ef == filter_get_by_name ("png-libart"))
+      ef->export_func(diagdata, outfname, infname, size);
+    else {
+      g_warning ("--size parameter unsupported for %s filter", 
+                 ef->unique_name ? ef->unique_name : "selected");
+      ef->export_func(diagdata, outfname, infname, ef->user_data);
+    }
+  }
+  else
+    ef->export_func(diagdata, outfname, infname, ef->user_data);
   /* if (!quiet) */ fprintf(stdout,
                       _("%s --> %s\n"),
                         infname,outfname);
@@ -217,35 +275,54 @@ app_is_interactive(void)
 
 /** Handle loading of diagrams given on command line, including conversions.
  * Returns TRUE if any automatic conversions were performed.
+ * Note to future hackers:  'size' is currently the only argument that can be
+ * sent to exporters.  If more arguments are desired, please don't just add
+ * even more arguments, but create a more general system.
  */
 static gboolean
 handle_initial_diagram(const char *in_file_name, 
 		       const char *out_file_name,
-		       const char *export_file_format) {
+		       const char *export_file_format,
+		       const char *size) {
   DDisplay *ddisp = NULL;
   Diagram *diagram = NULL;
   gboolean made_conversions = FALSE;
 
   if (export_file_format) {
     char *export_file_name = NULL;
-    DiaExportFilter *ef;
+    DiaExportFilter *ef = NULL;
+
     /* First try guessing based on extension */
     export_file_name = build_output_file_name(in_file_name,
 					      export_file_format);
-    ef = filter_guess_export_filter(export_file_name);
+
+    /* to make the --size hack even uglier but work again for the only filter supporting it */
+    if (   size && strcmp(export_file_format, "png") == 0)
+      ef = filter_get_by_name ("png-libart");
+    if (!ef)
+      ef = filter_guess_export_filter(export_file_name);
     if (ef == NULL) {
       ef = filter_get_by_name(export_file_format);
       if (ef == NULL) {
 	g_error(_("Can't find output format %s\n"), export_file_format);
 	return FALSE;
       }
+      g_free (export_file_name);
       export_file_name = build_output_file_name(in_file_name,
 						ef->extensions[0]);
     }
-    made_conversions |= do_convert(in_file_name, (out_file_name != NULL?out_file_name:export_file_name), ef);
+    made_conversions |= do_convert(in_file_name,
+      (out_file_name != NULL?out_file_name:export_file_name), ef, size);
     g_free(export_file_name);
   } else if (out_file_name) {
-    made_conversions |= do_convert(in_file_name, out_file_name, NULL);
+    DiaExportFilter *ef = NULL;
+
+    /* if this looks like an ugly hack to you, agreed ;)  */
+    if (size && strstr(out_file_name, ".png"))
+      ef = filter_get_by_name ("png-libart");
+    
+    made_conversions |= do_convert(in_file_name, out_file_name, ef,
+				   size);
   } else {
     if (g_file_test(in_file_name, G_FILE_TEST_EXISTS)) {
       diagram = diagram_load (in_file_name, NULL);
@@ -254,9 +331,10 @@ handle_initial_diagram(const char *in_file_name,
 	      
     if (diagram != NULL) {
       diagram_update_extents(diagram);
-      layer_dialog_set_diagram(diagram);
-                  
-      ddisp = new_display(diagram);
+      if (app_is_interactive()) {
+	layer_dialog_set_diagram(diagram);
+	ddisp = new_display(diagram);
+      }
     }
   }
   return made_conversions;
@@ -271,7 +349,7 @@ myXmlErrorReporting (void *ctx, const char* msg, ...)
 
   va_start(args, msg);
   string = g_strdup_vprintf (msg, args);
-  g_print (string);
+  g_print (string ? string : "xml error (null)?");
   va_end(args);
 
   g_free(string);
@@ -306,17 +384,17 @@ app_init (int argc, char **argv)
   gboolean nosplash = FALSE;
   gboolean credits = FALSE;
   gboolean version = FALSE;
+  gboolean log_to_stderr = FALSE;
 #ifdef GNOME
   GnomeClient *client;
 #endif
   char *export_file_name = NULL;
   char *export_file_format = NULL;
+  char *size = NULL;
   gboolean made_conversions = FALSE;
+  GSList *files = NULL;
 
 #ifdef HAVE_POPT
-#ifndef GNOME
-  int rc = 0;
-#endif  
   poptContext poptCtx = NULL;
   gchar *export_format_string = 
      /* Translators:  The argument is a list of options, not to be translated */
@@ -333,15 +411,17 @@ app_init (int argc, char **argv)
     {"export-to-format",'t', POPT_ARG_STRING, NULL /* &export_file_format */,
      0, export_format_string, N_("FORMAT")
     },
+    {"size", 's', POPT_ARG_STRING, NULL, 0,
+     N_("Export graphics size"), N_("WxH")},
     {"nosplash", 'n', POPT_ARG_NONE, &nosplash, 0,
      N_("Don't show the splash screen"), NULL },
+    {"log-to-stderr", 'l', POPT_ARG_NONE, &log_to_stderr, 0,
+     N_("Send error messages to stderr instead of showing dialogs."), NULL },
     {"credits", 'c', POPT_ARG_NONE, &credits, 0,
      N_("Display credits list and exit"), NULL },
     {"version", 'v', POPT_ARG_NONE, &version, 0,
      N_("Display version and exit"), NULL },
-#ifndef GNOME
     {"help", 'h', POPT_ARG_NONE, 0, 1, N_("Show this help message") },
-#endif
     {(char *) NULL, '\0', 0, NULL, 0}
   };
 #endif
@@ -349,20 +429,30 @@ app_init (int argc, char **argv)
 #ifdef HAVE_POPT
   options[0].arg = &export_file_name;
   options[1].arg = &export_file_format;
+  options[2].arg = &size;
 #endif
 
   argv0 = (argc > 0) ? argv[0] : "(none)";
-  
+
   gtk_set_locale();
   setlocale(LC_NUMERIC, "C");
   
   bindtextdomain(GETTEXT_PACKAGE, LOCALEDIR);
-#if defined ENABLE_NLS
+  textdomain(GETTEXT_PACKAGE);
+
+  process_opts(argc, argv, 
+#ifdef HAVE_POPT
+               poptCtx, options, 
+#endif
+               &files,
+	     &export_file_name, &export_file_format, &size);
+
+#if defined ENABLE_NLS && defined HAVE_BIND_TEXTDOMAIN_CODESET
   bind_textdomain_codeset(GETTEXT_PACKAGE,"UTF-8");  
 #endif
   textdomain(GETTEXT_PACKAGE);
 
-  if (argv) {
+  if (argv && dia_is_interactive && !version) {
 #ifdef GNOME
     GnomeProgram *program =
       gnome_program_init (PACKAGE, VERSION, LIBGNOMEUI_MODULE,
@@ -375,38 +465,27 @@ app_init (int argc, char **argv)
       g_warning(_("Can't connect to session manager!\n"));
     }
     else {
-      gtk_signal_connect(GTK_OBJECT (client), "save_yourself",
-			 GTK_SIGNAL_FUNC (save_state), NULL);
-      gtk_signal_connect(GTK_OBJECT (client), "die",
-			 GTK_SIGNAL_FUNC (session_die), NULL);
+      g_signal_connect(GTK_OBJECT (client), "save_yourself",
+		       G_CALLBACK (save_state), NULL);
+      g_signal_connect(GTK_OBJECT (client), "die",
+		       G_CALLBACK (session_die), NULL);
     }
 
     /* This smaller icon is 48x48, standard Gnome size */
     gnome_window_icon_set_default_from_file (GNOME_ICONDIR"/dia_gnome_icon.png");
-#else
-    gtk_init (&argc, &argv);
-#ifdef HAVE_POPT
-    poptCtx = poptGetContext(PACKAGE, argc, (const char **)argv, options, 0);
-    poptSetOtherOptionHelp(poptCtx, _("[OPTION...] [FILE...]"));
-    while (rc >= 0) {
-        if((rc = poptGetNextOpt(poptCtx)) < -1) {
-            fprintf(stderr, 
-                    _("Error on option %s: %s.\nRun '%s --help' to see a full list of available command line options.\n"),
-                    poptBadOption(poptCtx, 0),
-                    poptStrerror(rc),
-                    argv[0]);
-            exit(1);
-        }
-        if(rc == 1) {
-            poptPrintHelp(poptCtx, stderr, 0);
-            exit(0);
-        }
 
-    }
-#endif
+#else
+    gtk_init(&argc, &argv);
 #endif
   }
+  else
+    g_type_init();
 
+#ifdef HAVE_POPT
+  /* done with option parsing, don't leak */
+  g_free(export_format_string);
+#endif
+  
   if (version) {
 #if (defined __TIME__) && (defined __DATE__)
     /* TRANSLATOR: 2nd and 3rd %s are time and date respectively. */
@@ -417,43 +496,13 @@ app_init (int argc, char **argv)
     exit(0);
   }
 
-  /* --credits option. Added by Andrew Ferrier.
+  if (!dia_is_interactive)
+    log_to_stderr = TRUE;
   
-     Hopefully we're not ignoring anything too crucial by
-     quitting directly after the credits.
+  if (log_to_stderr)
+    set_message_func(stderr_message_internal);
 
-     The phrasing of the English here might need changing
-     if we switch from plural to non-plural (say, only
-     one maintainer).
-  */
-
-  if (credits) {
-      int i;
-      const gint nauthors = (sizeof(authors) / sizeof(authors[0])) - 1;
-      const gint ndocumentors = (sizeof(documentors) / sizeof(documentors[0])) - 1;
-
-      printf("The original author of Dia was:\n\n");
-      for (i = 0; i < NUMBER_OF_ORIG_AUTHORS; i++) {
-          printf(authors[i]); printf("\n");
-      }
-
-      printf("\nThe current maintainers of Dia are:\n\n");
-      for (i = NUMBER_OF_ORIG_AUTHORS; i < NUMBER_OF_ORIG_AUTHORS + NUMBER_OF_MAINTAINERS; i++) {
-	  printf(authors[i]); printf("\n");
-      }
-
-      printf("\nOther authors are:\n\n");
-      for (i = NUMBER_OF_ORIG_AUTHORS + NUMBER_OF_MAINTAINERS; i < nauthors; i++) {
-          printf(authors[i]); printf("\n");
-      }
-
-      printf("\nDia is documented by:\n\n");
-      for (i = 0; i < ndocumentors; i++) {
-          printf(documentors[i]); printf("\n");
-      }
-
-      exit(0);
-  }
+  print_credits(credits);
 
   LIBXML_TEST_VERSION;
 
@@ -463,22 +512,26 @@ app_init (int argc, char **argv)
 
   stdprops_init();
 
-  dia_image_init();
+  if (dia_is_interactive) {
+    dia_image_init();
 
-  gdk_rgb_init();
+    gdk_rgb_init();
 
-  gtk_rc_parse ("diagtkrc"); 
+    gtk_rc_parse("diagtkrc"); 
 
-  if (!nosplash)
-    app_splash_init("");
+    if (!nosplash) {
+      app_splash_init("");
+    }
+  }
 
   create_user_dirs();
 
-  color_init();
-
   /* Init cursors: */
-  default_cursor = gdk_cursor_new(GDK_LEFT_PTR);
-  ddisplay_set_all_cursor(default_cursor);
+  if (dia_is_interactive) {
+    color_init();
+    default_cursor = gdk_cursor_new(GDK_LEFT_PTR);
+    ddisplay_set_all_cursor(default_cursor);
+  }
 
   object_registry_init();
 
@@ -494,85 +547,76 @@ app_init (int argc, char **argv)
   if (object_get_type("Standard - Box") == NULL) {
     message_error(_("Couldn't find standard objects when looking for "
 		  "object-libs, exiting...\n"));
-    fprintf(stderr, _("Couldn't find standard objects when looking for "
+    g_error( _("Couldn't find standard objects when looking for "
 	    "object-libs, exiting...\n"));
     exit(1);
   }
 
-  prefs_load();
-
   persistence_load();
 
-  /* further initialization *before* reading files */  
-  active_tool = create_modify_tool();
+  /** Must load prefs after persistence */
+  prefs_init();
 
-  create_toolbox();
+  if (dia_is_interactive) {
 
-  persistence_register_window_create("layer_window",
-				     (NullaryFunc*)&create_layer_dialog);
+    /* further initialization *before* reading files */  
+    active_tool = create_modify_tool();
 
-  /*fill recent file menu */
-  recent_file_history_init();
+    create_toolbox();
 
-  /* Set up autosave to check every 5 minutes */
-  gtk_timeout_add(5*60*1000, autosave_check_autosave, NULL);
+    persistence_register_window_create("layer_window",
+				       (NullaryFunc*)&create_layer_dialog);
 
-  create_tree_window();
 
-  persistence_register_window_create("sheets_main_dialog",
-				     (NullaryFunc*)&sheets_dialog_create);
+    /*fill recent file menu */
+    recent_file_history_init();
 
-  /* In current setup, we can't find the autosaved files. */
-  /*autosave_restore_documents();*/
-  
-  if (argv) {
-#ifdef HAVE_POPT
-      while (poptPeekArg(poptCtx)) {
-          char *in_file_name = (char *)poptGetArg(poptCtx);
-	  
-	  made_conversions |= handle_initial_diagram(in_file_name,
-						     export_file_name,
-						     export_file_format);
-      }
-      poptFreeContext(poptCtx);
-#else
-      int i;
+    /* Set up autosave to check every 5 minutes */
+    gtk_timeout_add(5*60*1000, autosave_check_autosave, NULL);
+
+    create_tree_window();
+
+    persistence_register_window_create("sheets_main_dialog",
+				       (NullaryFunc*)&sheets_dialog_create);
+
+
+    /* In current setup, we can't find the autosaved files. */
+    /*autosave_restore_documents();*/
 
-      for (i=1; i<argc; i++) {
-          char *in_file_name = argv[i]; /* unless it's an option... */
-          
-          if (0==strcmp(argv[i],"-t")) {
-              if (i < (argc-1)) {
-                  i++;
-                  export_file_format = argv[i];
-                  continue;
-              }
-          } else if (0 == strcmp(argv[i],"-e")) {
-              if (i < (argc-1)) {
-                  i++;
-                  export_file_name = argv[i];
-                  continue;
-              }
-          }
-          
-	  made_conversions |= handle_initial_diagram(in_file_name,
-						     export_file_name,
-						     export_file_format);
-      }
-#endif
   }
+
+  made_conversions = handle_all_diagrams(files, export_file_name,
+					 export_file_format, size);
+  if (dia_is_interactive && files == NULL) {
+    gchar *filename = g_filename_from_utf8(_("Diagram1.dia"), -1, NULL, NULL, NULL);
+    Diagram *diagram = new_diagram (filename);
+    g_free(filename);
+	
+	      
+    if (diagram != NULL) {
+      diagram_update_extents(diagram);
+      if (app_is_interactive()) {
+	layer_dialog_set_diagram(diagram);
+	new_display(diagram);
+      }
+    }
+  }
+  g_slist_free(files);
   if (made_conversions) exit(0);
 
   dynobj_refresh_init();
 }
 
+#if 0
+/* app_procs.c: warning: `set_true_callback' defined but not used */
 static void
 set_true_callback(GtkWidget *w, int *data)
 {
   *data = TRUE;
 }
+#endif
 
-void
+gboolean
 app_exit(void)
 {
   GList *list;
@@ -587,7 +631,7 @@ app_exit(void)
 
   if (app_exit_once) {
     g_error(_("This shouldn't happen.  Please file a bug report at bugzilla.gnome.org\ndescribing how you can cause this message to appear.\n"));
-    return;
+    return FALSE;
   }
 
   if (diagram_modified_exists()) {
@@ -612,15 +656,14 @@ app_exit(void)
     button = gtk_button_new_from_stock (GTK_STOCK_QUIT);
     gtk_dialog_add_action_widget (GTK_DIALOG(dialog), button, GTK_RESPONSE_OK);
 
-    g_signal_connect_after (G_OBJECT (dialog), "response",
-		            G_CALLBACK(gtk_widget_destroy),
-		            NULL);
     gtk_widget_show_all (dialog);
 
-    if (gtk_dialog_run(GTK_DIALOG(dialog)) != GTK_RESPONSE_OK)
-      return;
+    if (gtk_dialog_run(GTK_DIALOG(dialog)) != GTK_RESPONSE_OK) {
+      gtk_widget_destroy(dialog);
+      return FALSE;
+    }
+    gtk_widget_destroy(dialog);
   }
-
   prefs_save();
 
   persistence_save();
@@ -651,14 +694,13 @@ app_exit(void)
   /* save pluginrc */
   dia_pluginrc_write();
 
-  /* save recent file history */
-  recent_file_history_write();
-  
   gtk_main_quit();
   /* This printf seems to prevent a race condition with unrefs. */
   /* Yuck.  -Lars */
-  printf(_("Thank you for using Dia.\n"));
+  g_print(_("Thank you for using Dia.\n"));
   app_exit_once = TRUE;
+
+  return TRUE;
 }
 
 static void create_user_dirs(void)
@@ -729,3 +771,143 @@ internal_plugin_init(PluginInfo *info)
   return DIA_PLUGIN_INIT_OK;
 }
 
+/* Note: running in locale encoding */
+static void
+process_opts(int argc, char **argv,
+#ifdef HAVE_POPT
+	     poptContext poptCtx, struct poptOption options[],
+#endif
+	     GSList **files, char **export_file_name,
+	     char **export_file_format, char **size)
+{
+#ifdef HAVE_POPT
+  int rc = 0;
+  poptCtx = poptGetContext(PACKAGE, argc, (const char **)argv, options, 0);
+  poptSetOtherOptionHelp(poptCtx, _("[OPTION...] [FILE...]"));
+  while (rc >= 0) {
+    if((rc = poptGetNextOpt(poptCtx)) < -1) {
+      fprintf(stderr,_("Error on option %s: %s.\nRun '%s --help' to see a full list of available command line options.\n"),
+	      poptBadOption(poptCtx, 0),
+	      poptStrerror(rc),
+	      argv[0]);
+      exit(1);
+    }
+    if(rc == 1) {
+      poptPrintHelp(poptCtx, stderr, 0);
+      exit(0);
+    }
+  }
+#endif
+  if (argv) {
+#ifdef HAVE_POPT
+      while (poptPeekArg(poptCtx)) {
+          char *in_file_name = (char *)poptGetArg(poptCtx);
+	  if (*in_file_name != '\0')
+	    *files = g_slist_append(*files, in_file_name);
+      }
+      poptFreeContext(poptCtx);
+#else
+      int i;
+
+      for (i=1; i<argc; i++) {
+          char *in_file_name = argv[i]; /* unless it's an option... */
+
+          if (0==strcmp(argv[i],"-t")) {
+              if (i < (argc-1)) {
+                  i++;
+                  *export_file_format = argv[i];
+                  continue;
+              }
+          } else if (0 == strcmp(argv[i],"-e")) {
+              if (i < (argc-1)) {
+                  i++;
+                  *export_file_name = argv[i];
+                  continue;
+              }
+          } else if (0 == strcmp(argv[i],"-s")) {
+              if (i < (argc-1)) {
+                  i++;
+                  *size = argv[i];
+                  continue;
+              }
+          }
+	  *files = g_slist_append(*files, in_file_name);
+      }
+#endif
+  }
+  if (*export_file_name || *export_file_format || *size)
+    dia_is_interactive = FALSE;
+}
+
+static gboolean
+handle_all_diagrams(GSList *files, char *export_file_name,
+		    char *export_file_format, char *size)
+{
+  GSList *node = NULL;
+  gboolean made_conversions = FALSE;
+
+  for (node = files; node; node = node->next) {
+    made_conversions |=
+      handle_initial_diagram(node->data, export_file_name,
+			     export_file_format, size);
+  }
+  return made_conversions;
+}
+
+/* --credits option. Added by Andrew Ferrier.
+  
+   Hopefully we're not ignoring anything too crucial by
+   quitting directly after the credits.
+
+   The phrasing of the English here might need changing
+   if we switch from plural to non-plural (say, only
+   one maintainer).
+*/
+static void
+print_credits(gboolean credits)
+{
+  if (credits) {
+      int i;
+      const gint nauthors = (sizeof(authors) / sizeof(authors[0])) - 1;
+      const gint ndocumentors = (sizeof(documentors) / sizeof(documentors[0])) - 1;
+
+      g_print(_("The original author of Dia was:\n\n"));
+      for (i = 0; i < NUMBER_OF_ORIG_AUTHORS; i++) {
+          g_print(authors[i]); g_print("\n");
+      }
+
+      g_print(_("\nThe current maintainers of Dia are:\n\n"));
+      for (i = NUMBER_OF_ORIG_AUTHORS; i < NUMBER_OF_ORIG_AUTHORS + NUMBER_OF_MAINTAINERS; i++) {
+	  g_print(authors[i]); g_print("\n");
+      }
+
+      g_print(_("\nOther authors are:\n\n"));
+      for (i = NUMBER_OF_ORIG_AUTHORS + NUMBER_OF_MAINTAINERS; i < nauthors; i++) {
+          g_print(authors[i]); g_print("\n");
+      }
+
+      g_print(_("\nDia is documented by:\n\n"));
+      for (i = 0; i < ndocumentors; i++) {
+          g_print(documentors[i]); g_print("\n");
+      }
+
+      exit(0);
+  }
+}
+
+/* parses a string of the form "[0-9]*x[0-9]*" and transforms it into
+   two long values width and height. */
+void
+parse_size(gchar *size, long *width, long *height)
+{
+  if (size) {
+    gchar** array = g_strsplit(size, "x", 3);
+    *width  = (array[0])? strtol(array[0], NULL, 10): 0;
+    *height = (array[1])? strtol(array[1], NULL, 10): 0;
+    g_strfreev(array);
+  }
+  else {
+    *width  = 0;
+    *height = 0;
+  }
+}
