@@ -18,11 +18,28 @@
 #ifndef APP_PROCS_H
 #define APP_PROCS_H
 
+#include <lib/filter.h>
+
 void app_init(int argc, char **argv);
-void app_exit(void);
+/** Exit the application, but asking the user for confirmation
+ * if there are changed diagrams.
+ * Returns TRUE if the application exits.
+ */
+gboolean app_exit(void);
 int app_is_embedded(void);
 
+gboolean do_convert(const char *infname,
+		    const char *outfname, DiaExportFilter *ef,
+		    const char *size);
+char *build_output_file_name(const char *infname, const char *format);
+
 void app_splash_init(const gchar* name);
 void app_splash_done(void);
 
+gboolean app_is_interactive(void);
+
+/* parses a string of the form "[0-9]*x[0-9]*" and transforms it into
+   two long values width and height. */
+void parse_size(gchar *size, long *width, long *height);
+
 #endif /* APP_PROCS_H */
