@@ -19,19 +19,31 @@
 #define DDISPLAY_H
 
 #include <gtk/gtk.h>
+#include <gtk/gtkimcontext.h>
 
 typedef struct _DDisplay DDisplay;
 
 #include "geometry.h"
 #include "diagram.h"
 #include "grid.h"
-#include "render_gdk.h"
-#include "render_libart.h"
+#include "menus.h"
+#include "diarenderer.h"
+
+/** Defines the pixels per cm, default is 20 pixels = 1 cm */
+/** This is close to, but not quite the same as, the physical display size
+ * in most cases */
 
 #define DDISPLAY_MAX_ZOOM 500.0
 #define DDISPLAY_NORMAL_ZOOM 20.0
 #define DDISPLAY_MIN_ZOOM 1.0
 
+/* The zoom amount should be uniform.  Pixels per cm should be defined by the
+ * renderer alone.  But that'd take a lot of fiddling in renderers. */
+/*
+#define DDISPLAY_MAX_ZOOM 100.0
+#define DDISPLAY_NORMAL_ZOOM 1.0
+#define DDISPLAY_MIN_ZOOM 0.01
+*/
 struct _DDisplay {
   Diagram *diagram;               /* pointer to the associated diagram */
 
@@ -40,9 +52,22 @@ struct _DDisplay {
   GtkWidget *hsb, *vsb;           /* widgets for scroll bars           */
   GtkWidget *hrule, *vrule;       /* widgets for rulers                */
   GtkWidget *origin;              /* widgets for rulers                */
+  GtkWidget *menu_bar;            /* widget for the menu bar           */
+  GtkItemFactory *mbar_item_factory; /* item factory used to create the menu bar */
+
+  /* menu bar widgets */
+  GtkMenuItem *rulers;
+  GtkMenuItem *visible_grid;
+  GtkMenuItem *snap_to_grid;
+  GtkMenuItem *show_cx_pts_mitem;
+#ifdef HAVE_LIBART
+  GtkMenuItem *antialiased;
+#endif
+  UpdatableMenuItems updatable_menu_items;
+    
 
-  GtkWidget *snap_status;
   GtkWidget *zoom_status;         
+  GtkWidget *grid_status;
   GtkWidget *modified_status;
 
   GtkAccelGroup *accel_group;
@@ -60,15 +85,17 @@ struct _DDisplay {
   gboolean autoscroll;
 
   int aa_renderer;
-  Renderer *renderer;
+  DiaRenderer *renderer;
   
   GSList *update_areas;           /* Update areas list                 */
   GSList *display_areas;          /* Display areas list                */
   guint update_id;                /* idle handler ID for redraws       */
 
-  /* input contexts */
-  GdkIC *ic;
-  GdkICAttr *ic_attr;
+  GtkIMContext *im_context;
+
+  /* Preedit String */
+  gchar *preedit_string;
+  PangoAttrList *preedit_attrs;
 };
 
 extern GdkCursor *default_cursor;
@@ -90,6 +117,8 @@ real ddisplay_untransform_length(DDisplay *ddisp, real len);
 void ddisplay_add_update_pixels(DDisplay *ddisp, Point *point,
 				       int pixel_width, int pixel_height);
 void ddisplay_add_update_all(DDisplay *ddisp);
+void ddisplay_add_update_with_border(DDisplay *ddisp, Rectangle *rect,
+				     int pixel_border);
 void ddisplay_add_update(DDisplay *ddisp, Rectangle *rect);
 void ddisplay_add_display_area(DDisplay *ddisp,
 			       int left, int top,
@@ -100,7 +129,7 @@ void ddisplay_set_origo(DDisplay *ddisp,
 			coord x, coord y);
 void ddisplay_zoom(DDisplay *ddisp, Point *point,
 		   real zoom_factor);
-
+void ddisplay_set_snap_to_grid(DDisplay *ddisp, gboolean snap);
 void ddisplay_set_renderer(DDisplay *ddisp, int aa_renderer);
 void ddisplay_resize_canvas(DDisplay *ddisp,
 			    int width,
@@ -117,18 +146,21 @@ void ddisplay_set_title(DDisplay *ddisp, char *title);
 void ddisplay_set_cursor(DDisplay *ddisp, GdkCursor *cursor);
 void ddisplay_set_all_cursor(GdkCursor *cursor);
 
-void ddisplay_scroll(DDisplay *ddisp, Point *delta);
+gboolean ddisplay_scroll(DDisplay *ddisp, Point *delta);
 gboolean ddisplay_autoscroll(DDisplay *ddisp, int x, int y);
 void ddisplay_scroll_up(DDisplay *ddisp);
 void ddisplay_scroll_down(DDisplay *ddisp);
 void ddisplay_scroll_left(DDisplay *ddisp);
 void ddisplay_scroll_right(DDisplay *ddisp);
+gboolean ddisplay_scroll_center_point(DDisplay *ddisp, Point *p);
+gboolean ddisplay_scroll_to_object(DDisplay *ddisp, DiaObject *obj);
 
 void display_update_menu_state(DDisplay *ddisp);
 void ddisplay_update_statusbar(DDisplay *ddisp);
-/* Have to be called from interface.c */
-GtkPixmap *snap_status_load_images(GdkWindow *window);
+void ddisplay_do_update_menu_sensitivity (DDisplay *ddisp);
 
 void display_set_active(DDisplay *ddisp);
 
+void ddisplay_im_context_preedit_reset(DDisplay *ddisp, Focus *focus);
+
 #endif /* DDISPLAY_H */
