@@ -46,7 +46,14 @@ static GtkWidget *compressbutton = NULL;
 static int
 file_dialog_hide (GtkWidget *filesel)
 {
+  gpointer user_data;
+
   gtk_widget_hide (filesel);
+  if ((user_data = gtk_object_get_user_data(GTK_OBJECT(filesel))) != NULL) {
+    diagram_remove_related_dialog(user_data, filesel);
+  }
+  g_object_unref(G_OBJECT(user_data)); 
+  gtk_object_set_user_data(GTK_OBJECT(filesel), NULL);
 
 #if 0
   menus_set_sensitive ("<Toolbox>/File/Open...", TRUE);
@@ -71,10 +78,11 @@ set_true_callback(GtkWidget *w, int *data)
 static void
 toggle_compress_callback(GtkWidget *w, gpointer data)
 {
-  // Changes prefs exactly when the user toggles the setting, i.e.
-  // the setting really remembers what the user chose last time, but
-  // lets diagrams of the opposite kind stay that way unless the user
-  // intervenes.
+  /* Changes prefs exactly when the user toggles the setting, i.e.
+      * the setting really remembers what the user chose last time, but
+     * lets diagrams of the opposite kind stay that way unless the user
+     * intervenes.
+     */
   prefs.new_diagram.compress_save =
     gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(compressbutton));
 }
@@ -167,6 +175,7 @@ file_open_callback(gpointer data, guint action, GtkWidget *widget)
   if (!opendlg) {
     DDisplay *ddisp;
     Diagram *dia = NULL;
+    gchar *filename = NULL;
     
     ddisp = ddisplay_active();
     if (ddisp) {
@@ -175,9 +184,12 @@ file_open_callback(gpointer data, guint action, GtkWidget *widget)
     opendlg = gtk_file_selection_new(_("Open Diagram"));
     gtk_window_set_role(GTK_WINDOW(opendlg), "open_diagram");
     gtk_window_set_position(GTK_WINDOW(opendlg), GTK_WIN_POS_MOUSE);
-    gtk_file_selection_set_filename(GTK_FILE_SELECTION(opendlg),
-				    dia && dia->filename ? dia->filename
-				    : "." G_DIR_SEPARATOR_S);
+    if (dia && dia->filename)
+      filename = g_filename_from_utf8(dia->filename, -1, NULL, NULL, NULL);
+    if (filename == NULL)
+      filename = g_strdup("." G_DIR_SEPARATOR_S);
+    gtk_file_selection_set_filename(GTK_FILE_SELECTION(opendlg), filename);
+    g_free(filename);
     g_signal_connect_swapped(
 		GTK_OBJECT(GTK_FILE_SELECTION(opendlg)->cancel_button),
 	    "clicked", G_CALLBACK(file_dialog_hide),
@@ -241,9 +253,8 @@ file_save_as_ok_callback(GtkWidget *w, GtkFileSelection *fs)
     GtkWidget *dialog = NULL;
     char buffer[300];
     char *utf8filename = NULL;
-
     if (!g_utf8_validate(filename, -1, NULL)) {
-      utf8filename = g_locale_to_utf8(filename, -1, NULL, NULL, NULL);
+      utf8filename = g_filename_to_utf8(filename, -1, NULL, NULL, NULL);
       if (utf8filename == NULL) {
 	message_warning(_("Some characters in the filename are neither UTF-8 nor your local encoding.\nSome things will break."));
       }
@@ -263,7 +274,7 @@ file_save_as_ok_callback(GtkWidget *w, GtkFileSelection *fs)
     gtk_dialog_set_default_response (GTK_DIALOG (dialog), GTK_RESPONSE_YES);
 
     if (gtk_dialog_run (GTK_DIALOG (dialog)) != GTK_RESPONSE_YES) {
-      gtk_widget_hide(savedlg);
+      file_dialog_hide(savedlg);
       gtk_widget_destroy(dialog);
       return;
     }
@@ -277,7 +288,7 @@ file_save_as_ok_callback(GtkWidget *w, GtkFileSelection *fs)
   diagram_set_filename(dia, filename);
   diagram_save(dia, filename);
 
-  gtk_widget_hide(savedlg);
+  file_dialog_hide(savedlg);
 }
 
 void
@@ -285,6 +296,7 @@ file_save_as_callback(gpointer data, guint action, GtkWidget *widget)
 {
   DDisplay *ddisp;
   Diagram *dia;
+  gchar *filename = NULL;
 
   ddisp = ddisplay_active();
   if (!ddisp) return;
@@ -324,28 +336,38 @@ file_save_as_callback(gpointer data, guint action, GtkWidget *widget)
     gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(compressbutton),
 				 dia->data->is_compressed);
     g_signal_handlers_unblock_by_func(G_OBJECT(compressbutton), toggle_compress_callback, NULL);
+    if (gtk_object_get_user_data(GTK_OBJECT(savedlg)) != NULL)
+      diagram_remove_related_dialog(gtk_object_get_user_data(GTK_OBJECT(savedlg)), savedlg);
+    gtk_object_set_user_data(GTK_OBJECT(savedlg), dia);
+    diagram_add_related_dialog(dia, savedlg);
     if (GTK_WIDGET_VISIBLE(savedlg))
       return;
   }
-  gtk_file_selection_set_filename(GTK_FILE_SELECTION(savedlg),
-				  dia->filename ? dia->filename
-				  : "." G_DIR_SEPARATOR_S);
+  if (dia && dia->filename)
+    filename = g_filename_from_utf8(dia->filename, -1, NULL, NULL, NULL);
+  if (filename == NULL)
+    filename = g_strdup("." G_DIR_SEPARATOR_S);
+  gtk_file_selection_set_filename(GTK_FILE_SELECTION(savedlg), filename);
+  g_free(filename);
   gtk_object_set_user_data(GTK_OBJECT(savedlg), dia);
+  diagram_add_related_dialog(dia, savedlg);
+  g_object_ref(dia); 
   gtk_widget_show(savedlg);
 }
 
 void
 file_save_callback(gpointer data, guint action, GtkWidget *widget)
 {
-  DDisplay *ddisp;
+  Diagram *diagram;
 
-  ddisp = ddisplay_active();
+  diagram = ddisplay_active_diagram();
+  if (!diagram) return;
 
-  if (ddisp->diagram->unsaved) {
+  if (diagram->unsaved) {
     file_save_as_callback(data, action, widget);
   } else {
-    diagram_update_extents(ddisp->diagram);
-    diagram_save(ddisp->diagram, ddisp->diagram->filename);
+    diagram_update_extents(diagram);
+    diagram_save(diagram, diagram->filename);
   }
 }
 
@@ -421,7 +443,6 @@ file_export_ok_callback(GtkWidget *w, GtkFileSelection *fs)
   if (stat(filename, &statbuf) == 0) {
     GtkWidget *dialog = NULL;
     char buffer[300];
-    int result;
 
     g_snprintf(buffer, 300,
 	       _("The file '%s' already exists.\n"
@@ -435,7 +456,7 @@ file_export_ok_callback(GtkWidget *w, GtkFileSelection *fs)
     gtk_dialog_set_default_response (GTK_DIALOG (dialog), GTK_RESPONSE_YES);
 
     if (gtk_dialog_run (GTK_DIALOG (dialog)) != GTK_RESPONSE_YES) {
-      gtk_widget_hide(exportdlg);
+      file_dialog_hide(exportdlg);
       gtk_widget_destroy(dialog);
       return;
     }
@@ -446,13 +467,14 @@ file_export_ok_callback(GtkWidget *w, GtkFileSelection *fs)
 					   ->menu_item));
   if (!ef)
     ef = filter_guess_export_filter(filename);
-  if (ef)
-      ef->export(dia->data, filename, dia->filename, ef->user_data);
-  else
-      message_error(_("Could not determine which export filter\n"
-		      "to use to save '%s'"), filename);
+  if (ef) {
+    g_object_ref(dia->data); 
+    ef->export_func(dia->data, filename, dia->filename, ef->user_data);
+  } else
+    message_error(_("Could not determine which export filter\n"
+		    "to use to save '%s'"), filename);
 
-  gtk_widget_hide(exportdlg);
+  file_dialog_hide(exportdlg);
 }
 
 void
@@ -461,6 +483,7 @@ file_export_callback(gpointer data, guint action, GtkWidget *widget)
   DDisplay *ddisp;
   Diagram *dia;
   GtkWidget *export_menu, *export_item;
+  gchar *filename = NULL;
 
   ddisp = ddisplay_active();
   if (!ddisp) return;
@@ -470,9 +493,12 @@ file_export_callback(gpointer data, guint action, GtkWidget *widget)
     exportdlg = gtk_file_selection_new(_("Export Diagram"));
     gtk_window_set_role(GTK_WINDOW(exportdlg), "export_diagram");
     gtk_window_set_position(GTK_WINDOW(exportdlg), GTK_WIN_POS_MOUSE);
-    gtk_file_selection_set_filename(GTK_FILE_SELECTION(exportdlg),
-				    dia->filename ? dia->filename
-				    : "." G_DIR_SEPARATOR_S);
+    if (dia && dia->filename)
+      filename = g_filename_from_utf8(dia->filename, -1, NULL, NULL, NULL);
+    if (filename == NULL)
+      filename = g_strdup("." G_DIR_SEPARATOR_S);
+    gtk_file_selection_set_filename(GTK_FILE_SELECTION(exportdlg), filename);
+	g_free(filename);
     g_signal_connect_swapped(
 		GTK_OBJECT(GTK_FILE_SELECTION(exportdlg)->cancel_button),
 	    "clicked", G_CALLBACK(file_dialog_hide),
@@ -512,12 +538,17 @@ file_export_callback(gpointer data, guint action, GtkWidget *widget)
   }
  
   gtk_object_set_user_data(GTK_OBJECT(exportdlg), dia);
+  g_object_ref(dia); 
+  diagram_add_related_dialog(dia, exportdlg);
   gtk_widget_set_sensitive(exportdlg, TRUE);
   if (GTK_WIDGET_VISIBLE(exportdlg))
     return;
-  gtk_file_selection_set_filename(GTK_FILE_SELECTION(exportdlg),
-				  dia->filename ? dia->filename
-				  : "." G_DIR_SEPARATOR_S);
+  if (dia && dia->filename)
+    filename = g_filename_from_utf8(dia->filename, -1, NULL, NULL, NULL);
+  if (filename == NULL)
+    filename = g_strdup("." G_DIR_SEPARATOR_S);
+  gtk_file_selection_set_filename(GTK_FILE_SELECTION(exportdlg), filename);
+  g_free(filename);
   export_menu = gtk_option_menu_get_menu(GTK_OPTION_MENU(export_omenu));
   export_item = gtk_menu_get_active(GTK_MENU(export_menu));
   if (export_item)
