@@ -27,22 +27,23 @@
 #include <fcntl.h>
 #include <string.h>
 #include <glib.h>
+#include <errno.h>
 
 #include "intl.h"
 
-#include <tree.h>
-#include <parser.h>
-#if defined(LIBXML_VERSION) && LIBXML_VERSION >= 20000
-#define root children
-#define childs children
-#endif
+#include <libxml/tree.h>
+#include <libxml/parser.h>
+#include <libxml/xmlmemory.h>
+#include "dia_xml_libxml.h"
+#include "dia_xml.h"
 
 #include "load_save.h"
-#include "dia_xml.h"
 #include "group.h"
+#include "diagramdata.h"
 #include "message.h"
 #include "preferences.h"
 #include "diapagelayout.h"
+#include "autosave.h"
 
 #ifdef G_OS_WIN32
 #include <io.h>
@@ -54,6 +55,30 @@
 #define fchmod(f,m) (0)
 #endif
 
+static void read_connections(GList *objects, xmlNodePtr layer_node,
+			     GHashTable *objects_hash);
+static void GHFuncUnknownObjects(gpointer key,
+				 gpointer value,
+				 gpointer user_data);
+static GList *read_objects(xmlNodePtr objects,
+			   GHashTable *objects_hash,
+			   const char *filename, DiaObject *parent);
+static void hash_free_string(gpointer       key,
+			     gpointer       value,
+			     gpointer       user_data);
+static xmlNodePtr find_node_named (xmlNodePtr p, const char *name);
+static gboolean diagram_data_load(const char *filename, DiagramData *data,
+				  void* user_data);
+static gboolean write_objects(GList *objects, xmlNodePtr objects_node,
+			      GHashTable *objects_hash, int *obj_nr, 
+			      const char *filename);
+static gboolean write_connections(GList *objects, xmlNodePtr layer_node,
+				  GHashTable *objects_hash);
+static xmlDocPtr diagram_data_write_doc(DiagramData *data, const char *filename);
+static int diagram_data_raw_save(DiagramData *data, const char *filename);
+static int diagram_data_save(DiagramData *data, const char *filename);
+
+
 static void
 GHFuncUnknownObjects(gpointer key,
                      gpointer value,
@@ -65,12 +90,30 @@ GHFuncUnknownObjects(gpointer key,
   g_free(key);
 }
 
+/**
+ * Recursive function to read objects from a specific level in the xml.
+ *
+ * Nowadays there are quite a few of them :
+ * - Layers : a diagram may have different layers, but this function does *not*
+ *   add the created objects to them as it does not know on which nesting level it
+ *   is called. So the topmost caller must add the returned objects to the layer.
+ * - Groups : groups in itself can have an arbitrary nesting level including other
+ *   groups or objects or both of them. A group not containing any objects is by 
+ *   definition useless. So it is not created. This is to avoid trouble with some older
+ *   diagrams which happen to be saved with empty groups.
+ * - Parents : if the parent relations would have been there from the beginning of
+ *   Dias file format they probably would have been added as nesting level 
+ *   themselves. But to maintain forward compatibility (allow to let older versions
+ *   of Dia to see as much as possible) they were added all on the same level and
+ *   the parent child relation is reconstructed from additional attributes.
+ */
 static GList *
-read_objects(xmlNodePtr objects, GHashTable *objects_hash,const char *filename)
+read_objects(xmlNodePtr objects, 
+             GHashTable *objects_hash,const char *filename, DiaObject *parent)
 {
   GList *list;
-  ObjectType *type;
-  Object *obj;
+  DiaObjectType *type;
+  DiaObject *obj;
   ObjectNode obj_node;
   char *typestr;
   char *versionstr;
@@ -78,15 +121,22 @@ read_objects(xmlNodePtr objects, GHashTable *objects_hash,const char *filename)
   int version;
   GHashTable* unknown_hash;
   GString*    unknown_str;
+  xmlNodePtr child_node;
 
   unknown_hash = g_hash_table_new(g_str_hash, g_str_equal);
   unknown_str  = g_string_new("Unknown types while reading diagram file");
 
   list = NULL;
 
-  obj_node = objects->childs;
-  
+  obj_node = objects->xmlChildrenNode;
+
   while ( obj_node != NULL) {
+    if (xmlIsBlankNode(obj_node)) {
+      obj_node = obj_node->next;
+      continue;
+    }
+
+    if (!obj_node) break;
 
     if (strcmp(obj_node->name, "object")==0) {
       typestr = xmlGetProp(obj_node, "type");
@@ -96,28 +146,50 @@ read_objects(xmlNodePtr objects, GHashTable *objects_hash,const char *filename)
       version = 0;
       if (versionstr != NULL) {
 	version = atoi(versionstr);
-	free(versionstr);
+	xmlFree(versionstr);
       }
 
       type = object_get_type((char *)typestr);
       
       if (!type) {
 	if (NULL == g_hash_table_lookup(unknown_hash, typestr))
-	    g_hash_table_insert(unknown_hash, strdup(typestr), 0);
+	    g_hash_table_insert(unknown_hash, g_strdup(typestr), 0);
       }
       else
       {
         obj = type->ops->load(obj_node, version, filename);
         list = g_list_append(list, obj);
-      
-        g_hash_table_insert(objects_hash, (char *)id, obj);
+
+        if (parent)
+        {
+          obj->parent = parent;
+          parent->children = g_list_append(parent->children, obj);
+        }
+
+        g_hash_table_insert(objects_hash, g_strdup((char *)id), obj);
+
+        child_node = obj_node->children;
+
+        while(child_node)
+        {
+          if (strcmp(child_node->name, "children") == 0)
+          {
+	    GList *children_read = read_objects(child_node, objects_hash, filename, obj);
+            list = g_list_concat(list, children_read);
+            break;
+          }
+          child_node = child_node->next;
+        }
       }
-      if (typestr) free(typestr);
-    } else if (strcmp(obj_node->name, "group")==0) {
-      obj = group_create(read_objects(obj_node, objects_hash, filename));
+      if (typestr) xmlFree(typestr);
+      if (id) xmlFree (id);
+    } else if (   strcmp(obj_node->name, "group")==0
+               && obj_node->children) {
+      /* don't create empty groups */
+      obj = group_create(read_objects(obj_node, objects_hash, filename, NULL));
       list = g_list_append(list, obj);
     } else {
-      message_error(_("Error reading diagram file\n"));
+      /* silently ignore other nodes */
     }
 
     obj_node = obj_node->next;
@@ -147,56 +219,97 @@ read_connections(GList *objects, xmlNodePtr layer_node,
   char *tostr;
   char *connstr;
   int handle, conn;
-  Object *to;
+  DiaObject *to;
   
   list = objects;
-  obj_node = layer_node->childs;
-  while (list != NULL) {
-    Object *obj = (Object *) list->data;
+  obj_node = layer_node->xmlChildrenNode;
+  while ((list != NULL) && (obj_node != NULL)) {
+    DiaObject *obj = (DiaObject *) list->data;
 
+    while (obj_node && xmlIsBlankNode(obj_node)) obj_node = obj_node->next;
+    if (!obj_node) break;
+    
     if IS_GROUP(obj) {
       read_connections(group_objects(obj), obj_node, objects_hash);
     } else {
-      connections = obj_node->childs;
+      connections = obj_node->xmlChildrenNode;
       while ((connections!=NULL) &&
 	     (strcmp(connections->name, "connections")!=0))
 	connections = connections->next;
       if (connections != NULL) {
-	connection = connections->childs;
+	connection = connections->xmlChildrenNode;
 	while (connection != NULL) {
+	  char *donestr;
+          if (xmlIsBlankNode(connection)) {
+            connection = connection->next;
+            continue;
+          }
 	  handlestr = xmlGetProp(connection, "handle");
 	  tostr = xmlGetProp(connection, "to");
 	  connstr = xmlGetProp(connection, "connection");
 	  
 	  handle = atoi(handlestr);
-	  conn = atoi(connstr);
-	  
-	  to = g_hash_table_lookup(objects_hash, tostr);
+	  conn = strtol(connstr, &donestr, 10);
+	  if (*donestr != '\0') { /* Didn't convert it all -- use string */
+	    conn = -1;
+	  }
 
-	  if (handlestr) free(handlestr);
-	  if (connstr) free(connstr);
-	  if (tostr) free(tostr);
+	  to = g_hash_table_lookup(objects_hash, tostr);
 
 	  if (to == NULL) {
 	    message_error(_("Error loading diagram.\n"
 			    "Linked object not found in document."));
-	  } else if (conn >= 0 && conn >= to->num_connections) {
-	    message_error(_("Error loading diagram.\n"
-			    "connection point does not exist."));
-	  } else if (handle >= 0 && handle >= obj->num_handles) {
+	  } else if (handle < 0 || handle >= obj->num_handles) {
 	    message_error(_("Error loading diagram.\n"
 			    "connection handle does not exist."));
 	  } else {
-	    object_connect(obj, obj->handles[handle],
-			   to->connections[conn]);
+	    if (conn == -1) { /* Find named connpoint */
+	      int i;
+	      for (i = 0; i < to->num_connections; i++) {
+		if (to->connections[i]->name != NULL &&
+		    strcmp(to->connections[i]->name, connstr) == 0) {
+		  conn = i;
+		  break;
+		}
+	      }
+	    }
+	    if (conn >= 0 && conn < to->num_connections) {
+	      object_connect(obj, obj->handles[handle],
+			     to->connections[conn]);
+	    } else {
+	      message_error(_("Error loading diagram.\n"
+			      "connection point %s does not exist."),
+			    connstr);
+	    }
 	  }
 
+	  if (handlestr) xmlFree(handlestr);
+	  if (tostr) xmlFree(tostr);
+	  if (connstr) xmlFree(connstr);
+
 	  connection = connection->next;
 	}
       }
-      
     }
     
+    /* Now set up parent relationships. */
+    connections = obj_node->xmlChildrenNode;
+    while ((connections!=NULL) &&
+	   (strcmp(connections->name, "childnode")!=0))
+      connections = connections->next;
+    if (connections != NULL) {
+      tostr = xmlGetProp(connections, "parent");
+      if (tostr) {
+	obj->parent = g_hash_table_lookup(objects_hash, tostr);
+	if (obj->parent == NULL) {
+	  message_error(_("Can't find parent %s of %s object\n"), 
+			tostr, obj->type->name);
+	} else {
+	  obj->parent->children = g_list_prepend(obj->parent->children, obj);
+	}
+      }
+    }
+
     list = g_list_next(list);
     obj_node = obj_node->next;
   }
@@ -207,7 +320,15 @@ hash_free_string(gpointer       key,
 		 gpointer       value,
 		 gpointer       user_data)
 {
-  free(key);
+  g_free(key);
+}
+
+static xmlNodePtr
+find_node_named (xmlNodePtr p, const char *name)
+{
+  while (p && 0 != strcmp(p->name, name))
+    p = p->next;
+  return p;
 }
 
 static gboolean
@@ -215,7 +336,6 @@ diagram_data_load(const char *filename, DiagramData *data, void* user_data)
 {
   GHashTable *objects_hash;
   int fd;
-  struct stat stat_buf;
   GList *list;
   xmlDocPtr doc;
   xmlNodePtr diagramdata;
@@ -224,7 +344,13 @@ diagram_data_load(const char *filename, DiagramData *data, void* user_data)
   AttributeNode attr;
   Layer *layer;
   xmlNsPtr namespace;
-  
+  gchar firstchar;
+
+  if (g_file_test (filename, G_FILE_TEST_IS_DIR)) {
+    message_error(_("You must specify a file, not a directory.\n"));
+    return FALSE;
+  }
+
   fd = open(filename, O_RDONLY);
 
   if (fd==-1) {
@@ -232,29 +358,31 @@ diagram_data_load(const char *filename, DiagramData *data, void* user_data)
     return FALSE;
   }
 
-  fstat(fd, &stat_buf);
-  if (S_ISDIR(stat_buf.st_mode)) {
-    message_error(_("You must specify a file, not a directory.\n"));
-    return FALSE;
+  if (read(fd, &firstchar, 1)) {
+    data->is_compressed = (firstchar != '<');
+  } else {
+    /* Couldn't read a single char?  Set to default. */
+    data->is_compressed = prefs.new_diagram.compress_save;
   }
   
+  /* Note that this closing and opening means we can't read from a pipe */
   close(fd);
 
-  doc = xmlParseFile(filename);
+  doc = xmlDiaParseFile(filename);
 
   if (doc == NULL){
     message_error(_("Error loading diagram %s.\nUnknown file type."),filename);
     return FALSE;
   }
   
-  if (doc->root == NULL) {
+  if (doc->xmlRootNode == NULL) {
     message_error(_("Error loading diagram %s.\nUnknown file type."),filename);
     xmlFreeDoc (doc);
     return FALSE;
   }
 
-  namespace = xmlSearchNs(doc, doc->root, "dia");
-  if (strcmp (doc->root->name, "diagram") || (namespace == NULL)){
+  namespace = xmlSearchNs(doc, doc->xmlRootNode, "dia");
+  if (strcmp (doc->xmlRootNode->name, "diagram") || (namespace == NULL)){
     message_error(_("Error loading diagram %s.\nNot a Dia file."), filename);
     xmlFreeDoc (doc);
     return FALSE;
@@ -264,14 +392,20 @@ diagram_data_load(const char *filename, DiagramData *data, void* user_data)
   g_ptr_array_remove(data->layers, data->active_layer);
   layer_destroy(data->active_layer);
   
-  diagramdata = doc->root->childs;
+  diagramdata = 
+    find_node_named (doc->xmlRootNode->xmlChildrenNode, "diagramdata");
 
   /* Read in diagram data: */
-  data->bg_color = color_white;
+  data->bg_color = prefs.new_diagram.bg_color;
   attr = composite_find_attribute(diagramdata, "background");
   if (attr != NULL)
     data_color(attribute_first_data(attr), &data->bg_color);
 
+  data->pagebreak_color = prefs.new_diagram.pagebreak_color;
+  attr = composite_find_attribute(diagramdata, "pagebreak");
+  if (attr != NULL)
+    data_color(attribute_first_data(attr), &data->pagebreak_color);
+
   /* load paper information from diagramdata section */
   attr = composite_find_attribute(diagramdata, "paper");
   if (attr != NULL) {
@@ -282,6 +416,9 @@ diagram_data_load(const char *filename, DiagramData *data, void* user_data)
       g_free(data->paper.name);
       data->paper.name = data_string(attribute_first_data(attr));
     }
+    if (data->paper.name == NULL || data->paper.name[0] == '\0') {
+      data->paper.name = g_strdup(prefs.new_diagram.papertype);
+    }
     /* set default margins for paper size ... */
     dia_page_layout_get_default_margins(data->paper.name,
 					&data->paper.tmargin,
@@ -360,6 +497,12 @@ diagram_data_load(const char *filename, DiagramData *data, void* user_data)
     attr = composite_find_attribute(gridinfo, "visible_y");
     if (attr != NULL)
       data->grid.visible_y = data_int(attribute_first_data(attr));
+
+    data->grid.colour = prefs.new_diagram.grid_color;
+    attr = composite_find_attribute(diagramdata, "color");
+    if (attr != NULL)
+      data_color(attribute_first_data(attr), &data->grid.colour);
+
   }
   attr = composite_find_attribute(diagramdata, "guides");
   if (attr != NULL) {
@@ -391,27 +534,39 @@ diagram_data_load(const char *filename, DiagramData *data, void* user_data)
   }
 
   /* Read in all layers: */
-  layer_node = diagramdata->next;
+  layer_node =
+    find_node_named (doc->xmlRootNode->xmlChildrenNode, "layer");
 
   objects_hash = g_hash_table_new(g_str_hash, g_str_equal);
 
   while (layer_node != NULL) {
-    char *name;
+    gchar *name;
     char *visible;
-    name = (char *)xmlGetProp(layer_node, "name");
-    visible = (char *)xmlGetProp(layer_node, "visible");
-    layer = new_layer(g_strdup(name));
-    if (name) free(name);
     
+    if (xmlIsBlankNode(layer_node)) {
+      layer_node = layer_node->next;
+      continue;
+    }
+
+    if (!layer_node) break;
+    
+    name = (char *)xmlGetProp(layer_node, "name");
+    if (!name) break; /* name is mandatory */
+
+    visible = (char *)xmlGetProp(layer_node, "visible");
+
+    layer = new_layer(g_strdup(name), data);
+    if (name) xmlFree(name);
+
     layer->visible = FALSE;
     if ((visible) && (strcmp(visible, "true")==0))
       layer->visible = TRUE;
-    if (visible) free(visible);
+    if (visible) xmlFree(visible);
 
     /* Read in all objects: */
     
-    list = read_objects(layer_node, objects_hash, filename);
-    layer->objects = list;
+    list = read_objects(layer_node, objects_hash, filename, NULL);
+    layer_add_objects (layer, list);
     read_connections( list, layer_node, objects_hash);
 
     data_add_layer(data, layer);
@@ -420,17 +575,24 @@ diagram_data_load(const char *filename, DiagramData *data, void* user_data)
   }
 
   data->active_layer = (Layer *) g_ptr_array_index(data->layers, 0);
-  
+
   xmlFreeDoc(doc);
   
   g_hash_table_foreach(objects_hash, hash_free_string, NULL);
   
   g_hash_table_destroy(objects_hash);
-  
+
+  if (data->layers->len < 1) {
+    message_error (_("Error loading diagram:\n%s.\n"
+                     "A valid Dia file defines at least one layer."),
+		     filename);
+    return FALSE;
+  }
+
   return TRUE;
 }
 
-static void
+static gboolean
 write_objects(GList *objects, xmlNodePtr objects_node,
 	      GHashTable *objects_hash, int *obj_nr, const char *filename)
 {
@@ -441,9 +603,15 @@ write_objects(GList *objects, xmlNodePtr objects_node,
 
   list = objects;
   while (list != NULL) {
-    Object *obj = (Object *) list->data;
+    DiaObject *obj = (DiaObject *) list->data;
 
-    if IS_GROUP(obj) {
+    if (g_hash_table_lookup(objects_hash, obj))
+    {
+      list = g_list_next(list);
+      continue;
+    }
+
+    if (IS_GROUP(obj) && group_objects(obj) != NULL) {
       group_node = xmlNewChild(objects_node, NULL, "group", NULL);
       write_objects(group_objects(obj), group_node,
 		    objects_hash, obj_nr, filename);
@@ -451,7 +619,6 @@ write_objects(GList *objects, xmlNodePtr objects_node,
       obj_node = xmlNewChild(objects_node, NULL, "object", NULL);
     
       xmlSetProp(obj_node, "type", obj->type->name);
-      
       g_snprintf(buffer, 30, "%d", obj->type->version);
       xmlSetProp(obj_node, "version", buffer);
       
@@ -464,13 +631,28 @@ write_objects(GList *objects, xmlNodePtr objects_node,
       g_hash_table_insert(objects_hash, obj, GINT_TO_POINTER(*obj_nr));
       (*obj_nr)++;
       
+      /*
+      if (obj->can_parent && obj->children)
+      {
+	int res;
+	xmlNodePtr parent_node;
+        parent_node = xmlNewChild(obj_node, NULL, "children", NULL);
+        res = write_objects(obj->children, parent_node, objects_hash, obj_nr, filename);
+	if (!res) return FALSE;
+      }
+      */
     }
-    
+
     list = g_list_next(list);
   }
+  return TRUE;
 }
 
-static int
+/* Parents broke assumption that both objects and xml nodes are laid out
+ * linearly.  
+ */
+
+static gboolean
 write_connections(GList *objects, xmlNodePtr layer_node,
 		  GHashTable *objects_hash)
 {
@@ -482,9 +664,12 @@ write_connections(GList *objects, xmlNodePtr layer_node,
   int i;
 
   list = objects;
-  obj_node = layer_node->childs;
-  while (list != NULL) {
-    Object *obj = (Object *) list->data;
+  obj_node = layer_node->xmlChildrenNode;
+  while ((list != NULL) && (obj_node != NULL)) {
+    DiaObject *obj = (DiaObject *) list->data;
+
+    while (obj_node && xmlIsBlankNode(obj_node)) obj_node = obj_node->next;
+    if (!obj_node) break;
 
     if IS_GROUP(obj) {
       write_connections(group_objects(obj), obj_node, objects_hash);
@@ -499,7 +684,7 @@ write_connections(GList *objects, xmlNodePtr layer_node,
 	con_point = handle->connected_to;
 	
 	if ( con_point != NULL ) {
-	  Object *other_obj;
+	  DiaObject *other_obj;
 	  int con_point_nr;
 	  
 	  other_obj = con_point->object;
@@ -524,13 +709,27 @@ write_connections(GList *objects, xmlNodePtr layer_node,
 	  g_snprintf(buffer, 30, "O%d",
 		     GPOINTER_TO_INT(g_hash_table_lookup(objects_hash,
 							 other_obj)));
+
 	  xmlSetProp(connection, "to", buffer);
 	  /* to what connection_point on that object */
-	  g_snprintf(buffer, 30, "%d", con_point_nr);
+	  if (other_obj->connections[con_point_nr]->name != NULL) {
+	    g_snprintf(buffer, 30, "%s", other_obj->connections[con_point_nr]->name);
+	  } else {
+	    g_snprintf(buffer, 30, "%d", con_point_nr);
+	  }
 	  xmlSetProp(connection, "connection", buffer);
 	}
       }
     }
+
+    if (obj->parent) {
+      DiaObject *other_obj = obj->parent;
+      xmlNodePtr parent;
+      g_snprintf(buffer, 30, "O%d",
+		 GPOINTER_TO_INT(g_hash_table_lookup(objects_hash, other_obj)));
+      parent = xmlNewChild(obj_node, NULL, "childnode", NULL);
+      xmlSetProp(parent, "parent", buffer);
+    }
     
     list = g_list_next(list);
     obj_node = obj_node->next;
@@ -538,67 +737,39 @@ write_connections(GList *objects, xmlNodePtr layer_node,
   return TRUE;
 }
 
-static int
-diagram_data_save(DiagramData *data, const char *filename)
+/* Filename seems to be junk, but is passed on to objects */
+static xmlDocPtr
+diagram_data_write_doc(DiagramData *data, const char *filename)
 {
-  FILE *file;
   xmlDocPtr doc;
   xmlNodePtr tree;
   xmlNodePtr pageinfo, gridinfo, guideinfo;
   xmlNodePtr layer_node;
   GHashTable *objects_hash;
-  int res;
+  gboolean res;
   int obj_nr;
   int i;
   Layer *layer;
   AttributeNode attr;
   xmlNs *name_space;
-  char *bakname,*tmpname,*dirname,*p;
-  int mode,_umask;
-  int fildes;
-  int ret;
-   
-  /* build the temporary and backup file names */
-  dirname = g_strdup(filename);
-  p = strrchr((char *)dirname,G_DIR_SEPARATOR);
-  if (p) {
-    *(p+1) = 0;
-  } else {
-    g_free(dirname);
-    dirname = g_strdup("." G_DIR_SEPARATOR_S);
-  }
-  tmpname = g_strconcat(dirname,"__diaXXXXXX",NULL);
-  bakname = g_strconcat(filename,"~",NULL);
-
-  /* open a temporary name, and fix the modes to match what fopen() would have
-     done (mkstemp() is (rightly so) a bit paranoid for what we do). */
-  fildes = mkstemp(tmpname);
-  _umask = umask(0); umask(_umask);
-  mode = 0666 & ~_umask;
-  ret = fchmod(fildes,mode);
-  file = fdopen(fildes,"wb");
-
-  /* Now write the data in the temporary file name. */
-
-  if (file==NULL) {
-    message_error(_("Couldn't open: '%s' for writing.\n"), tmpname);
-    return FALSE;
-  }
-  fclose(file);
 
   doc = xmlNewDoc("1.0");
+  doc->encoding = xmlStrdup("UTF-8");
+  doc->xmlRootNode = xmlNewDocNode(doc, NULL, "diagram", NULL);
 
-  doc->root = xmlNewDocNode(doc, NULL, "diagram", NULL);
-
-  name_space = xmlNewNs(doc->root, "http://www.lysator.liu.se/~alla/dia/",
+  name_space = xmlNewNs(doc->xmlRootNode, 
+                        DIA_XML_NAME_SPACE_BASE,
 			"dia");
-  xmlSetNs(doc->root, name_space);
+  xmlSetNs(doc->xmlRootNode, name_space);
 
-  tree = xmlNewChild(doc->root, name_space, "diagramdata", NULL);
+  tree = xmlNewChild(doc->xmlRootNode, name_space, "diagramdata", NULL);
   
   attr = new_attribute((ObjectNode)tree, "background");
   data_add_color(attr, &data->bg_color);
 
+  attr = new_attribute((ObjectNode)tree, "pagebreak");
+  data_add_color(attr, &data->pagebreak_color);
+
   attr = new_attribute((ObjectNode)tree, "paper");
   pageinfo = data_add_composite(attr, "paper");
   data_add_string(composite_add_attribute(pageinfo, "name"),
@@ -631,9 +802,13 @@ diagram_data_save(DiagramData *data, const char *filename)
   data_add_real(composite_add_attribute(gridinfo, "width_y"),
 		data->grid.width_y);
   data_add_int(composite_add_attribute(gridinfo, "visible_x"),
-		data->grid.visible_x);
+	       data->grid.visible_x);
   data_add_int(composite_add_attribute(gridinfo, "visible_y"),
-		data->grid.visible_y);
+	       data->grid.visible_y);
+  attr = new_attribute((ObjectNode)tree, "color");
+  data_add_composite(gridinfo, "color");
+  data_add_color(attr, &data->grid.colour);
+  
 
   attr = new_attribute((ObjectNode)tree, "guides");
   guideinfo = data_add_composite(attr, "guides");
@@ -649,9 +824,10 @@ diagram_data_save(DiagramData *data, const char *filename)
   obj_nr = 0;
 
   for (i = 0; i < data->layers->len; i++) {
-    layer_node = xmlNewChild(doc->root, name_space, "layer", NULL);
+    layer_node = xmlNewChild(doc->xmlRootNode, name_space, "layer", NULL);
     layer = (Layer *) g_ptr_array_index(data->layers, i);
     xmlSetProp(layer_node, "name", layer->name);
+
     if (layer->visible)
       xmlSetProp(layer_node, "visible", "true");
     else
@@ -661,19 +837,77 @@ diagram_data_save(DiagramData *data, const char *filename)
 		  objects_hash, &obj_nr, filename);
   
     res = write_connections(layer->objects, layer_node, objects_hash);
+    /* Why do we bail out like this?  It leaks! */
     if (!res)
-      return FALSE;
+      return NULL;
   }
   g_hash_table_destroy(objects_hash);
 
-  if (prefs.compress_save)
+  if (data->is_compressed)
     xmlSetDocCompressMode(doc, 9);
   else
     xmlSetDocCompressMode(doc, 0);
-    
-  ret = xmlSaveFile (tmpname, doc);
+
+  return doc;
+}
+
+/** This tries to save the diagram into a file, without any backup
+ * Returns >= 0 on success.
+ * Only for internal use. */
+static int
+diagram_data_raw_save(DiagramData *data, const char *filename)
+{
+  xmlDocPtr doc;
+  int ret;
+
+  doc = diagram_data_write_doc(data, filename);
+
+  ret = xmlDiaSaveFile (filename, doc);
   xmlFreeDoc(doc);
 
+  return ret;
+}
+
+/** This saves the diagram, using a backup in case of failure */
+static int
+diagram_data_save(DiagramData *data, const char *filename)
+{
+  FILE *file;
+  char *bakname,*tmpname,*dirname,*p;
+  int mode,_umask;
+  int fildes;
+  int ret;
+   
+  /* build the temporary and backup file names */
+  dirname = g_strdup(filename);
+  p = strrchr((char *)dirname,G_DIR_SEPARATOR);
+  if (p) {
+    *(p+1) = 0;
+  } else {
+    g_free(dirname);
+    dirname = g_strdup("." G_DIR_SEPARATOR_S);
+  }
+  tmpname = g_strconcat(dirname,"__diaXXXXXX",NULL);
+  bakname = g_strconcat(filename,"~",NULL);
+
+  /* open a temporary name, and fix the modes to match what fopen() would have
+     done (mkstemp() is (rightly so) a bit paranoid for what we do). */
+  fildes = mkstemp(tmpname);
+  _umask = umask(0); umask(_umask);
+  mode = 0666 & ~_umask;
+  ret = fchmod(fildes,mode);
+  file = fdopen(fildes,"wb");
+
+  /* Now write the data in the temporary file name. */
+
+  if (file==NULL) {
+    message_error(_("Can't open output file %s: %s\n"), tmpname, strerror(errno));
+    return FALSE;
+  }
+  fclose(file);
+
+  ret = diagram_data_raw_save(data, tmpname);
+
   if (ret < 0) {
     /* Save failed; we clean our stuff up, without touching the file named
        "filename" if it existed. */
@@ -705,11 +939,62 @@ diagram_save(Diagram *dia, const char *filename)
   }
 
   dia->unsaved = FALSE;
+  undo_mark_save(dia->undo);
   diagram_set_modified (dia, FALSE);
 
+  diagram_cleanup_autosave(dia);
+
   return TRUE;
 }
 
+/* Autosave stuff.  Needs to use low-level save to avoid setting and resetting flags */
+void
+diagram_cleanup_autosave(Diagram *dia)
+{
+  gchar *savefile;
+  struct stat statbuf;
+
+  savefile = dia->autosavefilename;
+  if (savefile == NULL) return;
+  printf("Cleaning up autosave %s for %s\n", savefile, dia->filename);
+
+  if (stat(savefile, &statbuf) == 0) { /* Success */
+    unlink(savefile);
+  }
+  g_free(savefile);
+  dia->autosavefilename = NULL;
+  dia->autosaved = FALSE;
+}
+
+/** Absolutely autosave a diagram.
+ * Called after a periodic check at the first idleness.
+ */
+void
+diagram_autosave(Diagram *dia)
+{
+  gchar *save_filename;
+
+  /* Must check if the diagram is still valid, or Death Ensues! */
+  GList *diagrams = dia_open_diagrams();
+  Diagram *diagram;
+  while (diagrams != NULL) {
+    diagram = (Diagram *)diagrams->data;
+    if (diagram == dia &&
+	diagram_is_modified(diagram) && 
+	!diagram->autosaved) {
+      save_filename = g_strdup_printf("%s.autosave", dia->filename);
+
+      if (dia->autosavefilename != NULL) 
+	g_free(dia->autosavefilename);
+      dia->autosavefilename = save_filename;
+      diagram_data_raw_save(dia->data, save_filename);
+      dia->autosaved = TRUE;
+      return;
+    }
+    diagrams = g_list_next(diagrams);
+  }
+}
+
 /* --- filter interfaces --- */
 static void
 export_native(DiagramData *data, const gchar *filename,
@@ -720,12 +1005,12 @@ export_native(DiagramData *data, const gchar *filename,
 
 static const gchar *extensions[] = { "dia", NULL };
 DiaExportFilter dia_export_filter = {
-  N_("Native Dia Diagram"),
+  N_("Dia Diagram File"),
   extensions,
   export_native
 };
 DiaImportFilter dia_import_filter = {
-  N_("Native Dia Diagram"),
+  N_("Dia Diagram File"),
   extensions,
   diagram_data_load
 };
