@@ -26,11 +26,16 @@
 #include "connectionpoint_ops.h"
 #include "message.h"
 #include "properties.h"
-#include "render_gdk.h"
 #include "select.h"
 #include "preferences.h"
+#include "cursor.h"
+#include "highlight.h"
 
-static Object *click_select_object(DDisplay *ddisp, Point *clickedpoint,
+#include "diacanvas.h"
+#include "prop_text.h"
+#include "gtk/gtk.h"
+
+static DiaObject *click_select_object(DDisplay *ddisp, Point *clickedpoint,
 				   GdkEventButton *event);
 static int do_if_clicked_handle(DDisplay *ddisp, ModifyTool *tool,
 				Point *clickedpoint,
@@ -43,13 +48,18 @@ static void modify_motion(ModifyTool *tool, GdkEventMotion *event,
 			  DDisplay *ddisp);
 static void modify_double_click(ModifyTool *tool, GdkEventButton *event,
 				DDisplay *ddisp);
+static void modify_make_text_edit(DDisplay *ddisp, DiaObject *obj, 
+                                  Point *clickedpoint);
+static void modify_start_text_edit(DDisplay *ddisp, Text *text, DiaObject *obj, 
+                                   Point *clickedpoint);
+
 
 Tool *
 create_modify_tool(void)
 {
   ModifyTool *tool;
 
-  tool = g_new(ModifyTool, 1);
+  tool = g_new0(ModifyTool, 1);
   tool->tool.type = MODIFY_TOOL;
   tool->tool.button_press_func = (ButtonPressFunc) &modify_button_press;
   tool->tool.button_release_func = (ButtonReleaseFunc) &modify_button_release;
@@ -58,13 +68,13 @@ create_modify_tool(void)
   tool->gc = NULL;
   tool->state = STATE_NONE;
   tool->break_connections = 0;
+  tool->auto_scrolled = FALSE;
 
   tool->orig_pos = NULL;
   
   return (Tool *)tool;
 }
 
-
 void
 free_modify_tool(Tool *tool)
 {
@@ -77,16 +87,16 @@ free_modify_tool(Tool *tool)
 /*
   This function is buggy. Fix it later!
 static void
-transitive_select(DDisplay *ddisp, Point *clickedpoint, Object *obj)
+transitive_select(DDisplay *ddisp, Point *clickedpoint, DiaObject *obj)
 {
   guint i;
   GList *j;
-  Object *obj1;
+  DiaObject *obj1;
 
   for(i = 0; i < obj->num_connections; i++) {
     printf("%d\n", i);
     j = obj->connections[i]->connected;
-    while(j != NULL && (obj1 = (Object *)j->data) != NULL) {
+    while(j != NULL && (obj1 = (DiaObject *)j->data) != NULL) {
       diagram_select(ddisp->diagram, obj1);
       obj1->ops->select(obj1, clickedpoint,
 			(Renderer *)ddisp->renderer);
@@ -97,13 +107,13 @@ transitive_select(DDisplay *ddisp, Point *clickedpoint, Object *obj)
 }
 */
 
-static Object *
+static DiaObject *
 click_select_object(DDisplay *ddisp, Point *clickedpoint,
 		    GdkEventButton *event)
 {
   Diagram *diagram;
   real click_distance;
-  Object *obj;
+  DiaObject *obj;
   
   diagram = ddisp->diagram;
   
@@ -130,7 +140,7 @@ click_select_object(DDisplay *ddisp, Point *clickedpoint,
       
       diagram_select(diagram, obj);
       obj->ops->selectf(obj, clickedpoint,
-		       (Renderer *)ddisp->renderer);
+		        ddisp->renderer);
 
       /*
 	This stuff is buggy, fix it later.
@@ -139,7 +149,7 @@ click_select_object(DDisplay *ddisp, Point *clickedpoint,
       }
       */
 
-      diagram_update_menu_sensitivity(diagram);
+      ddisplay_do_update_menu_sensitivity(ddisp);
       object_add_updates_list(diagram->data->selected, diagram);
       diagram_flush(diagram);
 
@@ -147,16 +157,20 @@ click_select_object(DDisplay *ddisp, Point *clickedpoint,
     } else { /* Clicked on already selected. */
       /*printf("Already selected\n");*/
       obj->ops->selectf(obj, clickedpoint,
-		       (Renderer *)ddisp->renderer);
+		       ddisp->renderer);
       object_add_updates_list(diagram->data->selected, diagram);
       diagram_flush(diagram);
       
       if (event->state & GDK_SHIFT_MASK) { /* Multi-select */
 	/* Remove the selected selected */
-	diagram_update_menu_sensitivity(diagram);
-	diagram_unselect_object(ddisp->diagram, (Object *)already->data);
+	ddisplay_do_update_menu_sensitivity(ddisp);
+	diagram_unselect_object(ddisp->diagram, (DiaObject *)already->data);
 	diagram_flush(ddisp->diagram);
       } else {
+	/* Maybe start editing text */
+#ifdef NEW_TEXT_EDIT
+	modify_make_text_edit(ddisp, obj, clickedpoint);
+#endif
 	return obj;
       }
     }
@@ -165,10 +179,19 @@ click_select_object(DDisplay *ddisp, Point *clickedpoint,
   return NULL;
 }
 
+static glong
+time_micro()
+{
+  GTimeVal tv;
+
+  g_get_current_time(&tv);
+  return tv.tv_sec*G_USEC_PER_SEC+tv.tv_usec;
+}
+
 static int do_if_clicked_handle(DDisplay *ddisp, ModifyTool *tool,
 				Point *clickedpoint, GdkEventButton *event)
 {
-  Object *obj;
+  DiaObject *obj;
   Handle *handle;
   real dist;
   
@@ -184,7 +207,9 @@ static int do_if_clicked_handle(DDisplay *ddisp, ModifyTool *tool,
     gdk_pointer_grab (ddisp->canvas->window, FALSE,
                       GDK_POINTER_MOTION_HINT_MASK | GDK_BUTTON1_MOTION_MASK | GDK_BUTTON_RELEASE_MASK,
                       NULL, NULL, event->time);
-    tool->start_at = *clickedpoint;
+    tool->start_at = handle->pos;
+    tool->start_time = time_micro();
+    ddisplay_set_all_cursor(get_cursor(CURSOR_SCROLL));
     return TRUE;
   }
   return FALSE;
@@ -195,20 +220,20 @@ modify_button_press(ModifyTool *tool, GdkEventButton *event,
 		     DDisplay *ddisp)
 {
   Point clickedpoint;
-  Object *clicked_obj;
+  DiaObject *clicked_obj;
   
   ddisplay_untransform_coords(ddisp,
 			      (int)event->x, (int)event->y,
 			      &clickedpoint.x, &clickedpoint.y);
 
+
+  if (do_if_clicked_handle(ddisp, tool, &clickedpoint, event))
+    return;
+  
+  clicked_obj = click_select_object(ddisp, &clickedpoint, event);
   if (do_if_clicked_handle(ddisp, tool, &clickedpoint, event))
     return;
 
-  clicked_obj = click_select_object(ddisp, &clickedpoint, event);
-  
-  if (do_if_clicked_handle(ddisp, tool, &clickedpoint, event))
-    return;
-  
   if ( clicked_obj != NULL ) {
     tool->state = STATE_MOVE_OBJECT;
     tool->object = clicked_obj;
@@ -219,13 +244,15 @@ modify_button_press(ModifyTool *tool, GdkEventButton *event,
                       GDK_POINTER_MOTION_HINT_MASK | GDK_BUTTON1_MOTION_MASK | GDK_BUTTON_RELEASE_MASK,
                       NULL, NULL, event->time);
     tool->start_at = clickedpoint;
+    tool->start_time = time_micro();
+    ddisplay_set_all_cursor(get_cursor(CURSOR_SCROLL));
   } else {
     tool->state = STATE_BOX_SELECT;
     tool->start_box = clickedpoint;
     tool->end_box = clickedpoint;
     tool->x1 = tool->x2 = (int) event->x;
     tool->y1 = tool->y2 = (int) event->y;
-    
+
     if (tool->gc == NULL) {
       tool->gc = gdk_gc_new(ddisp->canvas->window);
       gdk_gc_set_line_attributes(tool->gc, 1, GDK_LINE_ON_OFF_DASH, 
@@ -250,7 +277,7 @@ modify_double_click(ModifyTool *tool, GdkEventButton *event,
 		    DDisplay *ddisp)
 {
   Point clickedpoint;
-  Object *clicked_obj;
+  DiaObject *clicked_obj;
   
   ddisplay_untransform_coords(ddisp,
 			      (int)event->x, (int)event->y,
@@ -264,13 +291,65 @@ modify_double_click(ModifyTool *tool, GdkEventButton *event,
     /*printf("didn't select object\n");*/
     if (!(event->state & GDK_SHIFT_MASK)) {
       /* Not Multi-select => Remove all selected */
-      diagram_update_menu_sensitivity(ddisp->diagram);
+      ddisplay_do_update_menu_sensitivity(ddisp);
       diagram_remove_all_selected(ddisp->diagram, TRUE);
       diagram_flush(ddisp->diagram);
     }
   }
 }
 
+#define MIN_PIXELS 10
+
+/** Makes sure that objects aren't accidentally moved when double-clicking
+ * for properties.  Objects do not move unless double click time has passed
+ * or the move is 'significant'.  Allowing the 'significant' move makes a
+ * regular grab-and-move less jerky.
+ *
+ * There's a slight chance that the user could move outside the
+ * minimum movement limit and back in within the double click time
+ * (normally .25 seconds), but that's very, very unlikely.  If
+ * it happens, the cursor wouldn't reset just right.
+ */
+
+static gboolean
+modify_move_already(ModifyTool *tool, DDisplay *ddisp, Point *to)
+{
+  static gboolean settings_taken = FALSE;
+  static int double_click_time = 250;
+  real dist;
+
+  if (!settings_taken) {
+    /* One could argue that if the settings were updated while running,
+       we should re-read them.  But I don't see any way to get notified,
+       and I don't want to do this whole thing for each bit of the
+       move --Lars */
+    GtkSettings *settings = gtk_settings_get_default();
+    if (settings == NULL) {
+      g_message(_("Couldn't get GTK settings"));
+    } else {
+      g_object_get(G_OBJECT(settings), 
+		   "gtk-double-click-time", &double_click_time, NULL);
+    }
+    settings_taken = TRUE;
+  }
+  if (tool->start_time < time_micro()-double_click_time*1000) {
+    return TRUE;
+  }
+  dist = distance_point_point_manhattan(&tool->start_at, to);
+  if (ddisp->grid.snap) {
+    real grid_x = ddisp->diagram->data->grid.width_x;
+    real grid_y = ddisp->diagram->data->grid.width_y;
+    if (dist > grid_x || dist > grid_y) {
+      return TRUE;
+    }
+  }
+  if (ddisplay_transform_length(ddisp, dist) > MIN_PIXELS) {
+    return (ddisplay_transform_length(ddisp, dist) > MIN_PIXELS);
+  } else {
+    return FALSE;
+  }
+}
+
 
 static void
 modify_motion(ModifyTool *tool, GdkEventMotion *event,
@@ -280,6 +359,7 @@ modify_motion(ModifyTool *tool, GdkEventMotion *event,
   Point now, delta, full_delta;
   gboolean auto_scroll, vertical = FALSE;
   ConnectionPoint *connectionpoint;
+  ObjectChange *objchange;
 
   if (tool->state==STATE_NONE)
     return; /* Fast path... */
@@ -288,19 +368,22 @@ modify_motion(ModifyTool *tool, GdkEventMotion *event,
   
   ddisplay_untransform_coords(ddisp, event->x, event->y, &to.x, &to.y);
 
+  if (!modify_move_already(tool, ddisp, &to)) return;
+
   switch (tool->state) {
   case STATE_MOVE_OBJECT:
 
     if (tool->orig_pos == NULL) {
       GList *list;
       int i;
-      Object *obj;
-      
-      list = ddisp->diagram->data->selected;
+      DiaObject *obj;
+
+      /* consider non-selected children affected */
+      list = parent_list_affected(ddisp->diagram->data->selected);
       tool->orig_pos = g_new(Point, g_list_length(list));
       i=0;
       while (list != NULL) {
-	obj = (Object *)  list->data;
+	obj = (DiaObject *)  list->data;
 	tool->orig_pos[i] = obj->position;
 	list = g_list_next(list); i++;
       }
@@ -333,29 +416,45 @@ modify_motion(ModifyTool *tool, GdkEventMotion *event,
     }
 
     object_add_updates_list(ddisp->diagram->data->selected, ddisp->diagram);
-    object_list_move_delta(ddisp->diagram->data->selected, &delta);
+    objchange = object_list_move_delta(ddisp->diagram->data->selected, &delta);
+    if (objchange != NULL) {
+      undo_object_change(ddisp->diagram, tool->object, objchange);
+    }
     object_add_updates_list(ddisp->diagram->data->selected, ddisp->diagram);
   
     diagram_update_connections_selection(ddisp->diagram);
     diagram_flush(ddisp->diagram);
     break;
   case STATE_MOVE_HANDLE:
+    full_delta = to;
+    point_sub(&full_delta, &tool->start_at);
+
+    /* make sure resizing is restricted to its parent */
+
+
+    /* if resize was blocked by parent, that means the resizing was
+      outward, thus it won't bother the children so we don't have to
+      check the children */
+    if (!parent_handle_move_out_check(tool->object, &to))
+      parent_handle_move_in_check(tool->object, &to, &tool->start_at);
+
     /* Move to ConnectionPoint if near: */
     connectionpoint =
-      object_find_connectpoint_display(ddisp, &to);
+      object_find_connectpoint_display(ddisp, &to, tool->object);
 
-    if (event->state & GDK_CONTROL_MASK) {
-      full_delta = to;
-      point_sub(&full_delta, &tool->start_at);
+    if (event->state & GDK_CONTROL_MASK)
       vertical = (fabs(full_delta.x) < fabs(full_delta.y));
-    }
 
     if ( (tool->handle->connect_type != HANDLE_NONCONNECTABLE) &&
 	 (connectionpoint != NULL) ) {
       to = connectionpoint->pos;
+      highlight_object(connectionpoint->object, NULL, ddisp->diagram);
+      ddisplay_set_all_cursor(get_cursor(CURSOR_CONNECT));
     } else {
       /* No connectionopoint near, then snap to grid (if enabled) */
       snap_to_grid(ddisp, &to.x, &to.y);
+      highlight_reset_all(ddisp->diagram);
+      ddisplay_set_all_cursor(get_cursor(CURSOR_SCROLL));
     }
 
     if (tool->break_connections) {
@@ -383,8 +482,12 @@ modify_motion(ModifyTool *tool, GdkEventMotion *event,
     }
 
     object_add_updates(tool->object, ddisp->diagram);
-    tool->object->ops->move_handle(tool->object, tool->handle, &to,
-				   HANDLE_MOVE_USER,0);
+    objchange = tool->object->ops->move_handle(tool->object, tool->handle, 
+					       &to, connectionpoint,
+					       HANDLE_MOVE_USER, 0);
+    if (objchange != NULL) {
+      undo_object_change(ddisp->diagram, tool->object, objchange);
+    }
     object_add_updates(tool->object, ddisp->diagram);
   
     diagram_update_connections_selection(ddisp->diagram);
@@ -392,7 +495,7 @@ modify_motion(ModifyTool *tool, GdkEventMotion *event,
     break;
   case STATE_BOX_SELECT:
 
-    if (! auto_scroll)
+    if (!auto_scroll && !tool->auto_scrolled)
     {
       gdk_draw_rectangle (ddisp->canvas->window, tool->gc, FALSE,
 			  tool->x1, tool->y1,
@@ -422,6 +525,7 @@ modify_motion(ModifyTool *tool, GdkEventMotion *event,
   }
 
   tool->last_to = to;
+  tool->auto_scrolled = auto_scroll;
 }
 
 
@@ -429,28 +533,42 @@ static void
 modify_button_release(ModifyTool *tool, GdkEventButton *event,
 		      DDisplay *ddisp)
 {
-  Point *dest_pos;
+  Point *dest_pos, to;
   GList *list;
   int i;
-  Object *obj;
+  DiaObject *obj;
+  ObjectChange *objchange;
   
+  tool->break_connections = FALSE;
+  ddisplay_set_all_cursor(default_cursor);
+
   switch (tool->state) {
   case STATE_MOVE_OBJECT:
+    /* Return to normal state */
     gdk_pointer_ungrab (event->time);
+
+    ddisplay_untransform_coords(ddisp, event->x, event->y, &to.x, &to.y);
+    if (!modify_move_already(tool, ddisp, &to)) {
+      tool->orig_pos = NULL;
+      tool->state = STATE_NONE;
+      return;
+    }
+
     diagram_update_connections_selection(ddisp->diagram);
 
     if (tool->orig_pos != NULL) {
-      list = ddisp->diagram->data->selected;
+      /* consider the non-selected children affected */
+      list = parent_list_affected(ddisp->diagram->data->selected);
       dest_pos = g_new(Point, g_list_length(list));
       i=0;
       while (list != NULL) {
-	obj = (Object *)  list->data;
+	obj = (DiaObject *)  list->data;
 	dest_pos[i] = obj->position;
 	list = g_list_next(list); i++;
       }
-      
+
       undo_move_objects(ddisp->diagram, tool->orig_pos, dest_pos,
-			g_list_copy(ddisp->diagram->data->selected));
+			parent_list_affected(ddisp->diagram->data->selected));
     }
     
     ddisplay_connect_selected(ddisp); /* pushes UNDO info */
@@ -465,6 +583,7 @@ modify_button_release(ModifyTool *tool, GdkEventButton *event,
     break;
   case STATE_MOVE_HANDLE:
     gdk_pointer_ungrab (event->time);
+    tool->state = STATE_NONE;
 
     if (tool->orig_pos != NULL) {
       undo_move_handle(ddisp->diagram, tool->handle, tool->object,
@@ -473,9 +592,13 @@ modify_button_release(ModifyTool *tool, GdkEventButton *event,
     
     /* Final move: */
     object_add_updates(tool->object, ddisp->diagram);
-    tool->object->ops->move_handle(tool->object, tool->handle,
-				   &tool->last_to,
-				   HANDLE_MOVE_USER_FINAL,0);
+    objchange = tool->object->ops->move_handle(tool->object, tool->handle,
+					       &tool->last_to, NULL,
+					       HANDLE_MOVE_USER_FINAL,0);
+    if (objchange != NULL) {
+      undo_object_change(ddisp->diagram, tool->object, objchange);
+    }
+
     object_add_updates(tool->object, ddisp->diagram);
 
     /* Connect if possible: */
@@ -484,6 +607,7 @@ modify_button_release(ModifyTool *tool, GdkEventButton *event,
       diagram_update_connections_selection(ddisp->diagram);
     }
     
+    highlight_reset_all(ddisp->diagram);
     diagram_flush(ddisp->diagram);
     
     diagram_modified(ddisp->diagram);
@@ -496,21 +620,20 @@ modify_button_release(ModifyTool *tool, GdkEventButton *event,
       tool->orig_pos = NULL;
     }
 
-
-    
-    tool->state = STATE_NONE;
     break;
   case STATE_BOX_SELECT:
     gdk_pointer_ungrab (event->time);
     /* Remove last box: */
-    gdk_draw_rectangle (ddisp->canvas->window, tool->gc, FALSE,
+    if (!tool->auto_scrolled) {
+      gdk_draw_rectangle (ddisp->canvas->window, tool->gc, FALSE,
 			tool->x1, tool->y1,
 			tool->x2 - tool->x1, tool->y2 - tool->y1);
+    }
 
     {
       Rectangle r;
       GList *list, *list_to_free;
-      Object *obj;
+      DiaObject *obj;
 
       r.left = MIN(tool->start_box.x, tool->end_box.x);
       r.right = MAX(tool->start_box.x, tool->end_box.x);
@@ -540,7 +663,7 @@ modify_button_release(ModifyTool *tool, GdkEventButton *event,
         GList *intersection = NULL;
 
         while (list != NULL) {
-          obj = (Object *)list->data;
+          obj = (DiaObject *)list->data;
           
           if (diagram_is_selected(ddisp->diagram, obj)) {
             intersection = g_list_append(intersection, obj);
@@ -551,7 +674,7 @@ modify_button_release(ModifyTool *tool, GdkEventButton *event,
         list = intersection;
         diagram_remove_all_selected(ddisp->diagram, TRUE);
         while (list != NULL) {
-          obj = (Object *)list->data;
+          obj = (DiaObject *)list->data;
 
           diagram_select(ddisp->diagram, obj);
 
@@ -560,7 +683,7 @@ modify_button_release(ModifyTool *tool, GdkEventButton *event,
         g_list_free(intersection);
       } else {
         while (list != NULL) {
-          obj = (Object *)list->data;
+          obj = (DiaObject *)list->data;
           
           if (selection_style == SELECT_REMOVE) {
             if (diagram_is_selected(ddisp->diagram, obj))
@@ -583,7 +706,7 @@ modify_button_release(ModifyTool *tool, GdkEventButton *event,
       
     }
     
-    diagram_update_menu_sensitivity(ddisp->diagram);
+    ddisplay_do_update_menu_sensitivity(ddisp);
     ddisplay_flush(ddisp);
 
     tool->state = STATE_NONE;
@@ -594,11 +717,197 @@ modify_button_release(ModifyTool *tool, GdkEventButton *event,
     message_error("Internal error: Strange state in modify_tool\n");
       
   }
-  tool->break_connections = FALSE;
 }
 
+#define EDIT_BORDER_WIDTH 5
 
+gboolean
+modify_edit_end(GtkWidget *widget, GdkEventFocus *event, gpointer data)
+{
+  int foo = printf("Ending focus\n");
+  GtkTextView *view = GTK_TEXT_VIEW(widget);
+  DiaObject *obj = (DiaObject*)data;
+  GQuark quark = g_quark_from_string(PROP_TYPE_TEXT);
+  PropDescription *props = obj->ops->describe_props(obj);
+  int i;
 
+  for (i = 0; props[i].name != NULL; i++) {
+    printf("Testing to remove: %s\n", props[i].name);
+    if (props[i].type_quark == quark) {
+      GPtrArray *textprops = g_ptr_array_sized_new(1);
+      TextProperty *textprop;
+      Property *prop = props[i].ops->new_prop(&props[i], pdtpp_true);
+      GtkTextBuffer *buf;
+      GtkTextIter start, end;
+      ObjectChange *change;
 
+      printf("Going to stop %d\n", i);
+      buf = gtk_text_view_get_buffer(view);
+      g_ptr_array_add(textprops, prop);
+      obj->ops->get_props(obj, textprops);
+      textprop = (TextProperty*)prop;
+      if (textprop->text_data != NULL) g_free(textprop->text_data);
+      gtk_text_buffer_get_bounds(buf, &start, &end);
+      textprop->text_data = gtk_text_buffer_get_text(buf, &start, &end, TRUE);
+      printf("Setting text %s\n", textprop->text_data);
+      obj->ops->set_props(obj, textprops);
+      gtk_widget_destroy(widget);
+    }
+  }
+  return FALSE;
+}
 
+/** Start editing the first text among selected objects, if any */
+void
+modify_edit_first_text(DDisplay *ddisp)
+{
+  GList *edits = ddisp->diagram->data->text_edits;
 
+  if (edits != NULL) {
+    Text *text = (Text*)edits->data;
+    modify_start_text_edit(ddisp, text, text->parent_object, NULL);
+  }
+}
+
+void
+modify_start_text_edit(DDisplay *ddisp, Text *text, DiaObject *obj, Point *clickedpoint)
+{
+  GtkWidget *view = gtk_text_view_new();
+  int x, y, i;
+  GtkTextBuffer *buf;
+  GtkTextTag *fonttag;
+  GtkTextIter start, end;
+  real ascent;
+  int ascent_pixels;
+
+  printf("modify_start_text_edit\n");
+  ddisplay_transform_coords(ddisp,
+			    text->position.x,
+			    text->position.y,
+			    &x, &y);
+  ascent = dia_font_scaled_ascent(text->line[0], 
+				  text->font,
+				  text->height,
+				  ddisp->zoom_factor);
+  printf("Text prop string %s pos %d, %d ascent %f\n",
+	 text->line[0], x, y, ascent);
+  ascent_pixels = ddisplay_transform_length(ddisp, ascent);
+  y -= ascent_pixels;
+  dia_canvas_put(DIA_CANVAS(ddisp->canvas), view, 
+		 x-EDIT_BORDER_WIDTH, y-EDIT_BORDER_WIDTH);
+  buf = gtk_text_view_get_buffer(GTK_TEXT_VIEW(view));
+  for (i = 0; i < text->numlines; i++) {
+    gtk_text_buffer_insert_at_cursor(buf, text->line[i], -1);
+  }
+  fonttag = 
+    gtk_text_buffer_create_tag(buf,
+			       NULL,
+			       "font-desc",
+			       text->font->pfd,
+			       NULL);
+  gtk_text_buffer_get_bounds(buf, &start, &end);
+  gtk_text_buffer_apply_tag(buf, fonttag, &start, &end);
+
+  printf("Above lines %d below %d\n",
+	 gtk_text_view_get_pixels_above_lines(GTK_TEXT_VIEW(view)),
+	 gtk_text_view_get_pixels_below_lines(GTK_TEXT_VIEW(view)));
+
+  gtk_text_view_set_border_window_size(GTK_TEXT_VIEW(view), 
+				       GTK_TEXT_WINDOW_LEFT, 
+				       EDIT_BORDER_WIDTH);
+  gtk_text_view_set_border_window_size(GTK_TEXT_VIEW(view), 
+				       GTK_TEXT_WINDOW_RIGHT, 
+				       EDIT_BORDER_WIDTH);
+  gtk_text_view_set_border_window_size(GTK_TEXT_VIEW(view), 
+				       GTK_TEXT_WINDOW_TOP,
+				       EDIT_BORDER_WIDTH);
+  gtk_text_view_set_border_window_size(GTK_TEXT_VIEW(view), 
+				       GTK_TEXT_WINDOW_BOTTOM,
+				       EDIT_BORDER_WIDTH);
+  /* Using deprecated function because the fucking gobject documentation
+   * fucking sucks. */
+#ifdef NEW_TEXT_EDIT
+  gtk_signal_connect(GTK_OBJECT(view), "focus-out-event",
+		     modify_edit_end, obj);
+#endif
+  gtk_widget_grab_focus(view);
+  gtk_widget_show(view);
+}
+
+void
+modify_make_text_edit(DDisplay *ddisp, DiaObject *obj, Point *clickedpoint)
+{
+  PropDescription *props = obj->ops->describe_props(obj);
+  int i;
+  for (i = 0; props[i].name != NULL; i++) {
+    GQuark type = g_quark_from_string(PROP_TYPE_TEXT);
+    printf("Testing %s\n", props[i].type);
+    if (props[i].type_quark == type) {
+      GtkWidget *view = gtk_text_view_new();
+      GPtrArray *textprops = g_ptr_array_sized_new(1);
+      TextProperty *textprop;
+      Property *prop = props[i].ops->new_prop(&props[i], pdtpp_true);
+      int x, y;
+      GtkTextBuffer *buf;
+      GtkTextTag *fonttag;
+      GtkTextIter start, end;
+      real ascent;
+      int ascent_pixels;
+
+      g_ptr_array_add(textprops, prop);
+
+      printf("Found text prop %d\n", i);
+      obj->ops->get_props(obj, textprops);
+      textprop = (TextProperty*)prop;
+      ddisplay_transform_coords(ddisp,
+				textprop->attr.position.x,
+				textprop->attr.position.y,
+				&x, &y);
+      ascent = dia_font_scaled_ascent(textprop->text_data, 
+				      textprop->attr.font,
+				      textprop->attr.height,
+				      ddisp->zoom_factor);
+      printf("Text prop string %s pos %d, %d ascent %f\n",
+	     textprop->text_data, x, y, ascent);
+      ascent_pixels = ddisplay_transform_length(ddisp, ascent);
+      y -= ascent_pixels;
+      dia_canvas_put(DIA_CANVAS(ddisp->canvas), view, 
+		     x-EDIT_BORDER_WIDTH, y-EDIT_BORDER_WIDTH);
+      buf = gtk_text_view_get_buffer(GTK_TEXT_VIEW(view));
+      gtk_text_buffer_insert_at_cursor(buf, textprop->text_data, -1);
+      fonttag = 
+	gtk_text_buffer_create_tag(buf,
+				   NULL,
+				   "font-desc",
+				   textprop->attr.font->pfd,
+				   NULL);
+      gtk_text_buffer_get_bounds(buf, &start, &end);
+      gtk_text_buffer_apply_tag(buf, fonttag, &start, &end);
+
+      printf("Above lines %d below %d\n",
+	     gtk_text_view_get_pixels_above_lines(GTK_TEXT_VIEW(view)),
+	     gtk_text_view_get_pixels_below_lines(GTK_TEXT_VIEW(view)));
+
+      gtk_text_view_set_border_window_size(GTK_TEXT_VIEW(view), 
+					   GTK_TEXT_WINDOW_LEFT, 
+					   EDIT_BORDER_WIDTH);
+      gtk_text_view_set_border_window_size(GTK_TEXT_VIEW(view), 
+					   GTK_TEXT_WINDOW_RIGHT, 
+					   EDIT_BORDER_WIDTH);
+      gtk_text_view_set_border_window_size(GTK_TEXT_VIEW(view), 
+					   GTK_TEXT_WINDOW_TOP,
+					   EDIT_BORDER_WIDTH);
+      gtk_text_view_set_border_window_size(GTK_TEXT_VIEW(view), 
+					   GTK_TEXT_WINDOW_BOTTOM,
+					   EDIT_BORDER_WIDTH);
+      /* Using deprecated function because the fucking gobject documentation
+       * fucking sucks. */
+#ifdef NEW_TEXT_EDIT
+      gtk_signal_connect(GTK_OBJECT(view), "focus-out-event",
+			 modify_edit_end, obj);
+#endif
+      gtk_widget_grab_focus(view);
+      gtk_widget_show(view);
+    }
+  }  
+}
