@@ -1,8 +1,8 @@
 #ifndef _PAGINATE_GDIPRINT_H
 #define _PAGINATE_GDIPRINT_H
 
-#include "diagram.h"
+typedef struct _Diagram Diagram;
 
 void diagram_print_gdi(Diagram *dia);
 
-#endif
\ No newline at end of file
+#endif
