@@ -27,14 +27,17 @@
 #include <locale.h>
 #include <string.h> /* strlen */
 #include <signal.h>
+#include <errno.h>
 
 #include "intl.h"
 #include "message.h"
 #include "diagram.h"
 #include "diagramdata.h"
 #include "render_eps.h"
+#include "diapsrenderer.h"
 #include "paginate_psprint.h"
 #include "diapagelayout.h"
+#include "persistence.h"
 
 #include <gtk/gtk.h>
 
@@ -46,32 +49,33 @@
 
 /* keep track of print options between prints */
 typedef struct _dia_print_options {
-    int printer;
-    char *command;
-    char *output;
+  int printer;
 } dia_print_options;
 
 static dia_print_options last_print_options = 
 {
-    1, NULL, NULL
+    1
 };
 
 static void
-count_objs(Object *obj, Renderer *renderer, int active_layer, guint *nobjs)
+count_objs(DiaObject *obj, DiaRenderer *renderer, int active_layer, guint *nobjs)
 {
   (*nobjs)++;
 }
 
 static guint
-print_page(DiagramData *data, RendererEPS *rend, Rectangle *bounds)
+print_page(DiagramData *data, DiaRenderer *diarend, Rectangle *bounds)
 {
+  DiaPsRenderer *rend = DIA_PS_RENDERER(diarend);
   guint nobjs = 0;
   gfloat tmargin = data->paper.tmargin, bmargin = data->paper.bmargin;
   gfloat lmargin = data->paper.lmargin;
   gfloat scale = data->paper.scaling;
 
+  rend->paper = data->paper.name;
+
   /* count the number of objects in this region */
-  data_render(data, (Renderer *)rend, bounds,
+  data_render(data, diarend, bounds,
               (ObjectRenderer) count_objs, &nobjs);
 
   if (nobjs == 0)
@@ -102,10 +106,12 @@ print_page(DiagramData *data, RendererEPS *rend, Rectangle *bounds)
   fprintf(rend->file, "%f %f l ", bounds->right, bounds->bottom);
   fprintf(rend->file, "%f %f l ", bounds->left, bounds->bottom);
   fprintf(rend->file, "%f %f l ", bounds->left, bounds->top);
-  fprintf(rend->file, "clip\n");
+  /* Tip from Dov Grobgeld: Clip does not destroy the path, so we should
+     do a newpath afterwards */
+  fprintf(rend->file, "clip n\n"); 
 
   /* render the region */
-  data_render(data, (Renderer *)rend, bounds, NULL, NULL);
+  data_render(data, diarend, bounds, NULL, NULL);
 
   /* restore print context */
   fprintf(rend->file, "gr\n");
@@ -119,7 +125,7 @@ print_page(DiagramData *data, RendererEPS *rend, Rectangle *bounds)
 void
 paginate_psprint(Diagram *dia, FILE *file)
 {
-  RendererEPS *rend;
+  DiaRenderer *rend;
   Rectangle *extents;
   gfloat width, height;
   gfloat x, y, initx, inity;
@@ -129,6 +135,12 @@ paginate_psprint(Diagram *dia, FILE *file)
   old_locale = setlocale(LC_NUMERIC, "C");
   rend = new_psprint_renderer(dia, file);
 
+#ifdef DIA_PS_RENDERER_DUAL_PASS
+  /* Prepare the prolog (with fonts etc) */
+  data_render(dia->data, DIA_RENDERER(rend), NULL, NULL, NULL);
+  eps_renderer_prolog_done(rend);
+#endif
+
   /* the usable area of the page */
   width = dia->data->paper.width;
   height = dia->data->paper.height;
@@ -156,7 +168,7 @@ paginate_psprint(Diagram *dia, FILE *file)
       nobjs += print_page(dia->data,rend, &page_bounds);
     }
 
-  destroy_eps_renderer(rend);
+  g_object_unref(rend);
 
   setlocale(LC_NUMERIC, old_locale);
 }
@@ -181,6 +193,18 @@ pipe_handler(int signum)
   sigpipe_received = TRUE;
 }
 
+static gboolean
+diagram_print_destroy(GtkWidget *widget)
+{
+  Diagram **dia;
+
+  if ((dia = gtk_object_get_user_data(GTK_OBJECT(widget))) != NULL) {
+    diagram_remove_related_dialog(*dia, widget);
+    *dia = NULL;
+  }
+
+  return FALSE;
+}
 
 void
 diagram_print_ps(Diagram *dia)
@@ -190,7 +214,8 @@ diagram_print_ps(Diagram *dia)
   GtkWidget *iscmd, *isofile;
   GtkWidget *cmd, *ofile;
   gboolean cont = FALSE;
-  DDisplay *ddisp;
+  gchar *printcmd = NULL;
+  gchar *orig_command, *orig_file;
 
   FILE *file;
   gboolean is_pipe;
@@ -199,18 +224,19 @@ diagram_print_ps(Diagram *dia)
    * needed anymore because the pipe handling - which never worked on win32
    * anyway - is replace by "native" postscript printing now ...
    */
-  struct sigaction pipe_action, old_action;
+  void *old_action;
 #endif
 
-  ddisp = ddisplay_active();
-  dia = ddisp->diagram;
-
   /* create the dialog */
   dialog = gtk_dialog_new();
-  gtk_signal_connect(GTK_OBJECT(dialog), "delete_event",
-		     GTK_SIGNAL_FUNC(gtk_main_quit), NULL);
-  gtk_signal_connect(GTK_OBJECT(dialog), "delete_event",
-		     GTK_SIGNAL_FUNC(gtk_true), NULL);
+  diagram_add_related_dialog(dia, dialog);
+  gtk_object_set_user_data(GTK_OBJECT(dialog), &dia);
+  g_signal_connect(GTK_OBJECT(dialog), "delete_event",
+		   G_CALLBACK(diagram_print_destroy), NULL);
+  g_signal_connect(GTK_OBJECT(dialog), "delete_event",
+		   G_CALLBACK(gtk_main_quit), NULL);
+  g_signal_connect(GTK_OBJECT(dialog), "delete_event",
+		   G_CALLBACK(gtk_true), NULL);
   vbox = GTK_DIALOG(dialog)->vbox;
 
   frame = gtk_frame_new(_("Select Printer"));
@@ -234,8 +260,9 @@ diagram_print_ps(Diagram *dia)
   gtk_table_attach(GTK_TABLE(table), cmd, 1,2, 0,1,
 		   GTK_FILL|GTK_EXPAND, GTK_FILL|GTK_EXPAND, 0, 0);
   gtk_widget_show(cmd);
-  gtk_signal_connect(GTK_OBJECT(iscmd), "toggled",
-		     GTK_SIGNAL_FUNC(change_entry_state), cmd);
+
+  g_signal_connect(GTK_OBJECT(iscmd), "toggled",
+		   G_CALLBACK(change_entry_state), cmd);
 
   isofile = gtk_radio_button_new_with_label(GTK_RADIO_BUTTON(iscmd)->group,
 					    _("File"));
@@ -248,40 +275,52 @@ diagram_print_ps(Diagram *dia)
   gtk_table_attach(GTK_TABLE(table), ofile, 1,2, 1,2,
 		   GTK_FILL|GTK_EXPAND, GTK_FILL|GTK_EXPAND, 0, 0);
   gtk_widget_show(ofile);
-  gtk_signal_connect(GTK_OBJECT(isofile), "toggled",
-		     GTK_SIGNAL_FUNC(change_entry_state), ofile);
+  g_signal_connect(GTK_OBJECT(isofile), "toggled",
+		   G_CALLBACK(change_entry_state), ofile);
 
   box = GTK_DIALOG(dialog)->action_area;
 
   button = gtk_button_new_with_label(_("OK"));
-  gtk_signal_connect(GTK_OBJECT(button), "clicked", 
-		     GTK_SIGNAL_FUNC(ok_pressed), &cont);
+  g_signal_connect(GTK_OBJECT(button), "clicked", 
+		   G_CALLBACK(ok_pressed), &cont);
   GTK_WIDGET_SET_FLAGS(button, GTK_CAN_DEFAULT);
   gtk_box_pack_start(GTK_BOX(box), button, TRUE, TRUE, 0);
   gtk_widget_grab_default(button);
   gtk_widget_show(button);
 
   button = gtk_button_new_with_label(_("Cancel"));
-  gtk_signal_connect(GTK_OBJECT(button), "clicked", 
-		     GTK_SIGNAL_FUNC(gtk_main_quit), NULL);
+  g_signal_connect(GTK_OBJECT(button), "clicked", 
+		   G_CALLBACK(gtk_main_quit), NULL);
   GTK_WIDGET_SET_FLAGS(button, GTK_CAN_DEFAULT);
   gtk_box_pack_start(GTK_BOX(box), button, TRUE, TRUE, 0);
   gtk_widget_show(button);
 
   /* Set default or old dialog values: */
 #ifdef G_OS_WIN32
-  gtk_entry_set_text(GTK_ENTRY(cmd), 
-		     last_print_options.command 
-		     ? last_print_options.command : win32_printer_default ());
+  gtk_entry_set_text(GTK_ENTRY(cmd), win32_printer_default ());
 #else
-  gtk_entry_set_text(GTK_ENTRY(cmd), 
-		     last_print_options.command 
-		     ? last_print_options.command : "lpr");
+  {
+    gchar *printer = g_getenv("PRINTER");
+    
+    if (printer) {
+      printcmd = g_strdup_printf("lpr -P%s", printer);
+    } else {
+      printcmd = g_strdup("lpr");
+    }
+    
+    gtk_entry_set_text(GTK_ENTRY(cmd), printcmd);
+    g_free(printcmd);
+    printcmd = NULL;
+  }
 #endif
+  persistence_register_string_entry("printer-command", cmd);
+  printcmd = g_strdup(gtk_entry_get_text(GTK_ENTRY(cmd)));
+  orig_command = printcmd;
   /* Ought to use filename+extension here */
-  gtk_entry_set_text(GTK_ENTRY(ofile), 
-		     last_print_options.output 
-		     ? last_print_options.output : "output.ps");
+  gtk_entry_set_text(GTK_ENTRY(ofile), "output.ps");
+  persistence_register_string_entry("printer-file", ofile);
+  orig_file = g_strdup(gtk_entry_get_text(GTK_ENTRY(ofile)));
+    
   /* Scaling is already set at creation. */
   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(iscmd), last_print_options.printer);
   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(isofile), !last_print_options.printer);
@@ -289,20 +328,30 @@ diagram_print_ps(Diagram *dia)
   gtk_widget_show(dialog);
   gtk_main();
 
-  if (!cont) {
+  if(!dia) {
     gtk_widget_destroy(dialog);
     return;
   }
+  
+  if (!cont) {
+    persistence_change_string_entry("printer-command", orig_command, cmd);
+    persistence_change_string_entry("printer-file", orig_file, ofile);
+    gtk_widget_destroy(dialog);
+    g_free(orig_command);
+    g_free(orig_file);
+    return;
+  }
 
   if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(iscmd))) {
+    printcmd = g_strdup(gtk_entry_get_text(GTK_ENTRY(cmd)));
 #ifdef G_OS_WIN32
-    file = win32_printer_open (gtk_entry_get_text(GTK_ENTRY(cmd)));
+    file = win32_printer_open (printcmd);
 #else
-    file = popen(gtk_entry_get_text(GTK_ENTRY(cmd)), "w");
+    file = popen(printcmd, "w");
 #endif
     is_pipe = TRUE;
   } else {
-    gchar *filename = gtk_entry_get_text(GTK_ENTRY(ofile));
+    const gchar *filename = gtk_entry_get_text(GTK_ENTRY(ofile));
     if (!g_path_is_absolute(filename)) {
       char *diagram_dir;
       char *full_filename;
@@ -319,40 +368,45 @@ diagram_print_ps(Diagram *dia)
   }
 
   /* Store dialog values */
-  g_free( last_print_options.command );
-  g_free( last_print_options.output );
-  last_print_options.command = g_strdup( gtk_entry_get_text(GTK_ENTRY(cmd)) );
-  last_print_options.output = g_strdup( gtk_entry_get_text(GTK_ENTRY(ofile)) );
+  g_free(orig_command);
+  g_free(orig_file);
   last_print_options.printer = gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(iscmd));
   
   if (!file) {
-    if (is_pipe)
-      message_warning(_("Could not run command '%s'"),
-		      gtk_entry_get_text(GTK_ENTRY(cmd)));
-    else
-      message_warning(_("Could not open '%s' for writing"),
-		      gtk_entry_get_text(GTK_ENTRY(ofile)));
+    if (is_pipe) {
+      message_warning(_("Could not run command '%s': %s"), printcmd, strerror(errno));
+      g_free(printcmd);
+    } else
+      message_warning(_("Could not open '%s' for writing: %s"),
+		      gtk_entry_get_text(GTK_ENTRY(ofile)), strerror(errno));
+    gtk_widget_destroy(dialog);
     return;
   }
 
 #ifndef G_OS_WIN32
   /* set up a SIGPIPE handler to catch IO errors, rather than segfaulting */
-  memset(&pipe_action, '\0', sizeof(pipe_action));
   sigpipe_received = FALSE;
-  pipe_action.sa_handler = pipe_handler;
-  sigaction(SIGPIPE, &pipe_action, &old_action);
+  old_action = signal(SIGPIPE, pipe_handler);
 #endif
 
   paginate_psprint(dia, file);
   gtk_widget_destroy(dialog);
-  if (is_pipe)
-    pclose(file);
-  else
+  if (is_pipe) {
+    int exitval = pclose(file);
+    if (exitval != 0) {
+      message_error(_("Printing error: command '%s' returned %d\n"),
+		    printcmd, exitval);
+    }
+  } else
     fclose(file);
 
 #ifndef G_OS_WIN32
-  sigaction(SIGPIPE, &old_action, NULL);
+  /* restore original behaviour */
+  signal(SIGPIPE, old_action);
 #endif
   if (sigpipe_received)
-    message_error(_("Error occured while printing"));
+    message_error(_("Printing error: command '%s' caused sigpipe."),
+		  printcmd);
+
+  if (is_pipe) g_free(printcmd);
 }
