@@ -22,13 +22,13 @@
 
 #include <stdio.h>
 #include <stdlib.h>
-#include <assert.h>
 #include <math.h>
 #include <string.h> /* memcpy() */
 
 #include "bezier_conn.h"
 #include "intl.h"
 #include "message.h"
+#include "diarenderer.h"
 
 #define HANDLE_BEZMAJOR  (HANDLE_CUSTOM1)
 #define HANDLE_LEFTCTRL  (HANDLE_CUSTOM2)
@@ -80,7 +80,7 @@ bezierconn_create_corner_change(BezierConn *bez, Handle *handle,
 				BezCornerType new_corner_type);
 
 
-static void setup_corner_handle(Handle *handle, HandleId id)
+static void setup_handle(Handle *handle, HandleId id)
 {
   handle->id = id;
   handle->type = HANDLE_MINOR_CONTROL;
@@ -99,12 +99,15 @@ static int get_handle_nr(BezierConn *bez, Handle *handle)
   return -1;
 }
 
+void new_handles(BezierConn *bez, int num_points);
+
 #define get_comp_nr(hnum) (((int)(hnum)+2)/3)
 #define get_major_nr(hnum) (((int)(hnum)+1)/3)
 
-void
+ObjectChange*
 bezierconn_move_handle(BezierConn *bez, Handle *handle,
-		     Point *to, HandleMoveReason reason)
+		       Point *to, ConnectionPoint *cp,
+		       HandleMoveReason reason, ModifierKeys modifiers)
 {
   int handle_nr, comp_nr;
   Point delta, pt;
@@ -197,9 +200,10 @@ bezierconn_move_handle(BezierConn *bez, Handle *handle,
     message_error("Internal error in bezierconn_move_handle.\n");
     break;
   }
+  return NULL;
 }
 
-void
+ObjectChange*
 bezierconn_move(BezierConn *bez, Point *to)
 {
   Point p;
@@ -214,6 +218,7 @@ bezierconn_move(BezierConn *bez, Point *to)
     point_add(&bez->points[i].p2, &p);
     point_add(&bez->points[i].p3, &p);
   }
+  return NULL;
 }
 
 int
@@ -295,11 +300,11 @@ add_handles(BezierConn *bez, int pos, BezPoint *point,
 	    Handle *handle2, Handle *handle3)
 {
   int i;
-  Object *obj;
+  DiaObject *obj;
 
-  assert(pos > 0);
+  g_assert(pos > 0);
   
-  obj = (Object *)bez;
+  obj = (DiaObject *)bez;
   bez->numpoints++;
   bez->points = g_realloc(bez->points, bez->numpoints*sizeof(BezPoint));
   bez->corner_types = g_realloc(bez->corner_types,
@@ -327,13 +332,13 @@ static void
 remove_handles(BezierConn *bez, int pos)
 {
   int i;
-  Object *obj;
+  DiaObject *obj;
   Handle *old_handle1, *old_handle2, *old_handle3;
   Point tmppoint;
 
-  assert(pos > 0);
+  g_assert(pos > 0);
 
-  obj = (Object *)bez;
+  obj = (DiaObject *)bez;
 
   if (pos==obj->num_handles-1) {
     obj->handles[obj->num_handles-4]->type = HANDLE_MAJOR_CONTROL;
@@ -352,9 +357,9 @@ remove_handles(BezierConn *bez, int pos)
   bez->corner_types = g_realloc(bez->corner_types,
 				bez->numpoints * sizeof(BezCornerType));
 
-  old_handle1 = obj->handles[3*pos-2];
-  old_handle2 = obj->handles[3*pos-1];
-  old_handle3 = obj->handles[3*pos];
+  old_handle1 = obj->handles[3*pos-3];
+  old_handle2 = obj->handles[3*pos-2];
+  old_handle3 = obj->handles[3*pos-1];
   object_remove_handle(&bez->object, old_handle1);
   object_remove_handle(&bez->object, old_handle2);
   object_remove_handle(&bez->object, old_handle3);
@@ -396,9 +401,9 @@ bezierconn_add_segment(BezierConn *bez, int segment, Point *point)
   new_handle1 = g_malloc(sizeof(Handle));
   new_handle2 = g_malloc(sizeof(Handle));
   new_handle3 = g_malloc(sizeof(Handle));
-  setup_corner_handle(new_handle1, HANDLE_RIGHTCTRL);
-  setup_corner_handle(new_handle2, HANDLE_LEFTCTRL);
-  setup_corner_handle(new_handle3, HANDLE_BEZMAJOR);
+  setup_handle(new_handle1, HANDLE_RIGHTCTRL);
+  setup_handle(new_handle2, HANDLE_LEFTCTRL);
+  setup_handle(new_handle3, HANDLE_BEZMAJOR);
   add_handles(bez, segment+1, &realpoint, corner_type,
 	      new_handle1, new_handle2, new_handle3);
   return bezierconn_create_point_change(bez, TYPE_ADD_POINT,
@@ -416,8 +421,8 @@ bezierconn_remove_segment(BezierConn *bez, int pos)
   BezPoint old_point;
   BezCornerType old_ctype;
 
-  assert(pos > 0);
-  assert(bez->numpoints > 2);
+  g_assert(pos > 0);
+  g_assert(bez->numpoints > 2);
 
   if (pos == bez->numpoints-1) pos--;
 
@@ -431,9 +436,9 @@ bezierconn_remove_segment(BezierConn *bez, int pos)
   cpt2 = old_handle2->connected_to;
   cpt3 = old_handle3->connected_to;
   
-  object_unconnect((Object *)bez, old_handle1);
-  object_unconnect((Object *)bez, old_handle2);
-  object_unconnect((Object *)bez, old_handle3);
+  object_unconnect((DiaObject *)bez, old_handle1);
+  object_unconnect((DiaObject *)bez, old_handle2);
+  object_unconnect((DiaObject *)bez, old_handle3);
 
   remove_handles(bez, pos);
 
@@ -547,7 +552,22 @@ void
 bezierconn_update_data(BezierConn *bez)
 {
   int i;
+  DiaObject *obj = &bez->object;
   
+  /* handle the case of whole points array update (via set_prop) */
+  if (3*bez->numpoints-2 != obj->num_handles) {
+    g_assert(0 == obj->num_connections);
+
+    for (i = 0; i < obj->num_handles; i++)
+      g_free(obj->handles[i]);
+    g_free(obj->handles);
+
+    obj->num_handles = 3*bez->numpoints-2;
+    obj->handles = g_new(Handle*, obj->num_handles); 
+
+    new_handles(bez, bez->numpoints);
+  }
+
   /* Update handles: */
   bez->object.handles[0]->pos = bez->points[0].p1;
   for (i = 1; i < bez->numpoints; i++) {
@@ -560,213 +580,134 @@ bezierconn_update_data(BezierConn *bez)
 void
 bezierconn_update_boundingbox(BezierConn *bez)
 {
-  Rectangle *bb;
-  BezPoint *points;
-  int i;
-  Point pt1,pt0,vec;
-  BezierConnBBExtras *extra;
-  real trans;
+  g_assert(bez != NULL);
 
-  assert(bez != NULL);
-
-  extra = &bez->extra_spacing;
-  bb = &bez->object.bounding_box;
-  points = &bez->points[0];
-
-  bb->right = bb->left = points[0].p1.x;
-  bb->top = bb->bottom = points[0].p1.y;
-
-  trans = MAX(extra->start_trans,extra->middle_trans);
-  vec = points[0].p1;
-  if (points[1].type == BEZ_CURVE_TO) {
-    point_sub(&vec,&points[1].p1);
-  } else {
-    point_sub(&vec,&points[1].p1); 
-  }
-  check_bb_x(bb,points[0].p1.x + (extra->start_long * vec.x),vec.x);
-  check_bb_y(bb,points[0].p1.y + (extra->start_long * vec.y),vec.y);
-  check_bb_x(bb,points[0].p1.x + (trans * vec.y),vec.y);
-  check_bb_x(bb,points[0].p1.x - (trans * vec.y),vec.y);
-  check_bb_y(bb,points[0].p1.y + (trans * vec.x),vec.x);
-  check_bb_y(bb,points[0].p1.y - (trans * vec.x),vec.x);
-
-  
-  for (i=0;i<(bez->numpoints-1);i++) {
-    if (points[i].type == BEZ_CURVE_TO) {
-      pt1 = points[i].p3;
-    } else {
-      pt1 = points[i].p1;
-    }
-    if (points[i+1].type == BEZ_CURVE_TO) {
-      point_sub(&pt1,&(points[i+1].p3));
-    } else {
-      point_sub(&pt1,&(points[i+1].p1));
-    }
-    point_normalize(&pt1);
-
-    if (i!=0) {
-      if (points[i].type == BEZ_CURVE_TO) {
-        /* FIXME : dig up the math, do something closer to the curve (CC) */        
-        check_bb_x(bb,points[i].p1.x + extra->middle_trans * 1.41,1);
-        check_bb_x(bb,points[i].p1.x - extra->middle_trans * 1.41,1);
-        check_bb_x(bb,points[i].p2.x + extra->middle_trans * 1.41,1);
-        check_bb_x(bb,points[i].p2.x - extra->middle_trans * 1.41,1);
-        check_bb_x(bb,points[i].p3.x + extra->middle_trans * 1.41,1);
-        check_bb_x(bb,points[i].p3.x - extra->middle_trans * 1.41,1);
-
-        check_bb_y(bb,points[i].p1.y + extra->middle_trans * 1.41,1);
-        check_bb_y(bb,points[i].p1.y - extra->middle_trans * 1.41,1);
-        check_bb_y(bb,points[i].p2.y + extra->middle_trans * 1.41,1);
-        check_bb_y(bb,points[i].p2.y - extra->middle_trans * 1.41,1);
-        check_bb_y(bb,points[i].p3.y + extra->middle_trans * 1.41,1);
-        check_bb_y(bb,points[i].p3.y - extra->middle_trans * 1.41,1);
-      } else {
-        real co = point_dot(&pt1,&pt0);
-
-        check_bb_x(bb, points[i].p1.x + (extra->middle_trans * pt1.y),pt1.y);
-        check_bb_x(bb, points[i].p1.x - (extra->middle_trans * pt1.y),pt1.y);
-        check_bb_y(bb, points[i].p1.y + (extra->middle_trans * pt1.x),pt1.x);
-        check_bb_y(bb, points[i].p1.y - (extra->middle_trans * pt1.x),pt1.x);
-
-        if (co > -0.9816) { /* 0.9816 == cos(11deg) */
-          real alpha = fabs(acos(-co));
-          real overshoot = extra->middle_trans / (sin(alpha/2.0));
-          Point pts;
-
-          pts = pt1; point_sub(&pts,&pt0);
-          point_normalize(&pts);
-          
-          check_bb_x(bb,points[i].p1.x + (overshoot * pts.x),pts.x);
-          check_bb_y(bb,points[i].p1.y + (overshoot * pts.y),pts.y);
-        } else { /* this works because i>0 */
-          check_bb_x(bb, points[i].p1.x + (extra->middle_trans * pt0.y),pt0.y);
-          check_bb_x(bb, points[i].p1.x - (extra->middle_trans * pt0.y),pt0.y);
-          check_bb_y(bb, points[i].p1.y + (extra->middle_trans * pt0.x),pt0.x);
-          check_bb_y(bb, points[i].p1.y - (extra->middle_trans * pt0.x),pt0.x);
-        } 
-      }
-    }
-    pt0=pt1;
-  }  
-
-  trans = MAX(extra->start_trans,extra->middle_trans);
-  if (points[bez->numpoints-1].type == BEZ_CURVE_TO) {
-    vec = points[bez->numpoints-1].p3;
-    point_sub(&vec,&points[bez->numpoints-1].p2);
-
-    check_bb_x(bb,points[bez->numpoints-1].p3.x + (extra->end_long * vec.x),vec.x);
-    check_bb_y(bb,points[bez->numpoints-1].p3.y + (extra->end_long * vec.y),vec.y);
-    check_bb_x(bb,points[bez->numpoints-1].p3.x + (trans * vec.y),vec.y);
-    check_bb_x(bb,points[bez->numpoints-1].p3.x - (trans * vec.y),vec.y);
-    check_bb_y(bb,points[bez->numpoints-1].p3.y + (trans * vec.x),vec.x);
-    check_bb_y(bb,points[bez->numpoints-1].p3.y - (trans * vec.x),vec.x);
-  } else {
-    vec = points[bez->numpoints-1].p1;
-    point_sub(&vec,&points[bez->numpoints-2].p1); 
-
-    check_bb_x(bb,points[bez->numpoints-1].p1.x + (extra->end_long * vec.x),vec.x);
-    check_bb_y(bb,points[bez->numpoints-1].p1.y + (extra->end_long * vec.y),vec.y);
-    check_bb_x(bb,points[bez->numpoints-1].p1.x + (trans * vec.y),vec.y);
-    check_bb_x(bb,points[bez->numpoints-1].p1.x - (trans * vec.y),vec.y);
-    check_bb_y(bb,points[bez->numpoints-1].p1.y + (trans * vec.x),vec.x);
-    check_bb_y(bb,points[bez->numpoints-1].p1.y - (trans * vec.x),vec.x);
-  }
+  polybezier_bbox(&bez->points[0],
+                  bez->numpoints,
+                  &bez->extra_spacing, FALSE,
+                  &bez->object.bounding_box);
 }
 
 void
-bezierconn_simple_draw(BezierConn *bez, Renderer *renderer, real width)
+bezierconn_simple_draw(BezierConn *bez, DiaRenderer *renderer, real width)
 {
   BezPoint *points;
   
-  assert(bez != NULL);
-  assert(renderer != NULL);
+  g_assert(bez != NULL);
+  g_assert(renderer != NULL);
 
   points = &bez->points[0];
   
-  renderer->ops->set_linewidth(renderer, width);
-  renderer->ops->set_linestyle(renderer, LINESTYLE_SOLID);
-  renderer->ops->set_linejoin(renderer, LINEJOIN_ROUND);
-  renderer->ops->set_linecaps(renderer, LINECAPS_BUTT);
+  DIA_RENDERER_GET_CLASS(renderer)->set_linewidth(renderer, width);
+  DIA_RENDERER_GET_CLASS(renderer)->set_linestyle(renderer, LINESTYLE_SOLID);
+  DIA_RENDERER_GET_CLASS(renderer)->set_linejoin(renderer, LINEJOIN_ROUND);
+  DIA_RENDERER_GET_CLASS(renderer)->set_linecaps(renderer, LINECAPS_BUTT);
 
-  renderer->ops->draw_bezier(renderer, points, bez->numpoints, &color_black);
+  DIA_RENDERER_GET_CLASS(renderer)->draw_bezier(renderer, points, bez->numpoints, &color_black);
 }
 
 void
-bezierconn_draw_control_lines(BezierConn *bez, Renderer *renderer)
+bezierconn_draw_control_lines(BezierConn *bez, DiaRenderer *renderer)
 {
   Color line_colour = {0.0, 0.0, 0.6};
   Point startpoint;
   int i;
   
-  /* setup renderer ... */
-  renderer->ops->set_linewidth(renderer, 0);
-  renderer->ops->set_linestyle(renderer, LINESTYLE_DOTTED);
-  renderer->ops->set_dashlength(renderer, 1);
-  renderer->ops->set_linejoin(renderer, LINEJOIN_MITER);
-  renderer->ops->set_linecaps(renderer, LINECAPS_BUTT);
+  /* setup DiaRenderer ... */
+  DIA_RENDERER_GET_CLASS(renderer)->set_linewidth(renderer, 0);
+  DIA_RENDERER_GET_CLASS(renderer)->set_linestyle(renderer, LINESTYLE_DOTTED);
+  DIA_RENDERER_GET_CLASS(renderer)->set_dashlength(renderer, 1);
+  DIA_RENDERER_GET_CLASS(renderer)->set_linejoin(renderer, LINEJOIN_MITER);
+  DIA_RENDERER_GET_CLASS(renderer)->set_linecaps(renderer, LINECAPS_BUTT);
 
   startpoint = bez->points[0].p1;
   for (i = 1; i < bez->numpoints; i++) {
-    renderer->ops->draw_line(renderer, &startpoint, &bez->points[i].p1,
+    DIA_RENDERER_GET_CLASS(renderer)->draw_line(renderer, &startpoint, &bez->points[i].p1,
 			     &line_colour);
-    renderer->ops->draw_line(renderer, &bez->points[i].p2, &bez->points[i].p3,
+    DIA_RENDERER_GET_CLASS(renderer)->draw_line(renderer, &bez->points[i].p2, &bez->points[i].p3,
 			     &line_colour);
     startpoint = bez->points[i].p3;
   }
 }
 
 void
-bezierconn_init(BezierConn *bez)
+new_handles(BezierConn *bez, int num_points)
 {
-  Object *obj;
+  DiaObject *obj;
+  int i;
 
   obj = &bez->object;
 
-  object_init(obj, 4, 0);
-  
-  bez->numpoints = 2;
-
-  bez->points = g_new(BezPoint, 2);
-  bez->points[0].type = BEZ_MOVE_TO;
-  bez->points[1].type = BEZ_CURVE_TO;
-
-  bez->corner_types = g_new(BezCornerType, 2);
-  bez->corner_types[0] = BEZ_CORNER_SYMMETRIC;
-  bez->corner_types[1] = BEZ_CORNER_SYMMETRIC;
-
-  bez->object.handles[0] = g_new(Handle, 1);
-  bez->object.handles[1] = g_new(Handle, 1);
-  bez->object.handles[2] = g_new(Handle, 1);
-  bez->object.handles[3] = g_new(Handle, 1);
-
+  obj->handles[0] = g_new(Handle,1);
   obj->handles[0]->connect_type = HANDLE_CONNECTABLE;
   obj->handles[0]->connected_to = NULL;
   obj->handles[0]->type = HANDLE_MAJOR_CONTROL;
   obj->handles[0]->id = HANDLE_MOVE_STARTPOINT;
-  
-  obj->handles[1]->connect_type = HANDLE_NONCONNECTABLE;
-  obj->handles[1]->connected_to = NULL;
-  obj->handles[1]->type = HANDLE_MINOR_CONTROL;
-  obj->handles[1]->id = HANDLE_RIGHTCTRL;
 
-  obj->handles[2]->connect_type = HANDLE_NONCONNECTABLE;
-  obj->handles[2]->connected_to = NULL;
-  obj->handles[2]->type = HANDLE_MINOR_CONTROL;
-  obj->handles[2]->id = HANDLE_LEFTCTRL;
+  for (i = 1; i < num_points; i++) {
+    obj->handles[3*i-2] = g_new(Handle, 1);
+    obj->handles[3*i-1] = g_new(Handle, 1);
+    obj->handles[3*i] = g_new(Handle, 1);
   
-  obj->handles[3]->connect_type = HANDLE_CONNECTABLE;
-  obj->handles[3]->connected_to = NULL;
-  obj->handles[3]->type = HANDLE_MAJOR_CONTROL;
-  obj->handles[3]->id = HANDLE_MOVE_ENDPOINT;
+    setup_handle(obj->handles[3*i-2], HANDLE_RIGHTCTRL);
+    setup_handle(obj->handles[3*i-1], HANDLE_LEFTCTRL);
+  
+    obj->handles[3*i]->connect_type = HANDLE_CONNECTABLE;
+    obj->handles[3*i]->connected_to = NULL;
+    obj->handles[3*i]->type = HANDLE_MAJOR_CONTROL;
+    obj->handles[3*i]->id = HANDLE_MOVE_ENDPOINT;
+  }
+}
+
+void
+bezierconn_init(BezierConn *bez, int num_points)
+{
+  DiaObject *obj;
+  int i;
+
+  obj = &bez->object;
+
+  object_init(obj, 3*num_points-2, 0);
+  
+  bez->numpoints = num_points;
+
+  bez->points = g_new(BezPoint, num_points);
+  bez->corner_types = g_new(BezCornerType, num_points);
+  bez->points[0].type = BEZ_MOVE_TO;
+  bez->corner_types[0] = BEZ_CORNER_SYMMETRIC;
+  for (i = 1; i < num_points; i++) {
+    bez->points[i].type = BEZ_CURVE_TO;
+    bez->corner_types[i] = BEZ_CORNER_SYMMETRIC;
+  }
+
+  new_handles(bez, num_points);
 
   bezierconn_update_data(bez);
 }
 
+/** This function does *not* set up handles */
+void
+bezierconn_set_points(BezierConn *bez, int num_points, BezPoint *points)
+{
+  int i;
+
+  bez->numpoints = num_points;
+
+  if (bez->points)
+    g_free(bez->points);
+
+  bez->points = g_malloc((bez->numpoints)*sizeof(BezPoint));
+
+  for (i=0;i<bez->numpoints;i++) {
+    bez->points[i] = points[i];
+  }
+}
+
 void
 bezierconn_copy(BezierConn *from, BezierConn *to)
 {
   int i;
-  Object *toobj, *fromobj;
+  DiaObject *toobj, *fromobj;
 
   toobj = &to->object;
   fromobj = &from->object;
@@ -787,7 +728,7 @@ bezierconn_copy(BezierConn *from, BezierConn *to)
   *to->object.handles[0] = *from->object.handles[0];
   for (i = 1; i < to->object.num_handles - 1; i++) {
     to->object.handles[i] = g_new(Handle, 1);
-    setup_corner_handle(to->object.handles[i], from->object.handles[i]->id);
+    setup_handle(to->object.handles[i], from->object.handles[i]->id);
   }
   to->object.handles[to->object.num_handles-1] = g_new(Handle,1);
   *to->object.handles[to->object.num_handles-1] =
@@ -850,7 +791,7 @@ bezierconn_load(BezierConn *bez, ObjectNode obj_node) /* NOTE: Does object_init(
   AttributeNode attr;
   DataNode data;
   
-  Object *obj = &bez->object;
+  DiaObject *obj = &bez->object;
 
   object_load(obj, obj_node);
 
@@ -905,11 +846,11 @@ bezierconn_load(BezierConn *bez, ObjectNode obj_node) /* NOTE: Does object_init(
   
   for (i = 1; i < bez->numpoints; i++) {
     obj->handles[3*i-2] = g_new(Handle, 1);
-    setup_corner_handle(obj->handles[3*i-2], HANDLE_RIGHTCTRL);
+    setup_handle(obj->handles[3*i-2], HANDLE_RIGHTCTRL);
     obj->handles[3*i-1] = g_new(Handle, 1);
-    setup_corner_handle(obj->handles[3*i-1], HANDLE_LEFTCTRL);
+    setup_handle(obj->handles[3*i-1], HANDLE_LEFTCTRL);
     obj->handles[3*i]   = g_new(Handle, 1);
-    setup_corner_handle(obj->handles[3*i],   HANDLE_BEZMAJOR);
+    setup_handle(obj->handles[3*i],   HANDLE_BEZMAJOR);
   }
 
   obj->handles[obj->num_handles-1]->connect_type = HANDLE_CONNECTABLE;
@@ -935,7 +876,7 @@ bezierconn_point_change_free(struct PointChange *change)
 }
 
 static void
-bezierconn_point_change_apply(struct PointChange *change, Object *obj)
+bezierconn_point_change_apply(struct PointChange *change, DiaObject *obj)
 {
   change->applied = 1;
   switch (change->type) {
@@ -954,7 +895,7 @@ bezierconn_point_change_apply(struct PointChange *change, Object *obj)
 }
 
 static void
-bezierconn_point_change_revert(struct PointChange *change, Object *obj)
+bezierconn_point_change_revert(struct PointChange *change, DiaObject *obj)
 {
   switch (change->type) {
   case TYPE_ADD_POINT:
@@ -1010,7 +951,7 @@ bezierconn_create_point_change(BezierConn *bez, enum change_type type,
 
 static void
 bezierconn_corner_change_apply(struct CornerChange *change,
-			       Object *obj) {
+			       DiaObject *obj) {
   BezierConn *bez = (BezierConn *)obj;
   int handle_nr = get_handle_nr(bez, change->handle);
   int comp_nr = get_major_nr(handle_nr);
@@ -1024,7 +965,7 @@ bezierconn_corner_change_apply(struct CornerChange *change,
 
 static void
 bezierconn_corner_change_revert(struct CornerChange *change,
-				Object *obj) {
+				DiaObject *obj) {
   BezierConn *bez = (BezierConn *)obj;
   int handle_nr = get_handle_nr(bez, change->handle);
   int comp_nr = get_major_nr(handle_nr);
