@@ -21,37 +21,32 @@
 #ifndef BEZIERSHAPE_H
 #define BEZIERSHAPE_H
 
+#include "diatypes.h"
 #include "object.h"
 /* for BezCornerType */
 #include "bezier_conn.h"
 
 #define HANDLE_CORNER (HANDLE_CUSTOM1)
 
-typedef struct _BezierShape BezierShape;
-typedef struct _BezierShapeBBExtras BezierShapeBBExtras;
-
-struct _BezierShapeBBExtras {
-  real border_trans;
-};
-
-/* This is a subclass of Object used to help implementing objects
+/* This is a subclass of DiaObject used to help implementing objects
  * that form a polygon-like shape of line-segments.
  */
 struct _BezierShape {
-  /* Object must be first because this is a 'subclass' of it. */
-  Object object;
+  /* DiaObject must be first because this is a 'subclass' of it. */
+  DiaObject object;
 
   int numpoints; /* >= 2 */
   BezPoint *points;
   BezCornerType *corner_types;
-  BezierShapeBBExtras extra_spacing;
+  ElementBBExtras extra_spacing;
 };
 
 void beziershape_update_data(BezierShape *bezier);
 void beziershape_update_boundingbox(BezierShape *bezier);
-void beziershape_simple_draw(BezierShape *bezier, Renderer *renderer,
+void beziershape_simple_draw(BezierShape *bezier, DiaRenderer *renderer,
 			     real width);
-void beziershape_init(BezierShape *bezier);
+void beziershape_init(BezierShape *bezier, int num_points);
+void beziershape_set_points(BezierShape *bezier, int num_points, BezPoint *points);
 void beziershape_destroy(BezierShape *bezier);
 void beziershape_copy(BezierShape *from, BezierShape *to);
 void beziershape_save(BezierShape *bezier, ObjectNode obj_node);
@@ -62,14 +57,26 @@ ObjectChange *beziershape_add_segment(BezierShape *bezier, int segment,
 ObjectChange *beziershape_remove_segment(BezierShape *bezier, int point);
 ObjectChange *beziershape_set_corner_type(BezierShape *bez, Handle *handle,
 					  BezCornerType corner_type);
-void beziershape_move_handle(BezierShape *bezier, Handle *id,
-			     Point *to, HandleMoveReason reason);
-void beziershape_move(BezierShape *bezier, Point *to);
+ObjectChange *beziershape_move_handle(BezierShape *bezier, Handle *id,
+				      Point *to, ConnectionPoint *cp,
+				      HandleMoveReason reason,
+				      ModifierKeys modifiers);
+ObjectChange *beziershape_move(BezierShape *bezier, Point *to);
 real beziershape_distance_from(BezierShape *bezier, Point *point,
 			       real line_width);
 Handle *beziershape_closest_handle(BezierShape *bezier, Point *point);
 Handle *beziershape_closest_major_handle(BezierShape *bezier, Point *point);
 int beziershape_closest_segment(BezierShape *bezier, Point *point,
 				real line_width);
-void beziershape_draw_control_lines(BezierShape *bez, Renderer *renderer);
+void beziershape_draw_control_lines(BezierShape *bez, DiaRenderer *renderer);
+
+#define BEZSHAPE_COMMON_PROPERTIES \
+  OBJECT_COMMON_PROPERTIES, \
+  { "bez_points", PROP_TYPE_BEZPOINTARRAY, 0, "beziershape points", NULL} \
+
+#define BEZSHAPE_COMMON_PROPERTIES_OFFSETS \
+  OBJECT_COMMON_PROPERTIES_OFFSETS, \
+  { "bez_points", PROP_TYPE_BEZPOINTARRAY, \
+     offsetof(BezierShape,points), offsetof(BezierShape,numpoints)} \
+
 #endif /* BEZIER_SHAPE_H */
