@@ -23,8 +23,6 @@
 
 #include "config.h"
 
-#include <ctype.h>
-#include <locale.h>
 #include <string.h>
 #include <stdlib.h>
 
@@ -39,7 +37,6 @@ void
 dia_svg_parse_style(xmlNodePtr node, DiaSvgGraphicStyle *s)
 {
   char *str, *ptr;
-  char *old_locale;
   gchar temp[FONT_NAME_LENGTH_MAX+1]; /* font-family names will be limited to 40 characters */
   int i = 0;
   gboolean over = FALSE;  
@@ -51,12 +48,12 @@ dia_svg_parse_style(xmlNodePtr node, DiaSvgGraphicStyle *s)
  if (str) {
   while (ptr[0] != '\0') {
     /* skip white space at start */
-    while (ptr[0] != '\0' && isspace(ptr[0])) ptr++;
+    while (ptr[0] != '\0' && g_ascii_isspace(ptr[0])) ptr++;
     if (ptr[0] == '\0') break;
 
      if (!strncmp("font-family:", ptr, 12)) {
       ptr += 12;
-      while ((ptr[0] != '\0') && isspace(ptr[0])) ptr++;
+      while ((ptr[0] != '\0') && g_ascii_isspace(ptr[0])) ptr++;
       i = 0; over = FALSE;
       while (ptr[0] != '\0' && ptr[0] != ';' && !over) {
         if (i < FONT_NAME_LENGTH_MAX) {
@@ -70,7 +67,7 @@ dia_svg_parse_style(xmlNodePtr node, DiaSvgGraphicStyle *s)
       if (!over) family = g_strdup(temp);
      } else if (!strncmp("font-weight:", ptr, 12)) {
       ptr += 12;
-      while ((ptr[0] != '\0') && isspace(ptr[0])) ptr++;
+      while ((ptr[0] != '\0') && g_ascii_isspace(ptr[0])) ptr++;
       i = 0; over = FALSE;
       while (ptr[0] != '\0' && ptr[0] != ';' && !over) {
         if (i < FONT_NAME_LENGTH_MAX) {
@@ -84,7 +81,7 @@ dia_svg_parse_style(xmlNodePtr node, DiaSvgGraphicStyle *s)
       if (!over) weight = g_strdup(temp);
      } else if (!strncmp("font-style:", ptr, 11)) {
       ptr += 11;
-      while ((ptr[0] != '\0') && isspace(ptr[0])) ptr++;
+      while ((ptr[0] != '\0') && g_ascii_isspace(ptr[0])) ptr++;
       i = 0; over = FALSE;
       while (ptr[0] != '\0' && ptr[0] != ';' && !over) {
         if (i < FONT_NAME_LENGTH_MAX) {
@@ -98,7 +95,7 @@ dia_svg_parse_style(xmlNodePtr node, DiaSvgGraphicStyle *s)
       if (!over) style = g_strdup(temp);
      } else if (!strncmp("font-size:", ptr, 10)) {
       ptr += 10;
-      while ((ptr[0] != '\0') && isspace(ptr[0])) ptr++;
+      while ((ptr[0] != '\0') && g_ascii_isspace(ptr[0])) ptr++;
       i = 0; over = FALSE;
       while (ptr[0] != '\0' && ptr[0] != ';' && !over) {
         if (i < FONT_NAME_LENGTH_MAX) {
@@ -110,13 +107,11 @@ dia_svg_parse_style(xmlNodePtr node, DiaSvgGraphicStyle *s)
       temp[i] = '\0';
 
       if (!over) {
-          old_locale = setlocale(LC_NUMERIC, "C");
-          s->font_height = g_strtod(temp, NULL);
-          setlocale(LC_NUMERIC, old_locale);
+          s->font_height = g_ascii_strtod(temp, NULL);
       }
     } else if (!strncmp("text-anchor:", ptr, 12)) {
       ptr += 12;
-      while ((ptr[0] != '\0') && isspace(ptr[0])) ptr++;
+      while ((ptr[0] != '\0') && g_ascii_isspace(ptr[0])) ptr++;
       if (!strncmp(ptr, "start", 5))
         s->alignment = ALIGN_LEFT;
       else if (!strncmp(ptr, "end", 3))
@@ -126,11 +121,10 @@ dia_svg_parse_style(xmlNodePtr node, DiaSvgGraphicStyle *s)
 
     } else if (!strncmp("stroke-width:", ptr, 13)) {
       ptr += 13;
-      old_locale = setlocale(LC_NUMERIC, "C");
-      s->line_width = strtod(ptr, &ptr);
+      s->line_width = g_ascii_strtod(ptr, &ptr);
     } else if (!strncmp("stroke:", ptr, 7)) {
       ptr += 7;
-      while ((ptr[0] != '\0') && isspace(ptr[0])) ptr++;
+      while ((ptr[0] != '\0') && g_ascii_isspace(ptr[0])) ptr++;
       if (ptr[0] == '\0') break;
 
       if (!strncmp(ptr, "none", 4))
@@ -147,7 +141,7 @@ dia_svg_parse_style(xmlNodePtr node, DiaSvgGraphicStyle *s)
 	s->stroke = strtol(ptr+1, NULL, 16) & 0xffffff;
     } else if (!strncmp("fill:", ptr, 5)) {
       ptr += 5;
-      while (ptr[0] != '\0' && isspace(ptr[0])) ptr++;
+      while (ptr[0] != '\0' && g_ascii_isspace(ptr[0])) ptr++;
       if (ptr[0] == '\0') break;
 
       if (!strncmp(ptr, "none", 4))
@@ -164,7 +158,7 @@ dia_svg_parse_style(xmlNodePtr node, DiaSvgGraphicStyle *s)
 	s->fill = strtol(ptr+1, NULL, 16) & 0xffffff;
     } else if (!strncmp("stroke-linecap:", ptr, 15)) {
       ptr += 15;
-      while (ptr[0] != '\0' && isspace(ptr[0])) ptr++;
+      while (ptr[0] != '\0' && g_ascii_isspace(ptr[0])) ptr++;
       if (ptr[0] == '\0') break;
 
       if (!strncmp(ptr, "butt", 4))
@@ -177,7 +171,7 @@ dia_svg_parse_style(xmlNodePtr node, DiaSvgGraphicStyle *s)
 	s->linecap = DIA_SVG_LINECAPS_DEFAULT;
     } else if (!strncmp("stroke-linejoin:", ptr, 16)) {
       ptr += 16;
-      while (ptr[0] != '\0' && isspace(ptr[0])) ptr++;
+      while (ptr[0] != '\0' && g_ascii_isspace(ptr[0])) ptr++;
       if (ptr[0] == '\0') break;
 
       if (!strncmp(ptr, "miter", 5))
@@ -190,7 +184,7 @@ dia_svg_parse_style(xmlNodePtr node, DiaSvgGraphicStyle *s)
 	s->linejoin = DIA_SVG_LINEJOIN_DEFAULT;
     } else if (!strncmp("stroke-pattern:", ptr, 15)) {
       ptr += 15;
-      while (ptr[0] != '\0' && isspace(ptr[0])) ptr++;
+      while (ptr[0] != '\0' && g_ascii_isspace(ptr[0])) ptr++;
       if (ptr[0] == '\0') break;
 
       if (!strncmp(ptr, "solid", 5))
@@ -207,15 +201,13 @@ dia_svg_parse_style(xmlNodePtr node, DiaSvgGraphicStyle *s)
 	s->linestyle = DIA_SVG_LINESTYLE_DEFAULT;
     } else if (!strncmp("stroke-dashlength:", ptr, 18)) {
       ptr += 18;
-      while (ptr[0] != '\0' && isspace(ptr[0])) ptr++;
+      while (ptr[0] != '\0' && g_ascii_isspace(ptr[0])) ptr++;
       if (ptr[0] == '\0') break;
 
       if (!strncmp(ptr, "default", 7))
 	s->dashlength = 1.0;
       else {
-	old_locale = setlocale(LC_NUMERIC, "C");
-	s->dashlength = strtod(ptr, &ptr);
-	setlocale(LC_NUMERIC, old_locale);
+	s->dashlength = g_ascii_strtod(ptr, &ptr);
       }
     } else if (!strncmp("stroke-dasharray:", ptr, 17)) {
       /* FIXME? do we need to read an array here (not clear from
@@ -224,15 +216,13 @@ dia_svg_parse_style(xmlNodePtr node, DiaSvgGraphicStyle *s)
        */
       s->linestyle = LINESTYLE_DASHED;
       ptr += 17;
-      while (ptr[0] != '\0' && isspace(ptr[0])) ptr++;
+      while (ptr[0] != '\0' && g_ascii_isspace(ptr[0])) ptr++;
       if (ptr[0] == '\0') break;
 
       if (!strncmp(ptr, "default", 7))
 	s->dashlength = 1.0;
       else {
-	old_locale = setlocale(LC_NUMERIC, "C");
-	s->dashlength = strtod(ptr, &ptr);
-	setlocale(LC_NUMERIC, old_locale);
+	s->dashlength = g_ascii_strtod(ptr, &ptr);
       }
     }
 
