@@ -91,6 +91,7 @@ xml_file_check_encoding(const gchar *filename, const gchar *default_enc)
   /* first, we expect the magic <?xml string */
   if ((0 != strncmp(p,magic_xml,5)) || (len < 5)) {
     gzclose(zf);
+    g_free(buf);
     return filename; /* let libxml figure out what this is. */
   }
   /* now, we're sure we have some asciish XML file. */
@@ -99,16 +100,19 @@ xml_file_check_encoding(const gchar *filename, const gchar *default_enc)
          && (p<pmax)) p++;
   if (p>=pmax) { /* whoops ? */
     gzclose(zf);
+    g_free(buf);
     return filename;
   }
   if (0 != strncmp(p,"version=\"",9)) {
     gzclose(zf); /* chicken out. */
+    g_free(buf);
     return filename;
   }
   p += 9;
   /* The header is rather well formed. */
   if (p>=pmax) { /* whoops ? */
     gzclose(zf);
+    g_free(buf);
     return filename;
   }
   while ((*p != '"') && (p < pmax)) p++;
@@ -117,10 +121,12 @@ xml_file_check_encoding(const gchar *filename, const gchar *default_enc)
          && (p<pmax)) p++;
   if (p>=pmax) { /* whoops ? */
     gzclose(zf);
+    g_free(buf);
     return filename;
   }
   if (0 == strncmp(p,"encoding=\"",10)) {
     gzclose(zf); /* this file has an encoding string. Good. */
+    g_free(buf);
     return filename;
   }
   /* now let's read the whole file, to see if there are offending bits.
@@ -136,6 +142,7 @@ xml_file_check_encoding(const gchar *filename, const gchar *default_enc)
   } while (len > 0 && well_formed_utf8);
   if (well_formed_utf8) {
     gzclose(zf); /* this file is utf-8 compatible  */
+    g_free(buf);
     return filename;
   } else {
     gzclose(zf); /* poor man's fseek */
@@ -148,6 +155,7 @@ xml_file_check_encoding(const gchar *filename, const gchar *default_enc)
                       "assuming it is encoded in %s"),filename,default_enc);
   } else {
     gzclose(zf); /* we apply the standard here. */
+    g_free(buf);
     return filename;
   }
 
@@ -170,6 +178,7 @@ xml_file_check_encoding(const gchar *filename, const gchar *default_enc)
   }
   gzclose(zf);
   close(uf);
+  g_free(buf);
   return res; /* caller frees the name and unlinks the file. */
 }
 
@@ -375,7 +384,6 @@ data_real(DataNode data)
 {
   char *val;
   real res;
-  char *old_locale;
 
   if (data_type(data)!=DATATYPE_REAL) {
     message_error("Taking real value of non-real node.");
@@ -383,9 +391,7 @@ data_real(DataNode data)
   }
 
   val = xmlGetProp(data, "val");
-  old_locale = setlocale(LC_NUMERIC, "C");
-  res = strtod(val, NULL);
-  setlocale(LC_NUMERIC, old_locale);
+  res = g_ascii_strtod(val, NULL);
   if (val) xmlFree(val);
   
   return res;
@@ -459,7 +465,6 @@ data_point(DataNode data, Point *point)
 {
   char *val;
   char *str;
-  char *old_locale;
   real ax,ay;
 
   if (data_type(data)!=DATATYPE_POINT) {
@@ -468,8 +473,7 @@ data_point(DataNode data, Point *point)
   }
   
   val = xmlGetProp(data, "val");
-  old_locale = setlocale(LC_NUMERIC, "C");
-  point->x = strtod(val, &str);
+  point->x = g_ascii_strtod(val, &str);
   ax = fabs(point->x);
   if ((ax > 1e9) || ((ax < 1e-9) && (ax != 0.0)) || isnan(ax) || isinf(ax)) {
     /* there is no provision to keep values larger when saving, 
@@ -481,20 +485,18 @@ data_point(DataNode data, Point *point)
   while ((*str != ',') && (*str!=0))
     str++;
   if (*str==0){
-    setlocale(LC_NUMERIC, old_locale);
     point->y = 0.0;
-    g_error(_("Error parsing point."));
+    g_warning(_("Error parsing point."));
     xmlFree(val);
     return;
   }
-  point->y = strtod(str+1, NULL);
+  point->y = g_ascii_strtod(str+1, NULL);
   ay = fabs(point->y);
   if ((ay > 1e9) || ((ay < 1e-9) && (ay != 0.0)) || isnan(ay) || isinf(ay)) {
     if (!(ay < 1e-9)) /* don't bother with useless warnings (see above) */
       g_warning(_("Incorrect y Point value \"%s\" %f; discarding it."),str+1,point->y);
     point->y = 0.0;
   }
-  setlocale(LC_NUMERIC, old_locale);
   xmlFree(val);
 }
 
@@ -503,7 +505,6 @@ data_rectangle(DataNode data, Rectangle *rect)
 {
   char *val;
   char *str;
-  char *old_locale;
   
   if (data_type(data)!=DATATYPE_RECTANGLE) {
     message_error("Taking rectangle value of non-rectangle node.");
@@ -512,9 +513,7 @@ data_rectangle(DataNode data, Rectangle *rect)
   
   val = xmlGetProp(data, "val");
   
-  old_locale = setlocale(LC_NUMERIC, "C");
-  rect->left = strtod(val, &str);
-  setlocale(LC_NUMERIC, old_locale);
+  rect->left = g_ascii_strtod(val, &str);
   
   while ((*str != ',') && (*str!=0))
     str++;
@@ -525,9 +524,7 @@ data_rectangle(DataNode data, Rectangle *rect)
     return;
   }
     
-  old_locale = setlocale(LC_NUMERIC, "C");
-  rect->top = strtod(str+1, &str);
-  setlocale(LC_NUMERIC, old_locale);
+  rect->top = g_ascii_strtod(str+1, &str);
 
   while ((*str != ';') && (*str!=0))
     str++;
@@ -538,9 +535,7 @@ data_rectangle(DataNode data, Rectangle *rect)
     return;
   }
 
-  old_locale = setlocale(LC_NUMERIC, "C");
-  rect->right = strtod(str+1, &str);
-  setlocale(LC_NUMERIC, old_locale);
+  rect->right = g_ascii_strtod(str+1, &str);
 
   while ((*str != ',') && (*str!=0))
     str++;
@@ -551,9 +546,7 @@ data_rectangle(DataNode data, Rectangle *rect)
     return;
   }
 
-  old_locale = setlocale(LC_NUMERIC, "C");
-  rect->bottom = strtod(str+1, NULL);
-  setlocale(LC_NUMERIC, old_locale);
+  rect->bottom = g_ascii_strtod(str+1, NULL);
   
   xmlFree(val);
 }
@@ -620,13 +613,22 @@ data_string(DataNode data)
     str[len]=0; /* For safety */
 
     str[strlen(str)-1] = 0; /* Remove last '#' */
-    
+    xmlFree(p);
     return str;
   }
     
   return NULL;
 }
 
+char *
+data_filename(DataNode data)
+{
+  char *utf8 = data_string(data);
+  char *filename = g_filename_from_utf8(utf8, -1, NULL, NULL, NULL);
+  g_free(utf8);
+  return filename;
+}
+
 DiaFont *
 data_font(DataNode data)
 {
@@ -709,12 +711,9 @@ void
 data_add_real(AttributeNode attr, real data)
 {
   DataNode data_node;
-  char buffer[40+1]; /* Large enought? */
-  char *old_locale;
+  char buffer[G_ASCII_DTOSTR_BUF_SIZE]; /* Large enought */
 
-  old_locale = setlocale(LC_NUMERIC, "C");
-  g_snprintf(buffer, 40, "%g", data);
-  setlocale(LC_NUMERIC, old_locale);
+  g_ascii_dtostr(buffer, G_ASCII_DTOSTR_BUF_SIZE, data);
   
   data_node = xmlNewChild(attr, NULL, "real", NULL);
   xmlSetProp(data_node, "val", buffer);
@@ -820,6 +819,16 @@ data_add_string(AttributeNode attr, const char *str)
     g_free(sharped_str);
 }
 
+void
+data_add_filename(DataNode data, const char *str)
+{
+  char *utf8 = g_filename_to_utf8(str, -1, NULL, NULL, NULL);
+
+  data_add_string(data, utf8);
+
+  g_free(utf8);
+}
+
 void
 data_add_font(AttributeNode attr, const DiaFont *font)
 {
