@@ -35,6 +35,7 @@
 #include "message.h"
 #include "color.h"
 #include "font.h"
+#include "text.h"
 
 #ifdef HAVE_FREETYPE
 #include <pango/pango.h>
@@ -83,6 +84,8 @@ static void draw_string (DiaRenderer *renderer,
                          Point *pos,
                          Alignment alignment,
                          Color *color);
+static void draw_text (DiaRenderer *renderer,
+                         Text *text);
 static void draw_image (DiaRenderer *renderer,
                         Point *point,
                         real width, real height,
@@ -91,15 +94,19 @@ static void draw_image (DiaRenderer *renderer,
 static void draw_rect (DiaRenderer *renderer,
                        Point *ul_corner, Point *lr_corner,
                        Color *color);
-static void fill_rect (DiaRenderer *object,
-                       Point *ul_corner, Point *lr_corner,
-                       Color *color);
+static void fill_rect (DiaRenderer *renderer,
+		       Point *ul_corner, Point *lr_corner,
+		       Color *color);
+static void draw_fill_rect (DiaGdkRenderer *renderer,
+			    Point *ul_corner, Point *lr_corner,
+			    Color *color, gboolean fill);
 static void draw_polyline (DiaRenderer *renderer,
                            Point *points, int num_points,
                            Color *color);
 static void draw_polygon (DiaRenderer *renderer,
                           Point *points, int num_points,
                           Color *color);
+static void draw_object (DiaRenderer *renderer, DiaObject *object);
 
 static real get_text_width (DiaRenderer *renderer,
                             const gchar *text, int length);
@@ -143,11 +150,13 @@ renderer_init (DiaGdkRenderer *renderer, void* p)
   renderer->line_width = 1;
   renderer->line_style = GDK_LINE_SOLID;
   renderer->cap_style = GDK_CAP_BUTT;
-  renderer->join_style = GDK_JOIN_MITER;
+  renderer->join_style = GDK_JOIN_ROUND;
 
   renderer->saved_line_style = LINESTYLE_SOLID;
   renderer->dash_length = 10;
   renderer->dot_length = 2;
+
+  renderer->highlight_color = NULL;
 }
 
 static void
@@ -205,17 +214,30 @@ dia_gdk_renderer_class_init (DiaGdkRendererClass *klass)
   /* use <draw|fill>_bezier from DiaRenderer */
 
   renderer_class->draw_string  = draw_string;
+  renderer_class->draw_text    = draw_text;
   renderer_class->draw_image   = draw_image;
 
   /* medium level functions */
   renderer_class->draw_rect = draw_rect;
   renderer_class->draw_polyline  = draw_polyline;
   renderer_class->draw_polygon   = draw_polygon;
+  renderer_class->draw_object    = draw_object;
 
   /* Interactive functions */
   renderer_class->get_text_width = get_text_width;
 }
 
+static void
+renderer_color_convert(DiaGdkRenderer *renderer,
+		       Color *col, GdkColor *gdk_col)
+{
+  if (renderer->highlight_color != NULL) {
+    color_convert(renderer->highlight_color, gdk_col);
+  } else {
+    color_convert(col, gdk_col);
+  }
+}
+
 static void 
 begin_render (DiaRenderer *object)
 {
@@ -231,6 +253,12 @@ set_linewidth (DiaRenderer *object, real linewidth)
 {
   DiaGdkRenderer *renderer = DIA_GDK_RENDERER (object);
 
+  if (renderer->highlight_color != NULL) {
+    /* 6 pixels wide -> 3 pixels beyond normal obj */
+    real border = dia_untransform_length(renderer->transform, 6);
+    linewidth += border;
+  } 
+
   /* 0 == hairline **/
   renderer->line_width =
     dia_transform_length(renderer->transform, linewidth);
@@ -250,16 +278,20 @@ set_linecaps (DiaRenderer *object, LineCaps mode)
 {
   DiaGdkRenderer *renderer = DIA_GDK_RENDERER (object);
 
-  switch(mode) {
-  case LINECAPS_BUTT:
-    renderer->cap_style = GDK_CAP_BUTT;
-    break;
-  case LINECAPS_ROUND:
+  if (renderer->highlight_color != NULL) {
     renderer->cap_style = GDK_CAP_ROUND;
-    break;
-  case LINECAPS_PROJECTING:
-    renderer->cap_style = GDK_CAP_PROJECTING;
-    break;
+  } else {
+    switch(mode) {
+    case LINECAPS_BUTT:
+      renderer->cap_style = GDK_CAP_BUTT;
+      break;
+    case LINECAPS_ROUND:
+      renderer->cap_style = GDK_CAP_ROUND;
+      break;
+    case LINECAPS_PROJECTING:
+      renderer->cap_style = GDK_CAP_PROJECTING;
+      break;
+    }
   }
  
   gdk_gc_set_line_attributes(renderer->gc,
@@ -274,16 +306,20 @@ set_linejoin (DiaRenderer *object, LineJoin mode)
 {
   DiaGdkRenderer *renderer = DIA_GDK_RENDERER (object);
 
-  switch(mode) {
-  case LINEJOIN_MITER:
-    renderer->cap_style = GDK_JOIN_MITER;
-    break;
-  case LINEJOIN_ROUND:
-    renderer->cap_style = GDK_JOIN_ROUND;
-    break;
-  case LINEJOIN_BEVEL:
-    renderer->cap_style = GDK_JOIN_BEVEL;
-    break;
+  if (renderer->highlight_color != NULL) {
+    renderer->join_style = GDK_JOIN_ROUND;
+  } else {
+    switch(mode) {
+    case LINEJOIN_MITER:
+      renderer->join_style = GDK_JOIN_MITER;
+      break;
+    case LINEJOIN_ROUND:
+      renderer->join_style = GDK_JOIN_ROUND;
+      break;
+    case LINEJOIN_BEVEL:
+      renderer->join_style = GDK_JOIN_BEVEL;
+      break;
+    }
   }
  
   gdk_gc_set_line_attributes(renderer->gc,
@@ -293,27 +329,27 @@ set_linejoin (DiaRenderer *object, LineJoin mode)
 			     renderer->join_style);
 }
 
-static void 
-set_linestyle (DiaRenderer *object, LineStyle mode)
+/** Set the dashes for this renderer.
+ * offset determines where in the pattern the dashes will start.
+ * It is used by the grid in particular to make the grid dashes line up.
+ */
+void
+dia_gdk_renderer_set_dashes(DiaGdkRenderer *renderer, int offset)
 {
-  DiaGdkRenderer *renderer = DIA_GDK_RENDERER (object);
-
   char dash_list[6];
   int hole_width;
+  int pattern_length;
   
-  renderer->saved_line_style = mode;
-  switch(mode) {
+  switch(renderer->saved_line_style) {
   case LINESTYLE_SOLID:
-    renderer->line_style = GDK_LINE_SOLID;
     break;
   case LINESTYLE_DASHED:
-    renderer->line_style = GDK_LINE_ON_OFF_DASH;
     dash_list[0] = renderer->dash_length;
     dash_list[1] = renderer->dash_length;
-    gdk_gc_set_dashes(renderer->gc, 0, dash_list, 2);
+    pattern_length = renderer->dash_length*2;
+    gdk_gc_set_dashes(renderer->gc, offset, dash_list, 2);
     break;
   case LINESTYLE_DASH_DOT:
-    renderer->line_style = GDK_LINE_ON_OFF_DASH;
     hole_width = (renderer->dash_length - renderer->dot_length) / 2;
     if (hole_width==0)
       hole_width = 1;
@@ -321,10 +357,10 @@ set_linestyle (DiaRenderer *object, LineStyle mode)
     dash_list[1] = hole_width;
     dash_list[2] = renderer->dot_length;
     dash_list[3] = hole_width;
-    gdk_gc_set_dashes(renderer->gc, 0, dash_list, 4);
+    pattern_length = renderer->dash_length+renderer->dot_length+2*hole_width;
+    gdk_gc_set_dashes(renderer->gc, offset, dash_list, 4);
     break;
   case LINESTYLE_DASH_DOT_DOT:
-    renderer->line_style = GDK_LINE_ON_OFF_DASH;
     hole_width = (renderer->dash_length - 2*renderer->dot_length) / 3;
     if (hole_width==0)
       hole_width = 1;
@@ -334,13 +370,44 @@ set_linestyle (DiaRenderer *object, LineStyle mode)
     dash_list[3] = hole_width;
     dash_list[4] = renderer->dot_length;
     dash_list[5] = hole_width;
-    gdk_gc_set_dashes(renderer->gc, 0, dash_list, 6);
+    pattern_length = renderer->dash_length+2*renderer->dot_length+3*hole_width;
+    gdk_gc_set_dashes(renderer->gc, offset, dash_list, 6);
+    break;
+  case LINESTYLE_DOTTED:
+    dash_list[0] = renderer->dot_length;
+    dash_list[1] = renderer->dot_length;
+    pattern_length = renderer->dot_length;
+    gdk_gc_set_dashes(renderer->gc, offset, dash_list, 2);
+    break;
+  }
+
+}
+
+static void 
+set_linestyle (DiaRenderer *object, LineStyle mode)
+{
+  DiaGdkRenderer *renderer = DIA_GDK_RENDERER (object);
+
+  renderer->saved_line_style = mode;
+  switch(mode) {
+  case LINESTYLE_SOLID:
+    renderer->line_style = GDK_LINE_SOLID;
+    break;
+  case LINESTYLE_DASHED:
+    renderer->line_style = GDK_LINE_ON_OFF_DASH;
+    dia_gdk_renderer_set_dashes(renderer, 0);
+    break;
+  case LINESTYLE_DASH_DOT:
+    renderer->line_style = GDK_LINE_ON_OFF_DASH;
+    dia_gdk_renderer_set_dashes(renderer, 0);
+    break;
+  case LINESTYLE_DASH_DOT_DOT:
+    renderer->line_style = GDK_LINE_ON_OFF_DASH;
+    dia_gdk_renderer_set_dashes(renderer, 0);
     break;
   case LINESTYLE_DOTTED:
     renderer->line_style = GDK_LINE_ON_OFF_DASH;
-    dash_list[0] = renderer->dot_length;
-    dash_list[1] = renderer->dot_length;
-    gdk_gc_set_dashes(renderer->gc, 0, dash_list, 2);
+    dia_gdk_renderer_set_dashes(renderer, 0);
     break;
   }
   gdk_gc_set_line_attributes(renderer->gc,
@@ -377,8 +444,6 @@ set_dashlength (DiaRenderer *object, real length)
 static void 
 set_fillstyle (DiaRenderer *object, FillStyle mode)
 {
-  DiaGdkRenderer *renderer = DIA_GDK_RENDERER (object);
-
   switch(mode) {
   case FILLSTYLE_SOLID:
     break;
@@ -399,7 +464,7 @@ draw_line (DiaRenderer *object, Point *start, Point *end, Color *line_color)
   dia_transform_coords(renderer->transform, start->x, start->y, &x1, &y1);
   dia_transform_coords(renderer->transform, end->x, end->y, &x2, &y2);
   
-  color_convert(line_color, &color);
+  renderer_color_convert(renderer, line_color, &color);
   gdk_gc_set_foreground(gc, &color);
   
   gdk_draw_line(renderer->pixmap, gc,
@@ -423,7 +488,7 @@ fill_polygon (DiaRenderer *object, Point *points, int num_points, Color *line_co
     gdk_points[i].y = y;
   }
   
-  color_convert(line_color, &color);
+  renderer_color_convert(renderer, line_color, &color);
   gdk_gc_set_foreground(gc, &color);
   
   gdk_draw_polygon(renderer->pixmap, gc, TRUE, gdk_points, num_points);
@@ -454,7 +519,7 @@ draw_fill_arc (DiaRenderer *object,
   if ((left>right) || (top>bottom))
     return;
 
-  color_convert(color, &gdkcolor);
+  renderer_color_convert(renderer, color, &gdkcolor);
   gdk_gc_set_foreground(gc, &gdkcolor);
 
   dangle = angle2-angle1;
@@ -508,144 +573,190 @@ get_layout_first_baseline(PangoLayout* layout)
   return result;
 }
 
+/** Return the x adjustment needed for this text and alignment */
+static real
+get_alignment_adjustment(DiaRenderer *object, 
+			 const gchar *text,
+			 Alignment alignment)
+{
+  DiaGdkRenderer *renderer = DIA_GDK_RENDERER (object);
+
+   switch (alignment) {
+   case ALIGN_LEFT:
+     return 0.0;
+   case ALIGN_CENTER:
+     return
+       dia_font_scaled_string_width(text, object->font,
+				    object->font_height,
+				    dia_transform_length(renderer->transform, 10.0)/10.0)/2;
+     break;
+   case ALIGN_RIGHT:
+     return
+       dia_font_scaled_string_width(text, object->font,
+				    object->font_height,
+				    dia_transform_length(renderer->transform, 10.0)/10.0);
+     break;
+   }
+   return 0.0;
+}
+
+/* Draw a highlighted version of a string.
+ */
+static void
+draw_highlighted_string(DiaGdkRenderer *renderer,
+			PangoLayout *layout,
+			int x, int y,
+			GdkColor *color)
+{
+  gint width, height;
+
+  pango_layout_get_pixel_size(layout, &width, &height);
+
+  gdk_gc_set_foreground(renderer->gc, color);
+
+  gdk_draw_rectangle (renderer->pixmap,
+		      renderer->gc, TRUE,
+		      x-3, y-3,
+		      width+6, height+6);
+}
+
 static void 
 draw_string (DiaRenderer *object,
              const gchar *text, Point *pos, Alignment alignment,
              Color *color)
 {
   DiaGdkRenderer *renderer = DIA_GDK_RENDERER (object);
-  GdkGC *gc = renderer->gc;
   GdkColor gdkcolor;
 
   int x,y;
   Point start_pos;
-  PangoLayout* layout;
+  PangoLayout* layout = NULL;
   
-  long t1, t2;
+  if (text == NULL || *text == '\0') return; /* Don't render empty strings. */
 
   point_copy(&start_pos,pos);
 
-  color_convert(color, &gdkcolor);
+  renderer_color_convert(renderer, color, &gdkcolor);
+
+  start_pos.x -= get_alignment_adjustment(object, text, alignment);
+
+  {
+    int height_pixels = dia_transform_length(renderer->transform, object->font_height);
+    if (height_pixels < 2) { /* "Greeking" instead of making tiny font */
+      int width_pixels = dia_transform_length(
+                            renderer->transform, 
+                            dia_font_string_width(text, object->font, object->font_height));
+      gdk_gc_set_foreground(renderer->gc, &gdkcolor);
+      gdk_gc_set_dashes(renderer->gc, 0, "\1\2", 2);
+      dia_transform_coords(renderer->transform, start_pos.x, start_pos.y, &x, &y);
+      gdk_draw_line(renderer->pixmap, renderer->gc, x, y, x + width_pixels, y);
+      return;
+    }
+  }
 
   /* My apologies for adding more #hell, but the alternative is an abhorrent
    * kludge.
    */
-#ifdef HAVE_FREETYPE
+#if defined HAVE_FREETYPE
   {
    FT_Bitmap ftbitmap;
    guint8 *graybitmap;
    int width, height;
    int rowstride;
-   double font_height;
+   GdkPixbuf *rgba = NULL;
 
-   switch (alignment) {
-   case ALIGN_LEFT:
-     break;
-   case ALIGN_CENTER:
-     start_pos.x -= dia_font_scaled_string_width (
-						  text, object->font,
-						  object->font_height,
-						  dia_transform_length(renderer->transform, 1.0))/2;
-     break;
-   case ALIGN_RIGHT:
-     start_pos.x -= dia_font_scaled_string_width(
-					  text, object->font,
-					  object->font_height,
-					  dia_transform_length(renderer->transform, 1.0));
-     break;
-   }
-   
-   start_pos.y -= dia_font_scaled_ascent(text, object->font,
-					 object->font_height,
-					 dia_transform_length(renderer->transform, 1.0));
+
+   start_pos.y -= 
+     dia_font_scaled_ascent(text, object->font,
+			    object->font_height,
+			    dia_transform_length(renderer->transform, 1.0));
 
    dia_transform_coords(renderer->transform, 
 			start_pos.x, start_pos.y, &x, &y);
-
-   layout = dia_font_scaled_build_layout(text, object->font,
-					 object->font_height,
-					 dia_transform_length(renderer->transform, 1.0));
-   /*   y -= get_layout_first_baseline(layout);  */
-   pango_layout_get_pixel_size(layout, &width, &height);
-   if (width > 0) {
-      rowstride = 32*((width+31)/31);
+   
+     layout = dia_font_scaled_build_layout(text, object->font,
+					   object->font_height,
+					   dia_transform_length(renderer->transform, 1.0));
+     if (renderer->highlight_color != NULL) {
+       draw_highlighted_string(renderer, layout, x, y, &gdkcolor);
+     } else {
+       /*   y -= get_layout_first_baseline(layout);  */
+       pango_layout_get_pixel_size(layout, &width, &height);
      
-     graybitmap = (guint8*)g_new0(guint8, height*rowstride);
-     
-     ftbitmap.rows = height;
-     ftbitmap.width = width;
-     ftbitmap.pitch = rowstride;
-     ftbitmap.buffer = graybitmap;
-     ftbitmap.num_grays = 256;
-     ftbitmap.pixel_mode = ft_pixel_mode_grays;
-     ftbitmap.palette_mode = 0;
-     ftbitmap.palette = 0;
-     pango_ft2_render_layout(&ftbitmap, layout, 0, 0);
-
-     {
-       GdkPixbuf *rgba = 
-	 gdk_pixbuf_new(GDK_COLORSPACE_RGB, TRUE, 8, width, height);
-       int stride = gdk_pixbuf_get_rowstride(rgba);
-       guchar* pixels = gdk_pixbuf_get_pixels(rgba);
-       int i,j;
-       for (i = 0; i < height; i++) {
-	 for (j = 0; j < width; j++) {
-	   pixels[i*stride+j*4] = gdkcolor.red>>8;
-	   pixels[i*stride+j*4+1] = gdkcolor.green>>8;
-	   pixels[i*stride+j*4+2] = gdkcolor.blue>>8;
-	   pixels[i*stride+j*4+3] = graybitmap[i*rowstride+j];
+       if (width > 0) {
+	 rowstride = 32*((width+31)/31);
+       
+	 graybitmap = (guint8*)g_new0(guint8, height*rowstride);
+       
+	 ftbitmap.rows = height;
+	 ftbitmap.width = width;
+	 ftbitmap.pitch = rowstride;
+	 ftbitmap.buffer = graybitmap;
+	 ftbitmap.num_grays = 256;
+	 ftbitmap.pixel_mode = ft_pixel_mode_grays;
+	 ftbitmap.palette_mode = 0;
+	 ftbitmap.palette = 0;
+	 pango_ft2_render_layout(&ftbitmap, layout, 0, 0);
+       
+	 {
+	   int stride;
+	   guchar* pixels;
+	   int i,j;
+	   rgba = gdk_pixbuf_new(GDK_COLORSPACE_RGB, TRUE, 8, width, height);
+	   stride = gdk_pixbuf_get_rowstride(rgba);
+	   pixels = gdk_pixbuf_get_pixels(rgba);
+	   for (i = 0; i < height; i++) {
+	     for (j = 0; j < width; j++) {
+	       pixels[i*stride+j*4] = gdkcolor.red>>8;
+	       pixels[i*stride+j*4+1] = gdkcolor.green>>8;
+	       pixels[i*stride+j*4+2] = gdkcolor.blue>>8;
+	       pixels[i*stride+j*4+3] = graybitmap[i*rowstride+j];
+	     }
+	   }
+	   g_free(graybitmap);
 	 }
        }
        /*
 	 gdk_draw_pixbuf(renderer->pixmap, gc, rgba, 0, 0, x, y, width, height,
 	 GDK_RGB_DITHER_NONE, 0, 0);
        */
-       gdk_pixbuf_render_to_drawable_alpha(rgba, 
-					   renderer->pixmap,
-					   0, 0,
-					   x, y,
-					   width, height,
-					   GDK_PIXBUF_ALPHA_FULL,
-					   128,
-					   GDK_RGB_DITHER_NONE,
-					   0, 0);
-       g_object_unref(G_OBJECT(rgba));
-       g_free(graybitmap);
+       if (rgba != NULL) { /* Non-null width */
+	 gdk_pixbuf_render_to_drawable_alpha(rgba, 
+					     renderer->pixmap,
+					     0, 0,
+					     x, y,
+					     width, height,
+					     GDK_PIXBUF_ALPHA_FULL,
+					     128,
+					     GDK_RGB_DITHER_NONE,
+					     0, 0);
+	 g_object_unref(G_OBJECT(rgba));
+       }
+       /*
+	 gdk_gc_set_function(gc, GDK_COPY_INVERT);
+	 gdk_draw_gray_image(renderer->pixmap, gc, x, y, width, height, 
+	 GDK_RGB_DITHER_NONE, graybitmap, rowstride);
+	 gdk_gc_set_function(gc, GDK_COPY);
+       */
      }
-   }
-   /*
-   gdk_gc_set_function(gc, GDK_COPY_INVERT);
-   gdk_draw_gray_image(renderer->pixmap, gc, x, y, width, height, 
-		       GDK_RGB_DITHER_NONE, graybitmap, rowstride);
-   gdk_gc_set_function(gc, GDK_COPY);
-   */
- }
-#else
-  gdk_gc_set_foreground(gc, &gdkcolor);
-  switch (alignment) {
-  case ALIGN_LEFT:
-    break;
-  case ALIGN_CENTER:
-    start_pos.x -= dia_font_scaled_string_width (
-                      text, object->font,
-                      object->font_height,
-                      dia_transform_length (renderer->transform, 10.0) / 10.0)/2;
-    break;
-  case ALIGN_RIGHT:
-    start_pos.x -= dia_font_scaled_string_width(
-                      text, object->font,
-                      object->font_height,
-                      dia_transform_length (renderer->transform, 10.0) / 10.0);
-    break;
   }
-  dia_transform_coords(renderer->transform, start_pos.x, start_pos.y, &x, &y);
+#else
+  {
+    GdkGC *gc = renderer->gc;
+    gdk_gc_set_foreground(gc, &gdkcolor);
+    dia_transform_coords(renderer->transform, start_pos.x, start_pos.y, &x, &y);
 
-  layout = dia_font_scaled_build_layout(
-              text, object->font,
-              object->font_height,
-              dia_transform_length (renderer->transform, 10.0) / 10.0);
-  y -= get_layout_first_baseline(layout);  
-  gdk_draw_layout(renderer->pixmap,gc,x,y,layout);
+    layout = dia_font_scaled_build_layout(
+                text, object->font,
+                object->font_height,
+                dia_transform_length (renderer->transform, 10.0) / 10.0);
+    y -= get_layout_first_baseline(layout);  
+    if (renderer->highlight_color != NULL) {
+      draw_highlighted_string(renderer, layout, x, y, &gdkcolor);
+    } else {
+      gdk_draw_layout(renderer->pixmap,gc,x,y,layout);
+    }
+  }
 #endif
 
       /* abuse_layout_object(layout,text); */
@@ -653,6 +764,25 @@ draw_string (DiaRenderer *object,
   g_object_unref(G_OBJECT(layout));
 }
 
+/** Caching Pango renderer */
+static void
+draw_text(DiaRenderer *renderer, Text *text) {
+  Point pos;
+  int i;
+
+  DIA_RENDERER_GET_CLASS(renderer)->set_font(renderer, text->font, text->height);
+  pos = text->position;
+  
+  for (i=0;i<text->numlines;i++) {
+    DIA_RENDERER_GET_CLASS(renderer)->draw_string(renderer,
+						  text->line[i],
+						  &pos, text->alignment,
+						  &text->color);
+    pos.y += text->height;
+  }
+}
+
+
 /* Get the width of the given text in cm */
 static real
 get_text_width(DiaRenderer *object,
@@ -692,21 +822,31 @@ draw_image (DiaRenderer *object,
             DiaImage image)
 {
   DiaGdkRenderer *renderer = DIA_GDK_RENDERER (object);
-  int real_width, real_height, real_x, real_y;
-  
-  real_width = dia_transform_length(renderer->transform, width);
-  real_height = dia_transform_length(renderer->transform, height);
-  dia_transform_coords(renderer->transform, point->x, point->y,
-                       &real_x, &real_y);
+  if (renderer->highlight_color != NULL) {
+    Point lr;
+    DiaRendererClass *self_class = DIA_RENDERER_GET_CLASS (object);
 
-  dia_image_draw(image,  renderer->pixmap, real_x, real_y,
-		 real_width, real_height);
+    lr = *point;
+    lr.x += width;
+    lr.y += height;
+    self_class->fill_rect(object, point, &lr, renderer->highlight_color);
+  } else {
+    int real_width, real_height, real_x, real_y;
+  
+    real_width = dia_transform_length(renderer->transform, width);
+    real_height = dia_transform_length(renderer->transform, height);
+    dia_transform_coords(renderer->transform, point->x, point->y,
+			 &real_x, &real_y);
+
+    dia_image_draw(image,  renderer->pixmap, real_x, real_y,
+		   real_width, real_height);
+  }
 }
 
 /*
  * medium level functions
  */
-void
+static void
 draw_fill_rect (DiaGdkRenderer *renderer,
                 Point *ul_corner, Point *lr_corner,
                 Color *color, gboolean fill)
@@ -723,7 +863,7 @@ draw_fill_rect (DiaGdkRenderer *renderer,
   if ((left>right) || (top>bottom))
     return;
 
-  color_convert(color, &gdkcolor);
+  renderer_color_convert(renderer, color, &gdkcolor);
   gdk_gc_set_foreground(gc, &gdkcolor);
 
   gdk_draw_rectangle (renderer->pixmap,
@@ -754,33 +894,67 @@ fill_rect (DiaRenderer *object,
 }
 
 static void 
-draw_polyline (DiaRenderer *renderer,
+draw_polyline (DiaRenderer *self,
                Point *points, int num_points,
-               Color *color)
+               Color *line_color)
 {
-  DiaRendererClass *klass = DIA_RENDERER_GET_CLASS (renderer);
-  int i;
+  DiaGdkRenderer *renderer = DIA_GDK_RENDERER (self);
+  GdkGC *gc = renderer->gc;
+  GdkColor color;
+  GdkPoint *gdk_points;
+  int i,x,y;
+  
+  gdk_points = g_new(GdkPoint, num_points);
 
-  for (i = 0; i < num_points - 1; i++)
-    klass->draw_line (renderer, &points[i+0], &points[i+1], color);
+  for (i=0;i<num_points;i++) {
+    dia_transform_coords(renderer->transform, points[i].x, points[i].y, &x, &y);
+    gdk_points[i].x = x;
+    gdk_points[i].y = y;
+  }
+  
+  renderer_color_convert(renderer, line_color, &color);
+  gdk_gc_set_foreground(gc, &color);
+  
+  gdk_draw_lines(renderer->pixmap, gc, gdk_points, num_points);
+  g_free(gdk_points);
 }
 
 static void 
-draw_polygon (DiaRenderer *renderer,
+draw_polygon (DiaRenderer *self,
               Point *points, int num_points,
-              Color *color)
+              Color *line_color)
 {
-  DiaRendererClass *klass = DIA_RENDERER_GET_CLASS (renderer);
-  int i;
+  DiaGdkRenderer *renderer = DIA_GDK_RENDERER (self);
+  GdkGC *gc = renderer->gc;
+  GdkColor color;
+  GdkPoint *gdk_points;
+  int i,x,y;
+  
+  gdk_points = g_new(GdkPoint, num_points);
 
-  g_return_if_fail (1 < num_points);
+  for (i=0;i<num_points;i++) {
+    dia_transform_coords(renderer->transform, points[i].x, points[i].y, &x, &y);
+    gdk_points[i].x = x;
+    gdk_points[i].y = y;
+  }
+  
+  renderer_color_convert(renderer, line_color, &color);
+  gdk_gc_set_foreground(gc, &color);
+  
+  gdk_draw_polygon(renderer->pixmap, gc, FALSE, gdk_points, num_points);
+  g_free(gdk_points);
+}
 
-  for (i = 0; i < num_points - 1; i++)
-    klass->draw_line (renderer, &points[i+0], &points[i+1], color);
-  /* close it in any case */
-  if (   (points[0].x != points[num_points-1].x) 
-      || (points[0].y != points[num_points-1].y))
-    klass->draw_line (renderer, &points[num_points-1], &points[0], color);
+static void
+draw_object (DiaRenderer *renderer, DiaObject *object)
+{
+  if (object->highlight_color != NULL) {
+    DiaGdkRenderer *gdk_rend = DIA_GDK_RENDERER(renderer);
+    gdk_rend->highlight_color = object->highlight_color;
+    object->ops->draw(object, renderer);
+    gdk_rend->highlight_color = NULL;
+  }
+  object->ops->draw(object, renderer);
 }
 
 static int
