@@ -18,29 +18,35 @@
 #ifndef DIAMENU_H
 #define DIAMENU_H
 
+#include "diatypes.h"
 #include "object.h"
 
-typedef struct _DiaMenuItem DiaMenuItem;
-typedef struct _DiaMenu DiaMenu;
+/* Flags for DiaMenuItem->active */
+#define DIAMENU_ACTIVE (1<<0)
+#define DIAMENU_TOGGLE (1<<1)
+#define DIAMENU_TOGGLE_ON (1<<2)
 
 /* Note: The returned change is already applied. */
-typedef ObjectChange *(*DiaMenuCallback)(Object *obj, Point *pos, gpointer data);
+typedef ObjectChange *(*DiaMenuCallback)(DiaObject *obj, Point *pos, gpointer data);
 
 struct _DiaMenuItem {
   char *text;
   DiaMenuCallback callback;
   gpointer callback_data;
-  int active;
+  int active; /* Actually flags now, but keeps name for compatibility */
   /* Private for app:  */
   void *app_data; /* init to NULL */
 };
 
+typedef void (*DiaMenuAppDataFree) (DiaMenu *menu);
+
 struct _DiaMenu {
   char *title;
   int num_items;
   DiaMenuItem *items;
   /* Private for app:  */
   void *app_data; /* init to NULL */
+  DiaMenuAppDataFree app_data_free;
 };
 
 #endif /*DIAMENU_H*/
