@@ -21,31 +21,57 @@
 #ifndef FILTER_H
 #define FILTER_H
 
+#include "diatypes.h"
 #include <glib.h>
 #include <diagramdata.h>
 
-typedef struct _DiaExportFilter DiaExportFilter;
 typedef void (* DiaExportFunc) (DiagramData *dia, const gchar *filename,
-				const gchar *diafilename);
+				const gchar *diafilename, void* user_data);
 
 struct _DiaExportFilter {
   const gchar *description;
   /* NULL terminated array of extensions for this file format.  The first
    * one is the prefered extension for files of this type. */
   const gchar **extensions;
-  DiaExportFunc export;
+  DiaExportFunc export_func;
+  void* user_data;
+  /* A unique name that this filter can always be addressed as from the
+   * command line.  If more than one filter can handle an extension, this
+   * will allow disambiguation.  Note that import and export filters are
+   * treated seperately for this.
+   */
+  const gchar *unique_name;
 };
 
-typedef struct _DiaImportFilter DiaImportFilter;
 /* returns FALSE on error loading diagram */
-typedef gboolean (* DiaImportFunc) (const gchar *filename, DiagramData *dia);
+typedef gboolean (* DiaImportFunc) (const gchar *filename, DiagramData *dia, 
+                                    void* user_data);
 
 struct _DiaImportFilter {
   const gchar *description;
   /* NULL terminated array of extensions for this file format.  The first
    * one is the prefered extension for files of this type. */
   const gchar **extensions;
-  DiaImportFunc import;
+  DiaImportFunc import_func;
+  void* user_data;
+  /* A unique name that this filter can always be addressed as from the
+   * command line.  If more than one filter can handle an extension, this
+   * will allow disambiguation.  Note that import and export filters are
+   * treated seperately for this.
+   */
+  const gchar *unique_name;
+};
+
+/* gets called as menu callback */
+typedef void (* DiaCallbackFunc) (DiagramData *dia,
+                                  guint flags, /* further additions */
+                                  void* user_data);
+
+struct _DiaCallbackFilter {
+  const gchar *description;
+  const gchar *menupath;
+  DiaCallbackFunc callback;
+  void* user_data;
 };
 
 /* register an export filter.  The DiaExportFilter should be static, and
@@ -60,10 +86,16 @@ gchar *filter_get_export_filter_label(DiaExportFilter *efilter);
 
 /* guess the filter for a given filename. */
 DiaExportFilter *filter_guess_export_filter(const gchar *filename);
+/* Get the filter for the unique filename. */
+DiaExportFilter *filter_get_by_name(const gchar *name);
 
 void filter_register_import(DiaImportFilter *ifilter);
 GList *filter_get_import_filters(void);
 gchar *filter_get_import_filter_label(DiaImportFilter *ifilter);
 DiaImportFilter *filter_guess_import_filter(const gchar *filename);
 
+void filter_register_callback(DiaCallbackFilter *cbfilter);
+/* returns all registered callbacks */
+GList *filter_get_callbacks(void);
+
 #endif
