@@ -32,6 +32,7 @@
 #include <pango/pangoft2.h>
 #endif
 #include <gdk/gdk.h>
+#include <gtk/gtk.h> /* just for gtk_get_default_language() */
 
 #include "font.h"
 #include "message.h"
@@ -106,6 +107,7 @@ void
 dia_font_push_context(PangoContext *pcontext) {
   pango_contexts = g_list_prepend(pango_contexts, pango_context);
   pango_context = pcontext;
+  pango_context_set_language (pango_context, gtk_get_default_language ());
   g_object_ref(pcontext);
 }
 
@@ -113,6 +115,7 @@ void
 dia_font_pop_context() {
   g_object_unref(pango_context);
   pango_context = (PangoContext*)pango_contexts->data;
+  pango_context_set_language (pango_context, gtk_get_default_language ());
   pango_contexts = g_list_next(pango_contexts);
 }
 
@@ -286,6 +289,7 @@ dia_font_new_from_style(DiaFontStyle style, real height)
   
   retval = DIA_FONT(g_type_create_instance(dia_font_get_type()));
   retval->pfd = pfd;
+  retval->legacy_name = NULL;
   return retval;
 }
 
@@ -505,7 +509,6 @@ real dia_font_descent(const char* string, DiaFont* font, real height)
 typedef struct {
   gchar *string;
   DiaFont *font;
-  real height;
   PangoLayout *layout;
   int usecount;
 } LayoutCacheItem;
@@ -519,8 +522,7 @@ layout_cache_equals(gconstpointer e1, gconstpointer e2)
     *i2 = (LayoutCacheItem*)e2;
 
   return strcmp(i1->string, i2->string) == 0 &&
-    fabs(i1->height - i2->height) < 0.00001 &&
-    i1->font == i2->font;
+    pango_font_description_equal(i1->font->pfd, i2->font->pfd);
 }
 
 static guint
@@ -529,8 +531,7 @@ layout_cache_hash(gconstpointer el)
   LayoutCacheItem *item = (LayoutCacheItem*)el;
 
   return g_str_hash(item->string) ^
-    (int)(item->height*1000) ^
-    (int)(item->font);
+    pango_font_description_hash(item->font->pfd);
 }
 
 static long layout_cache_last_use;
@@ -552,7 +553,6 @@ layout_cache_cleanup_idle(gpointer data)
   GHashTable *table = (GHashTable*)(data);
 
   g_hash_table_foreach_remove(table, layout_cache_cleanup_entry, NULL);
-
   return FALSE;
 }
 
@@ -563,7 +563,7 @@ static gboolean
 layout_cache_cleanup(gpointer data)
 {
   /* Only cleanup if there has been font activity since last cleanup */
-  if (time(0) - layout_cache_last_use < 60*10) {
+  if (time(0) - layout_cache_last_use < 10) {
     /* Don't go directly to cleanup, wait till there's a pause. */
     g_idle_add(layout_cache_cleanup_idle, data);
   }
@@ -582,7 +582,7 @@ layout_cache_free_key(gpointer data)
     }
 
     if (item->font != NULL) {
-      g_object_unref(item->font);
+      dia_font_unref(item->font);
       item->font = NULL;
     }
 
@@ -603,7 +603,7 @@ dia_font_build_layout(const char* string, DiaFont* font, real height)
     guint length;
     gchar *desc = NULL;
 
-    LayoutCacheItem *cached;
+    LayoutCacheItem *cached, *item;
 
     layout_cache_last_use = time(0);
     if (layoutcache == NULL) {
@@ -612,33 +612,49 @@ dia_font_build_layout(const char* string, DiaFont* font, real height)
 					  layout_cache_equals,
 					  layout_cache_free_key,
 					  NULL);
-      g_timeout_add(10*60*1000, layout_cache_cleanup, (gpointer)layoutcache);
-    } else {
-      LayoutCacheItem item;
-      item.string = string;
-      item.font = font;
-      item.height = height;
-      cached = g_hash_table_lookup(layoutcache, &item);
-      if (cached != NULL) {
-	g_object_ref(cached->layout);
-	cached->usecount ++;
-	return cached->layout;
-      }
+      /** Check for cache cleanup every 10 seconds. */
+      /** This frequent a check is really a hack while we figure out the
+       *  exact problems with reffing the fonts.
+       *  Note to self:  The equals function should compare pfd's, but
+       *  then DiaFonts are freed too early. 
+       */
+      g_timeout_add(10*1000, layout_cache_cleanup, (gpointer)layoutcache);
     }
 
-    cached = g_new0(LayoutCacheItem,1);
-    cached->string = g_strdup(string);
-    cached->font = font;
-    g_object_ref(font);
-    cached->height = height;
-
     height *= 0.7;
+    dia_font_set_height(font, height);
+
+    item = g_new0(LayoutCacheItem,1);
+    item->string = g_strdup(string);
+    item->font = font;
+    
+    /* If it's in the cache, use that instead. */
+    cached = g_hash_table_lookup(layoutcache, item);
+    if (cached != NULL) {
+      g_object_ref(cached->layout);
+      g_free(item->string);
+      g_free(item);
+      cached->usecount ++;
+      return cached->layout;
+    }
+
+    dia_font_ref(font);
+
+    /* This could should account for DPI, but it doesn't do right.  Grrr...
+    {
+      GdkScreen *screen = gdk_screen_get_default();
+      real dpi = gdk_screen_get_width(screen)/
+		 (gdk_screen_get_width_mm(screen)/25.4);
+      printf("height = %f, dpi = %f, new height = %f\n",
+	     height, dpi, height * (dpi/100));
+      height *= dpi/100;
+    }
+    */
 
-    dia_font_set_height(font,height);
     layout = pango_layout_new(dia_font_get_context());
 
     length = string ? strlen(string) : 0;
-    pango_layout_set_text(layout,string,length);
+    pango_layout_set_text(layout, string, length);
         
     list = pango_attr_list_new();
     desc = g_utf8_strdown(pango_font_description_get_family(font->pfd), -1);
@@ -656,10 +672,10 @@ dia_font_build_layout(const char* string, DiaFont* font, real height)
     pango_layout_set_justify(layout,FALSE);
     pango_layout_set_alignment(layout,PANGO_ALIGN_LEFT);
   
-    cached->layout = layout;
+    item->layout = layout;
     g_object_ref(layout);
-    cached->usecount = 1;
-    g_hash_table_insert(layoutcache, cached, cached);
+    item->usecount = 1;
+    g_hash_table_replace(layoutcache, item, item);
 
     return layout;
 }
@@ -680,8 +696,13 @@ dia_font_scaled_string_width(const char* string, DiaFont *font, real height,
 {
     int lw,lh;
     real result;
-    PangoLayout* layout = dia_font_scaled_build_layout(string, font,
-						       height, zoom_factor);
+    PangoLayout* layout;
+
+    if (string == NULL || string[0] == '\0') {
+      return 0.0;
+    }
+
+    layout = dia_font_scaled_build_layout(string, font, height, zoom_factor);
     pango_layout_get_size(layout,&lw,&lh);
     g_object_unref(G_OBJECT(layout));
     
@@ -702,6 +723,10 @@ dia_font_vertical_extents(const char* string, DiaFont* font,
     PangoLayoutIter* iter;
     guint i;
 
+    if (string == NULL || string[0] == '\0') {
+      return FALSE;
+    }
+
     layout = dia_font_scaled_build_layout(string, font,
                                           height, zoom_factor);
     iter = pango_layout_get_iter(layout);
@@ -731,7 +756,7 @@ dia_font_scaled_ascent(const char* string, DiaFont* font, real height,
                       real zoom_factor)
 {
   real top,bline,bottom;
-  if (string[0] == '\0') {
+  if (!string || string[0] == '\0') {
     /* This hack won't work for fonts that don't cover ASCII */
     dia_font_vertical_extents("XjgM149",font,height,zoom_factor,
 			      0,&top,&bline,&bottom);
