@@ -18,19 +18,50 @@
 #ifndef GEOMETRY_H
 #define GEOMETRY_H
 
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
+#include "diatypes.h"
+
 #include <glib.h>
 #include <math.h>
 
-#ifdef _MSC_VER
+/* Solaris 2.4, 2.6, probably 2.5.x, and possibly others prototype
+   finite() in ieeefp.h instead of math.h.  finite() might not be
+   available at all on some HP-UX configurations (in which case,
+   you're on your own). */
+#ifdef HAVE_IEEEFP_H
+#include <ieeefp.h>
+#endif
+#ifndef HAVE_ISINF
+#define isinf(a) (!finite(a))
+#endif
+
+#ifdef _MSC_VER 
+/* #ifdef G_OS_WIN32  apparently _MSC_VER and mingw */
    /* there are some things more in the gcc headers */
 #  include <float.h>
 #  define finite(a) _finite(a)
 #  define isnan(a) _isnan(a)
+#endif
+#ifdef G_OS_WIN32
 #  define M_PI      3.14159265358979323846
 #  define M_SQRT2	1.41421356237309504880	/* sqrt(2) */
 #  define M_SQRT1_2 0.70710678118654752440	/* 1/sqrt(2) */
 #endif
 
+/* gcc -std=c89 doesn't have it either */
+#ifndef M_PI
+#define M_PI G_PI
+#endif
+#ifndef M_SQRT2
+#define M_SQRT2 G_SQRT2
+#endif
+#ifndef M_SQRT1_2
+#define M_SQRT1_2 (1.0/G_SQRT2)
+#endif
+
 /*
   Coordinate system used:
    +---> x
@@ -42,11 +73,6 @@
 typedef double real;
 typedef real coord;
 
-typedef struct _Point Point;
-typedef struct _Rectangle Rectangle;
-typedef struct _IntRectangle IntRectangle;
-typedef struct _BezPoint BezPoint;
-
 struct _Point {
   coord x;
   coord y;
@@ -81,39 +107,39 @@ struct _BezPoint {
 
 /* inline these functions if the platform supports it */
 
-G_INLINE_FUNC void point_add(Point *p1, Point *p2);
+G_INLINE_FUNC void point_add(Point *p1, const Point *p2);
 #ifdef G_CAN_INLINE
 G_INLINE_FUNC void
-point_add(Point *p1, Point *p2)
+point_add(Point *p1, const Point *p2)
 {
   p1->x += p2->x;
   p1->y += p2->y;
 }
 #endif
 
-G_INLINE_FUNC void point_sub(Point *p1, Point *p2);
+G_INLINE_FUNC void point_sub(Point *p1, const Point *p2);
 #ifdef G_CAN_INLINE
 G_INLINE_FUNC void
-point_sub(Point *p1, Point *p2)
+point_sub(Point *p1, const Point *p2)
 {
   p1->x -= p2->x;
   p1->y -= p2->y;
 }
 #endif
 
-G_INLINE_FUNC real point_dot(Point *p1, Point *p2);
+G_INLINE_FUNC real point_dot(const Point *p1, const Point *p2);
 #ifdef G_CAN_INLINE
 G_INLINE_FUNC real
-point_dot(Point *p1, Point *p2)
+point_dot(const Point *p1, const Point *p2)
 {
   return p1->x*p2->x + p1->y*p2->y;
 }
 #endif
 
-G_INLINE_FUNC real point_len(Point *p);
+G_INLINE_FUNC real point_len(const Point *p);
 #ifdef G_CAN_INLINE
 G_INLINE_FUNC real
-point_len(Point *p)
+point_len(const Point *p)
 {
   return sqrt(p->x*p->x + p->y*p->y);
 }
@@ -138,25 +164,36 @@ point_normalize(Point *p)
 
   len = sqrt(p->x*p->x + p->y*p->y);
   
-  p->x /= len;
-  p->y /= len;
+  /* One could call it a bug to try normalizing a vector with
+   * len 0 and the result at least requires definition. But
+   * this is what makes the beziergon bounding box calculation
+   * work. What's the mathematical correct result of 0.0/0.0 ? 
+   */
+  if (len > 0.0) {
+    p->x /= len;
+    p->y /= len;
+  } else {
+    p->x = 0.0;
+    p->y = 0.0;
+  }
+
 }
 #endif
 
-G_INLINE_FUNC void point_rotate(Point *p1, Point *p2);
+G_INLINE_FUNC void point_rotate(Point *p1, const Point *p2);
 #ifdef G_CAN_INLINE
 G_INLINE_FUNC void
-point_rotate(Point *p1, Point *p2)
+point_rotate(Point *p1, const Point *p2)
 {
   p1->x = p1->x*p2->x - p1->y*p2->y;
   p1->y = p1->x*p2->y + p1->y*p2->x;
 }
 #endif
 
-G_INLINE_FUNC void point_get_normed(Point *dst, Point *src);
+G_INLINE_FUNC void point_get_normed(Point *dst, const Point *src);
 #ifdef G_CAN_INLINE
 G_INLINE_FUNC void
-point_get_normed(Point *dst, Point *src)
+point_get_normed(Point *dst, const Point *src)
 {
   real len;
 
@@ -167,22 +204,22 @@ point_get_normed(Point *dst, Point *src)
 }
 #endif
 
-G_INLINE_FUNC void point_get_perp(Point *dst, Point *src);
+G_INLINE_FUNC void point_get_perp(Point *dst, const Point *src);
 #ifdef G_CAN_INLINE
 G_INLINE_FUNC void
-point_get_perp(Point *dst, Point *src)
+point_get_perp(Point *dst, const Point *src)
 {
-  /* dst = the src vector, rotated 90<B0> counter clowkwise. src *must* be 
+  /* dst = the src vector, rotated 90deg counter clowkwise. src *must* be 
      normalized before. */
   dst->y = src->x;
   dst->x = -src->y;
 }
 #endif
 
-G_INLINE_FUNC void point_copy(Point *dst, Point *src);
+G_INLINE_FUNC void point_copy(Point *dst, const Point *src);
 #ifdef G_CAN_INLINE
 G_INLINE_FUNC void 
-point_copy(Point *dst, Point *src)
+point_copy(Point *dst, const Point *src)
 {
   /* Unfortunately, the compiler is not clever enough. And copying using
      ints is faster if we don't computer based on the copied values, but
@@ -194,10 +231,10 @@ point_copy(Point *dst, Point *src)
 }
 #endif
 
-G_INLINE_FUNC void point_add_scaled(Point *dst, Point *src, real alpha);
+G_INLINE_FUNC void point_add_scaled(Point *dst, const Point *src, real alpha);
 #ifdef G_CAN_INLINE
 G_INLINE_FUNC void 
-point_add_scaled(Point *dst, Point *src, real alpha)
+point_add_scaled(Point *dst, const Point *src, real alpha)
 {
   /* especially useful if src is a normed vector... */
   dst->x += alpha * src->x;
@@ -205,11 +242,13 @@ point_add_scaled(Point *dst, Point *src, real alpha)
 }
 #endif
 
-G_INLINE_FUNC void point_copy_add_scaled(Point *dst, Point *src, Point *vct,
+G_INLINE_FUNC void point_copy_add_scaled(Point *dst, const Point *src, 
+                                         const Point *vct,
 					 real alpha);
 #ifdef G_CAN_INLINE
 G_INLINE_FUNC void 
-point_copy_add_scaled(Point *dst, Point *src, Point *vct, real alpha)
+point_copy_add_scaled(Point *dst, const Point *src, 
+                      const Point *vct, real alpha)
 {
   /* especially useful if vct is a normed vector... */
   dst->x = src->x + (alpha * vct->x);
@@ -217,6 +256,29 @@ point_copy_add_scaled(Point *dst, Point *src, Point *vct, real alpha)
 }
 #endif
 
+void point_convex(Point *dst, const Point *src1, const Point *src2, real alpha);
+
+void rectangle_union(Rectangle *r1, const Rectangle *r2);
+void int_rectangle_union(IntRectangle *r1, const IntRectangle *r2);
+void rectangle_intersection(Rectangle *r1, const Rectangle *r2);
+int rectangle_intersects(const Rectangle *r1, const Rectangle *r2);
+int point_in_rectangle(const Rectangle* r, const Point *p);
+int rectangle_in_rectangle(const Rectangle* outer, const Rectangle *inner);
+void rectangle_add_point(Rectangle *r, const Point *p);
+
+G_INLINE_FUNC gboolean rectangle_equals(const Rectangle *old_extents, 
+                                         const Rectangle *new_extents);
+#ifdef G_CAN_INLINE
+G_INLINE_FUNC gboolean 
+rectangle_equals(const Rectangle *r1, const Rectangle *r2)
+{
+  return ( (r2->left == r1->left) &&
+           (r2->right == r1->right) &&
+           (r2->top == r1->top) &&
+           (r2->bottom == r1->bottom) );  
+}
+#endif
+
 G_INLINE_FUNC void check_bb_x( Rectangle *bb, real val, real check );
 #ifdef G_CAN_INLINE
 G_INLINE_FUNC void 
@@ -243,18 +305,20 @@ check_bb_y( Rectangle *bb, real val, real check )
 }
 #endif
 
-void rectangle_union(Rectangle *r1, Rectangle *r2);
-void int_rectangle_union(IntRectangle *r1, IntRectangle *r2);
-void rectangle_intersection(Rectangle *r1, Rectangle *r2);
-int rectangle_intersects(Rectangle *r1, Rectangle *r2);
-int point_in_rectangle(Rectangle* r, Point *p);
-int rectangle_in_rectangle(Rectangle* outer, Rectangle *inner);
-void rectangle_add_point(Rectangle *r, Point *p);
+G_INLINE_FUNC void check_bb_pt( Rectangle *bb, const Point *pt, real check );
+#ifdef G_CAN_INLINE
+G_INLINE_FUNC void 
+check_bb_pt( Rectangle *bb, const Point *pt, real check )
+{
+  /* if pt is outside the bounding box, and check is real, adjust the bb */
+  if (finite(check)) rectangle_add_point(bb,pt);
+}
+#endif
 
-G_INLINE_FUNC real distance_point_point(Point *p1, Point *p2);
+G_INLINE_FUNC real distance_point_point(const Point *p1, const Point *p2);
 #ifdef G_CAN_INLINE
 G_INLINE_FUNC real
-distance_point_point(Point *p1, Point *p2)
+distance_point_point(const Point *p1, const Point *p2)
 {
   real dx = p1->x - p2->x;
   real dy = p1->y - p2->y;
@@ -262,10 +326,11 @@ distance_point_point(Point *p1, Point *p2)
 }
 #endif
 
-G_INLINE_FUNC real distance_point_point_manhattan(Point *p1, Point *p2);
+G_INLINE_FUNC real distance_point_point_manhattan(const Point *p1, 
+                                                  const Point *p2);
 #ifdef G_CAN_INLINE
 G_INLINE_FUNC real
-distance_point_point_manhattan(Point *p1, Point *p2)
+distance_point_point_manhattan(const Point *p1, const Point *p2)
 {
   real dx = p1->x - p2->x;
   real dy = p1->y - p2->y;
@@ -273,23 +338,24 @@ distance_point_point_manhattan(Point *p1, Point *p2)
 }
 #endif
 
-real distance_rectangle_point(Rectangle *rect, Point *point);
-real distance_line_point(Point *line_start, Point *line_end,
-			 real line_width, Point *point);
+real distance_rectangle_point(const Rectangle *rect, const Point *point);
+real distance_line_point(const Point *line_start, const Point *line_end,
+			 real line_width, const Point *point);
 
-real distance_polygon_point(Point *poly, guint npoints,
-			    real line_width, Point *point);
+real distance_polygon_point(const Point *poly, guint npoints,
+			    real line_width, const Point *point);
 
 /* bezier distance calculations */
-real distance_bez_seg_point(Point *b1, Point *b2, Point *b3, Point *b4,
-			    real line_width, Point *point);
-real distance_bez_line_point(BezPoint *b, guint npoints,
-			     real line_width, Point *point);
-real distance_bez_shape_point(BezPoint *b, guint npoints,
-			      real line_width, Point *point);
+real distance_bez_seg_point(const Point *b1, const Point *b2, 
+                            const Point *b3, const Point *b4,
+			    real line_width, const Point *point);
+real distance_bez_line_point(const BezPoint *b, guint npoints,
+			     real line_width, const Point *point);
+real distance_bez_shape_point(const BezPoint *b, guint npoints,
+			      real line_width, const Point *point);
 
-real distance_ellipse_point(Point *centre, real width, real height,
-			    real line_width, Point *point);
+real distance_ellipse_point(const Point *centre, real width, real height,
+			    real line_width, const Point *point);
 
 typedef real  Vector[3];
 typedef Vector  Matrix[3];
@@ -303,4 +369,12 @@ void          rotate_matrix               (Matrix, real);
 void          xshear_matrix               (Matrix, real);
 void          yshear_matrix               (Matrix, real);
 
+real dot2(Point *p1, Point *p2);
+void line_coef(real *a, real *b, real *c, Point *p1, Point *p2);
+real line_to_point(real a, real b , real c, Point *p);
+void point_perp(Point *p, real a, real b, real c, Point *perp);
+void fillet(Point *p1, Point *p2, Point *p3, Point *p4,
+	    real r, Point *c, real *pa, real *aa);
+real point_cross(Point *p1, Point *p2);
+
 #endif /* GEOMETRY_H */
