@@ -18,11 +18,10 @@
 #ifndef CHANGE_H
 #define CHANGE_H
 
-typedef struct _ObjectState ObjectState;
-typedef struct _ObjectChange ObjectChange;
+#include "diatypes.h"
 
-typedef void (*ObjectChangeApplyFunc)(ObjectChange *change, Object *obj);
-typedef void (*ObjectChangeRevertFunc)(ObjectChange *change, Object *obj);
+typedef void (*ObjectChangeApplyFunc)(ObjectChange *change, DiaObject *obj);
+typedef void (*ObjectChangeRevertFunc)(ObjectChange *change, DiaObject *obj);
 typedef void (*ObjectChangeFreeFunc)(ObjectChange *change);
 
 struct _ObjectChange {
@@ -50,7 +49,7 @@ struct _ObjectState {
   
   The calling function owns the returned reference.
 */
-typedef ObjectState * (*GetStateFunc) (Object* obj);
+typedef ObjectState * (*GetStateFunc) (DiaObject* obj);
 
 /*
   Sets the internal state from the object.
@@ -60,13 +59,13 @@ typedef ObjectState * (*GetStateFunc) (Object* obj);
   The called function owns the reference and is
   responsible for freeing it.
 */
-typedef void (*SetStateFunc) (Object* obj, ObjectState *state);
+typedef void (*SetStateFunc) (DiaObject* obj, ObjectState *state);
 
 
-extern ObjectChange *new_object_state_change(Object *obj,
-					     ObjectState *old_state,
-					     GetStateFunc get_state,
-					     SetStateFunc set_state );
+ObjectChange *new_object_state_change(DiaObject *obj,
+				      ObjectState *old_state,
+				      GetStateFunc get_state,
+				      SetStateFunc set_state );
 
 #endif /* CHANGE_H */
 
