@@ -29,6 +29,7 @@
 #include "text.h"
 #include "message.h"
 #include "diarenderer.h"
+#include "diagramdata.h"
 
 static int text_key_event(Focus *focus, guint keysym,
 			  const gchar *str, int strlen,
@@ -274,8 +275,9 @@ text_set_height(Text *text, real height)
 void
 text_set_font(Text *text, DiaFont *font)
 {
-  dia_font_unref(text->font);
-  text->font = dia_font_ref(text->font);
+  DiaFont *old_font = text->font;
+  text->font = dia_font_ref(font);
+  dia_font_unref(old_font);
   
   calc_width(text);
   calc_ascent_descent(text);
@@ -460,8 +462,31 @@ text_draw(Text *text, DiaRenderer *renderer)
   }
 }
 
+/** Register that an object has a text that can be edited.
+ *  This text will be inserted in the chain traversed by tabbing.
+ *  The registration is temporary, intended to be done when the 
+ *  object is selected.
+ */
 void
-text_grab_focus(Text *text, Object *object)
+text_register_editable(Text *text, DiaObject *obj)
+{
+  DiagramData *dia;
+
+  text->parent_object = obj;
+  dia = text->parent_object->parent_layer->parent_diagram;
+  dia->text_edits = g_list_append(dia->text_edits, text);
+}
+
+/** Reset the list of editable texts. */
+void
+text_reset_editable(DiagramData *dia)
+{
+  g_list_free(dia->text_edits);
+  dia->text_edits = NULL;
+}
+
+void
+text_grab_focus(Text *text, DiaObject *object)
 {
   text->focus.obj = object;
   request_focus(&text->focus);
@@ -616,7 +641,7 @@ text_delete_forward(Text *text)
   len = strlen (text->line[row]);
   memmove (start, end, text->line[row] + len - start);
 
-  text->strlen[row] = g_utf8_strlen(text->line[row], -1);//text->strlen[row]--;
+  text->strlen[row] = g_utf8_strlen(text->line[row], -1);/*text->strlen[row]--;*/
   
   if (text->cursor_pos > text->strlen[text->cursor_row])
     text->cursor_pos = text->strlen[text->cursor_row];
@@ -764,7 +789,7 @@ text_insert_char(Text *text, gunichar c)
   strncpy (str, ch, unilen);
   line[length + unilen] = 0; /* null terminate */
   text->cursor_pos += 1;
-  text->strlen[row] = g_utf8_strlen(text->line[row], -1);// length + unilen;
+  text->strlen[row] = g_utf8_strlen(text->line[row], -1);/* length + unilen; */
 
   text->row_width[row] =
       dia_font_string_width(text->line[row], text->font, text->height);
@@ -992,7 +1017,10 @@ data_text(AttributeNode text_attr)
 void
 text_get_attributes(Text *text, TextAttributes *attr)
 {    
-  attr->font = text->font;
+  DiaFont *old_font;
+  old_font = attr->font;
+  attr->font = dia_font_ref(text->font);
+  if (old_font != NULL) dia_font_unref(old_font);
   attr->height = text->height;
   attr->position = text->position;
   attr->color = text->color;
@@ -1003,8 +1031,9 @@ void
 text_set_attributes(Text *text, TextAttributes *attr)
 {
   if (text->font != attr->font) {
-      dia_font_unref(text->font);
-      text->font = dia_font_ref(attr->font);
+    DiaFont *old_font = text->font;
+    text->font = dia_font_ref(attr->font);
+    dia_font_unref(old_font);
   }
   text->height = attr->height;
   text->position = attr->position;
@@ -1013,7 +1042,7 @@ text_set_attributes(Text *text, TextAttributes *attr)
 }
 
 static void
-text_change_apply(struct TextObjectChange *change, Object *obj)
+text_change_apply(struct TextObjectChange *change, DiaObject *obj)
 {
   Text *text = change->text;
   switch (change->type) {
@@ -1049,7 +1078,7 @@ text_change_apply(struct TextObjectChange *change, Object *obj)
 }
 
 static void
-text_change_revert(struct TextObjectChange *change, Object *obj)
+text_change_revert(struct TextObjectChange *change, DiaObject *obj)
 {
   Text *text = change->text;
   switch (change->type) {
@@ -1097,7 +1126,7 @@ text_create_change(Text *text, enum change_type type,
 {
   struct TextObjectChange *change;
 
-  change = g_new(struct TextObjectChange, 1);
+  change = g_new0(struct TextObjectChange, 1);
 
   change->obj_change.apply = (ObjectChangeApplyFunc) text_change_apply;
   change->obj_change.revert = (ObjectChangeRevertFunc) text_change_revert;
