@@ -18,23 +18,26 @@
 #ifndef TEXT_H
 #define TEXT_H
 
-typedef struct _Text Text;
-typedef struct _TextAttributes TextAttributes;
-
-#include "font.h"
+#include <glib.h>
+#include "diatypes.h"
+#include "textattr.h"
 #include "focus.h"
+#include "properties.h"
+
 
 struct _Text {
+  /** The object that owns this text. */
+  DiaObject *parent_object;
   /* don't change these values directly, use the text_set* functions */
   
   /* Text data: */
-  char **line;
+  gchar **line;
   int numlines;
-  int *strlen;
-  int *alloclen;
+  int *strlen;  /* in characters */
+  int *alloclen; /* in bytes */
 
   /* Attributes: */
-  Font *font;
+  DiaFont *font;
   real height;
   Point position;
   Color color;
@@ -46,40 +49,32 @@ struct _Text {
   Focus focus;
   
   /* Computed values:  */
-  real ascent;
-  real descent;
+  real ascent; /* **average** ascent */
+  real descent; /* **average** descent */
   real max_width;
   real *row_width;
 };
 
 
-struct _TextAttributes {
-  Font *font;
-  real height;
-  Point position;
-  Color color;
-  Alignment alignment;
-};
-
 /* makes an internal copy of the string */
-Text *new_text(const char *string, Font *font, real height,
+Text *new_text(const char *string, DiaFont *font, real height,
 	       Point *pos, Color *color, Alignment align);
 void text_destroy(Text *text);
 Text *text_copy(Text *text);
 char *text_get_string_copy(Text *text);
 void text_set_string(Text *text, const char *string);
 void text_set_height(Text *text, real height);
-void text_set_font(Text *text, Font *font);
+void text_set_font(Text *text, DiaFont *font);
 void text_set_position(Text *text, Point *pos);
 void text_set_color(Text *text, Color *col);
 void text_set_alignment(Text *text, Alignment align);
 real text_distance_from(Text *text, Point *point);
 void text_calc_boundingbox(Text *text, Rectangle *box);
-void text_draw(Text *text, Renderer *renderer);
+void text_draw(Text *text, DiaRenderer *renderer);
 void text_set_cursor(Text *text, Point *clicked_point,
-		     Renderer *interactive_renderer);
+		     DiaRenderer *interactive_renderer);
 void text_set_cursor_at_end( Text* text );
-void text_grab_focus(Text *text, Object *object);
+void text_grab_focus(Text *text, DiaObject *object);
 int text_is_empty(Text *text);
 int text_delete_all(Text *text, ObjectChange **change);
 void text_get_attributes(Text *text, TextAttributes *attr);
@@ -87,6 +82,13 @@ void text_set_attributes(Text *text, TextAttributes *attr);
 
 void data_add_text(AttributeNode attr, Text *text);
 Text *data_text(AttributeNode attr);
+
+gboolean apply_textattr_properties(GPtrArray *props,
+                                   Text *text, const gchar *textname,
+                                   TextAttributes *attrs);
+gboolean apply_textstr_properties(GPtrArray *props,
+                                  Text *text, const gchar *textname,
+                                  const gchar *str);
 #endif /* TEXT_H */
 
 
