@@ -26,6 +26,7 @@
 #include "arrows.h"
 #include "diaarrowchooser.h"
 #include "dialinechooser.h"
+#include "persistence.h"
 
 #include <stdlib.h>
 #include <glib.h>
@@ -196,10 +197,10 @@ dia_size_selector_init (DiaSizeSelector *ss)
 		   G_CALLBACK(dia_size_selector_destroy_callback), NULL);
 }
 
-guint
+GtkType
 dia_size_selector_get_type (void)
 {
-  static guint dss_type = 0;
+  static GtkType dss_type = 0;
 
   if (!dss_type) {
     GtkTypeInfo dss_info = {
@@ -224,6 +225,7 @@ GtkWidget *
 dia_size_selector_new (real width, real height)
 {
   GtkWidget *wid;
+
   wid = GTK_WIDGET ( gtk_type_new (dia_size_selector_get_type ()));
   dia_size_selector_set_size(DIA_SIZE_SELECTOR(wid), width, height);
   return wid;
@@ -304,7 +306,6 @@ typedef struct {
 
 /* Hash table from font name to FontSelectorEntry */
 static GHashTable *font_hash_table = NULL;
-static GList *menu_entry_list = NULL;
 
 static void dia_font_selector_dialog_callback(GtkWidget *widget, int id, gpointer data);
 static void dia_font_selector_menu_callback(GtkWidget *button, gpointer data);
@@ -321,11 +322,8 @@ dia_font_selector_add_font(const char *lowername, const gchar *fontname,
   fse->name = fontname;
   fse->family = NULL;
   fse->last_select = time(0);
-  fse->entry_nr = g_list_length(menu_entry_list)+4; /* Skip first entries */
   g_hash_table_insert(font_hash_table, g_strdup(lowername), fse);
-  if (is_other_font) {
-    menu_entry_list = g_list_append(menu_entry_list, g_strdup(fontname));
-  } else {
+  if (!is_other_font) {
     if (!g_strcasecmp(fontname, "sans")) fse->entry_nr = 0;
     if (!g_strcasecmp(fontname, "serif")) fse->entry_nr = 1;
     if (!g_strcasecmp(fontname, "monospace")) fse->entry_nr = 2;
@@ -339,55 +337,6 @@ gboolean strcase_equal(gconstpointer s1, gconstpointer s2)
   return !(g_strcasecmp((char *)s1, (char *)s2));
 }
 
-static void
-dia_font_selector_read_persistence_file() {
-  gchar *file_contents;
-  gchar *persistence_name;
-  GError *error = NULL;
-
-  font_hash_table = g_hash_table_new(g_str_hash, strcase_equal);
-
-  dia_font_selector_add_font("sans", "Sans", FALSE);
-  dia_font_selector_add_font("serif", "Serif", FALSE);
-  dia_font_selector_add_font("monospace", "Monospace", FALSE);
-
-  persistence_name = dia_config_filename("font_menu");
-  
-  if (g_file_test(persistence_name, G_FILE_TEST_EXISTS) &&
-      g_file_get_contents(persistence_name, &file_contents, NULL, &error)) {
-    /* Should really use some macro for linebreak, but I can't find it */
-    gchar **lines = g_strsplit(file_contents, "\n", -1); 
-    int i;
-    for (i = 0; lines[i] != NULL; i++) {
-      gchar *lowername;
-      if (strlen(lines[i]) == 0) continue;
-      lowername = g_utf8_strdown(lines[i], -1);
-      dia_font_selector_add_font(lowername, lines[i], TRUE);
-      g_free(lowername);
-    }
-    g_free(file_contents);
-  }
-  if (error) g_error_free(error);
-  g_free(persistence_name);
-}
-
-static void
-dia_font_selector_write_persistence_file() {
-  gchar *persistence_name;
-  FILE *pfile;
-
-  persistence_name = dia_config_filename("font_menu");
-  if ((pfile = fopen(persistence_name, "wb")) != NULL) {
-    GList *entry;
-    for (entry = menu_entry_list; entry != NULL; entry = entry->next) {
-      fputs((gchar *)entry->data, pfile);
-      fputs("\n", pfile);
-    }
-    fclose(pfile);
-  }
-  g_free(persistence_name);
-}
-
 static GtkWidget *
 dia_font_selector_add_menu_item(gchar *label, GSList **group, GtkWidget *menu) 
 {
@@ -407,6 +356,7 @@ dia_font_selector_build_font_menu(DiaFontSelector *fs) {
   GtkWidget *menuitem;
   GSList *group;
   GList *entry;
+  int i;
 
   if (fs->font_omenu != NULL) {
     gtk_option_menu_remove_menu(fs->font_omenu);
@@ -428,8 +378,15 @@ dia_font_selector_build_font_menu(DiaFontSelector *fs) {
   gtk_menu_shell_append (GTK_MENU_SHELL (menu), menuitem);
   gtk_widget_show (menuitem);
 
-  for (entry = menu_entry_list; entry != NULL; entry = entry->next) {
+  for (i = 4, entry = persistent_list_get_glist("font-menu");
+       entry != NULL; entry = entry->next, i++) {
+    gchar *fontname = (gchar*)entry->data;
+    gchar *lowername = g_utf8_strdown(fontname, -1);
+    FontSelectorEntry *fse = 
+      (FontSelectorEntry*)g_hash_table_lookup(font_hash_table, lowername);
+    fse->entry_nr = i;
     menuitem = dia_font_selector_add_menu_item((gchar *)entry->data, &group, menu);
+    g_free(lowername);
   }
   menuitem = gtk_separator_menu_item_new();
   gtk_menu_shell_append (GTK_MENU_SHELL (menu), menuitem);
@@ -456,8 +413,8 @@ dia_font_selector_get_new_font(DiaFontSelector *fs, const gchar *fontname)
     (FontSelectorEntry*)g_hash_table_lookup(font_hash_table, lowername);
   if (fse == NULL) {
     fse = dia_font_selector_add_font(lowername, fontname, TRUE);
+    persistent_list_add("font-menu", fontname);
     dia_font_selector_build_font_menu(fs);
-    dia_font_selector_write_persistence_file();
   }
   g_free(lowername);
   return fse;
@@ -477,6 +434,29 @@ dia_font_selector_init (DiaFontSelector *fs)
   GtkWidget *menu;
   GtkWidget *omenu;
 
+  persistence_register_list("font-menu");
+
+  if (font_hash_table == NULL) {
+    GList *other_fonts;
+    font_hash_table = g_hash_table_new(g_str_hash, strcase_equal);
+
+    dia_font_selector_add_font("sans", "Sans", FALSE);
+    dia_font_selector_add_font("serif", "Serif", FALSE);
+    dia_font_selector_add_font("monospace", "Monospace", FALSE);
+
+    other_fonts = persistent_list_get_glist("font-menu");
+
+    for (other_fonts = g_list_last(other_fonts);
+	 other_fonts != NULL; other_fonts = g_list_previous(other_fonts)) {
+      
+      const gchar *name = other_fonts->data;
+      gchar *lowername = g_ascii_strdown (name, -1);
+      dia_font_selector_add_font(lowername, name, TRUE);
+
+      g_free(lowername);
+    }
+  }
+
   dia_font_selector_build_font_menu(fs);
   
   /* Now build the style menu button */
@@ -512,8 +492,6 @@ dia_font_selector_get_type        (void)
     };
     
     dfs_type = gtk_type_unique (gtk_hbox_get_type (), &dfs_info);
-
-    dia_font_selector_read_persistence_file();
   }
   
   return dfs_type;
@@ -535,10 +513,14 @@ dia_font_selector_get_family_from_name(GtkWidget *widget, const gchar *fontname)
 			       &families, &n_families);
   /* Doing it the slow way until I find a better way */
   for (i = 0; i < n_families; i++) {
-    if (!(g_strcasecmp(pango_font_family_get_name(families[i]), fontname)))
-      return families[i];
+    if (!(g_strcasecmp(pango_font_family_get_name(families[i]), fontname))) {
+      PangoFontFamily *fam = families[i];
+      g_free(families);
+      return fam;
+    }
   }
   g_warning(_("Couldn't find font family for %s\n"), fontname);
+  g_free(families);
   return NULL;
 }
 
@@ -653,9 +635,9 @@ dia_font_selector_set_styles(DiaFontSelector *fs, FontSelectorEntry *fse,
                              DiaFontStyle dia_style)
 {
   int i=0, select = 0;
-  PangoFontFace **faces;
-  int nfaces;
-  GtkWidget *menu = gtk_menu_new();
+  PangoFontFace **faces = NULL;
+  int nfaces = 0;
+  GtkWidget *menu = NULL;
   long stylebits = 0;
   int menu_item_nr = 0;
   GSList *group = NULL;
@@ -668,6 +650,7 @@ dia_font_selector_set_styles(DiaFontSelector *fs, FontSelectorEntry *fse,
       return;
   }
 
+  menu = gtk_menu_new ();
   pango_font_family_list_faces(fse->family, &faces, &nfaces);
 
   for (i = 0; i < nfaces; i++) {
@@ -689,6 +672,8 @@ dia_font_selector_set_styles(DiaFontSelector *fs, FontSelectorEntry *fse,
     pango_font_description_free(pfd);
   }
 
+  g_free(faces);
+
   for (i = DIA_FONT_NORMAL; i <= (DIA_FONT_HEAVY | DIA_FONT_ITALIC); i+=4) {
     GtkWidget *menuitem;
     /**
@@ -733,20 +718,20 @@ dia_font_selector_set_preview(DiaFontSelector *fs, gchar *text) {
 void
 dia_font_selector_set_font(DiaFontSelector *fs, DiaFont *font)
 {
-  int font_nr;
+  int font_nr = 0;
   FontSelectorEntry *fse;
-  const gchar *fontname = dia_font_get_family(font);
+  gchar *fontname = g_utf8_strdown(dia_font_get_family(font), -1);
 
   fse = dia_font_selector_get_new_font(fs, fontname);
   fse->last_select = time(0);
   dia_font_selector_set_styles(fs, fse, dia_font_get_style(font));
 
   font_nr = fse->entry_nr;
-
   gtk_option_menu_set_history(GTK_OPTION_MENU(fs->font_omenu), font_nr);
-  gtk_menu_set_active(fs->font_menu, font_nr);
+       /*  gtk_menu_set_active(fs->font_menu, font_nr);*/
   gtk_check_menu_item_set_active(GTK_CHECK_MENU_ITEM(gtk_menu_get_active(fs->font_menu)), TRUE);
   fs->old_font = font_nr;
+  g_free(fontname);
 }
 
 DiaFont *
@@ -1304,7 +1289,7 @@ set_size_sensitivity(DiaArrowSelector *as)
 static void
 arrow_type_change_callback(GtkObject *as, gboolean arg1, gpointer data)
 {
-  set_size_sensitivity(DIAARROWSELECTOR(as));
+  set_size_sensitivity(DIA_ARROW_SELECTOR(as));
 }
 
 /* This is actually quite general, but only used here */
@@ -1336,7 +1321,8 @@ static void dia_arrow_fill_menu(GtkMenu *menu, GSList **group,
 
 
 static void
-dia_arrow_selector_init (DiaArrowSelector *as)
+dia_arrow_selector_init (DiaArrowSelector *as,
+			 gpointer g_class)
 {
   GtkWidget *omenu;
   GtkWidget *menu;
@@ -1383,24 +1369,34 @@ dia_arrow_selector_init (DiaArrowSelector *as)
 
 }
 
-GtkType
+GType
 dia_arrow_selector_get_type        (void)
 {
-  static GtkType dfs_type = 0;
+  static GType dfs_type = 0;
 
   if (!dfs_type) {
-    GtkTypeInfo dfs_info = {
-      "DiaArrowSelector",
-      sizeof (DiaArrowSelector),
+    static const GTypeInfo dfs_info = {
+      /*      sizeof (DiaArrowSelector),*/
       sizeof (DiaArrowSelectorClass),
-      (GtkClassInitFunc) dia_arrow_selector_class_init,
+      (GBaseInitFunc) NULL,
+      (GBaseFinalizeFunc) NULL,
+      (GClassInitFunc) dia_arrow_selector_class_init,
+      NULL, /* class_finalize */
+      NULL, /* class_data */
+      sizeof (DiaArrowSelector),
+      0,    /* n_preallocs */
+      (GInstanceInitFunc)dia_arrow_selector_init,  /* init */
+      /*
       (GtkObjectInitFunc) dia_arrow_selector_init,
       NULL,
       NULL,
       (GtkClassInitFunc) NULL,
+      */
     };
     
-    dfs_type = gtk_type_unique (gtk_vbox_get_type (), &dfs_info);
+    dfs_type = g_type_register_static (GTK_TYPE_VBOX,
+				       "DiaArrowSelector",
+				       &dfs_info, 0);
   }
   
   return dfs_type;
@@ -1409,7 +1405,7 @@ dia_arrow_selector_get_type        (void)
 GtkWidget *
 dia_arrow_selector_new ()
 {
-  return GTK_WIDGET ( gtk_type_new (dia_arrow_selector_get_type ()));
+  return GTK_WIDGET ( g_object_new (DIA_TYPE_ARROW_SELECTOR, NULL));
 }
 
 
@@ -1429,16 +1425,8 @@ void
 dia_arrow_selector_set_arrow (DiaArrowSelector *as,
 			      Arrow arrow)
 {
-  int arrow_type_index = 0,i = 0;
-  const struct menudesc *md = arrow_types;
-  
-  while (md->name) {
-    if (md->enum_value == arrow.type) {
-      arrow_type_index = i;
-      break;
-    }
-    md++; i++;
-  }
+  int arrow_type_index = arrow_index_from_type(arrow.type);
+
   gtk_menu_set_active(GTK_MENU (as->arrow_type_menu), arrow_type_index);
   gtk_option_menu_set_history (GTK_OPTION_MENU(as->omenu), arrow_type_index);
 /* TODO: restore CheckMenu version of menu */
@@ -1621,3 +1609,87 @@ dia_get_image_from_file(gchar *filename)
   g_free(datadir);
   return image;
 }
+
+struct image_pair { GtkWidget *on; GtkWidget *off; };
+
+static void
+dia_toggle_button_swap_images(GtkToggleButton *widget,
+			      gpointer data) 
+{
+  struct image_pair *images = (struct image_pair *)data;
+  if (gtk_toggle_button_get_active(widget)) {
+    gtk_container_remove(GTK_CONTAINER(widget), 
+			 gtk_bin_get_child(GTK_BIN(widget)));
+    gtk_container_add(GTK_CONTAINER(widget),
+		      images->on);
+    
+  } else {
+    gtk_container_remove(GTK_CONTAINER(widget), 
+			 gtk_bin_get_child(GTK_BIN(widget)));
+    gtk_container_add(GTK_CONTAINER(widget),
+		      images->off);
+  }
+}
+
+static void
+dia_toggle_button_destroy(GtkWidget *widget, gpointer data)
+{
+  struct image_pair *images = (struct image_pair *)data;
+  g_object_unref(images->on);
+  g_object_unref(images->off);
+  g_free(images);
+}
+
+/** Create a toggle button with two images switching (on and off) */
+GtkWidget *
+dia_toggle_button_new_with_images(gchar *on_image, gchar *off_image)
+{
+  GtkWidget *button = gtk_toggle_button_new();
+  struct image_pair *images = g_new0(struct image_pair, 1);
+  GtkRcStyle *rcstyle;
+  GtkStyle *style;
+  GValue *prop;
+  gint i;
+
+  /* Since these may not be added at any point, make sure to
+   * sink them. */
+  images->on = dia_get_image_from_file(on_image);
+  g_object_ref(G_OBJECT(images->on));
+  gtk_object_sink(GTK_OBJECT(images->on));
+  gtk_widget_show(images->on);
+
+  images->off = dia_get_image_from_file(off_image);
+  g_object_ref(G_OBJECT(images->off));
+  gtk_object_sink(GTK_OBJECT(images->off));
+  gtk_widget_show(images->off);
+
+  /* Make border as small as possible */
+  gtk_misc_set_padding(GTK_MISC(images->on), 0, 0);
+  gtk_misc_set_padding(GTK_MISC(images->off), 0, 0);
+  GTK_WIDGET_UNSET_FLAGS(GTK_WIDGET(button), GTK_CAN_FOCUS);
+  GTK_WIDGET_UNSET_FLAGS(GTK_WIDGET(button), GTK_CAN_DEFAULT);
+
+  rcstyle = gtk_rc_style_new ();  
+  rcstyle->xthickness = rcstyle->ythickness = 0;       
+  gtk_widget_modify_style (button, rcstyle);
+  gtk_rc_style_unref (rcstyle);
+
+  prop = g_new0(GValue, 1);
+  g_value_init(prop, G_TYPE_INT);
+  gtk_widget_style_get_property(GTK_WIDGET(button), "focus-padding", prop);
+  i = g_value_get_int(prop);
+  g_value_set_int(prop, 0);
+
+  gtk_button_set_relief(GTK_BUTTON(button), GTK_RELIEF_NONE);
+  /*  gtk_button_set_focus_on_click(GTK_BUTTON(button), FALSE);*/
+  gtk_container_set_border_width(GTK_CONTAINER(button), 0);
+
+  gtk_container_add(GTK_CONTAINER(button), images->off);
+
+  g_signal_connect(G_OBJECT(button), "toggled", 
+		   dia_toggle_button_swap_images, images);
+  g_signal_connect(G_OBJECT(button), "destroy",
+		   dia_toggle_button_destroy, images);
+
+  return button;
+}
