@@ -25,7 +25,6 @@
 #endif
 
 #include <assert.h>
-#include <gtk/gtk.h>
 #include <math.h>
 #include <string.h>
 #include <stdio.h>
@@ -34,7 +33,7 @@
 #include "object.h"
 #include "objchange.h"
 #include "connection.h"
-#include "render.h"
+#include "diarenderer.h"
 #include "handle.h"
 #include "arrows.h"
 #include "diamenu.h"
@@ -47,7 +46,6 @@
 
 
 typedef struct _Orthflow Orthflow;
-typedef struct _OrthflowDialog OrthflowDialog;
 typedef struct _OrthflowChange OrthflowChange;
 
 typedef enum {
@@ -62,17 +60,9 @@ struct _Orthflow {
   Handle text_handle;
 
   Text* text;
+  TextAttributes attrs;
   OrthflowType type;
-};
-
-struct _OrthflowDialog {
-  GtkWidget *dialog;
-  
-  GtkWidget *text;
-
-  GtkWidget *m_energy;
-  GtkWidget *m_material;
-  GtkWidget *m_signal;
+  Point textpos; /* This is the master position, only overridden in load */
 };
 
 enum OrthflowChangeType {
@@ -101,40 +91,32 @@ Color orthflow_color_signal   = { 0.0f, 0.0f, 1.0f };
 #define ORTHFLOW_ARROWWIDTH 0.5
 #define HANDLE_MOVE_TEXT (HANDLE_CUSTOM2)
 
-static Font *orthflow_font = NULL;
+static DiaFont *orthflow_font = NULL;
 
-static OrthflowDialog *defaults_dialog;
-
-/* Remember the most recently applied orthflow type and use it to
-   set the type for any newly created orthflows
- */
-static OrthflowType orthflow_most_recent_type = ORTHFLOW_ENERGY ;
-static Text* orthflow_default_label = 0 ;
-
-static void orthflow_move_handle(Orthflow *orthflow, Handle *handle,
-				 Point *to, HandleMoveReason reason);
-static void orthflow_move(Orthflow *orthflow, Point *to);
+static ObjectChange* orthflow_move_handle(Orthflow *orthflow, Handle *handle,
+					  Point *to, ConnectionPoint *cp,
+					  HandleMoveReason reason, 
+					  ModifierKeys modifiers);
+static ObjectChange* orthflow_move(Orthflow *orthflow, Point *to);
 static void orthflow_select(Orthflow *orthflow, Point *clicked_point,
-			    Renderer *interactive_renderer);
-static void orthflow_draw(Orthflow *orthflow, Renderer *renderer);
-static Object *orthflow_create(Point *startpoint,
+			    DiaRenderer *interactive_renderer);
+static void orthflow_draw(Orthflow *orthflow, DiaRenderer *renderer);
+static DiaObject *orthflow_create(Point *startpoint,
 			       void *user_data,
 			       Handle **handle1,
 			       Handle **handle2);
 static real orthflow_distance_from(Orthflow *orthflow, Point *point);
 static void orthflow_update_data(Orthflow *orthflow);
 static void orthflow_destroy(Orthflow *orthflow);
-static Object *orthflow_copy(Orthflow *orthflow);
-static GtkWidget *orthflow_get_defaults(void);
-static void orthflow_apply_defaults(void);
+static DiaObject *orthflow_copy(Orthflow *orthflow);
 static PropDescription *orthflow_describe_props(Orthflow *mes);
 static void
-orthflow_get_props(Orthflow * orthflow, Property *props, guint nprops);
+orthflow_get_props(Orthflow * orthflow, GPtrArray *props);
 static void
-orthflow_set_props(Orthflow * orthflow, Property *props, guint nprops);
+orthflow_set_props(Orthflow * orthflow, GPtrArray *props);
 static void orthflow_save(Orthflow *orthflow, ObjectNode obj_node,
 			  const char *filename);
-static Object *orthflow_load(ObjectNode obj_node, int version,
+static DiaObject *orthflow_load(ObjectNode obj_node, int version,
 			     const char *filename);
 static DiaMenu *orthflow_get_object_menu(Orthflow *orthflow, Point *clickedpoint) ;
 
@@ -144,12 +126,12 @@ static ObjectTypeOps orthflow_type_ops =
   (CreateFunc)		orthflow_create,
   (LoadFunc)		orthflow_load,
   (SaveFunc)		orthflow_save,
-  (GetDefaultsFunc)	orthflow_get_defaults,
-  (ApplyDefaultsFunc)	orthflow_apply_defaults
+  (GetDefaultsFunc)	NULL,
+  (ApplyDefaultsFunc)	NULL,
   
 } ;
 
-ObjectType orthflow_type =
+DiaObjectType orthflow_type =
 {
   "FS - Orthflow",		/* name */
   0,				/* version */
@@ -184,6 +166,12 @@ static PropDescription orthflow_props[] = {
   ELEMENT_COMMON_PROPERTIES,
   { "type", PROP_TYPE_ENUM, PROP_FLAG_VISIBLE,
     N_("Type:"), NULL, prop_orthflow_type_data },
+  { "text", PROP_TYPE_TEXT, 0, NULL, NULL },
+  PROP_STD_TEXT_ALIGNMENT,
+  PROP_STD_TEXT_FONT,
+  PROP_STD_TEXT_HEIGHT,
+  /* Colour determined from type, don't show */
+  { "text_colour", PROP_TYPE_COLOUR, PROP_FLAG_DONT_SAVE, },
   PROP_DESC_END
 };
 
@@ -199,28 +187,35 @@ orthflow_describe_props(Orthflow *mes)
 static PropOffset orthflow_offsets[] = {
   OBJECT_COMMON_PROPERTIES_OFFSETS,
   { "type", PROP_TYPE_ENUM, offsetof(Orthflow, type) },
+  { "text", PROP_TYPE_TEXT, offsetof (Orthflow, text) },
+  { "text_alignment", PROP_TYPE_REAL, offsetof (Orthflow, attrs.alignment) },
+  { "text_font", PROP_TYPE_FONT, offsetof (Orthflow, attrs.font) },
+  { "text_height", PROP_TYPE_REAL, offsetof (Orthflow, attrs.height) },
+  { "text_colour", PROP_TYPE_COLOUR, offsetof (Orthflow, attrs.color) },
   { NULL, 0, 0 }
 };
 
 static void
-orthflow_get_props(Orthflow * orthflow, Property *props, guint nprops)
+orthflow_get_props(Orthflow * orthflow, GPtrArray *props)
 {
+  text_get_attributes (orthflow->text, &orthflow->attrs);
   object_get_props_from_offsets(&orthflow->orth.object, 
-                                orthflow_offsets, props, nprops);
+                                orthflow_offsets, props);
 }
 
 static void
-orthflow_set_props(Orthflow *orthflow, Property *props, guint nprops)
+orthflow_set_props(Orthflow *orthflow, GPtrArray *props)
 {
   object_set_props_from_offsets(&orthflow->orth.object, 
-                                orthflow_offsets, props, nprops);
+                                orthflow_offsets, props);
+  apply_textattr_properties (props, orthflow->text, "text", &orthflow->attrs);
   orthflow_update_data(orthflow);
 }
 
 
 
 static void
-orthflow_change_apply_revert(ObjectChange* objchg, Object* obj)
+orthflow_change_apply_revert(ObjectChange* objchg, DiaObject* obj)
 {
   struct _OrthflowChange* change = (struct _OrthflowChange*) objchg ;
   Orthflow* oflow = (Orthflow*) obj ;
@@ -286,7 +281,7 @@ orthflow_distance_from(Orthflow *orthflow, Point *point)
 
 static void
 orthflow_select(Orthflow *orthflow, Point *clicked_point,
-		Renderer *interactive_renderer)
+		DiaRenderer *interactive_renderer)
 {
   text_set_cursor(orthflow->text, clicked_point, interactive_renderer);
   text_grab_focus(orthflow->text, &orthflow->orth.object);
@@ -294,97 +289,118 @@ orthflow_select(Orthflow *orthflow, Point *clicked_point,
   orthconn_update_data(&orthflow->orth);
 }
 
-static void
+static ObjectChange*
 orthflow_move_handle(Orthflow *orthflow, Handle *handle,
-		     Point *to, HandleMoveReason reason)
+		     Point *to, ConnectionPoint *cp,
+		     HandleMoveReason reason, ModifierKeys modifiers)
 {
+  ObjectChange *change = NULL;
   assert(orthflow!=NULL);
   assert(handle!=NULL);
   assert(to!=NULL);
 
   if (handle->id == HANDLE_MOVE_TEXT) {
-    orthflow->text->position = *to;
+    orthflow->textpos = *to;
   } else {
     Point along ;
 
-    along = orthflow->text->position ;
+    along = orthflow->textpos ;
     point_sub( &along, &(orthconn_get_middle_handle(&orthflow->orth)->pos) ) ;
 
-    orthconn_move_handle( &orthflow->orth, handle, to, reason );
+    change = orthconn_move_handle( &orthflow->orth, handle, to, cp, 
+				   reason, modifiers);
     orthconn_update_data( &orthflow->orth ) ;
 
-    orthflow->text->position = orthconn_get_middle_handle(&orthflow->orth)->pos ;
-    point_add( &orthflow->text->position, &along ) ;
+    orthflow->textpos = orthconn_get_middle_handle(&orthflow->orth)->pos ;
+    point_add( &orthflow->textpos, &along ) ;
   }
 
   orthflow_update_data(orthflow);
+
+  return change;
 }
 
-static void
+static ObjectChange*
 orthflow_move(Orthflow *orthflow, Point *to)
 {
+  ObjectChange *change;
+
   Point *points = &orthflow->orth.points[0]; 
   Point delta;
 
   delta = *to;
   point_sub(&delta, &points[0]);
-  point_add(&orthflow->text->position, &delta);
+  point_add(&orthflow->textpos, &delta);
 
-  orthconn_move( &orthflow->orth, to ) ;
+  change = orthconn_move( &orthflow->orth, to ) ;
 
   orthflow_update_data(orthflow);
+
+  return change;
 }
 
 static void
-orthflow_draw(Orthflow *orthflow, Renderer *renderer)
+orthflow_draw(Orthflow *orthflow, DiaRenderer *renderer)
 {
-  ArrowType arrow_type;
+  DiaRendererClass *renderer_ops = DIA_RENDERER_GET_CLASS (renderer);
   int n = orthflow->orth.numpoints ;
   Color* render_color = &orthflow_color_signal;
   Point *points;
-  
+  real linewidth;
+  Arrow arrow;
+
   assert(orthflow != NULL);
   assert(renderer != NULL);
 
-  arrow_type = ARROW_FILLED_TRIANGLE;
+  arrow.type = ARROW_FILLED_TRIANGLE;
+  arrow.width = ORTHFLOW_ARROWWIDTH;
+  arrow.length = ORTHFLOW_ARROWLEN;
  
   points = &orthflow->orth.points[0];
 
-  renderer->ops->set_linecaps(renderer, LINECAPS_BUTT);
+  renderer_ops->set_linecaps(renderer, LINECAPS_BUTT);
 
   switch (orthflow->type) {
   case ORTHFLOW_SIGNAL:
-    renderer->ops->set_linewidth(renderer, ORTHFLOW_WIDTH);
-    renderer->ops->set_dashlength(renderer, ORTHFLOW_DASHLEN);
-    renderer->ops->set_linestyle(renderer, LINESTYLE_DASHED);
+    linewidth = ORTHFLOW_WIDTH;
+    renderer_ops->set_dashlength(renderer, ORTHFLOW_DASHLEN);
+    renderer_ops->set_linestyle(renderer, LINESTYLE_DASHED);
     render_color = &orthflow_color_signal ;
     break ;
   case ORTHFLOW_MATERIAL:
-    renderer->ops->set_linewidth(renderer, ORTHFLOW_MATERIAL_WIDTH ) ;
-    renderer->ops->set_linestyle(renderer, LINESTYLE_SOLID);
+    linewidth = ORTHFLOW_MATERIAL_WIDTH;
+    renderer_ops->set_linestyle(renderer, LINESTYLE_SOLID);
     render_color = &orthflow_color_material ;
     break ;
   case ORTHFLOW_ENERGY:
-    renderer->ops->set_linewidth(renderer, ORTHFLOW_WIDTH);
-    renderer->ops->set_linestyle(renderer, LINESTYLE_SOLID);
+    linewidth = ORTHFLOW_WIDTH;
+    renderer_ops->set_linestyle(renderer, LINESTYLE_SOLID);
     render_color = &orthflow_color_energy ;
+    break ;
+  default:
+    linewidth = 0.001;
+    break;
   }
 
-  renderer->ops->draw_polyline(renderer, points, n,
-			       render_color); 
-
-  arrow_draw(renderer, arrow_type,
+  renderer_ops->set_linewidth(renderer, linewidth);
+  renderer_ops->draw_polyline_with_arrows(renderer, points, n,
+					  ORTHFLOW_WIDTH,
+					  render_color,
+					  NULL, &arrow); 
+#if 0
+  arrow_draw(renderer, 
 	     &points[n-1], &points[n-2],
-	     ORTHFLOW_ARROWLEN, ORTHFLOW_ARROWWIDTH, ORTHFLOW_WIDTH,
+	     ORTHFLOW_ARROWLEN, ORTHFLOW_ARROWWIDTH, 
 	     render_color, &color_white);
+#endif
 
-  renderer->ops->set_font(renderer, orthflow_font,
-			  ORTHFLOW_FONTHEIGHT);
+  renderer_ops->set_font(renderer, orthflow_font,
+                          ORTHFLOW_FONTHEIGHT);
 
   text_draw(orthflow->text, renderer);
 }
 
-static Object *
+static DiaObject *
 orthflow_create(Point *startpoint,
 		void *user_data,
 		Handle **handle1,
@@ -392,11 +408,12 @@ orthflow_create(Point *startpoint,
 {
   Orthflow *orthflow;
   OrthConn *orth;
-  Object *obj;
+  DiaObject *obj;
   Point p;
-  OrthConnBBExtras *extra;
-  orthflow = g_new0(Orthflow,1);
+  PolyBBExtras *extra;
+  DiaFont *font;
 
+  orthflow = g_new0(Orthflow,1);
   orth = &orthflow->orth ;
   orthconn_init( orth, startpoint ) ;
  
@@ -406,20 +423,27 @@ orthflow_create(Point *startpoint,
   obj->type = &orthflow_type;
   obj->ops = &orthflow_ops;
 
-  orthflow->type = orthflow_most_recent_type ;
-
   /* Where to put the text */
   p = *startpoint ;
   p.y += 0.1 * ORTHFLOW_FONTHEIGHT ;
+  orthflow->textpos = p;
+  font = dia_font_new_from_style(DIA_FONT_SANS, 0.8);
 
+  orthflow->text = new_text("", font, 0.8, &p, &color_black, ALIGN_CENTER);
+  dia_font_unref(font);  
+  text_get_attributes(orthflow->text, &orthflow->attrs);
+
+#if 0
   if ( orthflow_default_label ) {
     orthflow->text = text_copy( orthflow_default_label ) ;
     text_set_position( orthflow->text, &p ) ;
   } else {
     Color* color = &orthflow_color_signal;
 
-    if (orthflow_font == NULL)
-      orthflow_font = font_getfont("Helvetica-Oblique");
+    if (orthflow_font == NULL) {
+	    orthflow_font = dia_font_new_from_style(DIA_FONT_SANS|DIA_FONT_ITALIC,
+                                              1.0);
+    }
 
     switch (orthflow->type) {
     case ORTHFLOW_ENERGY:
@@ -434,8 +458,9 @@ orthflow_create(Point *startpoint,
     }
 
     orthflow->text = new_text("", orthflow_font, ORTHFLOW_FONTHEIGHT, 
-			      &p, color, ALIGN_CENTER);
+                              &p, color, ALIGN_CENTER);
   }
+#endif
 
   orthflow->text_handle.id = HANDLE_MOVE_TEXT;
   orthflow->text_handle.type = HANDLE_MINOR_CONTROL;
@@ -453,7 +478,7 @@ orthflow_create(Point *startpoint,
 
   /*obj->handles[1] = orth->handles[2] ;*/
   *handle1 = obj->handles[0];
-  *handle2 = obj->handles[2];
+  *handle2 = obj->handles[1];
   return &orthflow->orth.object;
 }
 
@@ -465,12 +490,12 @@ orthflow_destroy(Orthflow *orthflow)
   text_destroy( orthflow->text ) ;
 }
 
-static Object *
+static DiaObject *
 orthflow_copy(Orthflow *orthflow)
 {
   Orthflow *neworthflow;
   OrthConn *orth, *neworth;
-  Object *newobj;
+  DiaObject *newobj;
 
   orth = &orthflow->orth;
   
@@ -494,7 +519,7 @@ static void
 orthflow_update_data(Orthflow *orthflow)
 {
   OrthConn *orth = &orthflow->orth ;
-  Object *obj = &orth->object;
+  DiaObject *obj = &orth->object;
   Rectangle rect;
   Color* color = &orthflow_color_signal;
   
@@ -511,7 +536,8 @@ orthflow_update_data(Orthflow *orthflow)
   }
   text_set_color( orthflow->text, color ) ;
 
-  orthflow->text_handle.pos = orthflow->text->position;
+  text_set_position( orthflow->text, &orthflow->textpos ) ;
+  orthflow->text_handle.pos = orthflow->textpos;
 
   orthconn_update_data(orth);
   obj->position = orth->points[0];
@@ -536,17 +562,19 @@ orthflow_save(Orthflow *orthflow, ObjectNode obj_node, const char *filename)
 	       orthflow->type);
 }
 
-static Object *
+static DiaObject *
 orthflow_load(ObjectNode obj_node, int version, const char *filename)
 {
   Orthflow *orthflow;
   AttributeNode attr;
   OrthConn *orth;
-  Object *obj;
-  OrthConnBBExtras *extra;
+  DiaObject *obj;
+  PolyBBExtras *extra;
 
-  if (orthflow_font == NULL)
-    orthflow_font = font_getfont("Helvetica-Oblique");
+  if (orthflow_font == NULL) {
+    orthflow_font = dia_font_new_from_style(DIA_FONT_SANS|DIA_FONT_ITALIC,
+                                            1.0);
+  }
 
   orthflow = g_malloc0(sizeof(Orthflow));
 
@@ -581,207 +609,27 @@ orthflow_load(ObjectNode obj_node, int version, const char *filename)
     extra->middle_trans = ORTHFLOW_WIDTH/2.0;
   extra->end_long = 
     extra->end_trans = ORTHFLOW_WIDTH/2 + ORTHFLOW_ARROWLEN;
-  
+  orthflow->textpos = orthflow->text->position;
+
   orthflow_update_data(orthflow);
   
   return &orthflow->orth.object;
 }
 
-
-
-static void
-fill_in_defaults_dialog()
-{
-  OrthflowDialog *prop_dialog;
-  GtkToggleButton *button=NULL;
-
-  prop_dialog = defaults_dialog;
-
-  if ( orthflow_default_label ) {
-    gtk_text_set_point( GTK_TEXT(prop_dialog->text), 0 ) ;
-    gtk_text_forward_delete( GTK_TEXT(prop_dialog->text), 
-			     gtk_text_get_length(GTK_TEXT(prop_dialog->text))) ;
-    gtk_text_insert( GTK_TEXT(prop_dialog->text),
-		     NULL, NULL, NULL,
-		     text_get_string_copy(orthflow_default_label),
-		     -1) ;
-  }
-
-  switch (orthflow_most_recent_type) {
-  case ORTHFLOW_ENERGY:
-    button = GTK_TOGGLE_BUTTON(prop_dialog->m_energy);
-    break;
-  case ORTHFLOW_MATERIAL:
-    button = GTK_TOGGLE_BUTTON(prop_dialog->m_material);
-    break;
-  case ORTHFLOW_SIGNAL:
-    button = GTK_TOGGLE_BUTTON(prop_dialog->m_signal);
-    break;
-  }
-  if (button)
-    gtk_toggle_button_set_active(button, TRUE);
-}
-
-#if 0
 static ObjectChange *
-orthflow_apply_properties(Orthflow *orthflow)
-{
-  OrthflowDialog *prop_dialog;
-  ObjectChange* change ;
-  
-  prop_dialog = properties_dialog;
-
-  change = orthflow_create_change( BOTH,
-				   orthflow->type, orthflow->text ) ;
-  text_set_string(orthflow->text,
-                  gtk_editable_get_chars( GTK_EDITABLE(prop_dialog->text),
-					  0, -1));
-
-  if (GTK_TOGGLE_BUTTON(prop_dialog->m_energy)->active)
-    orthflow->type = ORTHFLOW_ENERGY;
-  else if (GTK_TOGGLE_BUTTON( prop_dialog->m_material )->active) 
-    orthflow->type = ORTHFLOW_MATERIAL;
-  else if (GTK_TOGGLE_BUTTON( prop_dialog->m_signal )->active) 
-    orthflow->type = ORTHFLOW_SIGNAL;
-
-  orthflow_update_data(orthflow);
-
-  return change ;
-}
-
-static void
-fill_in_dialog(Orthflow *orthflow)
-{
-  OrthflowDialog *prop_dialog;
-  GtkToggleButton *button=NULL;
-
-  prop_dialog = properties_dialog;
-
-  gtk_text_set_point( GTK_TEXT(prop_dialog->text), 0 ) ;
-  gtk_text_forward_delete( GTK_TEXT(prop_dialog->text), 
-			   gtk_text_get_length(GTK_TEXT(prop_dialog->text))) ;
-  gtk_text_insert( GTK_TEXT(prop_dialog->text),
-                   NULL, NULL, NULL,
-                   text_get_string_copy(orthflow->text),
-                   -1);
-
-  switch (orthflow->type) {
-  case ORTHFLOW_ENERGY:
-    button = GTK_TOGGLE_BUTTON(prop_dialog->m_energy);
-    break;
-  case ORTHFLOW_MATERIAL:
-    button = GTK_TOGGLE_BUTTON(prop_dialog->m_material);
-    break;
-  case ORTHFLOW_SIGNAL:
-    button = GTK_TOGGLE_BUTTON(prop_dialog->m_signal);
-    break;
-  }
-  if (button)
-    gtk_toggle_button_set_active(button, TRUE);
-}
-
-
-
-static GtkWidget *
-orthflow_get_properties(Orthflow *orthflow)
-{
-  OrthflowDialog *prop_dialog;
-  GtkWidget *dialog;
-  GtkWidget *entry;
-  GtkWidget *hbox;
-  GtkWidget *label;
-  GSList *group;
-
-  if (properties_dialog == NULL) {
-
-    prop_dialog = g_new(OrthflowDialog, 1);
-    properties_dialog = prop_dialog;
-    
-    dialog = gtk_vbox_new(FALSE, 0);
-    gtk_object_ref(GTK_OBJECT(dialog));
-    gtk_object_sink(GTK_OBJECT(dialog));
-    prop_dialog->dialog = dialog;
-    
-    hbox = gtk_hbox_new(FALSE, 5);
-
-    label = gtk_label_new(_("Flow:"));
-    gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, TRUE, 0);
-    entry = gtk_text_new(NULL, NULL);
-    prop_dialog->text = entry ;
-    gtk_text_set_editable(GTK_TEXT(entry), TRUE);
-    gtk_box_pack_start (GTK_BOX (hbox), entry, TRUE, TRUE, 0);
-    gtk_widget_show (label);
-    gtk_widget_show (entry);
-    gtk_box_pack_start (GTK_BOX (dialog), hbox, TRUE, TRUE, 0);
-    gtk_widget_show(hbox);
-
-    label = gtk_hseparator_new ();
-    gtk_box_pack_start (GTK_BOX (dialog), label, FALSE, TRUE, 0);
-    gtk_widget_show (label);
-
-    label = gtk_label_new(_("Flow type:"));
-    gtk_box_pack_start (GTK_BOX (dialog), label, FALSE, TRUE, 0);
-    gtk_widget_show (label);
-
-    /* */
-    prop_dialog->m_energy = gtk_radio_button_new_with_label (NULL, _("Energy"));
-    gtk_box_pack_start (GTK_BOX (dialog), prop_dialog->m_energy, TRUE, TRUE, 0);
-    gtk_widget_show (prop_dialog->m_energy);
-    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (prop_dialog->m_energy), TRUE);
-
-    group = gtk_radio_button_group (GTK_RADIO_BUTTON (prop_dialog->m_energy));
-
-    prop_dialog->m_material = gtk_radio_button_new_with_label(group, _("Material"));
-    gtk_box_pack_start (GTK_BOX (dialog), prop_dialog->m_material, TRUE, TRUE, 0);
-    gtk_widget_show (prop_dialog->m_material);
-
-    group = gtk_radio_button_group (GTK_RADIO_BUTTON (prop_dialog->m_material));
-
-    prop_dialog->m_signal = gtk_radio_button_new_with_label(group, _("Signal"));
-    gtk_box_pack_start (GTK_BOX (dialog), prop_dialog->m_signal, TRUE, TRUE, 0);
-    gtk_widget_show (prop_dialog->m_signal);
-
-#if 0
-    group = gtk_radio_button_group (GTK_RADIO_BUTTON (prop_dialog->m_signal));
-#endif
-  }
-  
-  fill_in_dialog(orthflow);
-  gtk_widget_show (properties_dialog->dialog);
-
-  return properties_dialog->dialog;
-}
-#endif
-
-static void
-orthflow_update_defaults( Orthflow* orthflow, char update_text )
-{
-  orthflow_most_recent_type = orthflow->type ;
-
-  if (update_text) {
-    if ( orthflow_default_label )
-      text_destroy( orthflow_default_label ) ;
-    orthflow_default_label = text_copy( orthflow->text ) ;
-  }
-}
-
-static ObjectChange *
-orthflow_set_type_callback (Object* obj, Point* clicked, gpointer data)
+orthflow_set_type_callback (DiaObject* obj, Point* clicked, gpointer data)
 {
   ObjectChange* change ;
   change = orthflow_create_change( FLOW_TYPE, ((Orthflow*)obj)->type, 0 ) ;
 
   ((Orthflow*)obj)->type = (int) data ;
-  orthflow_update_defaults( (Orthflow*) obj, 1 ) ;
-  if ( defaults_dialog )
-    fill_in_defaults_dialog() ;
   orthflow_update_data((Orthflow*)obj);
 
   return change;
 }
 
 static ObjectChange *
-orthflow_segment_callback (Object* obj, Point* clicked, gpointer data)
+orthflow_segment_callback (DiaObject* obj, Point* clicked, gpointer data)
 {
   if ( (int)data )
      return orthconn_add_segment( (OrthConn*)obj, clicked ) ;
@@ -794,7 +642,8 @@ static DiaMenuItem orthflow_menu_items[] = {
   { N_("Material"), orthflow_set_type_callback, (void*)ORTHFLOW_MATERIAL, 1 },
   { N_("Signal"), orthflow_set_type_callback, (void*)ORTHFLOW_SIGNAL, 1 },
   { N_("Add segment"), orthflow_segment_callback, (void*)1, 1 },
-  { N_("Delete segment"), orthflow_segment_callback, (void*)0, 1 }
+  { N_("Delete segment"), orthflow_segment_callback, (void*)0, 1 },
+  ORTHCONN_COMMON_MENUS,
 };
 
 static DiaMenu orthflow_menu = {
@@ -807,111 +656,9 @@ static DiaMenu orthflow_menu = {
 static DiaMenu *
 orthflow_get_object_menu(Orthflow *orthflow, Point *clickedpoint)
 {
-  /* Set entries sensitive/selected etc here */
+  orthflow_menu_items[4].active = orthflow->orth.numpoints > 3;
+  orthconn_update_object_menu((OrthConn*)orthflow,
+			      clickedpoint, &orthflow_menu_items[5]);
   return &orthflow_menu;
 }
 
-static GtkWidget *
-orthflow_get_defaults(void)
-{
-  OrthflowDialog *prop_dialog;
-  GtkWidget *dialog;
-  GtkWidget *entry;
-  GtkWidget *hbox;
-  GtkWidget *label;
-  GSList *group;
-
-  if (defaults_dialog == NULL) {
-
-    prop_dialog = g_new(OrthflowDialog, 1);
-    defaults_dialog = prop_dialog;
-    
-    dialog = gtk_vbox_new(FALSE, 0);
-    prop_dialog->dialog = dialog;
-
-    gtk_object_ref(GTK_OBJECT(dialog));
-    gtk_object_sink(GTK_OBJECT(dialog));
-
-    hbox = gtk_hbox_new(FALSE, 5);
-
-    label = gtk_label_new(_("Orthflow:"));
-    gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, TRUE, 0);
-    entry = gtk_text_new(NULL, NULL);
-    prop_dialog->text = entry ;
-    gtk_text_set_editable(GTK_TEXT(entry), TRUE);
-    gtk_widget_set_usize( GTK_WIDGET(entry), 100, 50 ) ;
-    gtk_box_pack_start (GTK_BOX (hbox), entry, TRUE, TRUE, 0);
-    gtk_widget_show (label);
-    gtk_widget_show (entry);
-    gtk_box_pack_start (GTK_BOX (dialog), hbox, TRUE, TRUE, 0);
-    gtk_widget_show(hbox);
-
-    label = gtk_hseparator_new ();
-    gtk_box_pack_start (GTK_BOX (dialog), label, FALSE, TRUE, 0);
-    gtk_widget_show (label);
-
-    label = gtk_label_new(_("Orthflow type:"));
-    gtk_box_pack_start (GTK_BOX (dialog), label, FALSE, TRUE, 0);
-    gtk_widget_show (label);
-
-    /* */
-    prop_dialog->m_energy = gtk_radio_button_new_with_label (NULL, _("Energy"));
-    gtk_box_pack_start (GTK_BOX (dialog), prop_dialog->m_energy, TRUE, TRUE, 0);
-    gtk_widget_show (prop_dialog->m_energy);
-    gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (prop_dialog->m_energy), TRUE);
-
-    group = gtk_radio_button_group (GTK_RADIO_BUTTON (prop_dialog->m_energy));
-
-    prop_dialog->m_material = gtk_radio_button_new_with_label(group, _("Material"));
-    gtk_box_pack_start (GTK_BOX (dialog), prop_dialog->m_material, TRUE, TRUE, 0);
-    gtk_widget_show (prop_dialog->m_material);
-
-    group = gtk_radio_button_group (GTK_RADIO_BUTTON (prop_dialog->m_material));
-
-    prop_dialog->m_signal = gtk_radio_button_new_with_label(group, _("Signal"));
-    gtk_box_pack_start (GTK_BOX (dialog), prop_dialog->m_signal, TRUE, TRUE, 0);
-    gtk_widget_show (prop_dialog->m_signal);
-  }
-  
-  fill_in_defaults_dialog();
-  gtk_widget_show (defaults_dialog->dialog);
-
-  return defaults_dialog->dialog;
-}
-
-static void
-orthflow_apply_defaults(void)
-{
-  OrthflowDialog *prop_dialog;
-  Color* color = 0 ;
-
-  prop_dialog = defaults_dialog;
-
-  if (GTK_TOGGLE_BUTTON(prop_dialog->m_energy)->active) {
-    orthflow_most_recent_type = ORTHFLOW_ENERGY;
-    color = &orthflow_color_energy ;
-  } else if (GTK_TOGGLE_BUTTON( prop_dialog->m_material )->active) {
-    orthflow_most_recent_type = ORTHFLOW_MATERIAL;
-    color = &orthflow_color_material ;
-  } else if (GTK_TOGGLE_BUTTON( prop_dialog->m_signal )->active) {
-    orthflow_most_recent_type = ORTHFLOW_SIGNAL;
-    color = &orthflow_color_signal ;
-  }
-
-  if ( ! orthflow_default_label ) {
-    Point p ;
-
-    if (orthflow_font == NULL)
-      orthflow_font = font_getfont("Helvetica-Oblique");
-    orthflow_default_label =
-      new_text(
-	       gtk_editable_get_chars( GTK_EDITABLE(prop_dialog->text),0,-1), 
-	       orthflow_font, ORTHFLOW_FONTHEIGHT, &p, color, ALIGN_CENTER
-	       ) ;
-  } else {
-    text_set_string(orthflow_default_label,
-		    gtk_editable_get_chars( GTK_EDITABLE(prop_dialog->text),
-					    0, -1));
-    text_set_color( orthflow_default_label, color ) ;
-  }
-}
