@@ -22,16 +22,10 @@
  */
 
 #include <config.h>
-
 #include "boolequation.h"
 
 #define OVERLINE_RATIO .1
 
-static Font *symbol;
-static void init_symbolfont() {
-  if (!symbol) symbol = font_getfont("Symbol");
-}
-
 typedef enum {BLOCK_COMPOUND, BLOCK_OPERATOR, BLOCK_OVERLINE,
 	      BLOCK_PARENS, BLOCK_TEXT} BlockType;
 typedef enum {OP_AND, OP_OR, OP_XOR, OP_RISE, OP_FALL,
@@ -41,7 +35,7 @@ typedef void (*BlockGetBBFunc)(Block *block, Point *relpos, Boolequation *booleq
 			       Rectangle *rect);
 typedef void (*BlockDrawFunc)(Block *block, 
 			      Boolequation *booleq, 
-			      Renderer *render);
+			      DiaRenderer *render);
 typedef void (*BlockDestroyFunc)(Block *block);
 
 typedef struct {
@@ -64,7 +58,7 @@ struct _Block {
 
 
 /* utility */
-inline static gboolean isspecial(char c) 
+inline static gboolean isspecial(gunichar c) 
 {
   switch (c) {
   case '!':
@@ -93,11 +87,15 @@ textblock_get_boundingbox(Block *block, Point *relpos,
 
   block->pos = *relpos;
   block->bl.x = block->pos.x;
-  block->bl.y = block->pos.y + booleq->descent;
-  block->ur.y = block->bl.y - booleq->fontheight;
-  block->ur.x = block->bl.x + font_string_width(block->d.text,
-						booleq->font,
-						booleq->fontheight);
+  block->bl.y = block->pos.y + dia_font_descent(block->d.text,
+                                                booleq->font,
+                                                booleq->fontheight);
+  block->ur.y = block->pos.y - dia_font_ascent(block->d.text,
+                                               booleq->font,
+                                               booleq->fontheight);
+  block->ur.x = block->bl.x + dia_font_string_width(block->d.text,
+                                                    booleq->font,
+                                                    booleq->fontheight);
   rect->left = block->bl.x;
   rect->top = block->ur.y;
   rect->bottom = block->bl.y;
@@ -105,11 +103,12 @@ textblock_get_boundingbox(Block *block, Point *relpos,
 }
 
 static void 
-textblock_draw(Block *block,Boolequation *booleq,Renderer *renderer)
+textblock_draw(Block *block,Boolequation *booleq,DiaRenderer *renderer)
 {
+  DiaRendererClass *renderer_ops = DIA_RENDERER_GET_CLASS (renderer);
   g_assert(block); g_assert(block->type == BLOCK_TEXT);
-  renderer->ops->set_font(renderer,booleq->font,booleq->fontheight);
-  renderer->ops->draw_string(renderer,block->d.text,
+  renderer_ops->set_font(renderer,booleq->font,booleq->fontheight);
+  renderer_ops->draw_string(renderer,block->d.text,
 			     &block->pos,ALIGN_LEFT,&booleq->color);
 }
 
@@ -127,13 +126,19 @@ static BlockOps text_block_ops = {
   textblock_draw,
   textblock_destroy};
   
-static Block *textblock_create(const char **str) 
+static Block *textblock_create(const gchar **str) 
 {
-  const char *p = *str;
+  const gchar *p = *str;
   Block *block;
 
-  g_assert(!isspecial(**str));
-  while (**str && (!isspecial(**str))) (*str)++;
+  while (**str) {
+    gunichar c;
+    gchar *p1;
+    c = g_utf8_get_char(*str);
+    p1 = g_utf8_next_char(*str);
+    if (isspecial(c)) break;
+    *str = p1;
+  }
   
   block = g_new0(Block,1);
   block->type = BLOCK_TEXT;
@@ -144,11 +149,11 @@ static Block *textblock_create(const char **str)
 }
 
 /* Operator block */
-static const gchar xor_symbol[] = { 197, 0 };
+static const gchar xor_symbol[] = { 226, 138, 149, 0 }; /* 0x2295 */
+static const gchar and_symbol[] = { 226, 136, 153, 0 }; /* 0x2219 */
+static const gchar rise_symbol[] = { 226, 134, 145, 0 }; /* 0x2191 */
+static const gchar fall_symbol[] = { 226, 134, 147, 0 }; /* 0x2193 */
 static const gchar or_symbol[] = { 43, 0 };
-static const gchar and_symbol[] = { 215,0 };
-static const gchar rise_symbol[] = { 173,0 };
-static const gchar fall_symbol[] = { 175,0 };
 static const gchar lt_symbol[] = { 60,0 };
 static const gchar equal_symbol[] = { 61,0 };
 static const gchar gt_symbol[] = { 62,0 };
@@ -156,6 +161,7 @@ static const gchar gt_symbol[] = { 62,0 };
 static const gchar *opstring(OperatorType optype)
 {
   switch (optype) {
+
   case OP_AND: return and_symbol;
   case OP_OR: return or_symbol;
   case OP_XOR: return xor_symbol;
@@ -175,16 +181,18 @@ static void
 opblock_get_boundingbox(Block *block, Point *relpos,
 			Boolequation *booleq, Rectangle *rect)
 {
+  const gchar* ops;
   g_assert(block); g_assert(block->type == BLOCK_OPERATOR);
   
+  ops = opstring(block->d.operator);
   
   block->pos = *relpos;
   block->bl.x = block->pos.x;
-  block->bl.y = block->pos.y + font_descent(symbol,booleq->fontheight);
+  block->bl.y = block->pos.y +
+      dia_font_descent(ops,booleq->font,booleq->fontheight);
   block->ur.y = block->bl.y - booleq->fontheight;
-  block->ur.x = block->bl.x + font_string_width(opstring(block->d.operator),
-						symbol,
-						booleq->fontheight);
+  block->ur.x = block->bl.x + dia_font_string_width(ops, booleq->font,
+                                                    booleq->fontheight);
   rect->left = block->bl.x;
   rect->top = block->ur.y;
   rect->bottom = block->bl.y;
@@ -192,11 +200,11 @@ opblock_get_boundingbox(Block *block, Point *relpos,
 }
 
 static void 
-opblock_draw(Block *block, Boolequation *booleq,Renderer *renderer)
+opblock_draw(Block *block, Boolequation *booleq,DiaRenderer *renderer)
 {
+  DiaRendererClass *renderer_ops = DIA_RENDERER_GET_CLASS (renderer);
   g_assert(block); g_assert(block->type == BLOCK_OPERATOR);
-  renderer->ops->set_font(renderer,symbol,booleq->fontheight);
-  renderer->ops->draw_string(renderer,opstring(block->d.operator),&block->pos,
+  renderer_ops->draw_string(renderer,opstring(block->d.operator),&block->pos,
 			     ALIGN_LEFT,&booleq->color);
 }
     
@@ -212,16 +220,18 @@ static BlockOps operator_block_ops = {
   opblock_draw,
   opblock_destroy};
 
-static Block *opblock_create(const char **str) 
+static Block *opblock_create(const gchar **str) 
 {
-  const char *p = *str;
   Block *block;
+  gunichar c;
+
+  c = g_utf8_get_char(*str);
+  *str = g_utf8_next_char(*str);
 
-  (*str)++;
   block = g_new0(Block,1);
   block->type = BLOCK_OPERATOR;
   block->ops = &operator_block_ops;
-  switch(*p) {
+  switch(c) {
   case '&': 
   case '.':
     block->d.operator = OP_AND;
@@ -276,18 +286,22 @@ overlineblock_get_boundingbox(Block *block, Point *relpos,
 }
 
 static void 
-overlineblock_draw(Block *block,Boolequation *booleq,Renderer *renderer)
+overlineblock_draw(Block *block,Boolequation *booleq,DiaRenderer *renderer)
 {
+  DiaRendererClass *renderer_ops = DIA_RENDERER_GET_CLASS (renderer);
   Point ul,ur;
   g_assert(block); g_assert(block->type == BLOCK_OVERLINE);
   block->d.inside->ops->draw(block->d.inside,booleq,renderer);
-  renderer->ops->set_linestyle(renderer,LINESTYLE_SOLID);
-  renderer->ops->set_linewidth(renderer,booleq->fontheight * OVERLINE_RATIO);
+  renderer_ops->set_linestyle(renderer,LINESTYLE_SOLID);
+  renderer_ops->set_linewidth(renderer,booleq->fontheight * OVERLINE_RATIO);
   ul.x = block->bl.x;
   ur.y = ul.y = block->ur.y;
-  ur.x = 
-    block->ur.x - (font_string_width(" ",booleq->font,booleq->fontheight) / 2);
-  renderer->ops->draw_line(renderer,&ul,&ur,&booleq->color);
+
+      /* FIXME: try to get the actual block width */
+  ur.x = block->ur.x -
+      (dia_font_string_width("_", booleq->font,booleq->fontheight) / 2);
+  
+  renderer_ops->draw_line(renderer,&ul,&ur,&booleq->color);
 }
 
 static void
@@ -328,12 +342,12 @@ parensblock_get_boundingbox(Block *block, Point *relpos,
   temppos = block->pos = *relpos;
   block->d.inside->ops->get_boundingbox(block->d.inside,&temppos,booleq,rect);
   pheight = 1.1 * (block->d.inside->bl.y - block->d.inside->ur.y);
-  pwidth = font_string_width("()",booleq->font,pheight) / 2;
+  pwidth = dia_font_string_width("()",booleq->font,pheight) / 2;
   temppos.x += pwidth;
   block->d.inside->ops->get_boundingbox(block->d.inside,&temppos,booleq,rect);
   
   block->bl.x = block->pos.x;
-  block->bl.y = block->pos.y + font_descent(booleq->font,pheight);
+  block->bl.y = block->pos.y + dia_font_descent("()",booleq->font,pheight);
   block->ur.x = block->d.inside->ur.x + pwidth;
   block->ur.y = block->bl.y - pheight;
 
@@ -344,8 +358,9 @@ parensblock_get_boundingbox(Block *block, Point *relpos,
 }
 
 static void 
-parensblock_draw(Block *block,Boolequation *booleq,Renderer *renderer)
+parensblock_draw(Block *block,Boolequation *booleq,DiaRenderer *renderer)
 {
+  DiaRendererClass *renderer_ops = DIA_RENDERER_GET_CLASS (renderer);
   Point pt;
   real pheight;
 
@@ -354,12 +369,12 @@ parensblock_draw(Block *block,Boolequation *booleq,Renderer *renderer)
   pheight = block->d.inside->bl.y - block->d.inside->ur.y;
   block->d.inside->ops->draw(block->d.inside,booleq,renderer);
 
-  renderer->ops->set_font(renderer,booleq->font,pheight);
+  renderer_ops->set_font(renderer,booleq->font,pheight);
   pt.y = block->pos.y;
   pt.x = block->d.inside->ur.x;
 
-  renderer->ops->draw_string(renderer,"(",&block->pos,ALIGN_LEFT,&booleq->color);
-  renderer->ops->draw_string(renderer,")",&pt,ALIGN_LEFT,&booleq->color);
+  renderer_ops->draw_string(renderer,"(",&block->pos,ALIGN_LEFT,&booleq->color);
+  renderer_ops->draw_string(renderer,")",&pt,ALIGN_LEFT,&booleq->color);
 }
  
 static void
@@ -427,7 +442,7 @@ compoundblock_get_boundingbox(Block *block, Point *relpos,
 
 static void compoundblock_draw(Block *block,
 			       Boolequation *booleq,
-			       Renderer *renderer)
+			       DiaRenderer *renderer)
 {
   GSList *elem;
   Block *inblk;
@@ -460,6 +475,8 @@ compoundblock_destroy(Block *block)
 
     inblk->ops->destroy(inblk);
     elem->data = NULL;
+    
+    elem = g_slist_next(elem);
   }
 
   g_slist_free(block->d.contained);
@@ -472,7 +489,7 @@ static BlockOps compound_block_ops = {
   compoundblock_destroy};
 
 static Block *
-compoundblock_create(const char **str) 
+compoundblock_create(const gchar **str) 
 {
   Block *block, *inblk;
   
@@ -482,8 +499,13 @@ compoundblock_create(const char **str)
   block->d.contained = NULL;
 
   while (*str && **str) {
+    gunichar c;
+    gchar *p;
+    c = g_utf8_get_char(*str);
+    p = g_utf8_next_char(*str);
+
     inblk = NULL;
-    switch (**str) {
+    switch (c) {
     case '&':
     case '^':
     case '|':
@@ -498,20 +520,22 @@ compoundblock_create(const char **str)
       inblk = opblock_create(str);
       break;
     case '(':
-      (*str)++;
+      *str = p;
       inblk = parensblock_create(compoundblock_create(str));
       break;
     case '!':
-      (*str)++;
-      if ((**str) == '(') {
-	(*str)++;
+      *str = p;
+      c = g_utf8_get_char(*str);
+      p = g_utf8_next_char(*str);
+      if (c == '(') {
+	*str = p;
 	inblk = overlineblock_create(compoundblock_create(str));
       } else { /* single identifier "not" */
 	inblk = overlineblock_create(textblock_create(str));
       }
       break;
     case ')': /* Whooops ! outta here ! */
-      (*str)++;
+      *str = p;
       return block;
     default:
       inblk = textblock_create(str);
@@ -530,7 +554,7 @@ void
 boolequation_set_value(Boolequation *booleq, const gchar *value)
 {
   g_return_if_fail(booleq);
-  if (booleq->value) g_free((char *)booleq->value);
+  if (booleq->value) g_free((gchar *)booleq->value);
   if (booleq->rootblock) booleq->rootblock->ops->destroy(booleq->rootblock);
 
   booleq->value = g_strdup(value);
@@ -540,15 +564,13 @@ boolequation_set_value(Boolequation *booleq, const gchar *value)
 
 
 Boolequation *
-boolequation_create(const gchar *value, Font *font, real fontheight,
+boolequation_create(const gchar *value, DiaFont *font, real fontheight,
 		   Color *color)
 {
   Boolequation *booleq;
 
-  init_symbolfont();
-
   booleq = g_new0(Boolequation,1);
-  booleq->font = font;
+  booleq->font = dia_font_ref(font);
   booleq->fontheight = fontheight;
   booleq->color = *color;
   boolequation_set_value(booleq,value);
@@ -560,38 +582,41 @@ void
 boolequation_destroy(Boolequation *booleq)
 {
   g_return_if_fail(booleq);
-  if (booleq->value) g_free((char *)booleq->value);
+  dia_font_unref(booleq->font);
+  if (booleq->value) g_free((gchar *)booleq->value);
   if (booleq->rootblock) booleq->rootblock->ops->destroy(booleq->rootblock);
   g_free(booleq);
 }
 
-extern void save_boolequation(ObjectNode *obj_node, const gchar *attrname,
+void save_boolequation(ObjectNode obj_node, const gchar *attrname,
 			     Boolequation *booleq)
 {
-  save_string(obj_node,attrname,(char *)booleq->value);
+  data_add_string(new_attribute(obj_node,attrname),(gchar *)booleq->value);
 }
 
 Boolequation *
-load_boolequation(ObjectNode *obj_node,
+load_boolequation(ObjectNode obj_node,
 		 const gchar *attrname,
 		 const gchar *defaultvalue,
-		 Font *font,
+		 DiaFont *font,
 		 real fontheight, Color *color)
 {
   const gchar *value = NULL;
   Boolequation *booleq;
-
-  init_symbolfont();
+  AttributeNode attr;
 
   booleq = boolequation_create(NULL,font,fontheight,color);
-  value = load_string(obj_node,attrname,(char *)defaultvalue);
+  attr = object_find_attribute(obj_node,attrname);
+  if (attr) value = data_string(attribute_first_data(attr));
+  else value = (gchar *)defaultvalue;
   if (value) boolequation_set_value(booleq,value);
-  g_free((char *)value);
+  g_free((gchar *)value);
+
   return booleq;
 }
  
 void 
-boolequation_draw(Boolequation *booleq, Renderer *renderer)
+boolequation_draw(Boolequation *booleq, DiaRenderer *renderer)
 {
   if (booleq->rootblock) {
     booleq->rootblock->ops->draw(booleq->rootblock,booleq,renderer);
@@ -600,18 +625,20 @@ boolequation_draw(Boolequation *booleq, Renderer *renderer)
 
 void boolequation_calc_boundingbox(Boolequation *booleq, Rectangle *box)
 {
-  booleq->ascent = font_ascent(booleq->font,booleq->fontheight);
-  booleq->descent = font_descent(booleq->font,booleq->fontheight);
-
+        /*
+          booleq->ascent = dia_font_ascent(booleq->font,booleq->fontheight);
+          booleq->descent = dia_font_descent(booleq->font,booleq->fontheight);
+        */
+          
   box->left = box->right = booleq->pos.x;
   box->top = box->bottom = booleq->pos.y;
 
   if (booleq->rootblock) {
-    booleq->rootblock->ops->get_boundingbox(booleq->rootblock,&booleq->pos,booleq,box);
+    booleq->rootblock->ops->get_boundingbox(booleq->rootblock,
+                                            &booleq->pos,booleq,box);
   }  
 
   booleq->width = box->right - box->left;
   booleq->height = box->bottom - box->top;
 }
 
-
