@@ -2,7 +2,7 @@
  * Copyright (C) 1998 Alexander Larsson
  *
  * GRAFCET chart support 
- * Copyright(C) 2000 Cyrille Chepelov
+ * Copyright(C) 2000,2001 Cyrille Chepelov
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -24,14 +24,13 @@
 #endif
 
 #include <assert.h>
-#include <gtk/gtk.h>
 #include <math.h>
 
 #include "intl.h"
 #include "object.h"
 #include "element.h"
 #include "connectionpoint.h"
-#include "render.h"
+#include "diarenderer.h"
 #include "attributes.h"
 #include "widgets.h"
 #include "message.h"
@@ -50,7 +49,9 @@
 
 
 #define TRANSITION_LINE_WIDTH GRAFCET_GENERAL_LINE_WIDTH
-#define TRANSITION_FONT "Helvetica-Bold"
+
+#define TRANSITION_FONT "Arial"
+#define TRANSITION_STYLE STYLE_BOLD
 #define TRANSITION_FONT_HEIGHT 0.8
 #define TRANSITION_DECLAREDHEIGHT 2.0
 #define TRANSITION_DECLAREDWIDTH 2.0
@@ -61,7 +62,7 @@ typedef struct _Transition {
   Element element;
   
   Boolequation *receptivity;
-  Font *rcep_font;
+  DiaFont *rcep_font;
   real rcep_fontheight;
   Color rcep_color;
   char *rcep_value;
@@ -75,13 +76,14 @@ typedef struct _Transition {
   Point A,B,C,D,Z;
 } Transition;
 
-static void transition_move_handle(Transition *transition, Handle *handle,
-				   Point *to, HandleMoveReason reason, ModifierKeys modifiers);
-static void transition_move(Transition *transition, Point *to);
+static ObjectChange* transition_move_handle(Transition *transition, Handle *handle,
+					    Point *to, ConnectionPoint *cp,
+					    HandleMoveReason reason, ModifierKeys modifiers);
+static ObjectChange* transition_move(Transition *transition, Point *to);
 static void transition_select(Transition *transition, Point *clicked_point,
-			      Renderer *interactive_renderer);
-static void transition_draw(Transition *transition, Renderer *renderer);
-static Object *transition_create(Point *startpoint,
+			      DiaRenderer *interactive_renderer);
+static void transition_draw(Transition *transition, DiaRenderer *renderer);
+static DiaObject *transition_create(Point *startpoint,
 				 void *user_data,
 				 Handle **handle1,
 				 Handle **handle2);
@@ -89,13 +91,13 @@ static real transition_distance_from(Transition *transition, Point *point);
 static void transition_update_data(Transition *transition);
 static void transition_destroy(Transition *transition);
 
-static Object *transition_load(ObjectNode obj_node, int version,
+static DiaObject *transition_load(ObjectNode obj_node, int version,
 			       const char *filename);
 static PropDescription *transition_describe_props(Transition *transition);
 static void transition_get_props(Transition *transition, 
-                                 Property *props, guint nprops);
+                                 GPtrArray *props);
 static void transition_set_props(Transition *transition, 
-                                 Property *props, guint nprops);
+                                 GPtrArray *props);
 
 static ObjectTypeOps transition_type_ops =
 {
@@ -106,7 +108,7 @@ static ObjectTypeOps transition_type_ops =
   (ApplyDefaultsFunc) NULL
 };
 
-ObjectType transition_type =
+DiaObjectType transition_type =
 {
   "GRAFCET - Transition",   /* name */
   0,                         /* version */
@@ -134,7 +136,7 @@ static ObjectOps transition_ops = {
 
 static PropDescription transition_props[] = {
   ELEMENT_COMMON_PROPERTIES,
-  { "receptivity",PROP_TYPE_STRING, PROP_FLAG_VISIBLE,
+  { "receptivity",PROP_TYPE_STRING, PROP_FLAG_VISIBLE|PROP_FLAG_DONT_MERGE,
     N_("Receptivity"),N_("The boolean equation of the receptivity")},
   { "rcep_font",PROP_TYPE_FONT, PROP_FLAG_VISIBLE,
     N_("Font"),N_("The receptivity's font") },
@@ -145,7 +147,7 @@ static PropDescription transition_props[] = {
     N_("Color"),N_("The receptivity's color")},
   { "north_pos",PROP_TYPE_POINT,0,N_("North point"),NULL },
   { "south_pos",PROP_TYPE_POINT,0,N_("South point"),NULL },
-  PROP_DESC_END
+  PROP_DESC_END 
 };
 
 static PropDescription *
@@ -169,20 +171,21 @@ static PropOffset transition_offsets[] = {
 };
 
 static void
-transition_get_props(Transition *transition, Property *props, guint nprops)
+transition_get_props(Transition *transition, GPtrArray *props)
 {  
   object_get_props_from_offsets(&transition->element.object,
-                                transition_offsets,props,nprops);
+                                transition_offsets,props);
 }
 
 static void
-transition_set_props(Transition *transition, Property *props, guint nprops)
+transition_set_props(Transition *transition, GPtrArray *props)
 {
   object_set_props_from_offsets(&transition->element.object,
-                                transition_offsets,props,nprops);
+                                transition_offsets,props);
 
   boolequation_set_value(transition->receptivity,transition->rcep_value);
-  transition->receptivity->font = transition->rcep_font;
+  dia_font_unref(transition->receptivity->font);
+  transition->receptivity->font = dia_font_ref(transition->rcep_font);
   transition->receptivity->fontheight = transition->rcep_fontheight;
   transition->receptivity->color = transition->rcep_color;
 
@@ -193,7 +196,7 @@ static void
 transition_update_data(Transition *transition)
 {
   Element *elem = &transition->element;
-  Object *obj = &elem->object;
+  DiaObject *obj = &elem->object;
   Point *p;
 
   transition->element.extra_spacing.border_trans = TRANSITION_LINE_WIDTH / 2.0;
@@ -216,8 +219,8 @@ transition_update_data(Transition *transition)
     + (.3 * transition->receptivity->fontheight);
 
   transition->Z.x = transition->D.x + 
-    font_string_width(" ",transition->receptivity->font,
-		      transition->receptivity->fontheight);
+    dia_font_string_width("_",transition->receptivity->font,
+                          transition->receptivity->fontheight);
 
   for (p = &transition->A; p <= &transition->Z; p++) 
     point_add(p,&elem->corner);
@@ -240,7 +243,9 @@ transition_update_data(Transition *transition)
     (transition->south.pos.y + transition->B.y) / 2.0;
 
   obj->connections[0]->pos = transition->A;
+  obj->connections[0]->directions = DIR_EAST|DIR_WEST;
   obj->connections[1]->pos = transition->B;
+  obj->connections[1]->directions = DIR_EAST|DIR_WEST;
 
 
   element_update_boundingbox(elem);
@@ -279,14 +284,15 @@ transition_distance_from(Transition *transition, Point *point)
 
 static void
 transition_select(Transition *transition, Point *clicked_point,
-		  Renderer *interactive_renderer)
+		  DiaRenderer *interactive_renderer)
 {
   transition_update_data(transition);
 }
 
-static void
+static ObjectChange*
 transition_move_handle(Transition *transition, Handle *handle,
-		       Point *to, HandleMoveReason reason, 
+		       Point *to, ConnectionPoint *cp,
+		       HandleMoveReason reason, 
 		       ModifierKeys modifiers)
 {
   g_assert(transition!=NULL);
@@ -305,14 +311,17 @@ transition_move_handle(Transition *transition, Handle *handle,
       transition->south.pos.y = transition->B.y;
     break;
   default:
-    element_move_handle(&transition->element, handle->id, to, reason);
+    element_move_handle(&transition->element, handle->id, to, cp, 
+			reason, modifiers);
   }
 
   transition_update_data(transition);
+
+  return NULL;
 }
 
 
-static void
+static ObjectChange*
 transition_move(Transition *transition, Point *to)
 {
   Point delta = *to;
@@ -324,16 +333,19 @@ transition_move(Transition *transition, Point *to)
 
   element_update_handles(elem);
   transition_update_data(transition);
+
+  return NULL;
 }
 
 
 static void 
-transition_draw(Transition *transition, Renderer *renderer)
+transition_draw(Transition *transition, DiaRenderer *renderer)
 {
+  DiaRendererClass *renderer_ops = DIA_RENDERER_GET_CLASS (renderer);
   Point pts[6];
-  renderer->ops->set_linewidth(renderer, TRANSITION_LINE_WIDTH);
-  renderer->ops->set_linestyle(renderer, LINESTYLE_SOLID);
-  renderer->ops->set_linecaps(renderer, LINECAPS_BUTT);
+  renderer_ops->set_linewidth(renderer, TRANSITION_LINE_WIDTH);
+  renderer_ops->set_linestyle(renderer, LINESTYLE_SOLID);
+  renderer_ops->set_linecaps(renderer, LINECAPS_BUTT);
 
   pts[0] = transition->north.pos;
   pts[1] = transition->NU1;
@@ -341,27 +353,27 @@ transition_draw(Transition *transition, Renderer *renderer)
   pts[3] = transition->SD1;
   pts[4] = transition->SD2;
   pts[5] = transition->south.pos;
-  renderer->ops->draw_polyline(renderer,pts,sizeof(pts)/sizeof(pts[0]),
+  renderer_ops->draw_polyline(renderer,pts,sizeof(pts)/sizeof(pts[0]),
 			       &color_black);
 
-  renderer->ops->draw_line(renderer,
+  renderer_ops->draw_line(renderer,
 			     &transition->C,&transition->D,
 			     &color_black);
   
   boolequation_draw(transition->receptivity,renderer);
 }
 
-static Object *
+static DiaObject *
 transition_create(Point *startpoint,
 		  void *user_data,
 		  Handle **handle1,
 		  Handle **handle2)
 {
   Transition *transition;
-  Object *obj;
+  DiaObject *obj;
   int i;
   Element *elem;
-  Font *default_font; 
+  DiaFont *default_font = NULL; 
   real default_fontheight;
   Color fg_color;
 
@@ -388,10 +400,12 @@ transition_create(Point *startpoint,
                         &fg_color);
 
   transition->rcep_value = g_strdup("");
-  transition->rcep_font = default_font;
+  transition->rcep_font = dia_font_ref(default_font);
   transition->rcep_fontheight = default_fontheight;
   transition->rcep_color = fg_color;
 
+  dia_font_unref(default_font);
+  
 
   for (i=0;i<8;i++) {
     obj->handles[i]->type = HANDLE_NON_MOVABLE;
@@ -423,12 +437,13 @@ transition_create(Point *startpoint,
 static void
 transition_destroy(Transition *transition)
 {
+  dia_font_unref(transition->rcep_font);
   boolequation_destroy(transition->receptivity);
   g_free(transition->rcep_value);
   element_destroy(&transition->element);
 }
 
-static Object *
+static DiaObject *
 transition_load(ObjectNode obj_node, int version, const char *filename)
 {
   return object_load_using_properties(&transition_type,
