@@ -20,14 +20,13 @@
 #endif
 
 #include <assert.h>
-#include <gtk/gtk.h>
 #include <math.h>
 #include <string.h>
 
 #include "intl.h"
 #include "object.h"
 #include "element.h"
-#include "render.h"
+#include "diarenderer.h"
 #include "attributes.h"
 #include "text.h"
 #include "properties.h"
@@ -41,6 +40,10 @@ struct _Actor {
   ConnectionPoint connections[8];
 
   Text *text;
+  TextAttributes attrs;
+
+  Color line_color;
+  Color fill_color;
 };
 
 #define ACTOR_WIDTH 2.2
@@ -53,37 +56,36 @@ struct _Actor {
 
 static real actor_distance_from(Actor *actor, Point *point);
 static void actor_select(Actor *actor, Point *clicked_point,
-			Renderer *interactive_renderer);
-static void actor_move_handle(Actor *actor, Handle *handle,
-			     Point *to, HandleMoveReason reason, ModifierKeys modifiers);
-static void actor_move(Actor *actor, Point *to);
-static void actor_draw(Actor *actor, Renderer *renderer);
-static Object *actor_create(Point *startpoint,
+			DiaRenderer *interactive_renderer);
+static ObjectChange* actor_move_handle(Actor *actor, Handle *handle,
+				       Point *to, ConnectionPoint *cp,
+				       HandleMoveReason reason, ModifierKeys modifiers);
+static ObjectChange* actor_move(Actor *actor, Point *to);
+static void actor_draw(Actor *actor, DiaRenderer *renderer);
+static DiaObject *actor_create(Point *startpoint,
 			   void *user_data,
 			   Handle **handle1,
 			   Handle **handle2);
 static void actor_destroy(Actor *actor);
-static Object *actor_copy(Actor *actor);
-
-static void actor_save(Actor *actor, ObjectNode obj_node,
-		       const char *filename);
-static Object *actor_load(ObjectNode obj_node, int version,
+static DiaObject *actor_load(ObjectNode obj_node, int version,
 			  const char *filename);
 
 static PropDescription *actor_describe_props(Actor *actor);
-static void actor_get_props(Actor *actor, Property *props, guint nprops);
-static void actor_set_props(Actor *actor, Property *props, guint nprops);
+static void actor_get_props(Actor *actor, GPtrArray *props);
+static void actor_set_props(Actor *actor, GPtrArray *props);
 
 static void actor_update_data(Actor *actor);
 
 static ObjectTypeOps actor_type_ops =
 {
   (CreateFunc) actor_create,
-  (LoadFunc)   actor_load,
-  (SaveFunc)   actor_save
+  (LoadFunc)   actor_load,/*using_properties*/     /* load */
+  (SaveFunc)   object_save_using_properties,      /* save */
+  (GetDefaultsFunc)   NULL, 
+  (ApplyDefaultsFunc) NULL
 };
 
-ObjectType actor_type =
+DiaObjectType actor_type =
 {
   "UML - Actor",   /* name */
   0,                      /* version */
@@ -97,7 +99,7 @@ static ObjectOps actor_ops = {
   (DrawFunc)            actor_draw,
   (DistanceFunc)        actor_distance_from,
   (SelectFunc)          actor_select,
-  (CopyFunc)            actor_copy,
+  (CopyFunc)            object_copy_using_properties,
   (MoveFunc)            actor_move,
   (MoveHandleFunc)      actor_move_handle,
   (GetPropertiesFunc)   object_create_props_dialog,
@@ -112,121 +114,81 @@ static PropDescription actor_props[] = {
   ELEMENT_COMMON_PROPERTIES,
   PROP_STD_TEXT_FONT,
   PROP_STD_TEXT_HEIGHT,
-  PROP_STD_TEXT_COLOUR,
-  PROP_STD_TEXT,
-  
+  PROP_STD_TEXT_COLOUR_OPTIONAL,
+  PROP_STD_LINE_COLOUR_OPTIONAL, 
+  PROP_STD_FILL_COLOUR_OPTIONAL, 
+  { "text", PROP_TYPE_TEXT, 0, N_("Text"), NULL, NULL }, 
   PROP_DESC_END
 };
 
 static PropDescription *
 actor_describe_props(Actor *actor)
 {
-  if (actor_props[0].quark == 0)
+  if (actor_props[0].quark == 0) {
     prop_desc_list_calculate_quarks(actor_props);
+  }
   return actor_props;
 }
 
 static PropOffset actor_offsets[] = {
   ELEMENT_COMMON_PROPERTIES_OFFSETS,
-
+  {"text",PROP_TYPE_TEXT,offsetof(Actor,text)},
+  {"text_font",PROP_TYPE_FONT,offsetof(Actor,attrs.font)},
+  {"text_height",PROP_TYPE_REAL,offsetof(Actor,attrs.height)},
+  {"text_colour",PROP_TYPE_COLOUR,offsetof(Actor,attrs.color)},
+  {"line_colour",PROP_TYPE_COLOUR,offsetof(Actor,line_color)},
+  {"fill_colour",PROP_TYPE_COLOUR,offsetof(Actor,fill_color)},
   { NULL, 0, 0 },
 };
 
-static struct { const gchar *name; GQuark q; } quarks[] = {
-  { "text_font" },
-  { "text_height" },
-  { "text_colour" },
-  { "text" }
-};
-
 static void
-actor_get_props(Actor * actor, Property *props, guint nprops)
+actor_get_props(Actor * actor, GPtrArray *props)
 {
-  guint i;
-
-  if (object_get_props_from_offsets(&actor->element.object, 
-                                    actor_offsets, props, nprops))
-    return;
-  /* these props can't be handled as easily */
-  if (quarks[0].q == 0)
-    for (i = 0; i < sizeof(quarks)/sizeof(*quarks); i++)
-      quarks[i].q = g_quark_from_static_string(quarks[i].name);
-  for (i = 0; i < nprops; i++) {
-    GQuark pquark = g_quark_from_string(props[i].name);
-
-    if (pquark == quarks[0].q) {
-      props[i].type = PROP_TYPE_FONT;
-      PROP_VALUE_FONT(props[i]) = actor->text->font;
-    } else if (pquark == quarks[1].q) {
-      props[i].type = PROP_TYPE_REAL;
-      PROP_VALUE_REAL(props[i]) = actor->text->height;
-    } else if (pquark == quarks[2].q) {
-      props[i].type = PROP_TYPE_COLOUR;
-      PROP_VALUE_COLOUR(props[i]) = actor->text->color;
-    } else if (pquark == quarks[3].q) {
-      props[i].type = PROP_TYPE_STRING;
-      g_free(PROP_VALUE_STRING(props[i]));
-      PROP_VALUE_STRING(props[i]) = text_get_string_copy(actor->text);
-    }
-  }
+  text_get_attributes(actor->text,&actor->attrs);
+  object_get_props_from_offsets(&actor->element.object,
+                                actor_offsets,props);
 }
 
 static void
-actor_set_props(Actor *actor, Property *props, guint nprops)
+actor_set_props(Actor *actor, GPtrArray *props)
 {
-  if (!object_set_props_from_offsets(&actor->element.object, 
-                                     actor_offsets, props, nprops)) {
-    guint i;
-
-    if (quarks[0].q == 0)
-      for (i = 0; i < sizeof(quarks)/sizeof(*quarks); i++)
-	quarks[i].q = g_quark_from_static_string(quarks[i].name);
-
-    for (i = 0; i < nprops; i++) {
-      GQuark pquark = g_quark_from_string(props[i].name);
-
-      if (pquark == quarks[0].q && props[i].type == PROP_TYPE_FONT) {
-	text_set_font(actor->text, PROP_VALUE_FONT(props[i]));
-      } else if (pquark == quarks[1].q && props[i].type == PROP_TYPE_REAL) {
-	text_set_height(actor->text, PROP_VALUE_REAL(props[i]));
-      } else if (pquark == quarks[2].q && props[i].type == PROP_TYPE_COLOUR) {
-	text_set_color(actor->text, &PROP_VALUE_COLOUR(props[i]));
-      } else if (pquark == quarks[3].q && props[i].type == PROP_TYPE_STRING) {
-	text_set_string(actor->text, PROP_VALUE_STRING(props[i]));
-      }
-    }
-  }
+  object_set_props_from_offsets(&actor->element.object,
+                                actor_offsets,props);
+  apply_textattr_properties(props,actor->text,"text",&actor->attrs);
   actor_update_data(actor);
 }
 
 static real
 actor_distance_from(Actor *actor, Point *point)
 {
-  Object *obj = &actor->element.object;
+  DiaObject *obj = &actor->element.object;
   return distance_rectangle_point(&obj->bounding_box, point);
 }
 
 static void
 actor_select(Actor *actor, Point *clicked_point,
-	       Renderer *interactive_renderer)
+	       DiaRenderer *interactive_renderer)
 {
   text_set_cursor(actor->text, clicked_point, interactive_renderer);
   text_grab_focus(actor->text, &actor->element.object);
   element_update_handles(&actor->element);
 }
 
-static void
+static ObjectChange*
 actor_move_handle(Actor *actor, Handle *handle,
-		 Point *to, HandleMoveReason reason, ModifierKeys modifiers)
+		  Point *to, ConnectionPoint *cp,
+		  HandleMoveReason reason, ModifierKeys modifiers)
 {
   assert(actor!=NULL);
   assert(handle!=NULL);
   assert(to!=NULL);
 
   assert(handle->id < 8);
+
+  return NULL;
 }
 
-static void
+static ObjectChange*
 actor_move(Actor *actor, Point *to)
 {
   Element *elem = &actor->element;
@@ -236,11 +198,14 @@ actor_move(Actor *actor, Point *to)
   elem->corner.y -= elem->height / 2.0;
 
   actor_update_data(actor);
+
+  return NULL;
 }
 
 static void
-actor_draw(Actor *actor, Renderer *renderer)
+actor_draw(Actor *actor, DiaRenderer *renderer)
 {
+  DiaRendererClass *renderer_ops = DIA_RENDERER_GET_CLASS (renderer);
   Element *elem;
   real x, y, w, h;
   real r, r1;  
@@ -256,9 +221,9 @@ actor_draw(Actor *actor, Renderer *renderer)
   w = elem->width;
   h = elem->height;
   
-  renderer->ops->set_fillstyle(renderer, FILLSTYLE_SOLID);
-  renderer->ops->set_linewidth(renderer, ACTOR_LINEWIDTH);
-  renderer->ops->set_linestyle(renderer, LINESTYLE_SOLID);
+  renderer_ops->set_fillstyle(renderer, FILLSTYLE_SOLID);
+  renderer_ops->set_linewidth(renderer, ACTOR_LINEWIDTH);
+  renderer_ops->set_linestyle(renderer, LINESTYLE_SOLID);
 
   r = ACTOR_HEAD;
   r1 = 2*r;
@@ -268,40 +233,40 @@ actor_draw(Actor *actor, Renderer *renderer)
   cb.y = ch.y + r1 + r;
   
   /* head */
-  renderer->ops->fill_ellipse(renderer, 
+  renderer_ops->fill_ellipse(renderer, 
 			     &ch,
 			     r, r,
-			     &color_white);
-  renderer->ops->draw_ellipse(renderer, 
+			     &actor->fill_color);
+  renderer_ops->draw_ellipse(renderer, 
 			     &ch,
 			     r, r,
-			     &color_black);  
+			     &actor->line_color);  
   
   /* Arms */
   p1.x = ch.x - r1;
   p2.x = ch.x + r1;
   p1.y = p2.y = ch.y + r;
-  renderer->ops->draw_line(renderer, 
+  renderer_ops->draw_line(renderer, 
 			   &p1, &p2,
-			   &color_black);
+			   &actor->line_color);
 
   p1.x = ch.x;
   p1.y = ch.y + r*0.5;
   /* body & legs  */
-  renderer->ops->draw_line(renderer, 
+  renderer_ops->draw_line(renderer, 
 			   &p1, &cb,
-			   &color_black);
+			   &actor->line_color);
 
   p2.x = ch.x - r1;
   p2.y = y + ACTOR_BODY;
-  renderer->ops->draw_line(renderer, 
+  renderer_ops->draw_line(renderer, 
 			   &cb, &p2,
-			   &color_black);
+			   &actor->line_color);
   
   p2.x =  ch.x + r1;
-  renderer->ops->draw_line(renderer, 
+  renderer_ops->draw_line(renderer, 
 			   &cb, &p2,
-			   &color_black);
+			   &actor->line_color);
   
   text_draw(actor->text, renderer);
 }
@@ -310,40 +275,49 @@ static void
 actor_update_data(Actor *actor)
 {
   Element *elem = &actor->element;
-  Object *obj = &elem->object;
+  DiaObject *obj = &elem->object;
   Rectangle text_box;
   Point p;
   
+  text_calc_boundingbox(actor->text, &text_box);
+
   elem->width = ACTOR_WIDTH + ACTOR_MARGIN_X;
-  elem->height = ACTOR_HEIGHT;
+  elem->height = ACTOR_HEIGHT+actor->text->height;
 
   /* Update connections: */
   actor->connections[0].pos.x = elem->corner.x;
   actor->connections[0].pos.y = elem->corner.y;
+  actor->connections[0].directions = DIR_NORTH|DIR_WEST;
   actor->connections[1].pos.x = elem->corner.x + elem->width / 2.0;
   actor->connections[1].pos.y = elem->corner.y;
+  actor->connections[1].directions = DIR_NORTH;
   actor->connections[2].pos.x = elem->corner.x + elem->width;
   actor->connections[2].pos.y = elem->corner.y;
+  actor->connections[2].directions = DIR_NORTH|DIR_EAST;
   actor->connections[3].pos.x = elem->corner.x;
   actor->connections[3].pos.y = elem->corner.y + elem->height / 2.0;
+  actor->connections[3].directions = DIR_WEST;
   actor->connections[4].pos.x = elem->corner.x + elem->width;
   actor->connections[4].pos.y = elem->corner.y + elem->height / 2.0;
+  actor->connections[4].directions = DIR_EAST;
   actor->connections[5].pos.x = elem->corner.x;
   actor->connections[5].pos.y = elem->corner.y + elem->height;
+  actor->connections[5].directions = DIR_SOUTH|DIR_WEST;
   actor->connections[6].pos.x = elem->corner.x + elem->width / 2.0;
   actor->connections[6].pos.y = elem->corner.y + elem->height;
+  actor->connections[6].directions = DIR_SOUTH;
   actor->connections[7].pos.x = elem->corner.x + elem->width;
   actor->connections[7].pos.y = elem->corner.y + elem->height;
+  actor->connections[7].directions = DIR_SOUTH|DIR_EAST;
   
   element_update_boundingbox(elem);
 
   p = elem->corner;
   p.x += elem->width/2;
-  p.y +=  ACTOR_HEIGHT + actor->text->height;
+  p.y +=  ACTOR_HEIGHT + actor->text->ascent;
   text_set_position(actor->text, &p);
 
   /* Add bounding box for text: */
-  text_calc_boundingbox(actor->text, &text_box);
   rectangle_union(&obj->bounding_box, &text_box);
 
   obj->position = elem->corner;
@@ -353,7 +327,7 @@ actor_update_data(Actor *actor)
   element_update_handles(elem);
 }
 
-static Object *
+static DiaObject *
 actor_create(Point *startpoint,
 	       void *user_data,
   	       Handle **handle1,
@@ -361,9 +335,9 @@ actor_create(Point *startpoint,
 {
   Actor *actor;
   Element *elem;
-  Object *obj;
+  DiaObject *obj;
   Point p;
-  Font *font;
+  DiaFont *font;
   int i;
   
   actor = g_malloc0(sizeof(Actor));
@@ -376,12 +350,19 @@ actor_create(Point *startpoint,
   elem->width = ACTOR_WIDTH;
   elem->height = ACTOR_HEIGHT;
 
-  font = font_getfont("Helvetica");
+  actor->line_color = attributes_get_foreground();
+  actor->fill_color = attributes_get_background();
+
+  font = dia_font_new_from_style (DIA_FONT_SANS, 0.8);
   p = *startpoint;
   p.x += ACTOR_MARGIN_X;
-  p.y += ACTOR_HEIGHT - font_descent(font, 0.8);
+  p.y += ACTOR_HEIGHT - dia_font_descent(_("Actor"),font, 0.8);
+
+  actor->text = new_text(_("Actor"), 
+                         font, 0.8, &p, &color_black, ALIGN_CENTER);
+  dia_font_unref(font);
   
-  actor->text = new_text(_("Actor"), font, 0.8, &p, &color_black, ALIGN_CENTER);
+  text_get_attributes(actor->text,&actor->attrs);
   
   element_init(elem, 8, 8);
   
@@ -410,82 +391,10 @@ actor_destroy(Actor *actor)
   element_destroy(&actor->element);
 }
 
-static Object *
-actor_copy(Actor *actor)
-{
-  int i;
-  Actor *newactor;
-  Element *elem, *newelem;
-  Object *newobj;
-  
-  elem = &actor->element;
-  
-  newactor = g_malloc0(sizeof(Actor));
-  newelem = &newactor->element;
-  newobj = &newelem->object;
-
-  element_copy(elem, newelem);
-
-  newactor->text = text_copy(actor->text);
-  
-  for (i=0;i<8;i++) {
-    newobj->connections[i] = &newactor->connections[i];
-    newactor->connections[i].object = newobj;
-    newactor->connections[i].connected = NULL;
-    newactor->connections[i].pos = actor->connections[i].pos;
-    newactor->connections[i].last_pos = actor->connections[i].last_pos;
-  }
-  actor_update_data(newactor);
-  
-  return &newactor->element.object;
-}
-
-
-static void
-actor_save(Actor *actor, ObjectNode obj_node, const char *filename)
-{
-  element_save(&actor->element, obj_node);
-
-  data_add_text(new_attribute(obj_node, "text"),
-		actor->text);
-}
-
-static Object *
+static DiaObject *
 actor_load(ObjectNode obj_node, int version, const char *filename)
 {
-  Actor *actor;
-  Element *elem;
-  Object *obj;
-  int i;
-  AttributeNode attr;
-  
-  actor = g_malloc0(sizeof(Actor));
-  elem = &actor->element;
-  obj = &elem->object;
-  
-  obj->type = &actor_type;
-  obj->ops = &actor_ops;
-
-  element_load(elem, obj_node);
-  
-  attr = object_find_attribute(obj_node, "text");
-  if (attr != NULL)
-      actor->text = data_text(attribute_first_data(attr));
-
-  element_init(elem, 8, 8);
-
-  for (i=0;i<8;i++) {
-    obj->connections[i] = &actor->connections[i];
-    actor->connections[i].object = obj;
-    actor->connections[i].connected = NULL;
-  }
-  elem->extra_spacing.border_trans = ACTOR_LINEWIDTH/2.0;
-  actor_update_data(actor);
-
-  for (i=0;i<8;i++) {
-    obj->handles[i]->type = HANDLE_NON_MOVABLE;
-  }
-
-  return &actor->element.object;
+  return object_load_using_properties(&actor_type,
+                                      obj_node,version,filename);
 }
 
