@@ -16,6 +16,8 @@
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 
+/* DO NOT USE THIS OBJECT AS A BASIS FOR A NEW OBJECT. */
+
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
@@ -28,12 +30,13 @@
 #include "intl.h"
 #include "object.h"
 #include "orth_conn.h"
-#include "render.h"
+#include "diarenderer.h"
 #include "attributes.h"
 #include "arrows.h"
-
 #include "uml.h"
 
+#include "properties.h"
+
 #include "pixmaps/association.xpm"
 
 typedef struct _Association Association;
@@ -53,10 +56,14 @@ typedef enum {
 } AggregateType;
 
 typedef struct _AssociationEnd {
-  char *role; /* Can be NULL */
-  char *multiplicity; /* Can be NULL */
+  gchar *role; /* Can be NULL */
+  gchar *multiplicity; /* Can be NULL */
   Point text_pos;
   real text_width;
+  real role_ascent;
+  real role_descent;
+  real multi_ascent;
+  real multi_descent;
   Alignment text_align;
   
   int arrow;
@@ -66,12 +73,12 @@ typedef struct _AssociationEnd {
 struct _AssociationState {
   ObjectState obj_state;
 
-  char *name;
+  gchar *name;
   AssociationDirection direction;
 
   struct {
-    char *role;
-    char *multiplicity;
+    gchar *role;
+    gchar *multiplicity;
     int arrow;
     AggregateType aggregate;
   } end[2];
@@ -84,8 +91,10 @@ struct _Association {
   Point text_pos;
   Alignment text_align;
   real text_width;
-  
-  char *name;
+  real ascent;
+  real descent;
+    
+  gchar *name;
   AssociationDirection direction;
 
   AssociationEnd end[2];
@@ -112,27 +121,32 @@ struct _AssociationPropertiesDialog {
 #define ASSOCIATION_WIDTH 0.1
 #define ASSOCIATION_TRIANGLESIZE 0.8
 #define ASSOCIATION_DIAMONDLEN 1.4
+#define ASSOCIATION_DIAMONDWIDTH 0.7
 #define ASSOCIATION_FONTHEIGHT 0.8
 
-static Font *assoc_font = NULL;
+static DiaFont *assoc_font = NULL;
 
 static real association_distance_from(Association *assoc, Point *point);
 static void association_select(Association *assoc, Point *clicked_point,
-			       Renderer *interactive_renderer);
-static void association_move_handle(Association *assoc, Handle *handle,
-				    Point *to, HandleMoveReason reason, ModifierKeys modifiers);
-static void association_move(Association *assoc, Point *to);
-static void association_draw(Association *assoc, Renderer *renderer);
-static Object *association_create(Point *startpoint,
+			       DiaRenderer *interactive_renderer);
+static ObjectChange* association_move_handle(Association *assoc, Handle *handle,
+					     Point *to, ConnectionPoint *cp,
+					     HandleMoveReason reason, ModifierKeys modifiers);
+static ObjectChange* association_move(Association *assoc, Point *to);
+static void association_draw(Association *assoc, DiaRenderer *renderer);
+static DiaObject *association_create(Point *startpoint,
 				  void *user_data,
 				  Handle **handle1,
 				  Handle **handle2);
 static void association_destroy(Association *assoc);
-static Object *association_copy(Association *assoc);
-static GtkWidget *association_get_properties(Association *assoc);
+static DiaObject *association_copy(Association *assoc);
+static GtkWidget *association_get_properties(Association *assoc, gboolean is_default);
 static ObjectChange *association_apply_properties(Association *assoc);
 static DiaMenu *association_get_object_menu(Association *assoc,
 					    Point *clickedpoint);
+static PropDescription *association_describe_props(Association *assoc);
+static void association_get_props(Association *assoc, GPtrArray *props);
+static void association_set_props(Association *assoc, GPtrArray *props);
 
 static AssociationState *association_get_state(Association *assoc);
 static void association_set_state(Association *assoc,
@@ -140,10 +154,11 @@ static void association_set_state(Association *assoc,
 
 static void association_save(Association *assoc, ObjectNode obj_node,
 			     const char *filename);
-static Object *association_load(ObjectNode obj_node, int version,
+static DiaObject *association_load(ObjectNode obj_node, int version,
 				const char *filename);
 
 static void association_update_data(Association *assoc);
+static coord get_aggregate_pos_diff(AssociationEnd *end);
 
 static ObjectTypeOps association_type_ops =
 {
@@ -152,13 +167,15 @@ static ObjectTypeOps association_type_ops =
   (SaveFunc)   association_save
 };
 
-ObjectType association_type =
+DiaObjectType association_type =
 {
   "UML - Association",   /* name */
   0,                      /* version */
   (char **) association_xpm,  /* pixmap */
   
-  &association_type_ops       /* ops */
+  &association_type_ops,      /* ops */
+  NULL,                 /* pixmap_file */
+  0                     /* default_user_data */
 };
 
 static ObjectOps association_ops = {
@@ -171,9 +188,49 @@ static ObjectOps association_ops = {
   (MoveHandleFunc)      association_move_handle,
   (GetPropertiesFunc)   association_get_properties,
   (ApplyPropertiesFunc) association_apply_properties,
-  (ObjectMenuFunc)      association_get_object_menu
+  (ObjectMenuFunc)      association_get_object_menu,
+  (DescribePropsFunc)   association_describe_props,
+  (GetPropsFunc)        association_get_props,
+  (SetPropsFunc)        association_set_props
 };
 
+static PropDescription association_props[] = {
+  ORTHCONN_COMMON_PROPERTIES,
+  { "name", PROP_TYPE_STRING, PROP_FLAG_VISIBLE,
+    N_("Name:"), NULL, NULL },
+  PROP_DESC_END
+};
+
+static PropDescription *
+association_describe_props(Association *assoc)
+{
+  if (association_props[0].quark == 0) {
+    prop_desc_list_calculate_quarks(association_props);
+  }
+  return association_props;
+}
+
+static PropOffset association_offsets[] = {
+  ORTHCONN_COMMON_PROPERTIES_OFFSETS,
+  { "name", PROP_TYPE_STRING, offsetof(Association, name) },
+  { NULL, 0, 0 }
+};
+
+static void
+association_get_props(Association *assoc, GPtrArray *props)
+{
+  object_get_props_from_offsets(&assoc->orth.object,
+                                association_offsets,props);
+}
+
+static void
+association_set_props(Association *assoc, GPtrArray *props)
+{
+  object_set_props_from_offsets(&assoc->orth.object, 
+                                association_offsets, props);
+  association_update_data(assoc);
+}
+
 static real
 association_distance_from(Association *assoc, Point *point)
 {
@@ -183,57 +240,77 @@ association_distance_from(Association *assoc, Point *point)
 
 static void
 association_select(Association *assoc, Point *clicked_point,
-		  Renderer *interactive_renderer)
+		  DiaRenderer *interactive_renderer)
 {
   orthconn_update_data(&assoc->orth);
 }
 
-static void
+static ObjectChange*
 association_move_handle(Association *assoc, Handle *handle,
-		       Point *to, HandleMoveReason reason, ModifierKeys modifiers)
+			Point *to, ConnectionPoint *cp,
+			HandleMoveReason reason, ModifierKeys modifiers)
 {
+  ObjectChange *change;
   assert(assoc!=NULL);
   assert(handle!=NULL);
   assert(to!=NULL);
   
-  orthconn_move_handle(&assoc->orth, handle, to, reason);
+  change = orthconn_move_handle(&assoc->orth, handle, to, cp, reason, modifiers);
   association_update_data(assoc);
+
+  return change;
 }
 
-static void
+static ObjectChange*
 association_move(Association *assoc, Point *to)
 {
-  orthconn_move(&assoc->orth, to);
+  ObjectChange *change;
+
+  change = orthconn_move(&assoc->orth, to);
   association_update_data(assoc);
+
+  return change;
 }
 
 static void
-association_draw(Association *assoc, Renderer *renderer)
+association_draw(Association *assoc, DiaRenderer *renderer)
 {
+  DiaRendererClass *renderer_ops = DIA_RENDERER_GET_CLASS (renderer);
   OrthConn *orth = &assoc->orth;
   Point *points;
   Point poly[3];
   int n,i;
   Point pos;
+  Arrow startarrow, endarrow;
   
   points = &orth->points[0];
   n = orth->numpoints;
   
-  renderer->ops->set_linewidth(renderer, ASSOCIATION_WIDTH);
-  renderer->ops->set_linestyle(renderer, LINESTYLE_SOLID);
-  renderer->ops->set_linejoin(renderer, LINEJOIN_MITER);
-  renderer->ops->set_linecaps(renderer, LINECAPS_BUTT);
-
-  renderer->ops->draw_polyline(renderer, points, n, &color_black);
-
+  renderer_ops->set_linewidth(renderer, ASSOCIATION_WIDTH);
+  renderer_ops->set_linestyle(renderer, LINESTYLE_SOLID);
+  renderer_ops->set_linejoin(renderer, LINEJOIN_MITER);
+  renderer_ops->set_linecaps(renderer, LINECAPS_BUTT);
+  
   if (assoc->end[0].arrow) {
-    arrow_draw(renderer, ARROW_LINES,
-	       &points[0], &points[1],
-	       ASSOCIATION_TRIANGLESIZE, ASSOCIATION_TRIANGLESIZE,
-	       ASSOCIATION_WIDTH,
-	       &color_black, &color_white);
+    startarrow.type = ARROW_LINES;
+  } else {
+    startarrow.type = ARROW_NONE;
   }
+  if (assoc->end[1].arrow) {
+    endarrow.type = ARROW_LINES;
+  } else {
+    endarrow.type = ARROW_NONE;
+  }
+  startarrow.length = ASSOCIATION_TRIANGLESIZE;
+  startarrow.width = ASSOCIATION_TRIANGLESIZE;
+  endarrow.length = ASSOCIATION_TRIANGLESIZE;
+  endarrow.width = ASSOCIATION_TRIANGLESIZE;
+  renderer_ops->draw_polyline_with_arrows(renderer, points, n,
+					   ASSOCIATION_WIDTH,
+					   &color_black,
+					   &startarrow, &endarrow);
 
+  /* Shouldn't these be instead of the normal arrows? */
   switch (assoc->end[0].aggregate) {
   case AGGREGATE_NORMAL:
     arrow_draw(renderer, ARROW_HOLLOW_DIAMOND,
@@ -254,14 +331,6 @@ association_draw(Association *assoc, Renderer *renderer)
     break;
   }
 
-  if (assoc->end[1].arrow) {
-    arrow_draw(renderer, ARROW_LINES,
-	       &points[n-1], &points[n-2],
-	       ASSOCIATION_TRIANGLESIZE, ASSOCIATION_TRIANGLESIZE,
-	       ASSOCIATION_WIDTH,
-	       &color_black, &color_white);
-  }
-
   switch (assoc->end[1].aggregate) {
   case AGGREGATE_NORMAL:
     arrow_draw(renderer, ARROW_HOLLOW_DIAMOND,
@@ -283,17 +352,17 @@ association_draw(Association *assoc, Renderer *renderer)
   }
 
   /* Name: */
-  renderer->ops->set_font(renderer, assoc_font, ASSOCIATION_FONTHEIGHT);
+  renderer_ops->set_font(renderer, assoc_font, ASSOCIATION_FONTHEIGHT);
  
   if (assoc->name != NULL) {
     pos = assoc->text_pos;
-    renderer->ops->draw_string(renderer, assoc->name,
+    renderer_ops->draw_string(renderer, assoc->name,
 			       &pos, assoc->text_align,
 			       &color_black);
   }
 
   /* Direction: */
-  renderer->ops->set_fillstyle(renderer, FILLSTYLE_SOLID);
+  renderer_ops->set_fillstyle(renderer, FILLSTYLE_SOLID);
 
   switch (assoc->direction) {
   case ASSOC_NODIR:
@@ -307,7 +376,7 @@ association_draw(Association *assoc, Renderer *renderer)
     poly[1].y = poly[0].y - ASSOCIATION_FONTHEIGHT*0.5;
     poly[2].x = poly[0].x + ASSOCIATION_FONTHEIGHT*0.5;
     poly[2].y = poly[0].y - ASSOCIATION_FONTHEIGHT*0.5*0.5;
-    renderer->ops->fill_polygon(renderer, poly, 3, &color_black);
+    renderer_ops->fill_polygon(renderer, poly, 3, &color_black);
     break;
   case ASSOC_LEFT:
     poly[0].x = assoc->text_pos.x - 0.2;
@@ -318,7 +387,7 @@ association_draw(Association *assoc, Renderer *renderer)
     poly[1].y = poly[0].y - ASSOCIATION_FONTHEIGHT*0.5;
     poly[2].x = poly[0].x - ASSOCIATION_FONTHEIGHT*0.5;
     poly[2].y = poly[0].y - ASSOCIATION_FONTHEIGHT*0.5*0.5;
-    renderer->ops->fill_polygon(renderer, poly, 3, &color_black);
+    renderer_ops->fill_polygon(renderer, poly, 3, &color_black);
     break;
   }
 
@@ -328,13 +397,13 @@ association_draw(Association *assoc, Renderer *renderer)
     pos = end->text_pos;
 
     if (end->role != NULL) {
-      renderer->ops->draw_string(renderer, end->role,
+      renderer_ops->draw_string(renderer, end->role,
 				 &pos, end->text_align,
 				 &color_black);
       pos.y += ASSOCIATION_FONTHEIGHT;
     }
     if (end->multiplicity != NULL) {
-      renderer->ops->draw_string(renderer, end->multiplicity,
+      renderer_ops->draw_string(renderer, end->multiplicity,
 				 &pos, end->text_align,
 				 &color_black);
     }
@@ -387,9 +456,15 @@ association_set_state(Association *assoc, AssociationState *state)
   g_free(assoc->name);
   assoc->name = state->name;
   assoc->text_width = 0.0;
+  assoc->ascent = 0.0;
+  assoc->descent = 0.0;
   if (assoc->name != NULL) {
     assoc->text_width =
-      font_string_width(assoc->name, assoc_font, ASSOCIATION_FONTHEIGHT);
+      dia_font_string_width(assoc->name, assoc_font, ASSOCIATION_FONTHEIGHT);
+    assoc->ascent = 
+      dia_font_ascent(assoc->name, assoc_font, ASSOCIATION_FONTHEIGHT);
+    assoc->descent =     
+      dia_font_descent(assoc->name, assoc_font, ASSOCIATION_FONTHEIGHT);
   } 
   
   assoc->direction = state->direction;
@@ -404,14 +479,27 @@ association_set_state(Association *assoc, AssociationState *state)
     end->aggregate = state->end[i].aggregate;
 
     end->text_width = 0.0;
+    end->role_ascent = 0.0;
+    end->role_descent = 0.0;
+    end->multi_ascent = 0.0;
+    end->multi_descent = 0.0;
     if (end->role != NULL) {
       end->text_width = 
-	font_string_width(end->role, assoc_font, ASSOCIATION_FONTHEIGHT);
+          dia_font_string_width(end->role, assoc_font, ASSOCIATION_FONTHEIGHT);
+      end->role_ascent =
+          dia_font_ascent(end->role, assoc_font, ASSOCIATION_FONTHEIGHT);
+      end->role_descent =
+          dia_font_ascent(end->role, assoc_font, ASSOCIATION_FONTHEIGHT);          
     }
     if (end->multiplicity != NULL) {
-      end->text_width =
-	MAX(end->text_width,
-	    font_string_width(end->multiplicity, assoc_font, ASSOCIATION_FONTHEIGHT) );
+      end->text_width = MAX(end->text_width,
+                            dia_font_string_width(end->multiplicity,
+                                                  assoc_font,
+                                                  ASSOCIATION_FONTHEIGHT) );
+      end->role_ascent = dia_font_ascent(end->multiplicity,
+                                         assoc_font,ASSOCIATION_FONTHEIGHT);
+      end->role_descent = dia_font_descent(end->multiplicity,
+                                           assoc_font,ASSOCIATION_FONTHEIGHT);
     }
   }
 
@@ -424,9 +512,12 @@ association_set_state(Association *assoc, AssociationState *state)
 static void
 association_update_data(Association *assoc)
 {
+        /* FIXME: The ascent and descent computation logic here is
+           fundamentally slow. */
+
   OrthConn *orth = &assoc->orth;
-  Object *obj = &orth->object;
-  OrthConnBBExtras *extra = &orth->extra_spacing;
+  DiaObject *obj = &orth->object;
+  PolyBBExtras *extra = &orth->extra_spacing;
   int num_segm, i, n;
   Point *points;
   Rectangle rect;
@@ -444,6 +535,11 @@ association_update_data(Association *assoc)
                          ASSOCIATION_WIDTH/2.0:
                          (ASSOCIATION_WIDTH + ASSOCIATION_DIAMONDLEN)/2.0);
 
+  if (assoc->end[0].arrow)
+    extra->start_trans = MAX(extra->start_trans, ASSOCIATION_TRIANGLESIZE);
+  if (assoc->end[1].arrow)
+    extra->end_trans = MAX(extra->end_trans, ASSOCIATION_TRIANGLESIZE);
+
   orthconn_update_boundingbox(orth);
   
   /* Calc text pos: */
@@ -460,13 +556,12 @@ association_update_data(Association *assoc)
   case HORIZONTAL:
     assoc->text_align = ALIGN_CENTER;
     assoc->text_pos.x = 0.5*(points[i].x+points[i+1].x);
-    assoc->text_pos.y = points[i].y - font_descent(assoc_font, ASSOCIATION_FONTHEIGHT);
+    assoc->text_pos.y = points[i].y - assoc->descent;
     break;
   case VERTICAL:
     assoc->text_align = ALIGN_LEFT;
     assoc->text_pos.x = points[i].x + 0.1;
-    assoc->text_pos.y =
-      0.5*(points[i].y+points[i+1].y) - font_descent(assoc_font, ASSOCIATION_FONTHEIGHT);
+    assoc->text_pos.y = 0.5*(points[i].y+points[i+1].y) - assoc->descent;
     break;
   }
 
@@ -475,56 +570,78 @@ association_update_data(Association *assoc)
   if (assoc->text_align == ALIGN_CENTER)
     rect.left -= assoc->text_width/2.0;
   rect.right = rect.left + assoc->text_width;
-  rect.top = assoc->text_pos.y - font_ascent(assoc_font, ASSOCIATION_FONTHEIGHT);
+  rect.top = assoc->text_pos.y - assoc->ascent;
   rect.bottom = rect.top + ASSOCIATION_FONTHEIGHT;
 
   rectangle_union(&obj->bounding_box, &rect);
 
 
   /* Update the text-points of the ends: */
-  /* END 1: */
+  /* END 0: */
   end = &assoc->end[0];
   end->text_pos = points[0];
   switch (assoc->orth.orientation[0]) {
   case HORIZONTAL:
-    end->text_pos.y -= font_descent(assoc_font, ASSOCIATION_FONTHEIGHT);
-    if (points[0].x < points[1].x)
+    end->text_pos.y -= end->role_descent;
+    if (points[0].x < points[1].x) {
       end->text_align = ALIGN_LEFT;
-    else
-      end->text_align = ALIGN_RIGHT;
+      end->text_pos.x += get_aggregate_pos_diff(end);
+    } else {
+      end->text_align = ALIGN_RIGHT;    
+      end->text_pos.x -= get_aggregate_pos_diff(end);
+    }
     break;
   case VERTICAL:
-    end->text_pos.y += font_ascent(assoc_font, ASSOCIATION_FONTHEIGHT);
+    end->text_pos.y += end->role_ascent;
+    end->text_pos.y += ASSOCIATION_FONTHEIGHT;
+    end->text_pos.x += ASSOCIATION_DIAMONDWIDTH / 2;
+    if (points[0].y > points[1].y) {
+      if (end->role!=NULL)
+          end->text_pos.y -= ASSOCIATION_FONTHEIGHT;
+      if (end->multiplicity!=NULL)
+          end->text_pos.y -= ASSOCIATION_FONTHEIGHT;
+      end->text_pos.y -= get_aggregate_pos_diff(end);
+    } else {
+      end->text_pos.y += get_aggregate_pos_diff(end);
+    }
     end->text_align = ALIGN_LEFT;
     break;
   }
   /* Add the text recangle to the bounding box: */
   rect.left = end->text_pos.x;
   rect.right = rect.left + end->text_width;
-  rect.top = end->text_pos.y - font_ascent(assoc_font, ASSOCIATION_FONTHEIGHT);
+  rect.top = end->text_pos.y - end->role_ascent;
   rect.bottom = rect.top + 2*ASSOCIATION_FONTHEIGHT;
   
   rectangle_union(&obj->bounding_box, &rect);
 
-  /* END 2: */
+  /* END 1: */
   end = &assoc->end[1];
   n = assoc->orth.numpoints - 1;
   end->text_pos = points[n];
   switch (assoc->orth.orientation[n-1]) {
   case HORIZONTAL:
-    end->text_pos.y -= font_descent(assoc_font, ASSOCIATION_FONTHEIGHT);
-    if (points[n].x < points[n-1].x)
+    end->text_pos.y -= end->role_descent;
+    if (points[n].x < points[n-1].x) {
       end->text_align = ALIGN_LEFT;
-    else
+      end->text_pos.x += get_aggregate_pos_diff(end);
+    } else {
       end->text_align = ALIGN_RIGHT;
+      end->text_pos.x -= get_aggregate_pos_diff(end);
+    }
     break;
   case VERTICAL:
-    end->text_pos.y += font_ascent(assoc_font, ASSOCIATION_FONTHEIGHT);
+    end->text_pos.y += end->role_ascent;
+    end->text_pos.y += ASSOCIATION_FONTHEIGHT;
+    end->text_pos.x += ASSOCIATION_DIAMONDWIDTH / 2;
     if (points[n].y > points[n-1].y) {
       if (end->role!=NULL)
 	end->text_pos.y -= ASSOCIATION_FONTHEIGHT;
       if (end->multiplicity!=NULL)
 	end->text_pos.y -= ASSOCIATION_FONTHEIGHT;
+      end->text_pos.y -= get_aggregate_pos_diff(end);
+    } else {
+      end->text_pos.y += get_aggregate_pos_diff(end);
     }
     end->text_align = ALIGN_LEFT;
     break;
@@ -532,13 +649,30 @@ association_update_data(Association *assoc)
   /* Add the text rectangle to the bounding box: */
   rect.left = end->text_pos.x;
   rect.right = rect.left + end->text_width;
-  rect.top = end->text_pos.y - font_ascent(assoc_font, ASSOCIATION_FONTHEIGHT);
+  rect.top = end->text_pos.y - end->role_ascent;
   rect.bottom = rect.top + 2*ASSOCIATION_FONTHEIGHT;
   
   rectangle_union(&obj->bounding_box, &rect);
 }
 
-static Object *
+static coord get_aggregate_pos_diff(AssociationEnd *end)
+{
+  coord width=0;
+  if(end->arrow){
+    width = ASSOCIATION_TRIANGLESIZE;
+  }
+  switch(end->aggregate){
+  case AGGREGATE_COMPOSITION:
+  case AGGREGATE_NORMAL:
+    if(width!=0) width = MAX(ASSOCIATION_TRIANGLESIZE, ASSOCIATION_DIAMONDLEN);
+    else width = ASSOCIATION_DIAMONDLEN;
+  case AGGREGATE_NONE:
+    break;
+  }
+  return width;
+}
+
+static DiaObject *
 association_create(Point *startpoint,
 	       void *user_data,
   	       Handle **handle1,
@@ -546,12 +680,12 @@ association_create(Point *startpoint,
 {
   Association *assoc;
   OrthConn *orth;
-  Object *obj;
+  DiaObject *obj;
   int i;
   int user_d;
 
   if (assoc_font == NULL) {
-    assoc_font = font_getfont("Courier");
+    assoc_font = dia_font_new_from_style(DIA_FONT_MONOSPACE, ASSOCIATION_FONTHEIGHT);
   }
   
   assoc = g_malloc0(sizeof(Association));
@@ -596,7 +730,7 @@ association_create(Point *startpoint,
 }
 
 static ObjectChange *
-association_add_segment_callback(Object *obj, Point *clicked, gpointer data)
+association_add_segment_callback(DiaObject *obj, Point *clicked, gpointer data)
 {
   ObjectChange *change;
   change = orthconn_add_segment((OrthConn *)obj, clicked);
@@ -605,7 +739,7 @@ association_add_segment_callback(Object *obj, Point *clicked, gpointer data)
 }
 
 static ObjectChange *
-association_delete_segment_callback(Object *obj, Point *clicked, gpointer data)
+association_delete_segment_callback(DiaObject *obj, Point *clicked, gpointer data)
 {
   ObjectChange *change;
   change = orthconn_delete_segment((OrthConn *)obj, clicked);
@@ -617,6 +751,7 @@ association_delete_segment_callback(Object *obj, Point *clicked, gpointer data)
 static DiaMenuItem object_menu_items[] = {
   { N_("Add segment"), association_add_segment_callback, NULL, 1 },
   { N_("Delete segment"), association_delete_segment_callback, NULL, 1 },
+  ORTHCONN_COMMON_MENUS,
 };
 
 static DiaMenu object_menu = {
@@ -635,6 +770,7 @@ association_get_object_menu(Association *assoc, Point *clickedpoint)
   /* Set entries sensitive/selected etc here */
   object_menu_items[0].active = orthconn_can_add_segment(orth, clickedpoint);
   object_menu_items[1].active = orthconn_can_delete_segment(orth, clickedpoint);
+  orthconn_update_object_menu(orth, clickedpoint, &object_menu_items[2]);
   return &object_menu;
 }
 
@@ -658,12 +794,12 @@ association_destroy(Association *assoc)
   }
 }
 
-static Object *
+static DiaObject *
 association_copy(Association *assoc)
 {
   Association *newassoc;
   OrthConn *orth, *neworth;
-  Object *newobj;
+  DiaObject *newobj;
   int i;
   
   orth = &assoc->orth;
@@ -674,14 +810,14 @@ association_copy(Association *assoc)
 
   orthconn_copy(orth, neworth);
 
-  newassoc->name = (assoc->name != NULL)? strdup(assoc->name):NULL;
+  newassoc->name = (assoc->name != NULL)? g_strdup(assoc->name):NULL;
   newassoc->direction = assoc->direction;
   for (i=0;i<2;i++) {
     newassoc->end[i] = assoc->end[i];
     newassoc->end[i].role =
-      (assoc->end[i].role != NULL)?strdup(assoc->end[i].role):NULL;
+      (assoc->end[i].role != NULL)?g_strdup(assoc->end[i].role):NULL;
     newassoc->end[i].multiplicity =
-      (assoc->end[i].multiplicity != NULL)?strdup(assoc->end[i].multiplicity):NULL;
+      (assoc->end[i].multiplicity != NULL)?g_strdup(assoc->end[i].multiplicity):NULL;
   }
 
   newassoc->text_width = assoc->text_width;
@@ -723,18 +859,19 @@ association_save(Association *assoc, ObjectNode obj_node,
   }
 }
 
-static Object *
+static DiaObject *
 association_load(ObjectNode obj_node, int version, const char *filename)
 {
   Association *assoc;
   AttributeNode attr;
   DataNode composite;
   OrthConn *orth;
-  Object *obj;
+  DiaObject *obj;
   int i;
   
   if (assoc_font == NULL) {
-    assoc_font = font_getfont("Courier");
+	  assoc_font = dia_font_new_from_style(DIA_FONT_MONOSPACE,
+                                         ASSOCIATION_FONTHEIGHT);
   }
 
   assoc = g_new0(Association, 1);
@@ -755,7 +892,7 @@ association_load(ObjectNode obj_node, int version, const char *filename)
   assoc->text_width = 0.0;
   if (assoc->name != NULL) {
     assoc->text_width =
-      font_string_width(assoc->name, assoc_font, ASSOCIATION_FONTHEIGHT);
+      dia_font_string_width(assoc->name, assoc_font, ASSOCIATION_FONTHEIGHT);
   }
 
   assoc->direction = ASSOC_NODIR;
@@ -769,13 +906,22 @@ association_load(ObjectNode obj_node, int version, const char *filename)
 
     assoc->end[i].role = NULL;
     attr = composite_find_attribute(composite, "role");
-    if (attr != NULL)
+    if (attr != NULL) {
       assoc->end[i].role = data_string(attribute_first_data(attr));
+    }
+    if (assoc->end[i].role != NULL && !strcmp(assoc->end[i].role, "")) {
+      assoc->end[i].role = NULL;
+    }
     
     assoc->end[i].multiplicity = NULL;
     attr = composite_find_attribute(composite, "multiplicity");
-    if (attr != NULL)
+    if (attr != NULL) {
       assoc->end[i].multiplicity = data_string(attribute_first_data(attr));
+    }
+    if (assoc->end[i].multiplicity != NULL &&
+	!strcmp(assoc->end[i].multiplicity, "")) {
+      assoc->end[i].multiplicity = NULL;
+    }
     
     assoc->end[i].arrow = FALSE;
     attr = composite_find_attribute(composite, "arrow");
@@ -790,21 +936,21 @@ association_load(ObjectNode obj_node, int version, const char *filename)
     assoc->end[i].text_width = 0.0;
     if (assoc->end[i].role != NULL) {
       assoc->end[i].text_width = 
-	font_string_width(assoc->end[i].role, assoc_font,
-			  ASSOCIATION_FONTHEIGHT);
+          dia_font_string_width(assoc->end[i].role, assoc_font,
+                                ASSOCIATION_FONTHEIGHT);
     }
     if (assoc->end[i].multiplicity != NULL) {
       assoc->end[i].text_width =
-	MAX(assoc->end[i].text_width,
-	    font_string_width(assoc->end[i].multiplicity,
-			      assoc_font, ASSOCIATION_FONTHEIGHT) );
+          MAX(assoc->end[i].text_width,
+              dia_font_string_width(assoc->end[i].multiplicity,
+                                    assoc_font, ASSOCIATION_FONTHEIGHT) );
     }
     composite = data_next(composite);
   }
 
   assoc->properties_dialog = NULL;
   
-  association_update_data(assoc);
+  association_set_state(assoc, association_get_state(assoc));
 
   return &assoc->orth.object;
 }
@@ -813,7 +959,7 @@ static ObjectChange *
 association_apply_properties(Association *assoc)
 {
   AssociationPropertiesDialog *prop_dialog;
-  char *str;
+  const char *str;
   GtkWidget *menuitem;
   int i;
   ObjectState *old_state;
@@ -825,8 +971,8 @@ association_apply_properties(Association *assoc)
   /* Read from dialog and put in object: */
   g_free(assoc->name);
   str = gtk_entry_get_text(prop_dialog->name);
-  if (strlen(str) != 0)
-    assoc->name = strdup(str);
+  if (str && strlen (str) != 0)
+    assoc->name = g_strdup (str);
   else
     assoc->name = NULL;
 
@@ -834,7 +980,7 @@ association_apply_properties(Association *assoc)
 
   if (assoc->name != NULL) {
     assoc->text_width =
-      font_string_width(assoc->name, assoc_font, ASSOCIATION_FONTHEIGHT);
+      dia_font_string_width(assoc->name, assoc_font, ASSOCIATION_FONTHEIGHT);
   }
 
   menuitem = gtk_menu_get_active(prop_dialog->dir_menu);
@@ -846,16 +992,16 @@ association_apply_properties(Association *assoc)
     /* Role: */
     g_free(end->role);
     str = gtk_entry_get_text(prop_dialog->end[i].role);
-    if (strlen(str) != 0)
-      end->role = strdup(str);
+    if (str && strlen (str) != 0)
+      end->role = g_strdup (str);
     else
       end->role = NULL;
 
     /* Multiplicity: */
     g_free(end->multiplicity);
     str = gtk_entry_get_text(prop_dialog->end[i].multiplicity);
-    if (strlen(str) != 0)
-      end->multiplicity = strdup(str);
+    if (strlen (str) != 0)
+      end->multiplicity = g_strdup(str);
     else
       end->multiplicity = NULL;
 
@@ -863,12 +1009,13 @@ association_apply_properties(Association *assoc)
 
     if (end->role != NULL) {
       end->text_width = 
-	font_string_width(end->role, assoc_font, ASSOCIATION_FONTHEIGHT);
+          dia_font_string_width(end->role, assoc_font, ASSOCIATION_FONTHEIGHT);
     }
     if (end->multiplicity != NULL) {
       end->text_width =
-	MAX(end->text_width,
-	    font_string_width(end->multiplicity, assoc_font, ASSOCIATION_FONTHEIGHT) );
+          MAX(end->text_width,
+              dia_font_string_width(end->multiplicity,
+                                    assoc_font, ASSOCIATION_FONTHEIGHT) );
     }
 
     end->arrow = prop_dialog->end[i].draw_arrow->active;
@@ -881,7 +1028,7 @@ association_apply_properties(Association *assoc)
 
   }
 
-  association_update_data(assoc);
+  association_set_state(assoc, association_get_state(assoc));
   return new_object_state_change(&assoc->orth.object, old_state, 
 				 (GetStateFunc)association_get_state,
 				 (SetStateFunc)association_set_state);
@@ -897,7 +1044,7 @@ fill_in_dialog(Association *assoc)
 
   if (assoc->name != NULL)
     gtk_entry_set_text(prop_dialog->name, assoc->name);
-  else 
+  else
     gtk_entry_set_text(prop_dialog->name, "");
   
   gtk_option_menu_set_history(prop_dialog->dir_omenu, assoc->direction);
@@ -905,13 +1052,13 @@ fill_in_dialog(Association *assoc)
   for (i=0;i<2;i++) {
     if (assoc->end[i].role != NULL)
       gtk_entry_set_text(prop_dialog->end[i].role, assoc->end[i].role);
-    else 
+    else
       gtk_entry_set_text(prop_dialog->end[i].role, "");
     
     if (assoc->end[i].multiplicity != NULL)
-      gtk_entry_set_text(prop_dialog->end[i].multiplicity,
-			 assoc->end[i].multiplicity);
-    else 
+      gtk_entry_set_text(prop_dialog->end[i].multiplicity, 
+                         assoc->end[i].multiplicity);
+    else
       gtk_entry_set_text(prop_dialog->end[i].multiplicity, "");
 
     gtk_toggle_button_set_active(prop_dialog->end[i].draw_arrow,
@@ -946,7 +1093,7 @@ mutex_aggregate_callback(GtkWidget *widget,
 }
 
 static GtkWidget *
-association_get_properties(Association *assoc)
+association_get_properties(Association *assoc, gboolean is_default)
 {
   AssociationPropertiesDialog *prop_dialog;
   GtkWidget *dialog;
@@ -981,6 +1128,7 @@ association_get_properties(Association *assoc)
     entry = gtk_entry_new();
     prop_dialog->name = GTK_ENTRY(entry);
     gtk_box_pack_start (GTK_BOX (hbox), entry, TRUE, TRUE, 0);
+    gtk_widget_grab_focus(entry);
     gtk_widget_show (label);
     gtk_widget_show (entry);
     gtk_box_pack_start (GTK_BOX (dialog), hbox, TRUE, TRUE, 0);
