@@ -24,14 +24,13 @@
 #endif
 
 #include <assert.h>
-#include <gtk/gtk.h>
 #include <math.h>
 #include <string.h>
 
 #include "intl.h"
 #include "object.h"
 #include "element.h"
-#include "render.h"
+#include "diarenderer.h"
 #include "attributes.h"
 #include "text.h"
 #include "properties.h"
@@ -47,6 +46,10 @@ struct _Node
   Element element;
   ConnectionPoint connections[8];
   Text *name;
+  TextAttributes attrs;
+
+  Color line_color;
+  Color fill_color;
 };
 
 static const double NODE_BORDERWIDTH = 0.1;
@@ -57,37 +60,36 @@ static const double NODE_TEXT_MARGIN = 0.5;
 
 static real node_distance_from(Node *node, Point *point);
 static void node_select(Node *node, Point *clicked_point,
-				Renderer *interactive_renderer);
-static void node_move_handle(Node *node, Handle *handle,
-				     Point *to, HandleMoveReason reason, ModifierKeys modifiers);
-static void node_move(Node *node, Point *to);
-static void node_draw(Node *node, Renderer *renderer);
-static Object *node_create(Point *startpoint,
+				DiaRenderer *interactive_renderer);
+static ObjectChange* node_move_handle(Node *node, Handle *handle,
+				      Point *to, ConnectionPoint *cp,
+				      HandleMoveReason reason, ModifierKeys modifiers);
+static ObjectChange* node_move(Node *node, Point *to);
+static void node_draw(Node *node, DiaRenderer *renderer);
+static DiaObject *node_create(Point *startpoint,
 				   void *user_data,
 				   Handle **handle1,
 				   Handle **handle2);
 static void node_destroy(Node *node);
-static Object *node_copy(Node *node);
-
-static void node_save(Node *node, ObjectNode obj_node,
-			      const char *filename);
-static Object *node_load(ObjectNode obj_node, int version,
+static DiaObject *node_load(ObjectNode obj_node, int version,
 				 const char *filename);
 
 static PropDescription *node_describe_props(Node *node);
-static void node_get_props(Node *node, Property *props, guint nprops);
-static void node_set_props(Node *node, Property *props, guint nprops);
+static void node_get_props(Node *node, GPtrArray *props);
+static void node_set_props(Node *node, GPtrArray *props);
 
 static void node_update_data(Node *node);
 
 static ObjectTypeOps node_type_ops =
 {
   (CreateFunc) node_create,
-  (LoadFunc)   node_load,
-  (SaveFunc)   node_save
+  (LoadFunc)   node_load,/*using_properties*/     /* load */
+  (SaveFunc)   object_save_using_properties,      /* save */
+  (GetDefaultsFunc)   NULL, 
+  (ApplyDefaultsFunc) NULL
 };
 
-ObjectType node_type =
+DiaObjectType node_type =
 {
   "UML - Node",   /* name */
   0,                      /* version */
@@ -102,7 +104,7 @@ static ObjectOps node_ops =
   (DrawFunc)            node_draw,
   (DistanceFunc)        node_distance_from,
   (SelectFunc)          node_select,
-  (CopyFunc)            node_copy,
+  (CopyFunc)            object_copy_using_properties,
   (MoveFunc)            node_move,
   (MoveHandleFunc)      node_move_handle,
   (GetPropertiesFunc)   object_create_props_dialog,
@@ -115,93 +117,51 @@ static ObjectOps node_ops =
 
 static PropDescription node_props[] = {
   ELEMENT_COMMON_PROPERTIES,
+  PROP_STD_LINE_COLOUR_OPTIONAL, 
+  PROP_STD_FILL_COLOUR_OPTIONAL, 
   PROP_STD_TEXT_FONT,
   PROP_STD_TEXT_HEIGHT,
-  PROP_STD_TEXT_COLOUR,
-  PROP_STD_TEXT,
+  PROP_STD_TEXT_COLOUR_OPTIONAL,
+  { "name", PROP_TYPE_TEXT, 0, N_("Text"), NULL, NULL }, 
   
   PROP_DESC_END
 };
 
-static struct { const gchar *name; GQuark q; } quarks[] = {
-  { "text_font" },
-  { "text_height" },
-  { "text_colour" },
-  { "text" }
-};
 
 static PropDescription *
 node_describe_props(Node *node)
 {
-  if (node_props[0].quark == 0)
+  if (node_props[0].quark == 0) {
     prop_desc_list_calculate_quarks(node_props);
+  }
   return node_props;
 }
 
 static PropOffset node_offsets[] = {
   ELEMENT_COMMON_PROPERTIES_OFFSETS,
-  /*  { "name", PROP_TYPE_STRING, offsetof(Node, name) },*/
+  {"line_colour",PROP_TYPE_COLOUR,offsetof(Node,line_color)},
+  {"fill_colour",PROP_TYPE_COLOUR,offsetof(Node,fill_color)},
+  {"name",PROP_TYPE_TEXT,offsetof(Node,name)},
+  {"text_font",PROP_TYPE_FONT,offsetof(Node,attrs.font)},
+  {"text_height",PROP_TYPE_REAL,offsetof(Node,attrs.height)},
+  {"text_colour",PROP_TYPE_COLOUR,offsetof(Node,attrs.color)},
   { NULL, 0, 0 },
 };
 
 static void
-node_get_props(Node * node, Property *props, guint nprops)
+node_get_props(Node * node, GPtrArray *props)
 {
-  guint i;
-
-  if (object_get_props_from_offsets(&node->element.object, node_offsets, props, nprops))
-    return;
-
-  /* these props can't be handled as easily */
-  if (quarks[0].q == 0)
-    for (i = 0; i < sizeof(quarks)/sizeof(*quarks); i++)
-      quarks[i].q = g_quark_from_static_string(quarks[i].name);
-  for (i = 0; i < nprops; i++) {
-    GQuark pquark = g_quark_from_string(props[i].name);
-
-    if (pquark == quarks[0].q) {
-      props[i].type = PROP_TYPE_FONT;
-      PROP_VALUE_FONT(props[i]) = node->name->font;
-    } else if (pquark == quarks[1].q) {
-      props[i].type = PROP_TYPE_REAL;
-      PROP_VALUE_REAL(props[i]) = node->name->height;
-    } else if (pquark == quarks[2].q) {
-      props[i].type = PROP_TYPE_COLOUR;
-      PROP_VALUE_COLOUR(props[i]) = node->name->color;
-    } else if (pquark == quarks[3].q) {
-      props[i].type = PROP_TYPE_STRING;
-      g_free(PROP_VALUE_STRING(props[i]));
-      PROP_VALUE_STRING(props[i]) = text_get_string_copy(node->name);
-    }
-  }
-
+  text_get_attributes(node->name,&node->attrs);
+  object_get_props_from_offsets(&node->element.object,
+                                node_offsets,props);
 }
 
 static void
-node_set_props(Node *node, Property *props, guint nprops)
+node_set_props(Node *node, GPtrArray *props)
 {
-  if (!object_set_props_from_offsets(&node->element.object, 
-                                     node_offsets, props, nprops)) {
-    guint i;
-
-    if (quarks[0].q == 0)
-      for (i = 0; i < sizeof(quarks)/sizeof(*quarks); i++)
-	quarks[i].q = g_quark_from_static_string(quarks[i].name);
-
-    for (i = 0; i < nprops; i++) {
-      GQuark pquark = g_quark_from_string(props[i].name);
-
-      if (pquark == quarks[0].q && props[i].type == PROP_TYPE_FONT) {
-	text_set_font(node->name, PROP_VALUE_FONT(props[i]));
-      } else if (pquark == quarks[1].q && props[i].type == PROP_TYPE_REAL) {
-	text_set_height(node->name, PROP_VALUE_REAL(props[i]));
-      } else if (pquark == quarks[2].q && props[i].type == PROP_TYPE_COLOUR) {
-	text_set_color(node->name, &PROP_VALUE_COLOUR(props[i]));
-      } else if (pquark == quarks[3].q && props[i].type == PROP_TYPE_STRING) {
-	text_set_string(node->name, PROP_VALUE_STRING(props[i]));
-      }
-    }
-  }
+  object_set_props_from_offsets(&node->element.object,
+                                node_offsets,props);
+  apply_textattr_properties(props,node->name,"name",&node->attrs);
   node_update_data(node);
 }
 
@@ -209,22 +169,23 @@ node_set_props(Node *node, Property *props, guint nprops)
 static real
 node_distance_from(Node *node, Point *point)
 {
-  Object *obj = &node->element.object;
+  DiaObject *obj = &node->element.object;
   return distance_rectangle_point(&obj->bounding_box, point);
 }
 
 static void
 node_select(Node *node, Point *clicked_point,
-		    Renderer *interactive_renderer)
+		    DiaRenderer *interactive_renderer)
 {
   text_set_cursor(node->name, clicked_point, interactive_renderer);
   text_grab_focus(node->name, &node->element.object);
   element_update_handles(&node->element);
 }
 
-static void
+static ObjectChange*
 node_move_handle(Node *node, Handle *handle,
-			 Point *to, HandleMoveReason reason, ModifierKeys modifiers)
+		 Point *to, ConnectionPoint *cp,
+		 HandleMoveReason reason, ModifierKeys modifiers)
 {
   assert(node!=NULL);
   assert(handle!=NULL);
@@ -232,11 +193,13 @@ node_move_handle(Node *node, Handle *handle,
 
   assert(handle->id < 8);
   
-  element_move_handle(&node->element, handle->id, to, reason);
+  element_move_handle(&node->element, handle->id, to, cp, reason, modifiers);
   node_update_data(node);
+
+  return NULL;
 }
 
-static void
+static ObjectChange*
 node_move(Node *node, Point *to)
 {
   Point p;
@@ -249,10 +212,13 @@ node_move(Node *node, Point *to)
   text_set_position(node->name, &p);
 
   node_update_data(node);
+
+  return NULL;
 }
 
-static void node_draw(Node *node, Renderer *renderer)
+static void node_draw(Node *node, DiaRenderer *renderer)
 {
+  DiaRendererClass *renderer_ops = DIA_RENDERER_GET_CLASS (renderer);
   Element *elem;
   real x, y, w, h;
   Point points[4];
@@ -268,52 +234,55 @@ static void node_draw(Node *node, Renderer *renderer)
   w = elem->width;
   h = elem->height;
   
-  renderer->ops->set_fillstyle(renderer, FILLSTYLE_SOLID);
-  renderer->ops->set_linewidth(renderer, NODE_BORDERWIDTH);
-  renderer->ops->set_linestyle(renderer, LINESTYLE_SOLID);
+  renderer_ops->set_fillstyle(renderer, FILLSTYLE_SOLID);
+  renderer_ops->set_linewidth(renderer, NODE_BORDERWIDTH);
+  renderer_ops->set_linestyle(renderer, LINESTYLE_SOLID);
 
   points[0].x = x;     points[0].y = y;
   points[1].x = x + w; points[1].y = y + h;
 
-  renderer->ops->fill_rect(renderer, points, points + 1, &color_white);
-  renderer->ops->draw_rect(renderer, points, points + 1, &color_black);
+  renderer_ops->fill_rect(renderer, points, points + 1, &node->fill_color);
+  renderer_ops->draw_rect(renderer, points, points + 1, &node->line_color);
 
   points[1].x = x + NODE_DEPTH, points[1].y = y - NODE_DEPTH;
   points[2].x = x + w + NODE_DEPTH, points[2].y = y - NODE_DEPTH;
   points[3].x = x + w, points[3].y = y;
 
-  renderer->ops->fill_polygon(renderer, points, 4, &color_white);
-  renderer->ops->draw_polygon(renderer, points, 4, &color_black);
+  renderer_ops->fill_polygon(renderer, points, 4, &node->fill_color);
+  renderer_ops->draw_polygon(renderer, points, 4, &node->line_color);
 
   points[0].x = points[3].x, points[0].y = points[3].y;
   points[1].x = points[0].x + NODE_DEPTH, points[1].y = points[0].y - NODE_DEPTH;
   points[2].x = points[0].x + NODE_DEPTH, points[2].y = points[0].y - NODE_DEPTH + h;
   points[3].x = points[0].x, points[3].y = points[0].y + h;
 
-  renderer->ops->fill_polygon(renderer, points, 4, &color_white);
-  renderer->ops->draw_polygon(renderer, points, 4, &color_black);
+  renderer_ops->fill_polygon(renderer, points, 4, &node->fill_color);
+  renderer_ops->draw_polygon(renderer, points, 4, &node->line_color);
 
   text_draw(node->name, renderer);
   
-  renderer->ops->set_linewidth(renderer, NODE_LINEWIDTH);
+  renderer_ops->set_linewidth(renderer, NODE_LINEWIDTH);
   points[0].x = node->name->position.x;
   points[0].y = points[1].y = node->name->position.y + node->name->descent;
   for (i = 0; i < node->name->numlines; i++)
     { 
       points[1].x = points[0].x + node->name->row_width[i];
-      renderer->ops->draw_line(renderer, points, points + 1, &color_black);
+      renderer_ops->draw_line(renderer, points, points + 1, &color_black);
       points[0].y = points[1].y += node->name->height;
     }
 }
 
-static void node_update_data(Node *node)
+static void 
+node_update_data(Node *node)
 {
   Element *elem = &node->element;
-  Object *obj = &node->element.object;
-  Font *font;
+  DiaObject *obj = &node->element.object;
+  DiaFont *font;
   Point p1;
   real h, w = 0;
 
+  text_calc_boundingbox(node->name, NULL);
+
   font = node->name->font;
   h = elem->corner.y + NODE_TEXT_MARGIN;
 
@@ -328,20 +297,28 @@ static void node_update_data(Node *node)
   
   /* Update connections: */
   node->connections[0].pos = elem->corner;
+  node->connections[0].directions = DIR_NORTH|DIR_WEST;
   node->connections[1].pos.x = elem->corner.x + elem->width / 2.0;
   node->connections[1].pos.y = elem->corner.y;
+  node->connections[1].directions = DIR_NORTH;
   node->connections[2].pos.x = elem->corner.x + elem->width;
   node->connections[2].pos.y = elem->corner.y;
+  node->connections[2].directions = DIR_NORTH|DIR_EAST;
   node->connections[3].pos.x = elem->corner.x;
   node->connections[3].pos.y = elem->corner.y + elem->height / 2.0;
+  node->connections[3].directions = DIR_WEST;
   node->connections[4].pos.x = elem->corner.x + elem->width;
   node->connections[4].pos.y = elem->corner.y + elem->height / 2.0;
+  node->connections[4].directions = DIR_EAST;
   node->connections[5].pos.x = elem->corner.x;
   node->connections[5].pos.y = elem->corner.y + elem->height;
+  node->connections[5].directions = DIR_SOUTH|DIR_WEST;
   node->connections[6].pos.x = elem->corner.x + elem->width / 2.0;
   node->connections[6].pos.y = elem->corner.y + elem->height;
+  node->connections[6].directions = DIR_SOUTH;
   node->connections[7].pos.x = elem->corner.x + elem->width;
   node->connections[7].pos.y = elem->corner.y + elem->height;
+  node->connections[7].directions = DIR_SOUTH|DIR_EAST;
   
   element_update_boundingbox(elem);
   /* fix boundingbox for depth: */
@@ -353,13 +330,13 @@ static void node_update_data(Node *node)
   element_update_handles(elem);
 }
 
-static Object *node_create(Point *startpoint, void *user_data, Handle **handle1, Handle **handle2)
+static DiaObject *node_create(Point *startpoint, void *user_data, Handle **handle1, Handle **handle2)
 {
   Node *node;
   Element *elem;
-  Object *obj;
+  DiaObject *obj;
   Point p;
-  Font *font;
+  DiaFont *font;
   int i;
   
   node = g_malloc0(sizeof(Node));
@@ -371,12 +348,18 @@ static Object *node_create(Point *startpoint, void *user_data, Handle **handle1,
   obj->ops = &node_ops;
 
   elem->corner = *startpoint;
-  font = font_getfont("Helvetica");
+
+  node->line_color = attributes_get_foreground();
+  node->fill_color = attributes_get_background();
+
+  font = dia_font_new_from_style (DIA_FONT_SANS, 0.8);
   /* The text position is recalculated later */
   p.x = 0.0;
   p.y = 0.0;
   node->name = new_text("", font, 0.8, &p, &color_black, ALIGN_LEFT);
-
+  text_get_attributes(node->name,&node->attrs);
+  dia_font_unref(font);
+  
   element_init(elem, 8, 8);
 
   for (i=0;i<8;i++) {
@@ -388,7 +371,7 @@ static Object *node_create(Point *startpoint, void *user_data, Handle **handle1,
   node_update_data(node);
 
   *handle1 = NULL;
-  *handle2 = obj->handles[0];
+  *handle2 = obj->handles[7];
   return &node->element.object;
 }
 
@@ -398,71 +381,8 @@ static void node_destroy(Node *node)
   element_destroy(&node->element);
 }
 
-static Object *node_copy(Node *node)
+static DiaObject *node_load(ObjectNode obj_node, int version, const char *filename)
 {
-  int i;
-  Node *newnode;
-  Element *elem, *newelem;
-  Object *newobj;
-  
-  elem = &node->element;
-  
-  newnode = g_malloc0(sizeof(Node));
-  newelem = &newnode->element;
-  newobj = &newelem->object;
-
-  element_copy(elem, newelem);
-  newnode->name = text_copy(node->name);
-  
-  for (i=0;i<8;i++)
-    {
-      newobj->connections[i] = &newnode->connections[i];
-      newnode->connections[i].object = newobj;
-      newnode->connections[i].connected = NULL;
-      newnode->connections[i].pos = node->connections[i].pos;
-      newnode->connections[i].last_pos = node->connections[i].last_pos;
-    }
-
-  node_update_data(newnode);
-  return &newnode->element.object;
-}
-
-static void node_save(Node *node, ObjectNode obj_node, const char *filename)
-{
-  element_save(&node->element, obj_node);
-  data_add_text(new_attribute(obj_node, "name"), node->name);
-}
-
-static Object *node_load(ObjectNode obj_node, int version, const char *filename)
-{
-  Node *node;
-  AttributeNode attr;
-  Element *elem;
-  Object *obj;
-  int i;
-  
-  node = g_malloc0(sizeof(Node));
-  elem = &node->element;
-  obj = &elem->object;
-
-  obj->type = &node_type;
-  obj->ops = &node_ops;
-
-  element_load(elem, obj_node);
-
-  node->name = NULL;
-  attr = object_find_attribute(obj_node, "name");
-  if (attr != NULL) node->name = data_text(attribute_first_data(attr));
-  
-  element_init(elem, 8, 8);
-
-  for (i=0;i<8;i++)
-    {
-      obj->connections[i] = &node->connections[i];
-      node->connections[i].object = obj;
-      node->connections[i].connected = NULL;
-    }
-  elem->extra_spacing.border_trans = NODE_BORDERWIDTH/2.0;
-  node_update_data(node);
-  return &node->element.object;
+  return object_load_using_properties(&node_type,
+                                      obj_node,version,filename);
 }
