@@ -29,27 +29,31 @@
 #include "uml.h"
 #include "plug-ins.h"
 
-extern ObjectType umlclass_type;
-extern ObjectType note_type;
-extern ObjectType dependency_type;
-extern ObjectType realizes_type;
-extern ObjectType generalization_type;
-extern ObjectType association_type;
-extern ObjectType implements_type;
-extern ObjectType constraint_type;
-extern ObjectType smallpackage_type;
-extern ObjectType largepackage_type;
-extern ObjectType actor_type;
-extern ObjectType usecase_type;
-extern ObjectType lifeline_type;
-extern ObjectType objet_type;
-extern ObjectType umlobject_type;
-extern ObjectType message_type;
-extern ObjectType component_type;
-extern ObjectType classicon_type;
-extern ObjectType state_type;
-extern ObjectType node_type;
-extern ObjectType branch_type;
+extern DiaObjectType umlclass_type;
+extern DiaObjectType note_type;
+extern DiaObjectType dependency_type;
+extern DiaObjectType realizes_type;
+extern DiaObjectType generalization_type;
+extern DiaObjectType association_type;
+extern DiaObjectType implements_type;
+extern DiaObjectType constraint_type;
+extern DiaObjectType smallpackage_type;
+extern DiaObjectType largepackage_type;
+extern DiaObjectType actor_type;
+extern DiaObjectType usecase_type;
+extern DiaObjectType lifeline_type;
+extern DiaObjectType objet_type;
+extern DiaObjectType umlobject_type;
+extern DiaObjectType message_type;
+extern DiaObjectType component_type;
+extern DiaObjectType classicon_type;
+extern DiaObjectType state_type;
+extern DiaObjectType activity_type;
+extern DiaObjectType node_type;
+extern DiaObjectType branch_type;
+extern DiaObjectType fork_type;
+extern DiaObjectType state_term_type;
+extern DiaObjectType compfeat_type;
 
 DIA_PLUGIN_CHECK_INIT
 
@@ -79,51 +83,58 @@ dia_plugin_init(PluginInfo *info)
   object_register_type(&message_type);  
   object_register_type(&component_type);
   object_register_type(&classicon_type);
-  object_register_type(&state_type);    
+  object_register_type(&state_type);
+  object_register_type(&state_term_type);
+  object_register_type(&activity_type);
   object_register_type(&node_type);    
   object_register_type(&branch_type);    
-
+  object_register_type(&fork_type);
+  object_register_type(&compfeat_type);
+   
   return DIA_PLUGIN_INIT_OK;
 }
 
+/* Warning, the following *must* be strictly ASCII characters (or fix the 
+   following code for UTF-8 cleanliness */
+
 char visible_char[] = { '+', '-', '#', ' ' };
 
 char *
-uml_get_attribute_string(UMLAttribute *attribute)
+uml_get_attribute_string (UMLAttribute *attribute)
 {
   int len;
   char *str;
 
-  len = 1 + strlen(attribute->name) + strlen(attribute->type);
+  len = 1 + strlen (attribute->name) + strlen (attribute->type);
   if (attribute->name[0] && attribute->type[0]) {
     len += 2;
   }
-  if (attribute->value != NULL) {
-    len += 3 + strlen(attribute->value);
+  if (attribute->value != NULL && attribute->value[0] != '\0') {
+    len += 3 + strlen (attribute->value);
   }
-
-  str = g_malloc(sizeof(char)*(len+1));
+  
+  str = g_malloc (sizeof (char) * (len + 1));
 
   str[0] = visible_char[(int) attribute->visibility];
   str[1] = 0;
-  
-  strcat(str, attribute->name);
-  if (attribute->name[0] && attribute->type[0]) {
-    strcat(str, ": ");
-  }
-  strcat(str, attribute->type);
-  if (attribute->value != NULL) {
-    strcat(str, " = ");
-    strcat(str, attribute->value);
-  }
 
-  assert(strlen(str)==len);
+  strcat (str, attribute->name);
+  if (attribute->name[0] && attribute->type[0]) {
+    strcat (str, ": ");
+  }
+  strcat (str, attribute->type);
+  if (attribute->value != NULL && attribute->value[0] != '\0') {
+    strcat (str, " = ");
+    strcat (str, attribute->value);
+  }
+    
+  assert (strlen (str) == len);
 
   return str;
 }
 
 char *
-uml_get_operation_string(UMLOperation *operation)
+uml_get_operation_string (UMLOperation *operation)
 {
   int len;
   char *str;
@@ -131,121 +142,208 @@ uml_get_operation_string(UMLOperation *operation)
   UMLParameter *param;
 
   /* Calculate length: */
-  len = 1 + strlen(operation->name)  + 1;
+  len = 1 + strlen (operation->name)  + 1;
+  if(operation->stereotype != NULL && operation->stereotype[0] != '\0') {
+    len += 5 + strlen (operation->stereotype);
+  }   
   
   list = operation->parameters;
   while (list != NULL) {
     param = (UMLParameter  *) list->data;
-    list = g_list_next(list);
-
-    len += strlen(param->name) + strlen(param->type);
-    if (param->type[0] && param->name[0]) {
-      len += 1;
+    list = g_list_next (list);
+    
+    switch(param->kind)
+      {
+      case UML_UNDEF_KIND:
+	break;  
+      case UML_IN:
+	len += 3;
+	break;
+      case UML_OUT:
+	len += 4;
+	break;
+      case UML_INOUT:
+	len += 6;
+	break;	  
+      }
+    len += strlen (param->name);
+    if (param->type != NULL) {
+      len += strlen (param->type);
+      if (param->type[0] && param->name[0]) {
+        len += 1;
+      }
     }
-    if (param->value != NULL) {
-      len += 1 + strlen(param->value);
+    if (param->value != NULL && param->value[0] != '\0') {
+      len += 1 + strlen (param->value);
     }
-
+    
     if (list != NULL) {
       len += 1; /* ',' */
     }
   }
+
   len += 1; /* ')' */
-  if (operation->type != NULL) {
-    len += 2 + strlen(operation->type);
+  if (operation->type != NULL && operation->type[0]) {
+    len += 2 + strlen (operation->type);
   }
-  
+  if(operation->query != 0) {
+    len += 6;
+  }
+
   /* generate string: */
-  str = g_malloc(sizeof(char)*(len+1));
+  str = g_malloc (sizeof (char) * (len + 1));
 
   str[0] = visible_char[(int) operation->visibility];
   str[1] = 0;
 
-  strcat(str, operation->name);
-  strcat(str, "(");
+  if(operation->stereotype != NULL && operation->stereotype[0] != '\0') {
+    strcat(str, UML_STEREOTYPE_START);
+    strcat(str, operation->stereotype);
+    strcat(str, UML_STEREOTYPE_END);
+    strcat(str, " ");
+  }
+
+  strcat (str, operation->name);
+  strcat (str, "(");
   
   list = operation->parameters;
   while (list != NULL) {
     param = (UMLParameter  *) list->data;
-    list = g_list_next(list);
-
-    strcat(str, param->name);
-    if (param->type[0] && param->name[0]) {
-      strcat(str, ":");
-    }
-    strcat(str, param->type);
+    list = g_list_next (list);
     
-    if (param->value != NULL) {
-      strcat(str, "=");
-      strcat(str, param->value);
+    switch(param->kind)
+      {
+      case UML_UNDEF_KIND:
+	break;
+      case UML_IN:
+	strcat (str, "in ");
+	break;
+      case UML_OUT:
+	strcat (str, "out ");
+	break;
+      case UML_INOUT:
+	strcat (str, "inout ");
+	break;
+      }
+    strcat (str, param->name);
+
+    if (param->type != NULL) {
+      if (param->type[0] && param->name[0]) {
+        strcat (str, ":");
+      }
+      strcat (str, param->type);
+    }
+    
+    if (param->value != NULL && param->value[0] != '\0') {
+      strcat (str, "=");
+      strcat (str, param->value);
     }
 
     if (list != NULL) {
-      strcat(str, ",");
+      strcat (str, ",");
     }
   }
-  strcat(str, ")");
+  strcat (str, ")");
 
-  if (operation->type != NULL) {
-    strcat(str, ": ");
-    strcat(str, operation->type);
+  if (operation->type != NULL &&
+      operation->type[0]) {
+    strcat (str, ": ");
+    strcat (str, operation->type);
+  }
+ 
+  if (operation->query != 0) {
+    strcat(str, " const");
   }
 
-  assert(strlen(str)==len);
+  assert (strlen (str) == len);
   
   return str;
 }
 
 char *
-uml_get_parameter_string(UMLParameter *param)
+uml_get_parameter_string (UMLParameter *param)
 {
   int len;
   char *str;
 
   /* Calculate length: */
-  len = strlen(param->name) + 1 + strlen(param->type);
+  len = strlen (param->name) + 1 + strlen (param->type);
   
   if (param->value != NULL) {
-    len += 1 + strlen(param->value) ;
+    len += 1 + strlen (param->value) ;
   }
 
+  switch(param->kind)
+    {
+    case UML_UNDEF_KIND:
+      break;
+    case UML_IN:
+      len += 3;
+      break;
+    case UML_OUT:
+      len += 4;
+      break;
+    case UML_INOUT:
+      len += 6;
+      break;	  
+    }
+
   /* Generate string: */
-  str = g_malloc(sizeof(char)*(len+1));
-  strcpy(str, param->name);
-  strcat(str, ":");
-  strcat(str, param->type);
+  str = g_malloc (sizeof (char) * (len + 1));
+
+  strcpy(str, "");
+
+  switch(param->kind)
+    {
+    case UML_UNDEF_KIND:
+      break;     
+    case UML_IN:
+      strcat (str, "in ");
+      break;
+    case UML_OUT:
+      strcat (str, "out ");
+      break;
+    case UML_INOUT:
+      strcat (str, "inout ");
+      break;
+    }
+  
+
+  strcat (str, param->name);
+  strcat (str, ":");
+  strcat (str, param->type);
   if (param->value != NULL) {
-    strcat(str, "=");
-    strcat(str, param->value);
+    strcat (str, "=");
+    strcat (str, param->value);
   }
   
-  assert(strlen(str)==len);
+  assert (strlen (str) == len);
 
   return str;
 }
 
 char *
-uml_get_formalparameter_string(UMLFormalParameter *parameter)
+uml_get_formalparameter_string (UMLFormalParameter *parameter)
 {
   int len;
   char *str;
 
   /* Calculate length: */
-  len = strlen(parameter->name);
+  len = strlen (parameter->name);
   
   if (parameter->type != NULL) {
-    len += 1 + strlen(parameter->type);
+    len += 1 + strlen (parameter->type);
   }
 
   /* Generate string: */
-  str = g_malloc(sizeof(char)*(len+1));
-  strcpy(str, parameter->name);
+  str = g_malloc (sizeof (char) * (len + 1));
+  strcpy (str, parameter->name);
   if (parameter->type != NULL) {
-    strcat(str, ":");
-    strcat(str, parameter->type);
+    strcat (str, ":");
+    strcat (str, parameter->type);
   }
 
-  assert(strlen(str)==len);
+  assert (strlen (str) == len);
 
   return str;
 }
@@ -257,13 +355,18 @@ uml_attribute_copy(UMLAttribute *attr)
   UMLAttribute *newattr;
 
   newattr = g_new0(UMLAttribute, 1);
-  newattr->name = strdup(attr->name);
-  newattr->type = strdup(attr->type);
+  newattr->name = g_strdup(attr->name);
+  newattr->type = g_strdup(attr->type);
   if (attr->value != NULL) {
-    newattr->value = strdup(attr->value);
+    newattr->value = g_strdup(attr->value);
   } else {
     newattr->value = NULL;
   }
+  if (attr->comment != NULL)
+    newattr->comment = g_strdup (attr->comment);
+  else 
+    newattr->comment = NULL;
+
   newattr->visibility = attr->visibility;
   newattr->abstract = attr->abstract;
   newattr->class_scope = attr->class_scope;
@@ -283,15 +386,25 @@ uml_operation_copy(UMLOperation *op)
   GList *list;
   
   newop = g_new0(UMLOperation, 1);
-  newop->name = strdup(op->name);
+  newop->name = g_strdup(op->name);
   if (op->type != NULL) {
-    newop->type = strdup(op->type);
+    newop->type = g_strdup(op->type);
   } else {
     newop->type = NULL;
   }
+  if(op->stereotype != NULL) {
+    newop->stereotype = g_strdup(op->stereotype);
+  } else {
+    newop->stereotype = NULL;
+  }
+  
+  newop->comment = g_strdup(op->comment);
+
   newop->visibility = op->visibility;
-  newop->abstract = op->abstract;
   newop->class_scope = op->class_scope;
+  newop->inheritance_type = op->inheritance_type;
+  newop->query = op->query;
+
 
   newop->left_connection = op->left_connection;
   newop->right_connection = op->right_connection;
@@ -302,10 +415,12 @@ uml_operation_copy(UMLOperation *op)
     param = (UMLParameter *)list->data;
 
     newparam = g_new0(UMLParameter, 1);
-    newparam->name = strdup(param->name);
-    newparam->type = strdup(param->type);
+    newparam->name = g_strdup(param->name);
+    newparam->type = g_strdup(param->type);
+    newparam->comment = g_strdup(param->comment);
+
     if (param->value != NULL)
-      newparam->value = strdup(param->value);
+      newparam->value = g_strdup(param->value);
     else
       newparam->value = NULL;
     newparam->kind = param->kind;
@@ -325,13 +440,13 @@ uml_formalparameter_copy(UMLFormalParameter *param)
 
   newparam = g_new0(UMLFormalParameter, 1);
 
-  newparam->name = strdup(param->name);
+  newparam->name = g_strdup(param->name);
   if (param->type != NULL) {
-    newparam->type = strdup(param->type);
+    newparam->type = g_strdup(param->type);
   } else {
     newparam->type = NULL;
   }
-  
+
   return newparam;
 }
 
@@ -342,6 +457,9 @@ uml_attribute_destroy(UMLAttribute *attr)
   g_free(attr->type);
   if (attr->value != NULL)
     g_free(attr->value);
+  if (attr->comment != NULL)
+    g_free(attr->comment);
+
   g_free(attr);
 }
 
@@ -354,6 +472,10 @@ uml_operation_destroy(UMLOperation *op)
   g_free(op->name);
   if (op->type != NULL)
     g_free(op->type);
+  if (op->stereotype != NULL)
+    g_free(op->stereotype);
+
+  g_free(op->comment);
 
   list = op->parameters;
   while (list != NULL) {
@@ -371,6 +493,8 @@ uml_parameter_destroy(UMLParameter *param)
   g_free(param->type);
   if (param->value != NULL) 
     g_free(param->value);
+  g_free(param->comment);
+
   g_free(param);
 }
 
@@ -392,6 +516,7 @@ uml_attribute_new(void)
   attr->name = g_strdup("");
   attr->type = g_strdup("");
   attr->value = NULL;
+  attr->comment = g_strdup("");
   attr->visibility = UML_PUBLIC;
   attr->abstract = FALSE;
   attr->class_scope = FALSE;
@@ -409,9 +534,13 @@ uml_operation_new(void)
   op = g_new0(UMLOperation, 1);
   op->name = g_strdup("");
   op->type = NULL;
+  op->stereotype = NULL;
+  op->comment = g_strdup("");
   op->visibility = UML_PUBLIC;
-  op->abstract = FALSE;
   op->class_scope = FALSE;
+  op->inheritance_type = UML_LEAF;
+  op->query = FALSE;
+
   op->parameters = NULL;
 
   op->left_connection = NULL;
@@ -427,6 +556,7 @@ uml_parameter_new(void)
   param = g_new0(UMLParameter, 1);
   param->name = g_strdup("");
   param->type = g_strdup("");
+  param->comment = g_strdup("");
   param->value = NULL;
   param->kind = UML_UNDEF_KIND;
 
@@ -458,6 +588,8 @@ uml_attribute_write(AttributeNode attr_node, UMLAttribute *attr)
 		  attr->type);
   data_add_string(composite_add_attribute(composite, "value"),
 		  attr->value);
+  data_add_string(composite_add_attribute(composite, "comment"),
+		  attr->comment);
   data_add_enum(composite_add_attribute(composite, "visibility"),
 		attr->visibility);
   data_add_boolean(composite_add_attribute(composite, "abstract"),
@@ -479,12 +611,21 @@ uml_operation_write(AttributeNode attr_node, UMLOperation *op)
 
   data_add_string(composite_add_attribute(composite, "name"),
 		  op->name);
+  data_add_string(composite_add_attribute(composite, "stereotype"),
+		  op->stereotype);
   data_add_string(composite_add_attribute(composite, "type"),
 		  op->type);
   data_add_enum(composite_add_attribute(composite, "visibility"),
 		op->visibility);
+  data_add_string(composite_add_attribute(composite, "comment"),
+		  op->comment);
+  /* Backward compatibility */
   data_add_boolean(composite_add_attribute(composite, "abstract"),
-		   op->abstract);
+		   op->inheritance_type == UML_ABSTRACT);
+  data_add_enum(composite_add_attribute(composite, "inheritance_type"),
+		op->inheritance_type);
+  data_add_boolean(composite_add_attribute(composite, "query"),
+		   op->query);
   data_add_boolean(composite_add_attribute(composite, "class_scope"),
 		   op->class_scope);
   
@@ -502,6 +643,8 @@ uml_operation_write(AttributeNode attr_node, UMLOperation *op)
 		    param->type);
     data_add_string(composite_add_attribute(composite2, "value"),
 		    param->value);
+    data_add_string(composite_add_attribute(composite2, "comment"),
+		    param->comment);
     data_add_enum(composite_add_attribute(composite2, "kind"),
 		  param->kind);
     list = g_list_next(list);
@@ -544,6 +687,11 @@ uml_attribute_read(DataNode composite)
   if (attr_node != NULL)
     attr->value =  data_string( attribute_first_data(attr_node) );
   
+  attr->comment = NULL;
+  attr_node = composite_find_attribute(composite, "comment");
+  if (attr_node != NULL)
+    attr->comment =  data_string( attribute_first_data(attr_node) );
+
   attr->visibility = FALSE;
   attr_node = composite_find_attribute(composite, "visibility");
   if (attr_node != NULL)
@@ -587,15 +735,35 @@ uml_operation_read(DataNode composite)
   if (attr_node != NULL)
     op->type =  data_string( attribute_first_data(attr_node) );
 
+  op->stereotype = NULL;
+  attr_node = composite_find_attribute(composite, "stereotype");
+  if (attr_node != NULL)
+    op->stereotype = data_string( attribute_first_data(attr_node) );
+
+  op->comment = NULL;
+  attr_node = composite_find_attribute(composite, "comment");
+  if (attr_node != NULL)
+    op->comment = data_string( attribute_first_data(attr_node) );
+
   op->visibility = FALSE;
   attr_node = composite_find_attribute(composite, "visibility");
   if (attr_node != NULL)
     op->visibility =  data_enum( attribute_first_data(attr_node) );
   
-  op->abstract = FALSE;
+  op->inheritance_type = UML_LEAF;
+  /* Backward compatibility */
   attr_node = composite_find_attribute(composite, "abstract");
   if (attr_node != NULL)
-    op->abstract =  data_boolean( attribute_first_data(attr_node) );
+    if(data_boolean( attribute_first_data(attr_node) ))
+      op->inheritance_type = UML_ABSTRACT;
+  
+  attr_node = composite_find_attribute(composite, "inheritance_type");
+  if (attr_node != NULL)
+    op->inheritance_type = data_enum( attribute_first_data(attr_node) );
+  
+  attr_node = composite_find_attribute(composite, "query");
+  if (attr_node != NULL)
+    op->query =  data_boolean( attribute_first_data(attr_node) );
   
   op->class_scope = FALSE;
   attr_node = composite_find_attribute(composite, "class_scope");
@@ -624,6 +792,11 @@ uml_operation_read(DataNode composite)
     if (attr_node != NULL)
       param->value =  data_string( attribute_first_data(attr_node) );
     
+    param->comment = NULL;
+    attr_node = composite_find_attribute(composite2, "comment");
+    if (attr_node != NULL)
+      param->comment =  data_string( attribute_first_data(attr_node) );
+
     param->kind = UML_UNDEF_KIND;
     attr_node = composite_find_attribute(composite2, "kind");
     if (attr_node != NULL)
