@@ -21,14 +21,13 @@
 #endif
 
 #include <assert.h>
-#include <gtk/gtk.h>
 #include <math.h>
 
 #include "intl.h"
 #include "object.h"
 #include "connection.h"
 #include "connectionpoint.h"
-#include "render.h"
+#include "diarenderer.h"
 #include "attributes.h"
 #include "widgets.h"
 #include "arrows.h"
@@ -61,13 +60,14 @@ struct _Arc {
 
 };
 
-static void arc_move_handle(Arc *arc, Handle *handle,
-			    Point *to, HandleMoveReason reason, ModifierKeys modifiers);
-static void arc_move(Arc *arc, Point *to);
+static ObjectChange* arc_move_handle(Arc *arc, Handle *handle,
+				     Point *to, ConnectionPoint *cp,
+				     HandleMoveReason reason, ModifierKeys modifiers);
+static ObjectChange* arc_move(Arc *arc, Point *to);
 static void arc_select(Arc *arc, Point *clicked_point,
-		       Renderer *interactive_renderer);
-static void arc_draw(Arc *arc, Renderer *renderer);
-static Object *arc_create(Point *startpoint,
+		       DiaRenderer *interactive_renderer);
+static void arc_draw(Arc *arc, DiaRenderer *renderer);
+static DiaObject *arc_create(Point *startpoint,
 			  void *user_data,
 			  Handle **handle1,
 			  Handle **handle2);
@@ -75,27 +75,25 @@ static real arc_distance_from(Arc *arc, Point *point);
 static void arc_update_data(Arc *arc);
 static void arc_update_handles(Arc *arc);
 static void arc_destroy(Arc *arc);
-static Object *arc_copy(Arc *arc);
+static DiaObject *arc_copy(Arc *arc);
 
 static PropDescription *arc_describe_props(Arc *arc);
-static void arc_get_props(Arc *arc, Property *props, guint nprops);
-static void arc_set_props(Arc *arc, Property *props, guint nprops);
+static void arc_get_props(Arc *arc, GPtrArray *props);
+static void arc_set_props(Arc *arc, GPtrArray *props);
 
 static void arc_save(Arc *arc, ObjectNode obj_node, const char *filename);
-static Object *arc_load(ObjectNode obj_node, int version, const char *filename);
-/* static GtkWidget *arc_get_defaults();
-   static void arc_apply_defaults(); */
+static DiaObject *arc_load(ObjectNode obj_node, int version, const char *filename);
 
 static ObjectTypeOps arc_type_ops =
 {
   (CreateFunc) arc_create,
   (LoadFunc)   arc_load,
   (SaveFunc)   arc_save,
-  (GetDefaultsFunc)   NULL /*arc_get_defaults*/,
-  (ApplyDefaultsFunc) NULL /*arc_apply_defaults*/
+  (GetDefaultsFunc)   NULL,
+  (ApplyDefaultsFunc) NULL
 };
 
-ObjectType arc_type =
+DiaObjectType arc_type =
 {
   "Standard - Arc",  /* name */
   0,                 /* version */
@@ -104,7 +102,7 @@ ObjectType arc_type =
   &arc_type_ops      /* ops */
 };
 
-ObjectType *_arc_type = (ObjectType *) &arc_type;
+DiaObjectType *_arc_type = (DiaObjectType *) &arc_type;
 
 static ObjectOps arc_ops = {
   (DestroyFunc)         arc_destroy,
@@ -157,17 +155,17 @@ static PropOffset arc_offsets[] = {
 };
 
 static void
-arc_get_props(Arc *arc, Property *props, guint nprops)
+arc_get_props(Arc *arc, GPtrArray *props)
 {
   object_get_props_from_offsets(&arc->connection.object, 
-                                arc_offsets, props, nprops);
+                                arc_offsets, props);
 }
 
 static void
-arc_set_props(Arc *arc, Property *props, guint nprops)
+arc_set_props(Arc *arc, GPtrArray *props)
 {
   object_set_props_from_offsets(&arc->connection.object, 
-                                arc_offsets, props, nprops);
+                                arc_offsets, props);
   arc_update_data(arc);
 }
 
@@ -215,7 +213,7 @@ arc_distance_from(Arc *arc, Point *point)
 
 static void
 arc_select(Arc *arc, Point *clicked_point,
-	   Renderer *interactive_renderer)
+	   DiaRenderer *interactive_renderer)
 {
   arc_update_handles(arc);
 }
@@ -236,18 +234,21 @@ arc_update_handles(Arc *arc)
   dy = conn->endpoints[1].y - conn->endpoints[0].y;
   
   dist = sqrt(dx*dx + dy*dy);
-  middle_pos->x =
-    (conn->endpoints[0].x + conn->endpoints[1].x) / 2.0 -
-    arc->curve_distance*dy/dist;
-  middle_pos->y =
-    (conn->endpoints[0].y + conn->endpoints[1].y) / 2.0 +
-    arc->curve_distance*dx/dist;
+  if (dist > 0.000001) {
+    middle_pos->x =
+      (conn->endpoints[0].x + conn->endpoints[1].x) / 2.0 -
+      arc->curve_distance*dy/dist;
+    middle_pos->y =
+      (conn->endpoints[0].y + conn->endpoints[1].y) / 2.0 +
+      arc->curve_distance*dx/dist;
+  }
 }
 
 
-static void
+static ObjectChange*
 arc_move_handle(Arc *arc, Handle *handle,
-		Point *to, HandleMoveReason reason, ModifierKeys modifiers)
+		Point *to, ConnectionPoint *cp,
+		HandleMoveReason reason, ModifierKeys modifiers)
 {
   assert(arc!=NULL);
   assert(handle!=NULL);
@@ -271,13 +272,15 @@ arc_move_handle(Arc *arc, Handle *handle,
       arc->curve_distance = -arc->curve_distance;
 
   } else {
-    connection_move_handle(&arc->connection, handle->id, to, reason);
+    connection_move_handle(&arc->connection, handle->id, to, cp, reason, modifiers);
   }
 
   arc_update_data(arc);
+
+  return NULL;
 }
 
-static void
+static ObjectChange*
 arc_move(Arc *arc, Point *to)
 {
   Point start_to_end;
@@ -290,11 +293,14 @@ arc_move(Arc *arc, Point *to)
   point_add(&endpoints[1], &start_to_end);
 
   arc_update_data(arc);
+
+  return NULL;
 }
 
 static void
-arc_draw(Arc *arc, Renderer *renderer)
+arc_draw(Arc *arc, DiaRenderer *renderer)
 {
+  DiaRendererClass *renderer_ops = DIA_RENDERER_GET_CLASS (renderer);
   Point *endpoints;
   real width;
     
@@ -303,77 +309,35 @@ arc_draw(Arc *arc, Renderer *renderer)
 
   endpoints = &arc->connection.endpoints[0];
 
-  renderer->ops->set_linewidth(renderer, arc->line_width);
-  renderer->ops->set_linestyle(renderer, arc->line_style);
-  renderer->ops->set_dashlength(renderer, arc->dashlength);
-  renderer->ops->set_linecaps(renderer, LINECAPS_BUTT);
+  renderer_ops->set_linewidth(renderer, arc->line_width);
+  renderer_ops->set_linestyle(renderer, arc->line_style);
+  renderer_ops->set_dashlength(renderer, arc->dashlength);
+  renderer_ops->set_linecaps(renderer, LINECAPS_BUTT);
   
   /* Special case when almost line: */
   if (fabs(arc->curve_distance) <= 0.0001) {
-    renderer->ops->draw_line(renderer,
-			     &endpoints[0], &endpoints[1],
-			     &arc->arc_color);
+    renderer_ops->draw_line_with_arrows(renderer,
+					 &endpoints[0], &endpoints[1],
+					 arc->line_width,
+					 &arc->arc_color,
+					 &arc->start_arrow,
+					 &arc->end_arrow);
     return;
   }
-  
+
+  renderer_ops->draw_arc_with_arrows(renderer,
+				      &arc->connection.endpoints[0],
+				      &arc->connection.endpoints[1],
+				      &arc->middle_handle.pos,
+				      arc->line_width,
+				      &arc->arc_color,
+				      &arc->start_arrow,
+				      &arc->end_arrow);
   width = 2*arc->radius;
   
-  renderer->ops->draw_arc(renderer,
-			  &arc->center,
-			  width, width,
-			  arc->angle1, arc->angle2,
-			  &arc->arc_color);
-
-  if (arc->start_arrow.type != ARROW_NONE ||
-      arc->end_arrow.type != ARROW_NONE) {
-    Point reversepoint, centervec;
-    Point controlpoint0, controlpoint1;
-    real len, len2;
-
-    centervec = endpoints[0];
-    point_sub(&centervec, &endpoints[1]);
-    point_scale(&centervec, 0.5);
-    len = centervec.x*centervec.x+centervec.y*centervec.y;
-    point_add(&centervec, &endpoints[1]);
-    /* Centervec should now be the midpoint between [0] and [1] */
-    reversepoint = centervec;
-    point_sub(&centervec, &arc->center);
-    len2 = centervec.x*centervec.x+centervec.y*centervec.y;
-    if (len2 != 0) {
-      point_scale(&centervec, 1/len2);
-    }
-    point_scale(&centervec, len);
-    point_add(&reversepoint, &centervec);
-    controlpoint0 = controlpoint1 = reversepoint;
-    len = arc->angle2-arc->angle1;
-    if (len > 180 || (len < 0.0 && len > -180)) {
-      centervec = endpoints[0];
-      point_sub(&centervec, &reversepoint);
-      point_add(&controlpoint0, &centervec);
-      point_add(&controlpoint0, &centervec);
-      centervec = endpoints[1];
-      point_sub(&centervec, &reversepoint);
-      point_add(&controlpoint1, &centervec);
-      point_add(&controlpoint1, &centervec);
-    }
-    if (arc->start_arrow.type != ARROW_NONE) {
-      arrow_draw(renderer, arc->start_arrow.type,
-		 &endpoints[0],&controlpoint0,
-		 arc->start_arrow.length, arc->start_arrow.width,
-		 arc->line_width,
-		 &arc->arc_color, &color_white);
-    }
-    if (arc->end_arrow.type != ARROW_NONE) {
-      arrow_draw(renderer, arc->end_arrow.type,
-		 &endpoints[1], &controlpoint1,
-		 arc->end_arrow.length, arc->end_arrow.width,
-		 arc->line_width,
-		 &arc->arc_color, &color_white);
-    }
-  }
 }
 
-static Object *
+static DiaObject *
 arc_create(Point *startpoint,
 	   void *user_data,
 	   Handle **handle1,
@@ -381,7 +345,7 @@ arc_create(Point *startpoint,
 {
   Arc *arc;
   Connection *conn;
-  Object *obj;
+  DiaObject *obj;
   Point defaultlen = { 1.0, 1.0 };
 
   arc = g_malloc0(sizeof(Arc));
@@ -424,12 +388,12 @@ arc_destroy(Arc *arc)
   connection_destroy(&arc->connection);
 }
 
-static Object *
+static DiaObject *
 arc_copy(Arc *arc)
 {
   Arc *newarc;
   Connection *conn, *newconn;
-  Object *newobj;
+  DiaObject *newobj;
   
   conn = &arc->connection;
   
@@ -462,8 +426,8 @@ static void
 arc_update_data(Arc *arc)
 {
   Connection *conn = &arc->connection;
-  ConnectionBBExtras *extra =&conn->extra_spacing;
-  Object *obj = &conn->object;
+  LineBBExtras *extra =&conn->extra_spacing;
+  DiaObject *obj = &conn->object;
   Point *endpoints;
   real x1,y1,x2,y2,xc,yc;
   real lensq, alpha, radius;
@@ -580,12 +544,12 @@ arc_save(Arc *arc, ObjectNode obj_node, const char *filename)
   }
 }
 
-static Object *
+static DiaObject *
 arc_load(ObjectNode obj_node, int version, const char *filename)
 {
   Arc *arc;
   Connection *conn;
-  Object *obj;
+  DiaObject *obj;
   AttributeNode attr;
 
   arc = g_malloc0(sizeof(Arc));
@@ -625,8 +589,8 @@ arc_load(ObjectNode obj_node, int version, const char *filename)
 
 
   arc->start_arrow.type = ARROW_NONE;
-  arc->start_arrow.length = 0.8;
-  arc->start_arrow.width = 0.8;
+  arc->start_arrow.length = DEFAULT_ARROW_LENGTH;
+  arc->start_arrow.width = DEFAULT_ARROW_WIDTH;
   attr = object_find_attribute(obj_node, "start_arrow");
   if (attr != NULL)
     arc->start_arrow.type = data_enum(attribute_first_data(attr));
@@ -638,8 +602,8 @@ arc_load(ObjectNode obj_node, int version, const char *filename)
     arc->start_arrow.width = data_real(attribute_first_data(attr));
 
   arc->end_arrow.type = ARROW_NONE;
-  arc->end_arrow.length = 0.8;
-  arc->end_arrow.width = 0.8;
+  arc->end_arrow.length = DEFAULT_ARROW_LENGTH;
+  arc->end_arrow.width = DEFAULT_ARROW_WIDTH;
   attr = object_find_attribute(obj_node, "end_arrow");
   if (attr != NULL)
     arc->end_arrow.type = data_enum(attribute_first_data(attr));
