@@ -59,7 +59,6 @@ typedef struct _Line {
 struct _LineProperties {
   real absolute_start_gap, absolute_end_gap;
   real fractional_start_gap, fractional_end_gap;
-  gboolean object_edge_start, object_edge_end;
 };
 
 static LineProperties default_properties;
@@ -72,21 +71,21 @@ static ObjectChange* line_move(Line *line, Point *to);
 static void line_select(Line *line, Point *clicked_point,
 			DiaRenderer *interactive_renderer);
 static void line_draw(Line *line, DiaRenderer *renderer);
-static Object *line_create(Point *startpoint,
+static DiaObject *line_create(Point *startpoint,
 			   void *user_data,
 			   Handle **handle1,
 			   Handle **handle2);
 static real line_distance_from(Line *line, Point *point);
 static void line_update_data(Line *line);
 static void line_destroy(Line *line);
-static Object *line_copy(Line *line);
+static DiaObject *line_copy(Line *line);
 
 static PropDescription *line_describe_props(Line *line);
 static void line_get_props(Line *line, GPtrArray *props);
 static void line_set_props(Line *line, GPtrArray *props);
 
 static void line_save(Line *line, ObjectNode obj_node, const char *filename);
-static Object *line_load(ObjectNode obj_node, int version, const char *filename);
+static DiaObject *line_load(ObjectNode obj_node, int version, const char *filename);
 static DiaMenu *line_get_object_menu(Line *line, Point *clickedpoint);
 
 void calculate_gap_endpoints(Line *line, Point *gap_endpoints);
@@ -100,7 +99,7 @@ static ObjectTypeOps line_type_ops =
   (ApplyDefaultsFunc) NULL
 };
 
-ObjectType line_type =
+DiaObjectType line_type =
 {
   "Standard - Line",   /* name */
   0,                   /* version */
@@ -108,7 +107,7 @@ ObjectType line_type =
   &line_type_ops       /* ops */
 };
 
-ObjectType *_line_type = (ObjectType *) &line_type;
+DiaObjectType *_line_type = (DiaObjectType *) &line_type;
 
 static ObjectOps line_ops = {
   (DestroyFunc)         line_destroy,
@@ -151,10 +150,6 @@ static PropDescription line_props[] = {
     N_("Fractional start gap"), NULL, &gap_range },
   { "fractional_end_gap", PROP_TYPE_REAL, 0,
     N_("Fractional end gap"), NULL, &gap_range },
-  { "object_edge_start", PROP_TYPE_BOOL, 0,
-    N_("Start at object edge"), },
-  { "object_edge_end", PROP_TYPE_BOOL, 0,
-    N_("End at object edge"), },
   PROP_FRAME_END("gaps",0),
   */
   PROP_DESC_END
@@ -183,8 +178,6 @@ static PropOffset line_offsets[] = {
   { "absolute_end_gap", PROP_TYPE_REAL, offsetof(Line, absolute_end_gap) },
   { "fractional_start_gap", PROP_TYPE_REAL, offsetof(Line, fractional_start_gap) },
   { "fractional_end_gap", PROP_TYPE_REAL, offsetof(Line, fractional_end_gap) },
-  { "object_edge_start", PROP_TYPE_BOOL, offsetof(Line, object_edge_start) },
-  { "object_edge_end", PROP_TYPE_BOOL, offsetof(Line, object_edge_end) },
   */
   { NULL, 0, 0 }
 };
@@ -213,14 +206,12 @@ line_init_defaults() {
     default_properties.absolute_end_gap = 0.0;
     default_properties.fractional_start_gap = 0;
     default_properties.fractional_end_gap = 0;
-    default_properties.object_edge_start = FALSE;
-    default_properties.object_edge_end = FALSE;
     defaults_initialized = 1;
   }
 }
 
 static ObjectChange *
-line_add_connpoint_callback(Object *obj, Point *clicked, gpointer data) 
+line_add_connpoint_callback(DiaObject *obj, Point *clicked, gpointer data) 
 {
   ObjectChange *oc;
   oc = connpointline_add_point(((Line *)obj)->cpl,clicked);
@@ -229,7 +220,7 @@ line_add_connpoint_callback(Object *obj, Point *clicked, gpointer data)
 }
 
 static ObjectChange *
-line_remove_connpoint_callback(Object *obj, Point *clicked, gpointer data) 
+line_remove_connpoint_callback(DiaObject *obj, Point *clicked, gpointer data) 
 {
   ObjectChange *oc;
   oc = connpointline_remove_point(((Line *)obj)->cpl,clicked);
@@ -279,8 +270,9 @@ line_get_object_menu(Line *line, Point *clickedpoint)
   length which could be optionally added to the object intersection
   gap.
 */
+/* This is currently dead code, but may find new use in other places. */
 Point
-calculate_object_edge(Point *objmid, Point *end, Object *obj) 
+calculate_object_edge(Point *objmid, Point *end, DiaObject *obj) 
 {
 #define MAXITER 25
 #ifdef TRACE_DIST
@@ -463,7 +455,7 @@ line_draw(Line *line, DiaRenderer *renderer)
   }
 }
 
-static Object *
+static DiaObject *
 line_create(Point *startpoint,
 	    void *user_data,
 	    Handle **handle1,
@@ -471,7 +463,7 @@ line_create(Point *startpoint,
 {
   Line *line;
   Connection *conn;
-  Object *obj;
+  DiaObject *obj;
   Point defaultlen = { 1.0, 1.0 };
 
   line_init_defaults();
@@ -484,8 +476,6 @@ line_create(Point *startpoint,
   line->absolute_end_gap = default_properties.absolute_end_gap;
   line->fractional_start_gap = default_properties.fractional_start_gap;
   line->fractional_end_gap = default_properties.fractional_end_gap;
-  line->object_edge_start = default_properties.object_edge_start;
-  line->object_edge_end = default_properties.object_edge_end;
     
   conn = &line->connection;
   conn->endpoints[0] = *startpoint;
@@ -517,12 +507,12 @@ line_destroy(Line *line)
   connection_destroy(&line->connection);
 }
 
-static Object *
+static DiaObject *
 line_copy(Line *line)
 {
   Line *newline;
   Connection *conn, *newconn;
-  Object *newobj;
+  DiaObject *newobj;
   int rcc = 0;
 
   conn = &line->connection;
@@ -557,7 +547,7 @@ static void
 line_update_data(Line *line)
 {
   Connection *conn = &line->connection;
-  Object *obj = &conn->object;
+  DiaObject *obj = &conn->object;
   LineBBExtras *extra = &conn->extra_spacing;
   Point start, end;
 
@@ -644,24 +634,18 @@ line_save(Line *line, ObjectNode obj_node, const char *filename)
   if (line->fractional_end_gap)
     data_add_real(new_attribute(obj_node, "fractional_end_gap"),
                  line->fractional_end_gap);
-  if (line->object_edge_start)
-    data_add_boolean(new_attribute(obj_node, "object_edge_start"),
-		     line->object_edge_start);
-  if (line->object_edge_end)
-    data_add_boolean(new_attribute(obj_node, "object_edge_end"),
-		     line->object_edge_end);
 
   if (line->line_style != LINESTYLE_SOLID && line->dashlength != DEFAULT_LINESTYLE_DASHLEN)
     data_add_real(new_attribute(obj_node, "dashlength"),
 		  line->dashlength);
 }
 
-static Object *
+static DiaObject *
 line_load(ObjectNode obj_node, int version, const char *filename)
 {
   Line *line;
   Connection *conn;
-  Object *obj;
+  DiaObject *obj;
   AttributeNode attr;
 
   line = g_malloc0(sizeof(Line));
@@ -690,8 +674,8 @@ line_load(ObjectNode obj_node, int version, const char *filename)
     line->line_style = data_enum(attribute_first_data(attr));
 
   line->start_arrow.type = ARROW_NONE;
-  line->start_arrow.length = 0.8;
-  line->start_arrow.width = 0.8;
+  line->start_arrow.length = DEFAULT_ARROW_LENGTH;
+  line->start_arrow.width = DEFAULT_ARROW_WIDTH;
   attr = object_find_attribute(obj_node, "start_arrow");
   if (attr != NULL)
     line->start_arrow.type = data_enum(attribute_first_data(attr));
@@ -703,8 +687,8 @@ line_load(ObjectNode obj_node, int version, const char *filename)
     line->start_arrow.width = data_real(attribute_first_data(attr));
 
   line->end_arrow.type = ARROW_NONE;
-  line->end_arrow.length = 0.8;
-  line->end_arrow.width = 0.8;
+  line->end_arrow.length = DEFAULT_ARROW_LENGTH;
+  line->end_arrow.width = DEFAULT_ARROW_WIDTH;
   attr = object_find_attribute(obj_node, "end_arrow");
   if (attr != NULL)
     line->end_arrow.type = data_enum(attribute_first_data(attr));
@@ -731,14 +715,6 @@ line_load(ObjectNode obj_node, int version, const char *filename)
   attr = object_find_attribute(obj_node, "fractional_end_gap");
   if (attr != NULL)
     line->fractional_end_gap =  data_real( attribute_first_data(attr) );
-  line->object_edge_start = FALSE;
-  attr = object_find_attribute(obj_node, "object_edge_start");
-  if (attr != NULL)
-    line->object_edge_start = data_boolean( attribute_first_data(attr) );
-  line->object_edge_end = FALSE;
-  attr = object_find_attribute(obj_node, "object_edge_end");
-  if (attr != NULL)
-    line->object_edge_end = data_boolean( attribute_first_data(attr) );
 
   line->dashlength = DEFAULT_LINESTYLE_DASHLEN;
   attr = object_find_attribute(obj_node, "dashlength");
