@@ -41,14 +41,19 @@ extern "C" {
 }
 #endif
 
+#if defined HAVE_WINDOWS_H || defined G_OS_WIN32
 namespace W32 {
 // at least Rectangle conflicts ...
 #include <windows.h>
 }
+#else
+#include "wmf_gdi.h"
+#define SAVE_EMF
+#endif
 
 /* force linking with gdi32 */
 #pragma comment( lib, "gdi32" )
- 
+
 
 // #define SAVE_EMF
 
@@ -97,6 +102,9 @@ struct _WmfRenderer
     PLACEABLEMETAHEADER pmh;
     double xoff, yoff;
     double scale;
+
+    int nDashLen; /* the scaled dash length */
+    gboolean platform_is_nt; /* advanced line styles supported */
 };
 
 struct _WmfRendererClass
@@ -110,13 +118,13 @@ G_END_DECLS
  * helper macros
  */
 #define W32COLOR(c) \
-	 0xff * c->red + \
+	(W32::COLORREF)( 0xff * c->red + \
 	((unsigned char)(0xff * c->green)) * 256 + \
-	((unsigned char)(0xff * c->blue)) * 65536;
+	((unsigned char)(0xff * c->blue)) * 65536)
 
-#define SC(a) ((a)*renderer->scale)
-#define SCX(a) (((a)+renderer->xoff)*renderer->scale)
-#define SCY(a) (((a)+renderer->yoff)*renderer->scale)
+#define SC(a) ((int)((a)*renderer->scale))
+#define SCX(a) ((int)(((a)+renderer->xoff)*renderer->scale))
+#define SCY(a) ((int)(((a)+renderer->yoff)*renderer->scale))
 
 /*
  * helper functions
@@ -127,9 +135,59 @@ UsePen(WmfRenderer* renderer, Color* colour)
     W32::HPEN hOldPen;
     if (colour) {
 	W32::COLORREF rgb = W32COLOR(colour);
-	renderer->hPen = W32::CreatePen(renderer->fnPenStyle, 
-					renderer->nLineWidth,
-					rgb);
+#ifdef G_OS_WIN32
+	if (renderer->platform_is_nt && renderer->hPrintDC) {
+          W32::LOGBRUSH logbrush;
+	  W32::DWORD    dashes[6];
+	  int num_dashes = 0;
+	  int dashlen = renderer->nDashLen;
+	  int dotlen  = renderer->nDashLen / 10;
+	  
+	  logbrush.lbStyle = BS_SOLID;
+	  logbrush.lbColor = rgb;
+	  logbrush.lbHatch = 0;
+
+          switch (renderer->fnPenStyle & PS_STYLE_MASK) {
+	  case PS_SOLID :
+	    break;
+	  case PS_DASH :
+	    num_dashes = 2;
+	    dashes[0] = dashes[1] = dashlen; 
+	    break;
+	  case PS_DASHDOT :
+	    num_dashes = 4;
+	    dashes[1] = dashes[3] = MAX((dashlen - dotlen) / 2, 1);
+	    dashes[0] = dashlen;
+	    dashes[2] = dotlen;
+	    break;
+	  case PS_DASHDOTDOT :
+	    num_dashes = 6;
+	    dashes[0] = dashlen;
+	    dashes[1] = dashes[3] = dashes[5] = MAX((dashlen - 2 * dotlen)/3, 1);
+	    dashes[2] = dashes[4] = dotlen;
+	    break;
+	  case PS_DOT :
+	    num_dashes = 2;
+	    dashes[0] = dashes[1] = dotlen;
+	    break;
+	  default :
+	    g_assert_not_reached ();
+	  }
+
+	  renderer->hPen = W32::ExtCreatePen (
+	    renderer->fnPenStyle & PS_SOLID ? renderer->fnPenStyle :
+	      (renderer->fnPenStyle & ~(PS_STYLE_MASK)) | (PS_GEOMETRIC | PS_USERSTYLE),
+	    renderer->nLineWidth,
+	    &logbrush, num_dashes, dashes);
+
+	}
+	else 
+#endif /* G_OS_WIN32 */
+	{
+	  renderer->hPen = W32::CreatePen(renderer->fnPenStyle, 
+					  renderer->nLineWidth,
+					  rgb);
+	}
     } else {
 	renderer->hPen = (W32::HPEN)W32::GetStockObject(NULL_PEN);
     }
@@ -141,7 +199,8 @@ static void
 DonePen(WmfRenderer* renderer, W32::HPEN hPen)
 {
     /* restore the OLD one ... */
-    if (hPen) W32::SelectObject(renderer->hFileDC, hPen);
+    if (hPen) 
+      W32::SelectObject(renderer->hFileDC, hPen);
     /* ... before deleting the last active */
     if (renderer->hPen)
     {
@@ -179,6 +238,10 @@ begin_render(DiaRenderer *self)
 
     DIAG_NOTE(renderer, "begin_render\n");
 
+    /* FIXME: still not sure if the renderer output should be platform dependent */
+    if (renderer->platform_is_nt)
+      renderer->fnPenStyle = PS_GEOMETRIC;
+
     /* make unfilled the default */
     W32::SelectObject(renderer->hFileDC, 
 	                W32::GetStockObject (HOLLOW_BRUSH) );
@@ -252,14 +315,22 @@ set_linecaps(DiaRenderer *self, LineCaps mode)
     WmfRenderer *renderer = WMF_RENDERER (self);
 
     DIAG_NOTE(renderer, "set_linecaps %d\n", mode);
+	
+    // all the advanced line rendering is unsupported on win9x
+    if (!renderer->platform_is_nt)
+      return;
 
+    renderer->fnPenStyle &= ~(PS_ENDCAP_MASK);
     switch(mode) {
     case LINECAPS_BUTT:
-	break;
+      renderer->fnPenStyle |= PS_ENDCAP_FLAT;
+      break;
     case LINECAPS_ROUND:
-	break;
+      renderer->fnPenStyle |= PS_ENDCAP_ROUND;
+      break;
     case LINECAPS_PROJECTING:
-	break;
+      renderer->fnPenStyle |= PS_ENDCAP_SQUARE;
+      break;
     default:
 	message_error("WmfRenderer : Unsupported fill mode specified!\n");
     }
@@ -272,15 +343,22 @@ set_linejoin(DiaRenderer *self, LineJoin mode)
 
     DIAG_NOTE(renderer, "set_join %d\n", mode);
 
+    if (!renderer->platform_is_nt)
+      return;
+
+    renderer->fnPenStyle &= ~(PS_JOIN_MASK);
     switch(mode) {
     case LINEJOIN_MITER:
-	break;
+      renderer->fnPenStyle |= PS_JOIN_MITER;
+      break;
     case LINEJOIN_ROUND:
-	break;
+      renderer->fnPenStyle |= PS_JOIN_ROUND;
+      break;
     case LINEJOIN_BEVEL:
-	break;
+      renderer->fnPenStyle |= PS_JOIN_BEVEL;
+      break;
     default:
-	message_error("WmfRenderer : Unsupported fill mode specified!\n");
+      message_error("WmfRenderer : Unsupported fill mode specified!\n");
     }
 }
 
@@ -292,28 +370,34 @@ set_linestyle(DiaRenderer *self, LineStyle mode)
     DIAG_NOTE(renderer, "set_linestyle %d\n", mode);
 
     /* line type */
+    renderer->fnPenStyle &= ~(PS_STYLE_MASK);
     switch (mode) {
     case LINESTYLE_SOLID:
-      renderer->fnPenStyle = PS_SOLID;
+      renderer->fnPenStyle |= PS_SOLID;
       break;
     case LINESTYLE_DASHED:
-      renderer->fnPenStyle = PS_DASH;
+      renderer->fnPenStyle |= PS_DASH;
       break;
     case LINESTYLE_DASH_DOT:
-      renderer->fnPenStyle = PS_DASHDOT;
+      renderer->fnPenStyle |= PS_DASHDOT;
       break;
     case LINESTYLE_DASH_DOT_DOT:
-      renderer->fnPenStyle = PS_DASHDOTDOT;
+      renderer->fnPenStyle |= PS_DASHDOTDOT;
       break;
     case LINESTYLE_DOTTED:
-      renderer->fnPenStyle = PS_DOT;
+      renderer->fnPenStyle |= PS_DOT;
       break;
     default:
 	message_error("WmfRenderer : Unsupported fill mode specified!\n");
     }
     
+    if (renderer->platform_is_nt)
+      return;
+
     /* Non-solid linestyles are only displayed if width <= 1. 
-     * Better implementation will require custom linestyles */
+     * Better implementation will require custom linestyles
+     * not available on win9x ...
+     */
     switch (mode) {
     case LINESTYLE_DASHED:
     case LINESTYLE_DASH_DOT:
@@ -331,7 +415,8 @@ set_dashlength(DiaRenderer *self, real length)
 
     DIAG_NOTE(renderer, "set_dashlength %f\n", length);
 
-    /* dot = 20% of len */
+    /* dot = 10% of len */
+    renderer->nDashLen = SC(length);
 }
 
 static void
@@ -722,6 +807,8 @@ draw_bezier(DiaRenderer *self,
     g_free(pts);
 }
 
+#ifndef SAVE_EMF
+/* not defined in compatibility layer */
 static void
 fill_bezier(DiaRenderer *self, 
 	    BezPoint *points, /* Last point must be same as first point */
@@ -747,6 +834,7 @@ fill_bezier(DiaRenderer *self,
                       W32::GetStockObject (HOLLOW_BRUSH) );
     W32::DeleteObject(hBrush);
 }
+#endif
 
 static void
 draw_string(DiaRenderer *self,
@@ -821,6 +909,9 @@ draw_image(DiaRenderer *self,
 	   DiaImage image)
 {
     WmfRenderer *renderer = WMF_RENDERER (self);
+#ifdef SAVE_EMF
+    /* not yet supported in compatibility mode */
+#else	
     W32::HBITMAP hBmp;
     int iWidth, iHeight;
     unsigned char* pData = NULL;
@@ -914,8 +1005,11 @@ draw_image(DiaRenderer *self,
         g_free (pData);
     if (pImg)
         g_free (pImg);
+#endif /* SAVE_EMF */
 }
 
+//XXX: use RoundRect
+
 /* GObject boiler plate */
 static void wmf_renderer_class_init (WmfRendererClass *klass);
 
@@ -998,7 +1092,9 @@ wmf_renderer_class_init (WmfRendererClass *klass)
   renderer_class->draw_polygon   = draw_polygon;
 
   renderer_class->draw_bezier   = draw_bezier;
+#ifndef SAVE_EMF
   renderer_class->fill_bezier   = fill_bezier;
+#endif
 }
 
 /* plug-in export api */
@@ -1024,8 +1120,8 @@ export_data(DiagramData *data, const gchar *filename,
     else {
         scale = floor (32000.0 / (data->extents.bottom - data->extents.top));
     }
-    bbox.right = (data->extents.right - data->extents.left) * scale;
-    bbox.bottom = (data->extents.bottom - data->extents.top) * scale;
+    bbox.right = (int)((data->extents.right - data->extents.left) * scale);
+    bbox.bottom = (int)((data->extents.bottom - data->extents.top) * scale);
 
     file = (W32::HDC)W32::CreateEnhMetaFile(
                     W32::GetDC(NULL), // handle to a reference device context
@@ -1047,7 +1143,10 @@ export_data(DiagramData *data, const gchar *filename,
     renderer->hFileDC = file;
     renderer->sFileName = g_strdup(filename);
     renderer->hPrintDC = (W32::HDC)user_data;
-           
+
+    /* printing is platform dependent */
+    renderer->platform_is_nt = (W32::GetVersion () < 0x80000000);
+
     extent = &data->extents;
 
     /* calculate offsets */
@@ -1061,8 +1160,8 @@ export_data(DiagramData *data, const gchar *filename,
     renderer->pmh.Handle = 0;
     renderer->pmh.Left   = 0;
     renderer->pmh.Top    = 0;
-    renderer->pmh.Right  = SC(data->extents.right - data->extents.left) * 25.4;
-    renderer->pmh.Bottom = SC(data->extents.bottom - data->extents.top) * 25.4;
+    renderer->pmh.Right  = (gint16)(SC(data->extents.right - data->extents.left) * 25.4);
+    renderer->pmh.Bottom = (gint16)(SC(data->extents.bottom - data->extents.top) * 25.4);
     renderer->pmh.Inch   = 1440 * 10;
     renderer->pmh.Reserved = 0;
 
@@ -1071,11 +1170,16 @@ export_data(DiagramData *data, const gchar *filename,
     for (ptr = (guint16 *)&renderer->pmh; ptr < (guint16 *)&(renderer->pmh.Checksum); ptr++)
         renderer->pmh.Checksum ^= *ptr;
 
+#ifdef SAVE_EMF
+    /* write the placeable header */
+    fwrite(&renderer->pmh, 1, 22 /* NOT: sizeof(PLACEABLEMETAHEADER) */, file->file);
+#endif
+    
     /* bounding box in device units */
     bbox.left = 0;
     bbox.top = 0;
-    bbox.right = SC(data->extents.right - data->extents.left) * 25.4;
-    bbox.bottom = SC(data->extents.bottom - data->extents.top) * 25.4;
+    bbox.right = (int)(SC(data->extents.right - data->extents.left) * 25.4);
+    bbox.bottom = (int)(SC(data->extents.bottom - data->extents.top) * 25.4);
 
     /* initialize drawing */
     W32::SetBkMode(renderer->hFileDC, TRANSPARENT);
@@ -1099,7 +1203,9 @@ static const gchar *extensions[] = { "wmf", NULL };
 static DiaExportFilter my_export_filter = {
     N_("Windows Meta File"),
     extensions,
-    export_data
+    export_data,
+    NULL, /* user data */
+    "wmf::native"
 };
 
 
@@ -1115,7 +1221,14 @@ dia_plugin_init(PluginInfo *info)
                               NULL, NULL))
         return DIA_PLUGIN_INIT_ERROR;
 
+#ifdef G_OS_WIN32
+    /*
+     * On non windoze platforms this plug-in currently is only
+     * useful at compile/develoment time. The output is broken
+     * when processed by wmf_gdi.cpp ...
+     */
     filter_register_export(&my_export_filter);
+#endif
 
     return DIA_PLUGIN_INIT_OK;
 }
